/* 
 *  linux/arch/arm/mach-pxa/imm/pasr-stack.S
 *
 *  Intel Memory Management
 *
 *  Const kernel stack/data for use with PASR implementation
 *
 *  Todd Brandt
 *  Copyright (c) 2004, Intel Corporation (todd.e.brandt@intel.com)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation. 

 *(C) Copyright 2006 Marvell International Ltd.  
 * All Rights Reserved 
 */

#include <linux/config.h>
#include <linux/linkage.h>
#include <asm/assembler.h>
#include <asm/hardware.h>
#include <asm/arch/pxa-regs.h>

		.text
/*
 * imm_power_down is the function which compresses and prepares memory for powerdown.
 * it is designed to transparently usurp all subsequently accessed memory:
 * most especially the stack and the page table as those regions could
 * lie in physical memory that is not pasr safe.
 */
ENTRY(imm_power_down)
	ldr     r10, =imm_pasr_asm_data		@ get the address of the imm asm data
	str     sp, [r10, #8]			@ store the current sp
	ldr	sp, =imm_pasr_stack_begin	@ load up the pasr safe sp value
        stmfd   sp!, {r0 - r9, lr}              @ save registers on pasr-safe stack
	bl	pasr_calculate			@ create the compression table
#ifdef CONFIG_IMM_API_SRAM
	bl	sram_powersave_enable		@ shut unused SRAM banks down
#endif
	bl	pasr_memory_compress		@ perform memory compression
	bl	xscale_flush_kern_cache_all	@ clean data cache
#ifdef CONFIG_IMM_POWER_TEST
	ldr	r2, =OSCR			@ get the address of the counter
	ldr     r0, =imm_pasr_cycles		@ get the address of the cycle count data
	ldr	r1, [r2]			@ get the value of the counter
	str	r1, [r0, #4]			@ store the value indicating the stamp
#endif
	ldmfd   sp!, {r0 - r9, pc}              @ Restore regs and return to caller


/*
 * imm_power_up is the function which decompresses and prepares memory for powerup.
 * it is designed to replace all usurped memory regions to their original values
 * transparently to any code that executes in between imm_power_down and imm_power_up.
 */
ENTRY(imm_power_up)
        stmfd   sp!, {r0 - r9, lr}              @ save registers on pasr-safe stack
#ifdef CONFIG_IMM_POWER_TEST
	ldr	r2, =OSCR			@ get the address of the counter
	ldr     r0, =imm_pasr_cycles		@ get the address of the cycle count data
	ldr	r1, [r2]			@ get the value of the counter
	str	r1, [r0, #8]			@ store the value indicating the stamp
#endif
	bl	pasr_memory_decompress		@ perform memory decompress
	bl	xscale_flush_kern_cache_all	@ clean data cache
#ifdef CONFIG_IMM_API_SRAM
	bl	sram_powersave_disable		@ turn SRAM back on
#endif
	ldmfd   sp!, {r0 - r10}			@ Restore regs from pasr-safe stack
	ldr	sp, =imm_pasr_asm_data		@ grab the address of the imm asm data
	ldr	sp, [sp, #8]			@ load up the old sp
	mov	pc, r10				@ return to caller

        .data
	.global imm_pasr_asm_data
	.global imm_pasr_stack_begin
	.global imm_pasr_stack_end
	.global imm_pasr_cycles
	.align  2

imm_pasr_cycles:	@ used to time IMM power functionality
	.word	0	@ OSCR at start of powerdown
	.word	0	@ OSCR immediately before actual powerdown instruction
	.word	0	@ OSCR immediately after actual powerdown instruction
	.word	0	@ OSCR at exit of powerdown

imm_pasr_asm_data:			@ data used for PASR
	.word   imm_pasr_safe_pgd	@ the pasr safe page table
	.word	imm_pasr_disable_once	@ flag telling us if pasr is to be used
	.word	0			@ storage for the pasr-unsafe SP

imm_pasr_stack_end:	@ 32 entries of PASR safe stack, this may need to increase
	.word	0	@ depending on how many functions are called with it
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0
	.word	0	
	.word	0
	.word	0
	.word	0	@ depending on how many functions are called with it
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0
	.word	0	
	.word	0
	.word	0
	.word	0	@ depending on how many functions are called with it
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0
	.word	0	
	.word	0
	.word	0
	.word	0	@ depending on how many functions are called with it
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0
	.word	0	
	.word	0
	.word	0
	.word	0	@ depending on how many functions are called with it
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0
	.word	0	
	.word	0
	.word	0
	.word	0	@ depending on how many functions are called with it
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0
	.word	0	
	.word	0
	.word	0
	.word	0	@ depending on how many functions are called with it
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0
	.word	0	
	.word	0
	.word	0
	.word	0	@ depending on how many functions are called with it
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0	
	.word	0
	.word	0	
	.word	0
	.word	0
imm_pasr_stack_begin:
