#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/errno.h>
#include <linux/netlink.h>
#include <net/sock.h>
#include <linux/msgevent.h>

#ifdef _USE_FILE_NOTE_
#include <linux/cdev.h>
#include <linux/sched.h>
#include <linux/fs.h>
#include <linux/fcntl.h>
#endif

#ifdef _USE_NETLINK_

#define NETLINK_EXTPOWER  17
static struct sock *nlfd = NULL; 

void nl_data_recv(struct sk_buff *__skb)  
{  
	return;       
}

void msgevent_report_broadcast(struct MSG_DATA * msg)
{
	if(!nlfd)
		return;
		
	struct MSG_DATA *put_data=NULL;
	int len = sizeof(struct MSG_DATA);
	
	struct sk_buff *skb; 
	struct nlmsghdr *nlhdr;  
   
	skb = alloc_skb(NLMSG_SPACE(len), GFP_ATOMIC);
    if(skb) 
    {
		nlhdr = (struct nlmsghdr *)skb->data;
		/* add header */
		nlhdr = skb_put(skb, NLMSG_SPACE(len));
		nlhdr->nlmsg_len = NLMSG_LENGTH(len);
		
		nlhdr->nlmsg_pid = 0; // Send from kernel
		nlhdr->nlmsg_flags = 0;

		put_data = NLMSG_DATA(nlhdr);
		memset(put_data, 0, len);
		put_data->msgID=msg->msgID;
		MESSAGE_ID_TYPE msgtype=(MESSAGE_ID_TYPE)put_data->msgID;
		switch (msgtype)
		{
			case PowerMessage://power
				put_data->data.powerstate.ischarging = msg->data.powerstate.ischarging;
				put_data->data.powerstate.battery_power = msg->data.powerstate.battery_power;
				break;
			case AudioMessage://audio
				put_data->data.jackstate = msg->data.jackstate;
				break;
			case UsbMessage://usb
				put_data->data.usbcabstate = msg->data.usbcabstate;
				break;	
			case SDMessage://sd
				put_data->data.sdstate=msg->data.sdstate;
				break;
			case WifiMessage://wifi
				put_data->data.wifistate=msg->data.wifistate;
				break;
			case SensorBoardDataMessage://sensor board send data
				put_data->data.senvalue = msg->data.senvalue;
				break;
			case sensorboradLockMessage://sensor board enable/disable
				put_data->data.senlock = msg->data.senlock;
				break;
			case SensorBoardSwitchMessage://sensor board switch state
				put_data->data.senswitchstate = msg->data.senswitchstate;
				break;
			case PowerModeMessage: //power mode state
				put_data->data.powermodestate=msg->data.powermodestate;
				break;
		}
		
		NETLINK_CB(skb).dst_group = 1;//1 for group id 
		netlink_broadcast(nlfd, skb, 0, 1, GFP_ATOMIC/*GFP_KERNEL*/);
	  } 
	return;
}
EXPORT_SYMBOL(msgevent_report_broadcast);

#endif //_USE_NETLINK_

#ifdef _USE_FILE_NOTE_

static wait_queue_head_t msgevent_que;
static LIST_HEAD(msg_event_list);
spinlock_t lock;
int bwaiting=0;

int msgevent_report_broadcast(struct MSG_DATA * msgdata){
	if(!msgdata)
		return 0;
	
	printk(KERN_INFO "msgevent: report msgid: %d\n",msgdata->msgID);
	int flags;
	spin_lock_irqsave(&lock,flags);
	struct MSG_DATA *msg=(struct MSG_DATA* )kmalloc(sizeof(struct MSG_DATA),GFP_KERNEL);
	memcpy(msg,msgdata,sizeof(struct MSG_DATA));
	
	printk(KERN_INFO "msgevent: list_add_tail start\n");
	list_add_tail(&msg->note,&msg_event_list);
	printk(KERN_INFO "msgevent: list_add_tail end\n");
	
	if(waitqueue_active(&msgevent_que))
		wake_up_interruptible(&msgevent_que);
	
	printk(KERN_INFO "msgevent: wake_up_interruptible end\n");
	spin_unlock_irqrestore(&lock,flags);
}
EXPORT_SYMBOL(msgevent_report_broadcast);

ssize_t msgevent_open(struct inode *inode,struct file *file)
{
	//MOD_INC_USE_COUNT
	return 0;
}
ssize_t msgevent_release(struct inode *inode,struct file *file)
{
	if(!list_empty(&msg_event_list)){
		struct list_head *pos;
		list_for_each(pos, &msg_event_list){
		
			struct MSG_DATA *msgdata = list_entry(pos, struct MSG_DATA,
				note);
			if(msgdata)	
				kfree(msgdata);
		}
	}
	
	struct list_head *p_report_head=&msg_event_list;
	while (!list_empty(p_report_head)) {
		list_del_init(p_report_head->next);
	}
		
	printk(KERN_INFO "msgevent_release\n");
	//MOD_DEC_USE_COUNT
	return 0;
}
ssize_t msgevent_read(struct file *file,char *buf,size_t count,loff_t *f_ops)
{
	DECLARE_WAITQUEUE(wait, current);
	add_wait_queue(&msgevent_que, &wait);
	
	printk(KERN_INFO "msgevent: wait event start\n");
	bwaiting=1;
	wait_event_interruptible(msgevent_que,!list_empty(&msg_event_list));
	if(signal_pending(current))
		goto out1;
		
	bwaiting=0;
	printk(KERN_INFO "msgevent: wait event end\n");
	 
	if(list_empty(&msg_event_list)){
		printk(KERN_INFO "msgevent: read empty\n");
		return 0;
	}
	
	int flags;
	
	struct list_head *pos;
	struct MSG_DATA *msgdata=0;
	list_for_each(pos, &msg_event_list){
		
		msgdata = list_entry(pos, struct MSG_DATA,note);
		if(msgdata)
			break;
	}	
	//struct NetLink_MSG_DATA *msgdata = container_of(msg_event_list.next, struct NetLink_MSG_DATA,note);
	if(!msgdata){
		printk(KERN_INFO "msgdata is null\n");
	}
	printk(KERN_INFO "msgevent: copy_to_user msgid: %d\n",msgdata->msgID);
				
	if(copy_to_user(buf,msgdata,sizeof(struct MSG_DATA))){ 
		printk(KERN_ALERT "msgevent_read fail\n");
	}
	
	printk(KERN_INFO "list_del_init start: 0x%lx\n",msgdata);
	
	//list_del(&msgdata->note);
	spin_lock_irqsave(&lock,flags);
    list_del_init(&msgdata->note);
    spin_unlock_irqrestore(&lock,flags);
    
    printk(KERN_INFO "list_del_init end\n");
    
	kfree(msgdata);
	
	printk(KERN_INFO "kfree end\n");
	
out1:
    remove_wait_queue(&msgevent_que, &wait);
    set_current_state(TASK_RUNNING);
    
	return sizeof(struct MSG_DATA);
}

struct file_operations msgevent_ops = {
       open:   msgevent_open,
       read:   msgevent_read,
       write:  NULL,
       ioctl:  NULL,
       release:  msgevent_release,
};

#endif

#define msgevent_major 100

static int __init msgevent_init(void)
{
#ifdef _USE_NETLINK_
	nlfd = netlink_kernel_create(NETLINK_EXTPOWER, 0, nl_data_recv, THIS_MODULE);  
    if(!nlfd)   
    {  
        printk("KNL: Can not create a netlink socket!\n");  
        return -1;  
    }
#endif
    
#ifdef _USE_FILE_NOTE_  
	int ret = register_chrdev(msgevent_major,"msgevent",&msgevent_ops);	
	init_waitqueue_head(&msgevent_que);
	spin_lock_init(&lock);
#endif
}

static void __exit msgevent_init_exit(void)
{
#ifdef _USE_NETLINK_
	  if(nlfd != NULL)  
      {  
          sock_release(nlfd->sk_socket);  
      }  
#endif

#ifdef _USE_FILE_NOTE_ 
	unregister_chrdev(msgevent_major,"msgevent");
#endif
}

module_init(msgevent_init);
module_exit(msgevent_init_exit);
