#include <linux/module.h>
#include <linux/cdev.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/fs.h>
#include <linux/fcntl.h>
#include <linux/errno.h>
#include <linux/i2c.h>
#include <linux/platform_device.h>
#include <linux/delay.h>

#include <asm/arch/pxa3xx_gpio.h>
#include <asm/arch/zylonite.h>
#include <asm/uaccess.h>
#include <asm/arch/bq27541.h>
#include <linux/irq.h>
#include <linux/interrupt.h>
#include <linux/msgevent.h>
#include <asm/arch/gpio.h>

#define I2C_REGDATE_REORDER(x) (x>>8|x<<8) 

extern struct sysdev_class cpu_sysdev_class;

int max_ttf = 360;
int max_tte = 1100;
struct bq27541 {
	struct mutex lock;
	struct i2c_client *client;
	//u8 shadow_regs[MAX8660_N_REGS];
};
static struct bq27541 *the_bq27541;
static int bq27541_i2c_read(u8 command);
static int bq27541_i2c_write(u8 command,u16 val);

/******************************************************************************
 *                                                                            *
 *                              bq27541 fops                     *
 *                                                                            *
 ******************************************************************************/
int bq27541_major = 0;
int bq27541_minor = 0;
struct cdev *bq27541_cdev;
int bq27541_devno;
static const char bq27541_name[] = "bq27541";
int gauge_exist = 0;
static const int delay = 4 * HZ / 10;
static struct timer_list notify_timer;

int bq27541_open(struct inode *inode, struct file *filep)
{
	if(gauge_exist)
		return 0;
	else
		return -1;
}
int bq27541_close(struct inode *inode, struct file *filep)
{
	return 0;
}
ssize_t bq27541_read(struct file *filep, char *buf, size_t count,loff_t *f_pos)
{
	return 0;
}
ssize_t bq27541_write(struct file *filep, const char *buf, size_t count,loff_t *f_pos)
{
	return 0;
}
int get_battery_power()
{
	if(gauge_exist == 0)
		return -1;
	return bq27541_i2c_read(BQ27541_COMMAND_SOC);
}
EXPORT_SYMBOL(get_battery_power);
int bq27541_ioctl(struct inode *inode, struct file *file, unsigned int cmd,unsigned long arg)
{
	if(gauge_exist == 0)
		return -1;
	int ret=0;
	switch (cmd) {
	case BQ27541_IOCTL_GET_CHG_BAT:
	{
		int battery_charging;
		battery_charging = pxa3xx_gpio_get_level(MFP_BAT_CHG_CHGING);
		if(battery_charging != 0&&battery_charging != 1)
		{
			ret = -1;
			break;
		}
		
		if (copy_to_user((unsigned long *)arg, &battery_charging, sizeof(battery_charging)))
		{
			ret = -1;
			break;
		}
		break;
	}
	case BQ27541_IOCTL_GET_ARTTE:
	{
		int remaining_time = bq27541_i2c_read(BQ27541_COMMAND_ARTTE);
		if(remaining_time<0)
		{
			ret = -1;
			break;
		}
		if (copy_to_user((unsigned long *)arg, &remaining_time, sizeof(remaining_time)))
		{
			ret = -1;
			break;
		}
		break;
	}
	case BQ27541_IOCTL_GET_TEMP:
	{
		int temperature = bq27541_i2c_read(BQ27541_COMMAND_TEMP);
		if(temperature<0)
		{
			ret = -1;
			break;
		}
		if (copy_to_user((unsigned long *)arg, &temperature, sizeof(temperature)))
		{
			ret = -1;
			break;
		}
		break;
	}
	case BQ27541_IOCTL_GET_VOLT:
	{
		int voltage = bq27541_i2c_read(BQ27541_COMMAND_VOLT);
		if(voltage<0)
		{
			ret = -1;
			break;
		}
		if (copy_to_user((unsigned long *)arg, &voltage, sizeof(voltage)))
		{
			ret = -1;
			break;
		}
		break;
	}
	case BQ27541_IOCTL_GET_FLAGS:
	{
		int flags = bq27541_i2c_read(BQ27541_COMMAND_FLAGS);
		if (copy_to_user((unsigned long *)arg, &flags, sizeof(flags)))
		{
			ret = -1;
			break;
		}
		break;
	}
	case BQ27541_IOCTL_GET_NAC:
	{
		int nominal_available_capacity = bq27541_i2c_read(BQ27541_COMMAND_NAC);
		if(nominal_available_capacity<0)
		{
			ret = -1;
			break;
		}
		if (copy_to_user((unsigned long *)arg, &nominal_available_capacity, sizeof(nominal_available_capacity)))
		{
			ret = -1;
			break;
		}
		break;
	}
	case BQ27541_IOCTL_GET_FAC:
	{
		int full_available_capacity = bq27541_i2c_read(BQ27541_COMMAND_FAC);
		if(full_available_capacity<0)
		{
			ret = -1;
			break;
		}
		if (copy_to_user((unsigned long *)arg, &full_available_capacity, sizeof(full_available_capacity)))
		{
			ret = -1;
			break;
		}
		break;
	}
	case BQ27541_IOCTL_GET_RM:
	{
		int remaining_capacity = bq27541_i2c_read(BQ27541_COMMAND_RM);
		if(remaining_capacity<0)
		{
			ret = -1;
			break;
		}
		if (copy_to_user((unsigned long *)arg, &remaining_capacity, sizeof(remaining_capacity)))
		{
			ret = -1;
			break;
		}
		break;
	}
	case BQ27541_IOCTL_GET_FCC:
	{
		int full_charge_capacity = bq27541_i2c_read(BQ27541_COMMAND_FCC);
		if(full_charge_capacity<0)
		{
			ret = -1;
			break;
		}
		if (copy_to_user((unsigned long *)arg, &full_charge_capacity, sizeof(full_charge_capacity)))
		{
			ret = -1;
			break;
		}
		break;
	}
	case BQ27541_IOCTL_GET_AI:
	{
		int average_current = bq27541_i2c_read(BQ27541_COMMAND_AI);
		if (copy_to_user((unsigned long *)arg, &average_current, sizeof(average_current)))
		{
			ret = -1;
			break;
		}
		break;
	}
	case BQ27541_IOCTL_GET_TTE:
	{
		int timer_to_empty = bq27541_i2c_read(BQ27541_COMMAND_TTE);
		if(timer_to_empty<0)
		{
			ret = -1;
			break;
		}
		if(timer_to_empty > max_tte)
		{
			timer_to_empty = max_tte;
		}
		if (copy_to_user((unsigned long *)arg, &timer_to_empty, sizeof(timer_to_empty)))
		{
			ret = -1;
			break;
		}
		break;
	}
	case BQ27541_IOCTL_GET_TTF:
	{
		int time_to_full = bq27541_i2c_read(BQ27541_COMMAND_TTF);
		if(time_to_full<0)
		{
			ret = -1;
			break;
		}
		if(time_to_full > max_ttf)
		{
			time_to_full = max_ttf;
		}
		if (copy_to_user((unsigned long *)arg, &time_to_full, sizeof(time_to_full)))
		{
			ret = -1;
			break;
		}
		break;
	}
	case BQ27541_IOCTL_GET_SI:
	{
		int standby_current = bq27541_i2c_read(BQ27541_COMMAND_SI);
		if (copy_to_user((unsigned long *)arg, &standby_current, sizeof(standby_current)))
		{
			ret = -1;
			break;
		}
		break;
	}
	case BQ27541_IOCTL_GET_STTE:
	{
		int standby_time_to_empty = bq27541_i2c_read(BQ27541_COMMAND_AI);
		if(standby_time_to_empty<0)
		{
			ret = -1;
			break;
		}
		if (copy_to_user((unsigned long *)arg, &standby_time_to_empty, sizeof(standby_time_to_empty)))
		{
			ret = -1;
			break;
		}
		break;
	}
	case BQ27541_IOCTL_GET_MLI:
	{
		int max_load_current = bq27541_i2c_read(BQ27541_COMMAND_MLI);
		if (copy_to_user((unsigned long *)arg, &max_load_current, sizeof(max_load_current)))
		{
			ret = -1;
			break;
		}
		break;
	}
	case BQ27541_IOCTL_GET_MLTTE:
	{
		int max_load_time_to_empty = bq27541_i2c_read(BQ27541_COMMAND_MLTTE);
		if(max_load_time_to_empty<0)
		{
			ret = -1;
			break;
		}
		if (copy_to_user((unsigned long *)arg, &max_load_time_to_empty, sizeof(max_load_time_to_empty)))
		{
			ret = -EFAULT;
			break;
		}
		break;
	}
	case BQ27541_IOCTL_GET_AE:
	{
		int available_energy = bq27541_i2c_read(BQ27541_COMMAND_AE);
		if(available_energy<0)
		{
			ret = -1;
			break;
		}
		if (copy_to_user((unsigned long *)arg, &available_energy, sizeof(available_energy)))
		{
			ret = -1;
			break;
		}
		break;
	}
	case BQ27541_IOCTL_GET_AP:
	{
		int average_power = bq27541_i2c_read(BQ27541_COMMAND_AP);
		if(average_power<0)
		{
			ret = -1;
			break;
		}
		if (copy_to_user((unsigned long *)arg, &average_power, sizeof(average_power)))
		{
			ret = -1;
			break;
		}
		break;
	}
	case BQ27541_IOCTL_GET_TTECP:
	{
		int TTE_at_constant_power = bq27541_i2c_read(BQ27541_COMMAND_TTECP);
		if(TTE_at_constant_power<0)
		{
			ret = -1;
			break;
		}
		if (copy_to_user((unsigned long *)arg, &TTE_at_constant_power, sizeof(TTE_at_constant_power)))
		{
			ret = -1;
			break;
		}
		break;
	}
	case BQ27541_IOCTL_GET_CC:
	{
		int cycle_count = bq27541_i2c_read(BQ27541_COMMAND_CC);
		if(cycle_count<0)
		{
			ret = -1;
			break;
		}
		if (copy_to_user((unsigned long *)arg, &cycle_count, sizeof(cycle_count)))
		{
			ret = -1;
			break;
		}
		break;
	}
	case BQ27541_IOCTL_GET_SOC:
	{
		int state_of_charge = bq27541_i2c_read(BQ27541_COMMAND_SOC);
		if(state_of_charge<0)
		{
			ret = -1;
			break;
		}
		if (copy_to_user((unsigned long *)arg, &state_of_charge, sizeof(state_of_charge)))
		{
			ret = -1;
			break;
		}
		break;
	}
	case BQ27541_IOCTL_GET_DCAP:
	{
		int design_capacity = bq27541_i2c_read(BQ27541_COMMAND_DCAP);
		if(design_capacity<0)
		{
			ret = -1;
			break;
		}
		if (copy_to_user((unsigned long *)arg, &design_capacity, sizeof(design_capacity)))
		{
			ret = -1;
			break;
		}
		break;
	}
	default:
		ret = -1;
		break;
  }
	return ret;
}

static struct file_operations bq27541_fops = {
	owner:	THIS_MODULE,
	open:	bq27541_open,
	read:	bq27541_read,
	write:	bq27541_write,
	ioctl:	bq27541_ioctl,
	release: bq27541_close,
};

/******************************************************************************
 *                                                                            *
 *                              bq2754 Platform Driver                                *
 *                                                                            *
 ******************************************************************************/

static int bq27541_probe(struct platform_device *_dev)
{
	printk("bq27541_probe.\n");
	
	return 0;
}

static int bq27541_remove(struct platform_device *_dev)
{	
	
	return 0;
}

static int bq27541_suspend(struct platform_device *pdev, pm_message_t state)
{
	return 0;
}

static int bq27541_resume(struct platform_device *pdev)
{
	return 0;
}

static struct platform_driver bq27541_driver = {
         .driver = {
                 .name   = "guage_ic",
         },
         .probe          = bq27541_probe,
         .remove         = bq27541_remove,
         .suspend        = bq27541_suspend,
         .resume         = bq27541_resume,
};
/******************************************************************************
 *                                                                            *
 *                              bq27541 I2C Client Driver                     *
 *                                                                            *
 ******************************************************************************/

static int bq27541_i2c_read(u8 command)
{
	int data = i2c_smbus_read_word_data(the_bq27541->client,command);
	return data;
}

static int bq27541_i2c_write(u8 command,u16 val)
{
	int ret = 0;
	ret = i2c_smbus_write_word_data(the_bq27541->client,command,val);
	return ret;
}
static int bq27541_i2c_block_read(u8 block_ctl,u8 sub_class,u8 block,u8 offset,u8 is_word)
{
	int ret = 0;
	ret = i2c_smbus_write_byte_data(the_bq27541->client,BQ27541_COMMAND_DFDCNTL,block_ctl);
	ret = i2c_smbus_write_byte_data(the_bq27541->client,BQ27541_COMMAND_DFCLS,sub_class);
	ret = i2c_smbus_write_byte_data(the_bq27541->client,BQ27541_COMMAND_DFBLK,block);
	u16 data;
	if(is_word)
	{
		data = i2c_smbus_read_word_data(the_bq27541->client,BQ27541_COMMAND_DF+offset%32);
		data = I2C_REGDATE_REORDER(data);
	}
	else
		data = i2c_smbus_read_byte_data(the_bq27541->client,BQ27541_COMMAND_DF+offset%32);
	return data;
}
static int bq27541_i2c_block_write(u8 block_ctl,u8 sub_class,u8 block,u8 offset,u8 is_word,u16 data)
{
	int ret = 0;
	ret = i2c_smbus_write_byte_data(the_bq27541->client,BQ27541_COMMAND_DFDCNTL,block_ctl);
	ret = i2c_smbus_write_byte_data(the_bq27541->client,BQ27541_COMMAND_DFCLS,sub_class);
	ret = i2c_smbus_write_byte_data(the_bq27541->client,BQ27541_COMMAND_DFBLK,block);
	if(is_word)
	{
		data = I2C_REGDATE_REORDER(data);
		ret = i2c_smbus_write_word_data(the_bq27541->client,BQ27541_COMMAND_DF+offset%32,data);
	}
	else
	{
		u8 data_byte = 0x00ff&data;
		ret = i2c_smbus_write_byte_data(the_bq27541->client,BQ27541_COMMAND_DF+offset%32,data_byte);
	}
	return ret;
}

static int i2c_bq27541_attach_adapter(struct i2c_adapter *adapter);
static int i2c_bq27541_detach_client(struct i2c_client *client);

struct i2c_driver i2c_bq27541_driver  =
{
        .driver = {
                .name   = "bq27541 i2c client driver",
        },
        .id             = 0,
        .attach_adapter = &i2c_bq27541_attach_adapter,
        .detach_client  = &i2c_bq27541_detach_client,
};

static unsigned short normal_i2c[] = { BQ27541_ADDRESS, I2C_CLIENT_END };
I2C_CLIENT_INSMOD_1(bq27541);

static int i2c_bq27541_detect_client(struct i2c_adapter *adapter, int address, int kind)
{
	int ret = 0;
	struct i2c_client *new_client;
	struct bq27541 *bq27541;
	
	if (the_bq27541) {
		printk("only one bq27541 for now\n");
		return -ENODEV;
	}
        
	if ( !i2c_check_functionality(adapter,I2C_FUNC_SMBUS_WRITE_BYTE_DATA) ) {
		pr_info("bq27541: write byte op is not permited.\n");
			goto ERROR0;
	}
	
	new_client = kzalloc(sizeof(struct i2c_client), GFP_KERNEL );
	if (!new_client) {
		pr_info("bq27541: insufficient memory\n");
			ret = -ENOMEM;
			goto ERROR0;
	}
	new_client->addr = address;
	new_client->adapter = adapter;
	new_client->driver = &i2c_bq27541_driver;
	new_client->flags = 0;

	int chip_id = i2c_smbus_read_byte(new_client);
	printk(KERN_INFO "Gauge IC chip_id=%d\n",chip_id);
	if(chip_id<0)
	{
		printk("The Gauge IC is not exist.\n");
		goto ERROR0;
	}
		
	gauge_exist = 1;
	
	bq27541 = kzalloc(sizeof(struct bq27541), GFP_KERNEL);
	if (bq27541 == NULL) {
		pr_info("bq27541: insufficient memory\n");
		ret = -ENOMEM;
                goto ERROR1;
	}
	strcpy(new_client->name, "bq27541");
	bq27541->client = new_client;
	the_bq27541 = bq27541;
	
	if ((ret = i2c_attach_client(the_bq27541->client)))
		goto ERROR2;
        
	i2c_set_clientdata(the_bq27541->client, the_bq27541);
	
	ret = bq27541_i2c_write(BQ27541_COMMAND_CNTL,BQ27541_CNTL_DEVICE_TYPE);
	int device_type = bq27541_i2c_read(BQ27541_COMMAND_CNTL);
	if(ret==0 && device_type>=0)
	{
		if(device_type==0x0541)
			printk("The Gauge IC is bq27541.\n");
		else
			printk("The Gauge IC is unknow type.\n");
	}

	return 0;

ERROR2:
	kfree(the_bq27541);
	the_bq27541 = NULL;
ERROR1:
        kfree(new_client);
ERROR0:
        return ret;
}

static int i2c_bq27541_attach_adapter(struct i2c_adapter *adap)
{
	return i2c_probe(adap,&addr_data, &i2c_bq27541_detect_client);
}

static int i2c_bq27541_detach_client(struct i2c_client *client)
{
	int ret;
	struct bq27541 *bq27541;
	
	bq27541 = i2c_get_clientdata(client);
	kfree(bq27541);
	i2c_set_clientdata(client, NULL);
	the_bq27541 = NULL;

	if ((ret = i2c_detach_client(client))) {
			printk("bq27541: Client deregistration failed, client not detached.\n");
			return ret;
	}
	kfree(client);
	return 0;
}

static ssize_t bq27541_show(struct sys_device *sys_dev, char *buf)
{
	int len = 0;
	if(gauge_exist == 0)
	{
		len = sprintf(buf,"gauge no exist.\n");
		return len;
	}
		
	len = sprintf(buf,"battery_charging = %d\n",pxa3xx_gpio_get_level(MFP_BAT_CHG_CHGING));
	len += sprintf(buf+len,"battery_alert = %d\n",pxa3xx_gpio_get_level(MFP_BAT_BAT_ALERT));
	len += sprintf(buf+len,"voltage = %dmV\n",bq27541_i2c_read(BQ27541_COMMAND_VOLT));
	len += sprintf(buf+len,"nominal_available_capacity = %dmAh\n",bq27541_i2c_read(BQ27541_COMMAND_NAC));
	len += sprintf(buf+len,"full_available_capacity = %dmAh\n",bq27541_i2c_read(BQ27541_COMMAND_FAC));
	len += sprintf(buf+len,"remaining_capacity = %dmAh\n",bq27541_i2c_read(BQ27541_COMMAND_RM));
	len += sprintf(buf+len,"full_charge_capacity = %dmAh\n",bq27541_i2c_read(BQ27541_COMMAND_FCC));
	len += sprintf(buf+len,"timer_to_empty = %dmin\n",bq27541_i2c_read(BQ27541_COMMAND_TTE));
	len += sprintf(buf+len,"time_to_full = %dmin\n",bq27541_i2c_read(BQ27541_COMMAND_TTF));
	len += sprintf(buf+len,"state_of_charge = %d%\n",bq27541_i2c_read(BQ27541_COMMAND_SOC));
	return len;
}
SYSDEV_ATTR(bq27541, 0644, bq27541_show, NULL);
static struct attribute *gauge_ic_attr[] = {
	&attr_bq27541.attr,
	NULL
};

static int gauge_ic_add(struct sys_device *sys_dev)
{
	if (sysfs_create_file(&(sys_dev->kobj), gauge_ic_attr[0]))
			pr_debug("sysfs_create_file failed for %s\n",
					gauge_ic_attr[0]->name);
	return 0;
}

static int gauge_ic_rm(struct sys_device *sysdev)
{
	sysfs_remove_file(&(sysdev->kobj), gauge_ic_attr[0]);
	return 0;
}
static struct sysdev_driver bq27541_sysdev_driver = {
	.add		= gauge_ic_add,
	.remove		= gauge_ic_rm,
};

/******************************************************************************
 *                                                                            *
 *                              bq27541 Init & Exit                     *
 *                                                                            *
 ******************************************************************************/
int pre_ischarging;
static irqreturn_t ext_power_isr(int irq, void *dev_id)
{
	struct MSG_DATA power_msg;
	
	if((irq == gpio_to_irq(MFP2GPIO(MFP_BAT_BAT_ALERT)))&&(!pxa3xx_gpio_get_level(MFP_BAT_CHG_CHGING))&&(!pxa3xx_gpio_get_level(MFP_BAT_BAT_ALERT)))
	{
		power_msg.msgID = PowerModeMessage;
		power_msg.data.powermodestate = 5;
		msgevent_report_broadcast(&power_msg);
	}	
	else if(irq == gpio_to_irq(MFP2GPIO(MFP_BAT_CHG_CHGING))){
		int ischarging = pxa3xx_gpio_get_level(MFP_BAT_CHG_CHGING);
		if(pre_ischarging != ischarging)
		{
			pre_ischarging = ischarging;
			//~ printk("=====================pre_ischarging: %d\n", pre_ischarging);
			mod_timer(&notify_timer, jiffies + delay);
		}
	}
	return IRQ_HANDLED;
}  
int irq_bat_alert;
int irq_bat_chging;
int request_power_gpio_irq(int gpio)
{
	int irq;
	irq = gpio_to_irq(gpio);
	set_irq_type(irq, IRQ_TYPE_EDGE_BOTH);            
	int error = request_irq(irq, ext_power_isr, IRQF_SAMPLE_RANDOM, "power isr",NULL);
	if (error) 
	{
	   printk("request gpio%d irq fail\n",gpio);
	   return -1;
	}
	printk("request gpio%d irq success\n",gpio);
	return irq;
}
static void timer_function(unsigned long data)
{
	struct MSG_DATA power_msg;
	unsigned long flags;
	local_irq_save(flags);
	
	power_msg.msgID = PowerMessage;
	power_msg.data.powerstate.ischarging = pre_ischarging;
	power_msg.data.powerstate.battery_power = get_battery_power();
	msgevent_report_broadcast(&power_msg);
	
	//~ printk("=======================Power Report Msg, %d\n", power_msg.data.powerstate.ischarging);
	
	local_irq_restore(flags);
}


static int __init bq27541_init(void)
{
  int ret;
  pre_ischarging = pxa3xx_gpio_get_level(MFP_BAT_CHG_CHGING);
  init_timer(&notify_timer);
  notify_timer.data = 0;
  notify_timer.function = timer_function;
  
  if(bq27541_major)
  {
  		bq27541_devno = MKDEV(bq27541_major,bq27541_minor);
  		ret = register_chrdev_region(bq27541_devno,1,bq27541_name);
  }
  else
  {
  		ret = alloc_chrdev_region(&bq27541_devno,bq27541_minor,1,bq27541_name);
  		bq27541_major = MAJOR(bq27541_devno);
  }
  bq27541_cdev = cdev_alloc();
  bq27541_cdev->owner = THIS_MODULE;
  bq27541_cdev->ops = &bq27541_fops;
  cdev_init(bq27541_cdev,&bq27541_fops);
  ret = cdev_add(bq27541_cdev,bq27541_devno,1);
   if ( (ret = i2c_add_driver(&i2c_bq27541_driver)) ) {
                printk("bq27541: Driver registration failed, module not inserted.\n");
                return ret;
        }
	
  ret = platform_driver_register(&bq27541_driver);
  sysdev_driver_register(&cpu_sysdev_class, &bq27541_sysdev_driver);
  irq_bat_alert = request_power_gpio_irq(MFP2GPIO(MFP_BAT_BAT_ALERT));
  irq_bat_chging = request_power_gpio_irq(MFP2GPIO(MFP_BAT_CHG_CHGING));
  printk("************bq27541 init************\n");
  
  return 0;
}

static void bq27541_exit(void)
{
	free_irq(irq_bat_alert,ext_power_isr);
	free_irq(irq_bat_chging,ext_power_isr);
	unregister_chrdev_region(bq27541_devno, 1);
	cdev_del(bq27541_cdev);
	
	 platform_driver_unregister(&bq27541_driver);
	 sysdev_driver_unregister(&cpu_sysdev_class, &bq27541_sysdev_driver);

   if (i2c_del_driver(&i2c_bq27541_driver)) {
           printk("bq27541: Driver deregistration failed, module not removed.\n");
      }
	printk("************bq27541 exit************\n");
}

module_init(bq27541_init);
module_exit(bq27541_exit);

MODULE_DESCRIPTION("Gauge IC Driver");
MODULE_LICENSE("GPL");
