/*
 *  linux/arch/arm/mach-pxa/imm.c
 *
 *  Intel Memory Management
 *
 *  User level driver interface to IMM APIs
 *
 *  Todd Brandt
 *  Copyright (c) 2004, Intel Corporation (todd.e.brandt@intel.com)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.

 *(C) Copyright 2006 Marvell International Ltd.
 * All Rights Reserved
 */

#include <linux/mm.h>
#include <linux/miscdevice.h>
#include <linux/slab.h>
#include <linux/vmalloc.h>
#include <linux/mman.h>
#include <linux/random.h>
#include <linux/version.h>
#include <linux/init.h>
#include <linux/raw.h>
#include <linux/tty.h>
#include <linux/capability.h>
#include <linux/shm.h>

#include <asm/uaccess.h>
#include <asm/io.h>
#include <asm/pgalloc.h>
#include <asm/arch/imm.h>
#include <asm/arch/immcontrol.h>

static int num_instances = 0;

static int
imm_control_ioctl(struct inode *inode, struct file *filp,
		unsigned int cmd, unsigned long arg)
{
struct imm_client_info cl, *pcl = (struct imm_client_info *)arg;
int res = 0;

	if(arg == 0) return 0;
	memcpy(&cl, pcl, sizeof(struct imm_client_info));
	res = imm_client_control(&cl, cmd);
	copy_to_user(pcl, &cl, sizeof(struct imm_client_info));
        return res;
}

static int imm_control_driver_open(struct inode * inode, struct file * filp)
{
	num_instances++;
	imm_debug("imm_control_driver_open(current->pid = %d)\n", current->pid);
	return 0;
}

static int imm_control_driver_release(struct inode * inode, struct file * filp)
{
	num_instances--;
	if(num_instances < 0) num_instances = 0;
        return 0;
}

static struct file_operations imm_control_fops = {
	open:		imm_control_driver_open,
	release:	imm_control_driver_release,
	ioctl:		imm_control_ioctl,
};

static struct miscdevice imm_control_misc = {
	minor : IMMCONTROL_MINOR,
	name  : "immcontrol",
	fops  : &imm_control_fops,
};

static int __init imm_control_init(void) {
	if (misc_register(&imm_control_misc)) {
		printk(KERN_ERR "Error registering device /dev/%s\n", imm_control_misc.name);
		return -EFAULT;
	}
	return 0;
}

static void __exit imm_control_exit(void) {
	misc_deregister(&imm_control_misc);
}

module_init(imm_control_init)
module_exit(imm_control_exit)
