/*
 * Driver for keys on GPIO lines capable of generating interrupts.
 *
 * Copyright 2005 Phil Blundell
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include <linux/module.h>
#include <linux/version.h>

#include <linux/init.h>
#include <linux/fs.h>
#include <linux/interrupt.h>
#include <linux/irq.h>
#include <linux/sched.h>
#include <linux/pm.h>
#include <linux/sysctl.h>
#include <linux/proc_fs.h>
#include <linux/delay.h>
#include <linux/platform_device.h>
#include <linux/input.h>
#include <linux/irq.h>
#include <linux/gpio_keys.h>
#include <linux/mutex.h>

#include <linux/i2c.h>
#include <asm/io.h>
#include <asm/gpio.h>
#include <asm/uaccess.h>

#include <asm/arch/ipmc.h>
#include <linux/time.h>

#include <linux/msgevent.h>
#define gpio76_readl(mmio, off)		__raw_readl(mmio + off)

extern int isExt0Waked;

static int pDown = 0;
static int pValue = 0;
static wait_queue_head_t gpio_127_q;
static struct timer_list stimer,sentimer;


static wait_queue_head_t sen_q;

static int pSen = 0;

struct senbod_event {
	int pSen;
	int code;
	int value;
};

struct senbod_event eventout;


struct timer_list gpio_sleeptimer;
EXPORT_SYMBOL(gpio_sleeptimer);
/* After gpiosleep_timeout long without reset, we will sleep. */
unsigned int  gpiosleep_timeout= 5; /* In seconds. */
EXPORT_SYMBOL(gpiosleep_timeout);

static struct timer_list sSBtimer;

static struct i2c_client *g_client;
struct i2c_driver i2c_gpio_key_driver;

static spinlock_t bus_lock;
//struct mutex bus_lock1;

static unsigned short normal_i2c[] = {0x2c, I2C_CLIENT_END };
I2C_CLIENT_INSMOD_1(gpiokey);

static int add = 100;


struct MSG_DATA  msgsenbor,msgsenlock,msgswitch;
static int i2c_gpio_key_detect_client(struct i2c_adapter *adapter,
				int address, int kind)
{
       
	struct i2c_client *new_client;
	int err = 0;
	int chip_id;

	/* Let's see whether this adapter can support what we need.
	       Please substitute the things you need here!  */
	//if ( !i2c_check_functionality(adapter,I2C_FUNC_SMBUS_BYTE_DATA|I2C_FUNC_SMBUS_READ_WORD_DATA|I2C_FUNC_SMBUS_READ_I2C_BLOCK) ) {
if ( !i2c_check_functionality(adapter,I2C_FUNC_SMBUS_BYTE_DATA|I2C_FUNC_SMBUS_READ_I2C_BLOCK) ) {
		
		goto ERROR0;
	}

	/* OK. For now, we presume we have a valid client. We now create the
	   client structure, even though we cannot fill it completely yet.
	   But it allows us to access several i2c functions safely */

	/* Note that we reserve some space for arava_data too. If you don't
	   need it, remove it. We do it here to help to lessen memory
	   fragmentation. */

	new_client = kzalloc(sizeof(struct i2c_client), GFP_KERNEL );

	if (!new_client) {
             
		err = -ENOMEM;
		goto ERROR0;
	}

	new_client->addr = address;
 	new_client->adapter = adapter;
	new_client->driver = &i2c_gpio_key_driver;
	new_client->flags = 0;

        int cat = 0;
       
        chip_id = i2c_smbus_read_byte_exp(new_client,REG_ADDRESS_SIZE_16);
      
	if (chip_id < 0){
		
		goto ERROR1;
	} 

	g_client = new_client;

	strcpy(new_client->name, "GPIOKEY");

	/* Tell the i2c layer a new client has arrived */
	if ((err = i2c_attach_client(new_client)))
		goto ERROR1;

	return 0;

ERROR1:
  
	g_client = NULL;
	kfree(new_client);
ERROR0:
       
        return err;

}

static int i2c_gpio_key_attach_adapter(struct i2c_adapter *adap)
{
       
	return i2c_probe(adap, &addr_data, &i2c_gpio_key_detect_client);
}

static int i2c_gpio_key_detach_client(struct i2c_client *client)
{
	int err;
  	/* Try to detach the client from i2c space */
	if ((err = i2c_detach_client(client))) {
		printk(KERN_WARNING "arava.o: Client deregistration failed, \
					client not detached.\n");
	        return err;
	}

	/* Frees client data too, if allocated at the same time */
	kfree(client);
	g_client = NULL;
	return 0;
}

struct i2c_driver i2c_gpio_key_driver = {
	.driver		= {
		.name	= "gpio key i2c client driver",
	},
	.attach_adapter	= &i2c_gpio_key_attach_adapter,
	.detach_client	= &i2c_gpio_key_detach_client,
};
static int state127,state80 = 0,qtstate = 1;
int wait_127event =0;


static void timer_handle(unsigned long arg)
{

       if(state127==1)
       {
		    struct MSG_DATA power_mode_msg;
			power_mode_msg.msgID = PowerModeMessage;
			power_mode_msg.data.powermodestate = 4;
			msgevent_report_broadcast(&power_mode_msg);
					
		    pValue = 3;
		    wait_127event =1;	
			wake_up_interruptible(&gpio_127_q);  
		}
     
         
}


static struct resource *res;

#define	ISP1301_BCD_DEVICE		0xd8	/* u16 read */

static int bDown  = 0;
static int SiderIn  =0 ;
static int SiderTemp[4]  ={0} ;
static int ButtonCode[] = {KEY_F1,KEY_F2,KEY_F3,KEY_F4,KEY_F5,KEY_F6,KEY_F7,KEY_F8,KEY_F9,KEY_F10,KEY_F11,KEY_F12};

static int irqtemp1 = -1;

struct irqSB{
             int button ;
             int sider  ;
             int num    ;
};

static struct irqSB irqSB1;


static int i2c_pxa_polling_txempty(int timeout)
{
       unsigned int isr;

       while (timeout--) {
               isr = ISR;
               if (isr & ISR_ITE) {
                       ISR = ISR_ITE;
                       if ( isr & ISR_ALD)
                               ISR |= ISR_ALD;

                       return 0;
               }
       }

       return 1; /* time out*/
}

static int i2c_pxa_polling_rxfull(int timeout)
{
       unsigned int isr;

       while (timeout--) {
               isr = ISR;
               if (isr & ISR_IRF) {
                       ISR = ISR_IRF;
                       return 0;
               }
       }

       return 1; /* TIME OUT*/
}


static void timer_handleS(unsigned long arg)
{
    SiderIn = 0;SiderTemp[0] = 0;SiderTemp[1] = 0;SiderTemp[2] = 0;SiderTemp[3] = 0;
}
static int IsEnterStandby = 0,IsSenLock = 0;

struct delayed_work SBwork,SBlockwork;
struct input_dev *inputSB = NULL;    

static int  RButtonCopy = 0,RsliderCopy = 0;
static int senflag = 1,sentestflag = 0;

void findrslider(int i,int bdu)
{

   if(bdu==1)
   bdu = 0;
   else if(bdu==0)
   bdu = 1;

 switch(i) {
         case 0:
		// pr_info("r46 c8 %d\n",bdu);  
                input_report_key(inputSB,ButtonCode[11], bdu); 
		break;
         case 1:
		break;
		
         case 2:
		break;
         case 3:
           input_report_key(inputSB,ButtonCode[8], bdu); 
		break;
		
         case 4:
		break;
		
         case 6:
		//pr_info("r46 c13 %d\n",bdu);  
                input_report_key(inputSB,ButtonCode[5], bdu); 
                
		break;
	default:
		break;
        } 

}

void findrbutton(int i,int bdu)
{

   if(bdu==1)
   bdu = 0;
   else if(bdu==0)
   bdu = 1;

 switch(i) {
         case 0:
                input_report_key(inputSB,ButtonCode[0], bdu); 
		break;
         case 1:
                input_report_key(inputSB,ButtonCode[1], bdu); 
		break;
         case 2:
                input_report_key(inputSB,ButtonCode[2], bdu); 
		break;
         case 3:
                input_report_key(inputSB,ButtonCode[3], bdu); 
               
		break;
         case 4:
                input_report_key(inputSB,ButtonCode[4], bdu); 
		break;
 
	default:
		break;
        } 

}

void find45(int rButton,int rButtoncopy)
{

     int tempchange = rButton^rButtoncopy;
     int i = 0;
     for(i=0;i<6;i++)
     {
        int tempcch = 0;
        tempcch = tempchange>>i;
        tempcch = tempcch&0x1;
        if(tempcch)
        {
         int tempccr = rButtoncopy>>i;
         tempccr = tempccr&0x1;
         findrbutton(i,tempccr);
              }       
        }    
}

void find46(int rslider,int rslidercopy)
{

     int tempchange = rslider^rslidercopy;
     int i = 0;
     for(i=0;i<7;i++)
     {
        int tempcch = 0;
        tempcch = tempchange>>i;
        tempcch = tempcch&0x1;
        if(tempcch)
        {
         int tempccr = rslidercopy>>i;
         tempccr = tempccr&0x1;
         findrslider(i,tempccr);
        }       
      }    
}

static int p_id_numbercopy = 0,p_id_countcopy = 0;


static void gpio_sblock_work(struct work_struct *work)
{
     msgswitch.msgID = SensorBoardSwitchMessage;
     msgswitch.data.senswitchstate = 1;
     msgevent_report_broadcast(&msgswitch);
}

int dokeystatus(int key_status)
{
    if(key_status&0x40)
    {
       input_report_key(inputSB,ButtonCode[5], 1);
       input_report_key(inputSB,ButtonCode[5], 0);
       return 0;
       }
     else if(key_status&0x20)
     {
       input_report_key(inputSB,ButtonCode[6], 1);
       input_report_key(inputSB,ButtonCode[6], 0);
       return 0;
       }
     else if(key_status&0x10)
     {
       input_report_key(inputSB,ButtonCode[7], 1);
       input_report_key(inputSB,ButtonCode[7], 0);
       return 0;
       } 
     else if(key_status&0x08)
     {
       input_report_key(inputSB,ButtonCode[8], 1);
       input_report_key(inputSB,ButtonCode[8], 0);
       return 0;
       } 
     else if(key_status&0x04)
     {
       input_report_key(inputSB,ButtonCode[9], 1);
       input_report_key(inputSB,ButtonCode[9], 0);
       return 0;
       } 
     else if(key_status&0x02)
     {
       input_report_key(inputSB,ButtonCode[10], 1);
       input_report_key(inputSB,ButtonCode[10], 0);
       return 0;
       } 
     else if(key_status&0x01)
     {
       input_report_key(inputSB,ButtonCode[11], 1);
       input_report_key(inputSB,ButtonCode[11], 0);
       return 0;
       } 
return -1;
     
}



static void gpio_sb_work(struct work_struct *work)
{ 
        if(g_client){ 
    u8 buf0 = 0;
 if (i2c_check_functionality(g_client->adapter,
					    I2C_FUNC_SMBUS_WORD_DATA))
{
        unsigned char tpAddress = 0x2C;
        unsigned char tpDataWriteb = 0x45,tpDataWrites = 0x46; 
        unsigned char RButton[2] = {0x00,0x00};
        unsigned char Rslider[2] = {0x00,0x00};  
     u16 rev = i2c_smbus_read_word_exp(g_client,0x45,REG_ADDRESS_SIZE_16,1);
     RButton[0] = (u8)(rev>>8);
     RButton[1] = (u8)rev;
     rev = i2c_smbus_read_word_exp(g_client,0x46,REG_ADDRESS_SIZE_16,1);
     Rslider[0] = (u8)(rev>>8);
     Rslider[1] = (u8)rev;
     
    int keybutton = 0, keyslider = 0;
    
    keybutton = (RButton[1]&0x1f);
    keyslider = (((RButton[1]|(Rslider[1]<<6))>>5)&0x7f);
    
    if((keybutton&0x1f)&&(keyslider&0x7f))
    {
      return;
    }    
     
    if(keybutton != RButtonCopy)
    {      
      find45(keybutton,RButtonCopy);
      }
    RButtonCopy = keybutton;
    if(keyslider != RsliderCopy)
    {      
      find46(keyslider,RsliderCopy);
     }

    
    RsliderCopy = keyslider;

	}
}
}

static int senflagcopy = 1;

int set_Sensorboard(int benable)
{
        unsigned long   flags;
        spin_lock_irqsave(&bus_lock, flags);
     
         if(sentestflag)
        {
         spin_unlock_irqrestore(&bus_lock, flags);
        return 0;
        }
        if(benable<2&&qtstate==0)
        {
       
         spin_unlock_irqrestore(&bus_lock, flags);
       
        return 0;
        }
        if(g_client){
        unsigned char tpAddress = 0x2C;
        unsigned char tpDataWrite; 
        unsigned int  len = 3;    
       
        
        if(benable == 1||benable==3)
        tpDataWrite = 0x3F;
        else
        if(benable == 0||benable==2)
        tpDataWrite = 0x00;
        i2c_smbus_write_word_exp(g_client, 0x0041, tpDataWrite,REG_ADDRESS_SIZE_16);	
       
        i2c_smbus_write_word_exp(g_client, 0x0042, tpDataWrite,REG_ADDRESS_SIZE_16);	
        senflag = get_Sensorboardstate();
    
       spin_unlock_irqrestore(&bus_lock, flags);
       
        return 0;
        }
      
	spin_unlock_irqrestore(&bus_lock, flags);
       
	return -1;
}
int get_Sensorboardstate(void)
{
     if(g_client){
     int benalbe = 0;
     unsigned char tpAddress = 0x2C;
     unsigned char tpDataWrite = 0x00; 
     unsigned char RButton[2] = {0x00,0x00};
     unsigned char Rslider[2] = {0x00,0x00};  
      u16 rev = i2c_smbus_read_word_exp(g_client,0x41,REG_ADDRESS_SIZE_16,1);
     RButton[0] = (u8)(rev>>8);
     RButton[1] = (u8)rev;
     rev = i2c_smbus_read_word_exp(g_client,0x42,REG_ADDRESS_SIZE_16,1);
     Rslider[0] = (u8)(rev>>8);
     Rslider[1] = (u8)rev;
      
     if(RButton[1]==0x3f&&Rslider[1]==0x3f)
     benalbe = 1;

     return benalbe;
     }
     return -1;
}

int get_senflag()
{
     if(g_client){
       return senflag;
     }   
     return -1;
}

EXPORT_SYMBOL(set_Sensorboard);
EXPORT_SYMBOL(get_senflag);

static void sentimer_handle(unsigned long arg)
{
      if(g_client){
     int benalbe = 0;
     unsigned char tpAddress = 0x2C;
     unsigned char tpDataWrite = 0x00; 
     unsigned char RButton[2] = {0x00,0x00};
     unsigned char Rslider[2] = {0x00,0x00};  
 
      u16 rev = i2c_smbus_read_word_exp(g_client,0x45,REG_ADDRESS_SIZE_16,1);
     RButton[0] = (u8)(rev>>8);
     RButton[1] = (u8)rev;
     rev = i2c_smbus_read_word_exp(g_client,0x46,REG_ADDRESS_SIZE_16,1);
     Rslider[0] = (u8)(rev>>8);
     Rslider[1] = (u8)rev;
     return ;
     } 
}
extern int pm_updatetimer(int);
extern int bAllow_input;
extern int entering_suspend;
extern int suspending_irq;
static irqreturn_t gpio_keys_isr(int irq, void *dev_id)
{            
		if(bAllow_input)
		{
			bAllow_input = 0;
			pm_updatetimer(IPM_COUNT_LIMIT_TIMER);
			if(irq != gpio_to_irq(127))
			{
				ipm_event_notify(IPM_EVENT_UI, IPM_EVENT_DEVICE_KEYPAD, NULL, 0);
			}
		}
		
		int i;    
	
        unsigned long   flags;
        spin_lock_irqsave(&bus_lock, flags);

		struct platform_device *pdev = dev_id;
		struct gpio_keys_platform_data *pdata = pdev->dev.platform_data;
		struct input_dev *input = platform_get_drvdata(pdev);                

		
		if (irq == gpio_to_irq(127)) {  	

            int gpio = pdata->buttons[0].gpio;   
            state127 = (gpio_get_value(gpio) ? 1 : 0) ^ (pdata->buttons[0].active_low);   					
			if(state127 ==0&&pDown==1)
           	{
				if(isExt0Waked==0)
				{
					if(pValue!=3)
					{
						if(pxa3xx_gpio_get_level(MFP_PIN_GPIO77))
						{	
							ipm_event_notify(IPM_EVENT_UI, IPM_EVENT_DEVICE_KEYPAD127_UP_LCDON, NULL, 0);
							pValue =1;
							mod_timer(&gpio_sleeptimer,jiffies+gpiosleep_timeout*HZ);
						}
						else
						{
							if(entering_suspend)
							{
								suspending_irq = 1;
								printk("suspending_irq = 1\n");
							}
							pValue =0;
							ipm_event_notify(IPM_EVENT_UI, IPM_EVENT_DEVICE_KEYPAD127_UP_LCDOFF, NULL, 0);
							del_timer(&gpio_sleeptimer);
						}
						del_timer(&stimer);
						wait_127event =1;	
						wake_up_interruptible(&gpio_127_q); 
					}
					else
						pValue = -1;
				}
				else
					isExt0Waked = 0;
				
				           
		    }	
			else if (state127==1)
			{                                                                            
				pDown = 1;							
				if(pValue!=1)
				{
					mod_timer(&stimer, jiffies + 2*HZ);	
				}								
			}			              
			
		}
       else if(irq == gpio_to_irq(76))
       {    
		   inputSB = platform_get_drvdata(pdev); 
           int gpio = pdata->buttons[1].gpio;
			int state  = 0;
			state = (gpio_get_value(gpio) ? 1 : 0) ^ (pdata->buttons[1].active_low);    
			bDown = state;
			if(bDown)
			schedule_delayed_work(&SBwork, 5);
			 
			mod_timer(&sentimer, jiffies + HZ/2);	                        
	    }

    spin_unlock_irqrestore(&bus_lock, flags);

	return IRQ_HANDLED;
}

void set_IsEnterStandby(void)
{
	IsEnterStandby = 1;
	return;
}
void reset_IsEnterStandby(void)
{	
	IsEnterStandby = 0;
	return;
}
EXPORT_SYMBOL(set_IsEnterStandby);
EXPORT_SYMBOL(reset_IsEnterStandby);

void set_pValue(void)
{
	pValue = 1;
	return;
}
void reset_pValue(void)
{	
	pValue = 0;
	return;
}
EXPORT_SYMBOL(set_pValue);
EXPORT_SYMBOL(reset_pValue);

void set_Senlock(void)
{
IsSenLock = 1;
return;
}
int get_Senlock(void)
{
return IsSenLock;

}
void reset_Senlock(void)
{	
	IsSenLock = 0;
	return;
}
EXPORT_SYMBOL(set_Senlock);
EXPORT_SYMBOL(get_Senlock);
EXPORT_SYMBOL(reset_Senlock);

extern void pxa3xx_enable_gpiokeyp_pins(); 



int i2c_smbus_write_word_dataSenBor(unsigned char addr, unsigned char *buf, int len)
{
     unsigned int icr;

       /*
        * Step 1: target slave address into IDBR
        */
       IDBR = (addr << 1);

       /*
        * Step 2: initiate the write.
        */
       icr = ICR & ~(ICR_STOP | ICR_ALDIE);
       ICR = icr | ICR_START | ICR_TB;

       if (i2c_pxa_polling_txempty(4000))
       {
               return 1;
		}

        IDBR = 0x00;
               icr = (ICR & ~ICR_START) | ICR_ALDIE | ICR_TB;       
               ICR = icr;
               if (i2c_pxa_polling_txempty(50000))
                       return 1;


       while (len--) {
               IDBR = (unsigned int) (*buf++);
               icr = (ICR & ~ICR_START) | ICR_ALDIE | ICR_TB;
               if (len == 0)
                       icr |= ICR_STOP;

               ICR = icr;
               if (i2c_pxa_polling_txempty(50000))
                       return 1;
       }

       ICR &= ~ICR_STOP;
       return 0;
}


int set_Senled(int bShar)
{
  if(!g_client)
        return -1;

       unsigned char tpAddress = 0x2C;
       unsigned char tpDataWrite[3] = {0x00,0x00,0x00}; 
       unsigned int  len = 3; 
       u16 value;   
	   u16 setled;	
		
      if(bShar)
      {
            tpDataWrite[0] = 0x39; tpDataWrite[1] = 0x81;tpDataWrite[2] = 0x04;  
            value=0x43c4;
            setled=0x9000;
       }
       else 
       {
             tpDataWrite[0] = 0x39; tpDataWrite[1] = 0x01;tpDataWrite[2] = 0x04;
             value=0x03c4;
             setled=0x0000;
        }
	int ret = i2c_smbus_write_word_exp(g_client, 0x0029, setled,REG_ADDRESS_SIZE_16);
	ret = i2c_smbus_write_word_exp(g_client, 0x0039, value,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
return 0;
}

EXPORT_SYMBOL(set_Senled);


int set_threshold1(int thld)
{
     int ret = 0;
	u8 v1, v2;
        struct timeval start,end;
        do_gettimeofday(&start);
        if(!g_client)
        return -1;
	if (!i2c_check_functionality(g_client->adapter,I2C_FUNC_SMBUS_WORD_DATA))
        return -1;
	// COLD RESET
        pr_info("start gpio_init_sensor_board\n");
        unsigned char tpAddress = 0x2C;
        unsigned char tpDataWrite[3] = {0x00,0x00,0x00}; 
        unsigned int  len = 3;       

	ret = i2c_smbus_write_word_exp(g_client, 0x00, 0x0000,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}

	// TOUCH DISABLE
        tpDataWrite[0] = 0x40; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x30;
        ret = i2c_smbus_write_word_exp(g_client, 0x0040, 0x0030,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}	
	// DCM CONFIG
        tpDataWrite[0] = 0x0A; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x00;
        ret = i2c_smbus_write_word_exp(g_client, 0x000a, 0x0000,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
	// SCROLL CONFIG
        tpDataWrite[0] = 0x74; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x00;
        ret = i2c_smbus_write_word_exp(g_client, 0x0074, 0x0000,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
	// PSEL
        tpDataWrite[0] = 0x41; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x3F;
        ret = i2c_smbus_write_word_exp(g_client, 0x0041, 0x003f,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
         tpDataWrite[0] = 0x42; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x3F;
        ret = i2c_smbus_write_word_exp(g_client, 0x0042, 0x003f,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
	// INTERRUPT
         tpDataWrite[0] = 0x43; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x3F;
        ret = i2c_smbus_write_word_exp(g_client, 0x0043, 0x003f,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
        tpDataWrite[0] = 0x44; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x3F;
        ret = i2c_smbus_write_word_exp(g_client, 0x0044, 0x003f,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
	// INTERRUPT PIN DEFINE
        tpDataWrite[0] = 0x08; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x03;
        ret = i2c_smbus_write_word_exp(g_client, 0x0008, 0x0003,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
	// LED PSEL
        tpDataWrite[0] = 0x3F; tpDataWrite[1] = 0x03;tpDataWrite[2] = 0x00;
        ret = i2c_smbus_write_word_exp(g_client, 0x003f, 0x0300,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
        tpDataWrite[0] = 0x3E; tpDataWrite[1] = 0x82;tpDataWrite[2] = 0x00;
        ret = i2c_smbus_write_word_exp(g_client, 0x003e, 0x8200,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
	// LED DWC
        tpDataWrite[0] = 0x2F; tpDataWrite[1] = 0xC0;tpDataWrite[2] = 0x20;
        ret = i2c_smbus_write_word_exp(g_client, 0x002f, 0xc028,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
	// LED6
        //tpDataWrite[0] = 0x28; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x00;
        //ret = i2c_smbus_write_word_dataSenBor(tpAddress, tpDataWrite, len);
	if(ret != 0)
	{
		return -1;
	}
        //tpDataWrite[0] = 0x38; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x40;
        //ret = i2c_smbus_write_word_dataSenBor(tpAddress, tpDataWrite, len);
	if(ret != 0)
	{
		return -1;
	}
	// LED7
        tpDataWrite[0] = 0x29; tpDataWrite[1] = 0x80;tpDataWrite[2] = 0x00;
        ret = i2c_smbus_write_word_exp(g_client, 0x0029, 0x9000,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}	
        tpDataWrite[0] = 0x39; tpDataWrite[1] = 0x01;tpDataWrite[2] = 0x04;
        ret = i2c_smbus_write_word_exp(g_client, 0x0039, 0x010F,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
	// Ambient Config	
        tpDataWrite[0] = 0x51; tpDataWrite[1] = 0x0A;tpDataWrite[2] = 0x1F;
        ret = i2c_smbus_write_word_exp(g_client, 0x0051, 0x0a1f,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}

	// Recalib Config
         tpDataWrite[0] = 0x52; tpDataWrite[1] = 0x07;tpDataWrite[2] = 0xFF;
        ret = i2c_smbus_write_word_exp(g_client, 0x0052, 0x07ff,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}

	// Long Touch
        tpDataWrite[0] = 0x53; tpDataWrite[1] = 0x07;tpDataWrite[2] = 0xFF;
        ret = i2c_smbus_write_word_exp(g_client, 0x0053, 0x07ff,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}

	// C0 ~ C4 Threshold : 60
         tpDataWrite[0] = 0x5F; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x01;
        ret = i2c_smbus_write_word_exp(g_client, 0x005f, 0x0001,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
        if(thld<6) thld = 6;else if(thld>255) thld = 255;
        tpDataWrite[0] = 0x60; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x24;
        ret = i2c_smbus_write_word_exp(g_client, 0x0060, 0x0024,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
        tpDataWrite[0] = 0x61; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x24;
        ret = i2c_smbus_write_word_exp(g_client, 0x0061, 0x0024,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
        tpDataWrite[0] = 0x62; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x24;
        ret = i2c_smbus_write_word_exp(g_client, 0x0062, 0x0024,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
        tpDataWrite[0] = 0x63; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x24;
        ret = i2c_smbus_write_word_exp(g_client, 0x0063, 0x0024,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
        tpDataWrite[0] = 0x64; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x24;
        ret = i2c_smbus_write_word_exp(g_client, 0x0064, 0x0024,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
        tpDataWrite[0] = 0x65; tpDataWrite[1] = 0x00;tpDataWrite[2] = thld;
        ret = i2c_smbus_write_word_exp(g_client, 0x0065, thld,REG_ADDRESS_SIZE_16);     
	if(ret != 0)
	{
		return -1;
	}
        tpDataWrite[0] = 0x6A; tpDataWrite[1] = 0x00;tpDataWrite[2] = thld;
       ret = i2c_smbus_write_word_exp(g_client, 0x006a, thld,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}
        tpDataWrite[0] = 0x6B; tpDataWrite[1] = 0x00;tpDataWrite[2] = thld;
        ret = i2c_smbus_write_word_exp(g_client, 0x006b, thld,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}
         tpDataWrite[0] = 0x6C; tpDataWrite[1] = 0x00;tpDataWrite[2] = thld;
        ret = i2c_smbus_write_word_exp(g_client, 0x006c, thld,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}
 tpDataWrite[0] = 0x6D; tpDataWrite[1] = 0x00;tpDataWrite[2] = thld;
        ret = i2c_smbus_write_word_exp(g_client, 0x006d, thld,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}
   tpDataWrite[0] = 0x6E; tpDataWrite[1] = 0x00;tpDataWrite[2] = thld;
        ret = i2c_smbus_write_word_exp(g_client, 0x006e, thld,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}
 tpDataWrite[0] = 0x6F; tpDataWrite[1] = 0x00;tpDataWrite[2] = thld;
        ret = i2c_smbus_write_word_exp(g_client, 0x006f, thld,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}

	//Debounce & Hysteresis
 tpDataWrite[0] = 0x57; tpDataWrite[1] = 0x10;tpDataWrite[2] = 0x00;
        ret = i2c_smbus_write_word_exp(g_client, 0x0057, 0x1000,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}
 tpDataWrite[0] = 0x75; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x02;
        ret = i2c_smbus_write_word_exp(g_client, 0x0075, 0x0002,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}
	
	// Guard Channel Enable : C7
 tpDataWrite[0] = 0x7C; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x00;
        ret = i2c_smbus_write_word_exp(g_client, 0x007c, 0x0000,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}
 tpDataWrite[0] = 0x7D; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x00;
        ret = i2c_smbus_write_word_exp(g_client, 0x007d, 0x0000,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}

	// Guard Channel Mask
 tpDataWrite[0] = 0x7E; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x00;
         ret = i2c_smbus_write_word_exp(g_client, 0x007e, 0x0000,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}
 tpDataWrite[0] = 0x7F; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x00;
         ret = i2c_smbus_write_word_exp(g_client, 0x007f, 0x0000,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}

	// TOUCH ENABLE
 tpDataWrite[0] = 0x40; tpDataWrite[1] = 0x80;tpDataWrite[2] = 0x38;
        ret = i2c_smbus_write_word_exp(g_client, 0x0040, 0x8038,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}

	// SOFT RESET
 tpDataWrite[0] = 0x01; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x00;
         ret = i2c_smbus_write_word_exp(g_client, 0x0001, 0x0000,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}
        do_gettimeofday(&end);
	return 0;
}

int set_threshold2(int thld)
{
     int ret = 0;
	u8 v1, v2;
        struct timeval start,end;
        do_gettimeofday(&start);
        if(!g_client)
        return -1;
	if (!i2c_check_functionality(g_client->adapter,I2C_FUNC_SMBUS_WORD_DATA))
        return -1;
	// COLD RESET
        pr_info("start gpio_init_sensor_board\n");
        unsigned char tpAddress = 0x2C;
        unsigned char tpDataWrite[3] = {0x00,0x00,0x00}; 
        unsigned int  len = 3;       

	ret = i2c_smbus_write_word_exp(g_client, 0x0000, 0x0000,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}

	// TOUCH DISABLE
        tpDataWrite[0] = 0x40; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x30;
        ret = i2c_smbus_write_word_exp(g_client, 0x0040, 0x0030,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}	
	// DCM CONFIG
        tpDataWrite[0] = 0x0A; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x00;
        ret = i2c_smbus_write_word_exp(g_client, 0x000a, 0x0000,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}
	// SCROLL CONFIG
        tpDataWrite[0] = 0x74; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x00;
        ret = i2c_smbus_write_word_exp(g_client, 0x0074, 0x0000,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}
	// PSEL
        tpDataWrite[0] = 0x41; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x3F;
        ret = i2c_smbus_write_word_exp(g_client, 0x0041, 0x003f,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}
         tpDataWrite[0] = 0x42; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x3F;
        ret = i2c_smbus_write_word_exp(g_client, 0x0042, 0x003f,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}
	// INTERRUPT
         tpDataWrite[0] = 0x43; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x3F;
       ret = i2c_smbus_write_word_exp(g_client, 0x0043, 0x003f,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}
        tpDataWrite[0] = 0x44; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x3F;
        ret = i2c_smbus_write_word_exp(g_client, 0x0044, 0x003f,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}
	// INTERRUPT PIN DEFINE
        tpDataWrite[0] = 0x08; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x03;
        ret = i2c_smbus_write_word_exp(g_client, 0x0008, 0x0003,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}
	// LED PSEL
        tpDataWrite[0] = 0x3F; tpDataWrite[1] = 0x03;tpDataWrite[2] = 0x00;
        ret = i2c_smbus_write_word_exp(g_client, 0x003f, 0x0300,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}
        tpDataWrite[0] = 0x3E; tpDataWrite[1] = 0x82;tpDataWrite[2] = 0x00;
        ret = i2c_smbus_write_word_exp(g_client, 0x003e, 0x8200,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}
	// LED DWC
        tpDataWrite[0] = 0x2F; tpDataWrite[1] = 0xC0;tpDataWrite[2] = 0x20;
        ret = i2c_smbus_write_word_exp(g_client, 0x002f, 0xc028,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}
	// LED6
        //tpDataWrite[0] = 0x28; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x00;
        //ret = i2c_smbus_write_word_dataSenBor(tpAddress, tpDataWrite, len);
	if(ret != 0)
	{
		return -1;
	}
        //tpDataWrite[0] = 0x38; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x40;
        //ret = i2c_smbus_write_word_dataSenBor(tpAddress, tpDataWrite, len);
	if(ret != 0)
	{
		return -1;
	}
	// LED7
        tpDataWrite[0] = 0x29; tpDataWrite[1] = 0x80;tpDataWrite[2] = 0x00;
        ret = i2c_smbus_write_word_exp(g_client, 0x0029, 0x9000,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}	
        tpDataWrite[0] = 0x39; tpDataWrite[1] = 0x01;tpDataWrite[2] = 0x04;
        ret = i2c_smbus_write_word_exp(g_client, 0x0039, 0x010F,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}
	// Ambient Config	
        tpDataWrite[0] = 0x51; tpDataWrite[1] = 0x0A;tpDataWrite[2] = 0x1F;
        ret = i2c_smbus_write_word_exp(g_client, 0x0051, 0x0a1f,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}

	// Recalib Config
         tpDataWrite[0] = 0x52; tpDataWrite[1] = 0x07;tpDataWrite[2] = 0xFF;
        ret = i2c_smbus_write_word_exp(g_client, 0x0052, 0x07ff,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}

	// Long Touch
        tpDataWrite[0] = 0x53; tpDataWrite[1] = 0x07;tpDataWrite[2] = 0xFF;
        ret = i2c_smbus_write_word_exp(g_client, 0x0053, 0x07ff,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}

	// C0 ~ C4 Threshold : 60
         tpDataWrite[0] = 0x5F; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x01;
        ret = i2c_smbus_write_word_exp(g_client, 0x005f, 0x0001,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}
        if(thld<17) thld = 17;else if(thld>255) thld = 255;
        tpDataWrite[0] = 0x60; tpDataWrite[1] = 0x00;tpDataWrite[2] = thld;
        ret = i2c_smbus_write_word_exp(g_client, 0x0060, thld,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}
        tpDataWrite[0] = 0x61; tpDataWrite[1] = 0x00;tpDataWrite[2] = thld;
         ret = i2c_smbus_write_word_exp(g_client, 0x0061, thld,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}
        tpDataWrite[0] = 0x62; tpDataWrite[1] = 0x00;tpDataWrite[2] = thld;
        ret = i2c_smbus_write_word_exp(g_client, 0x0062, thld,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}
        tpDataWrite[0] = 0x63; tpDataWrite[1] = 0x00;tpDataWrite[2] = thld;
         ret = i2c_smbus_write_word_exp(g_client, 0x0063, thld,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}
        tpDataWrite[0] = 0x64; tpDataWrite[1] = 0x00;tpDataWrite[2] = thld;
        ret = i2c_smbus_write_word_exp(g_client, 0x0064, thld,REG_ADDRESS_SIZE_16); 
	if(ret != 0)
	{
		return -1;
	}
        tpDataWrite[0] = 0x65; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x12;
         ret = i2c_smbus_write_word_exp(g_client, 0x0065, 0x0012,REG_ADDRESS_SIZE_16);  
	if(ret != 0)
	{
		return -1;
	}
        tpDataWrite[0] = 0x6A; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x12;
        ret = i2c_smbus_write_word_exp(g_client, 0x006a, 0x0012,REG_ADDRESS_SIZE_16);  
	if(ret != 0)
	{
		return -1;
	}
        tpDataWrite[0] = 0x6B; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x12;
        ret = i2c_smbus_write_word_exp(g_client, 0x006b, 0x0012,REG_ADDRESS_SIZE_16);  
	if(ret != 0)
	{
		return -1;
	}
         tpDataWrite[0] = 0x6C; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x12;
        ret = i2c_smbus_write_word_exp(g_client, 0x006c, 0x0012,REG_ADDRESS_SIZE_16);  
	if(ret != 0)
	{
		return -1;
	}
 tpDataWrite[0] = 0x6D; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x12;
        ret = i2c_smbus_write_word_exp(g_client, 0x006d, 0x0012,REG_ADDRESS_SIZE_16);  
	if(ret != 0)
	{
		return -1;
	}
   tpDataWrite[0] = 0x6E; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x12;
        ret = i2c_smbus_write_word_exp(g_client, 0x006e, 0x0012,REG_ADDRESS_SIZE_16);  
	if(ret != 0)
	{
		return -1;
	}
 tpDataWrite[0] = 0x6F; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x12;
        ret = i2c_smbus_write_word_exp(g_client, 0x006f, 0x0012,REG_ADDRESS_SIZE_16);  
	if(ret != 0)
	{
		return -1;
	}

	//Debounce & Hysteresis
 tpDataWrite[0] = 0x57; tpDataWrite[1] = 0x10;tpDataWrite[2] = 0x00;
        ret = i2c_smbus_write_word_exp(g_client, 0x0057, 0x1000,REG_ADDRESS_SIZE_16);  
	if(ret != 0)
	{
		return -1;
	}
 tpDataWrite[0] = 0x75; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x02;
        ret = i2c_smbus_write_word_exp(g_client, 0x0075, 0x0002,REG_ADDRESS_SIZE_16);  
	if(ret != 0)
	{
		return -1;
	}

	// Guard Channel Enable : C7
 tpDataWrite[0] = 0x7C; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x00;
        ret = i2c_smbus_write_word_exp(g_client, 0x007c, 0x0000,REG_ADDRESS_SIZE_16);  
	if(ret != 0)
	{
		return -1;
	}
 tpDataWrite[0] = 0x7D; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x00;
        ret = i2c_smbus_write_word_exp(g_client, 0x007d, 0x00,REG_ADDRESS_SIZE_16);  
	if(ret != 0)
	{
		return -1;
	}

	// Guard Channel Mask
 tpDataWrite[0] = 0x7E; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x00;
        ret = i2c_smbus_write_word_exp(g_client, 0x007e, 0x00,REG_ADDRESS_SIZE_16);  
	if(ret != 0)
	{
		return -1;
	}
 tpDataWrite[0] = 0x7F; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x00;
        ret = i2c_smbus_write_word_exp(g_client, 0x007f, 0x00,REG_ADDRESS_SIZE_16);  
	if(ret != 0)
	{
		return -1;
	}

	// TOUCH ENABLE
 tpDataWrite[0] = 0x40; tpDataWrite[1] = 0x80;tpDataWrite[2] = 0x38;
        ret = i2c_smbus_write_word_exp(g_client, 0x0040, 0x8038,REG_ADDRESS_SIZE_16);  

	if(ret != 0)
	{
		return -1;
	}

	// SOFT RESET
 tpDataWrite[0] = 0x01; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x00;
        ret = i2c_smbus_write_word_exp(g_client, 0x0001, 0x00,REG_ADDRESS_SIZE_16);  
	if(ret != 0)
	{
		return -1;
	}
        do_gettimeofday(&end);
	return 0;
}

int gpio_init_sensor_board(void)
{
	int ret = 0;
	u8 v1, v2;

        if(!g_client)
        return -1;
	if (!i2c_check_functionality(g_client->adapter,I2C_FUNC_SMBUS_WORD_DATA))
        return -1;
	// COLD RESET
        pr_info("start gpio_init_sensor_board\n");
        unsigned char tpAddress = 0x2C;
        unsigned char tpDataWrite[3] = {0x00,0x00,0x00}; 
        unsigned int  len = 3;       

	ret = i2c_smbus_write_word_exp(g_client, 0x0000, 0x00,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}

	// TOUCH DISABLE
        tpDataWrite[0] = 0x40; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x30;
        ret = i2c_smbus_write_word_exp(g_client, 0x0040, 0x30,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}	
	// DCM CONFIG
        tpDataWrite[0] = 0x0A; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x00;
        ret = i2c_smbus_write_word_exp(g_client, 0x000A, 0x00,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
	// SCROLL CONFIG
        tpDataWrite[0] = 0x74; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x00;
        ret = i2c_smbus_write_word_exp(g_client, 0x0074, 0x00,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
	// PSEL
        tpDataWrite[0] = 0x41; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x3F;
        ret = i2c_smbus_write_word_exp(g_client, 0x0041, 0x3F,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
         tpDataWrite[0] = 0x42; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x3F;
        ret = i2c_smbus_write_word_exp(g_client, 0x0042, 0x3F,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
	// INTERRUPT
         tpDataWrite[0] = 0x43; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x3F;
        ret = i2c_smbus_write_word_exp(g_client, 0x0043, 0x3f,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
        tpDataWrite[0] = 0x44; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x3F;
        ret = i2c_smbus_write_word_exp(g_client, 0x0044, 0x3f,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
	// INTERRUPT PIN DEFINE
        tpDataWrite[0] = 0x08; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x03;
        ret = i2c_smbus_write_word_exp(g_client, 0x0008, 0x03,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
	// LED PSEL
        tpDataWrite[0] = 0x3F; tpDataWrite[1] = 0x03;tpDataWrite[2] = 0x00;
        ret = i2c_smbus_write_word_exp(g_client, 0x003f, 0x0300,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
        tpDataWrite[0] = 0x3E; tpDataWrite[1] = 0x82;tpDataWrite[2] = 0x00;
        ret = i2c_smbus_write_word_exp(g_client, 0x003e, 0x8200,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
	// LED DWC
        tpDataWrite[0] = 0x2F; tpDataWrite[1] = 0xC0;tpDataWrite[2] = 0x20;
        ret = i2c_smbus_write_word_exp(g_client, 0x002f, 0xc028,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
	// LED6
        //tpDataWrite[0] = 0x28; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x00;
        //ret = i2c_smbus_write_word_dataSenBor(tpAddress, tpDataWrite, len);
	if(ret != 0)
	{
		return -1;
	}
        //tpDataWrite[0] = 0x38; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x40;
        //ret = i2c_smbus_write_word_dataSenBor(tpAddress, tpDataWrite, len);
	if(ret != 0)
	{
		return -1;
	}
	// LED7
        tpDataWrite[0] = 0x29; tpDataWrite[1] = 0x80;tpDataWrite[2] = 0x00;
        //ret = i2c_smbus_write_word_exp(g_client, 0x0029, 0x9000,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}	
        tpDataWrite[0] = 0x39; tpDataWrite[1] = 0x01;tpDataWrite[2] = 0x04;
        ret = i2c_smbus_write_word_exp(g_client, 0x0039, 0x03c4,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
	// Ambient Config	
        tpDataWrite[0] = 0x51; tpDataWrite[1] = 0x0A;tpDataWrite[2] = 0x1F;
        ret = i2c_smbus_write_word_exp(g_client, 0x0051, 0x0a1f,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}

	// Recalib Config
         tpDataWrite[0] = 0x52; tpDataWrite[1] = 0x07;tpDataWrite[2] = 0xFF;
        ret = i2c_smbus_write_word_exp(g_client, 0x0052, 0x07ff,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}

	// Long Touch
        tpDataWrite[0] = 0x53; tpDataWrite[1] = 0x07;tpDataWrite[2] = 0xFF;
        ret = i2c_smbus_write_word_exp(g_client, 0x0053, 0x07ff,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}

	// C0 ~ C4 Threshold : 60
         tpDataWrite[0] = 0x5F; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x01;
        ret = i2c_smbus_write_word_exp(g_client, 0x005f, 0x0001,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
        tpDataWrite[0] = 0x60; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x1A;
        ret = i2c_smbus_write_word_exp(g_client, 0x0060, 0x001a,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
        tpDataWrite[0] = 0x61; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x22;
        ret = i2c_smbus_write_word_exp(g_client, 0x0061, 0x0022,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
        tpDataWrite[0] = 0x62; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x27;
        ret = i2c_smbus_write_word_exp(g_client, 0x0062, 0x0027,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
        tpDataWrite[0] = 0x63; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x26;
        ret = i2c_smbus_write_word_exp(g_client, 0x0063, 0x0026,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
        tpDataWrite[0] = 0x64; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x21;
        ret = i2c_smbus_write_word_exp(g_client, 0x0064, 0x0021,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
        tpDataWrite[0] = 0x65; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x14;
        ret = i2c_smbus_write_word_exp(g_client, 0x0065, 0x0014,REG_ADDRESS_SIZE_16);     
	if(ret != 0)
	{
		return -1;
	}
        tpDataWrite[0] = 0x6A; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x14;
        ret = i2c_smbus_write_word_exp(g_client, 0x006a, 0x0014,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
        tpDataWrite[0] = 0x6B; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x14;
        ret = i2c_smbus_write_word_exp(g_client, 0x006b, 0x0014,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
         tpDataWrite[0] = 0x6C; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x14;
        ret = i2c_smbus_write_word_exp(g_client, 0x006c, 0x0014,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
 tpDataWrite[0] = 0x6D; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x14;
        ret = i2c_smbus_write_word_exp(g_client, 0x006d, 0x0014,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
   tpDataWrite[0] = 0x6E; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x14;
        ret = i2c_smbus_write_word_exp(g_client, 0x006e, 0x0014,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
 tpDataWrite[0] = 0x6F; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x14;
        ret = i2c_smbus_write_word_exp(g_client, 0x006f, 0x0014,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}

	//Debounce & Hysteresis
 tpDataWrite[0] = 0x57; tpDataWrite[1] = 0x10;tpDataWrite[2] = 0x00;
        ret = i2c_smbus_write_word_exp(g_client, 0x0057, 0x1000,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
 tpDataWrite[0] = 0x75; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x02;
        ret = i2c_smbus_write_word_exp(g_client, 0x0075, 0x0002,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}

	// Guard Channel Enable : C7
 tpDataWrite[0] = 0x7C; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x00;
        ret = i2c_smbus_write_word_exp(g_client, 0x007c, 0x0000,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
 tpDataWrite[0] = 0x7D; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x00;
        ret = i2c_smbus_write_word_exp(g_client, 0x007d, 0x0000,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}

	// Guard Channel Mask
 tpDataWrite[0] = 0x7E; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x00;
        ret = i2c_smbus_write_word_exp(g_client, 0x007e, 0x0000,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
 tpDataWrite[0] = 0x7F; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x00;
        ret = i2c_smbus_write_word_exp(g_client, 0x007f, 0x0000,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}

	// TOUCH ENABLE
 tpDataWrite[0] = 0x40; tpDataWrite[1] = 0x80;tpDataWrite[2] = 0x38;
       ret = i2c_smbus_write_word_exp(g_client, 0x0040, 0x8038,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
	
	// SOFT RESET
 tpDataWrite[0] = 0x01; tpDataWrite[1] = 0x00;tpDataWrite[2] = 0x00;
        ret = i2c_smbus_write_word_exp(g_client, 0x0001, 0x0000,REG_ADDRESS_SIZE_16);
	if(ret != 0)
	{
		return -1;
	}
	return 0;
}
static int __devinit gpio_keys_probe(struct platform_device *pdev)
{
	struct gpio_keys_platform_data *pdata = pdev->dev.platform_data;
	struct input_dev *input;
	int i, error;
        init_waitqueue_head(&gpio_127_q);
	input = input_allocate_device();
	if (!input)
		return -ENOMEM;
	platform_set_drvdata(pdev, input);

	input->evbit[0] = BIT(EV_KEY);

	input->name = pdev->name;
	input->phys = "gpio-keys/input0";
	input->cdev.dev = &pdev->dev;
	input->private = pdata;

	input->id.bustype = BUS_HOST;
	input->id.vendor = 0x0001;
	input->id.product = 0x0001;
	input->id.version = 0x0100;

        if(gpio_init_sensor_board() < 0)
	{
		printk(KERN_ERR "init sensor board failed\n");
	}
        else
         {
               printk(KERN_ERR "init sensor board ok\n");
          }
         msgsenbor.msgID = SensorBoardDataMessage;
         msgsenbor.data.senvalue = 0;
         msgsenlock.msgID = sensorboradLockMessage;
         msgsenlock.data.senlock = 1;
         msgswitch.msgID = SensorBoardSwitchMessage;
         msgswitch.data.senswitchstate = 0;
        //FIXME, not needed, really?
	//pxa3xx_enable_gpiokeyp_pins();    

        pxa3xx_mfp_set_pull(MFP_PIN_GPIO80, MFP_PULL_NONE);
    
        INIT_DELAYED_WORK(&SBwork, gpio_sb_work);
         INIT_DELAYED_WORK(&SBlockwork, gpio_sblock_work);
        
	for (i = 0; i < pdata->nbuttons; i++) {
		int code = pdata->buttons[i].keycode;
		int irq = gpio_to_irq(pdata->buttons[i].gpio);
		set_irq_type(irq, IRQ_TYPE_EDGE_BOTH);            
                error = request_irq(irq, gpio_keys_isr, IRQF_SAMPLE_RANDOM,
				     pdata->buttons[i].desc ? pdata->buttons[i].desc : "gpio_keys",
				     pdev);
		if (error) {
			pr_info("request_irq fail\n");
			goto fail;
		}
                irqtemp1 = irq;
		
	}
       for (i = 0; i <12; i++) {		
		set_bit(ButtonCode[i], input->keybit);
	}
     
        state80 = (gpio_get_value(80) ? 1 : 0) ^ 1;  
       
        res = platform_get_resource(pdev, IORESOURCE_MEM, 0);
	error = input_register_device(input);
	if (error) {
		
		goto fail;
	}

	return 0;

 fail:
	for (i = i - 1; i >= 0; i--)
		free_irq(gpio_to_irq(pdata->buttons[i].gpio), pdev);

	input_free_device(input);

	return error;
}

static int __devexit gpio_keys_remove(struct platform_device *pdev)
{
	struct gpio_keys_platform_data *pdata = pdev->dev.platform_data;
	struct input_dev *input = platform_get_drvdata(pdev);
	int i;

	for (i = 0; i < pdata->nbuttons; i++) {
		int irq = gpio_to_irq(pdata->buttons[i].gpio);
		free_irq(irq, pdev);
	}
	input_unregister_device(input);

	return 0;
}

static int gpio_keys_resume(struct platform_device *dev)
{                         
       if(qtstate != senflagcopy){
        msgsenlock.msgID = sensorboradLockMessage;
        if(senflag)
        msgsenlock.data.senvalue = 0;
        else 
        msgsenlock.data.senvalue = 1;
        msgevent_report_broadcast(&msgsenlock);
        senflagcopy = qtstate;}      
	return 0;
}

struct platform_driver gpio_keys_device_driver = {
	.probe		= gpio_keys_probe,
        .resume		= gpio_keys_resume,
	.remove		= __devexit_p(gpio_keys_remove),
	.driver		= {
		.name	= "gpio-keys",
	}
};

ssize_t GPIO_127_read(struct file *file,char *buf,size_t count,loff_t *f_ops)
{     
       if(!wait_127event && (file->f_flags & O_NONBLOCK))
            return count;
              
       DECLARE_WAITQUEUE(wait, current);
       add_wait_queue(&gpio_127_q, &wait);       
       wait_event_interruptible(gpio_127_q,
			(wait_127event != 0));
       if(copy_to_user(buf,&pValue,sizeof(int)))
           { printk(KERN_ALERT "read fail\n");
             goto out;      
             count = -EFAULT;      
           }
        wait_127event =0;
    
     if(file->f_flags & O_NONBLOCK)  
		pValue = 0;
   
       
out:
       set_current_state(TASK_RUNNING);
       remove_wait_queue(&gpio_127_q, &wait);
       return count;
}

ssize_t GPIO_127_write(struct file *file,const char *buf,size_t count,loff_t *f_ops)
{   
       pValue = 0;
       return count;
}

ssize_t GPIO_127_open(struct inode *inode,struct file *file)
{       
       pValue = 0;
       pDown = 0;
       MOD_INC_USE_COUNT:
       return 0;
}

ssize_t GPIO_127_release(struct inode *inode,struct file *file)
{
       sentestflag = 0;
       pValue = 0;
       MOD_DEC_USE_COUNT:
       return 0;
}

#define SENTEST_STATE 0x2856

ssize_t GPIO_127_ioctl(struct inode *inode,struct file *file,unsigned int cmd,long data)
{
         switch(cmd){
			case SENTEST_STATE:
				           set_Sensorboard(3);                                           
                                           sentestflag = 1;
			break;
                        default:
		        break;
		}
		
         return 0;
}

struct file_operations GPIO_127_ctl_ops = {
       open:   GPIO_127_open,
       read:   GPIO_127_read,
       write:  GPIO_127_write,
       ioctl:  GPIO_127_ioctl,
       release:   GPIO_127_release,
};

void gpio_sleeptimer_handle(unsigned long arg)
 {
 	 printk( KERN_ALERT "**gpio_timer handle**\n"); 
 	 if(allow_suspend())
 	 {
		ipm_event_notify(IPM_EVENT_ENTERSLEEPTIME, 0, NULL, 0);
		pValue = 2; 
		wait_127event =1; 
		wake_up_interruptible(&gpio_127_q);
	 }
		
}

EXPORT_SYMBOL(gpio_sleeptimer_handle);


ssize_t sen_ctl_read(struct file *file,char *buf,size_t count,loff_t *f_ops)
{
	   /*DECLARE_WAITQUEUE(wait, current);
       add_wait_queue(&sen_q, &wait);       
       wait_event_interruptible_timeout(sen_q,
			(pSen != 0),HZ/5);
	   if(copy_to_user(buf,&eventout,sizeof(struct senbod_event)))
           { printk(KERN_ALERT "read fail\n");
             goto out;      
             count = -EFAULT;      
           }
       pSen = 0;eventout.pSen = 0;eventout.code = 0;eventout.value = 0;
out:
       set_current_state(TASK_RUNNING);
       remove_wait_queue(&sen_q, &wait);*/
       return count;
}

ssize_t sen_ctl_write(struct file *file,const char *buf,size_t count,loff_t *f_ops)
{      pSen = 0;

       eventout.pSen = 0;
       eventout.code = 0;
       eventout.value = 0;

       return count;
}

ssize_t sen_ctl_open(struct inode *inode,struct file *file)
{      pSen = 0;
       eventout.pSen = 0;
       eventout.code = 0;
       eventout.value = 0;
       MOD_INC_USE_COUNT:
       return 0;
}

ssize_t sen_ctl_release(struct inode *inode,struct file *file)
{
	   pSen = 0;
	   eventout.pSen = 0;
       eventout.code = 0;
       eventout.value = 0;
       MOD_DEC_USE_COUNT:
       return 0;
}

#define SENIOCTL_SETSTATE 0x4660
#define SENIOCTL_GETSTATE 0x4681
#define SENIOCTL_SETSENSOR 0x4682
#define SENIOCTL_SETTHOLDS 0x4683
#define SENIOCTL_SETTHOLDB 0x4684
ssize_t sen_ctl_ioctl(struct inode *inode,struct file *file,unsigned int cmd,long data)
{
                void __user *argp = (void __user *)data;
		int ret=-1;
                int setsensor = 0;
		switch(cmd){
			case SENIOCTL_SETSTATE:
				if(data==0){
					//disable cap sensor
					set_Senlock();
					
				}else if(data==1){
					//enable cap sensor
					reset_Senlock();
					
				}
				ret=0;
				break;
             case SENIOCTL_GETSTATE:
                  if(copy_to_user(argp,&qtstate,sizeof(int)))
                  { printk(KERN_ALERT "SENIOCTL_GETSTATE fail\n");     }
					ret= 0;
					break;
             case SENIOCTL_SETSENSOR:
				if (copy_from_user(&setsensor, argp, sizeof(int)))
				{ printk(KERN_ALERT "SENIOCTL_SETSENSOR fail\n");  break;   }
                               
                      set_Sensorboard(setsensor+2);
                      if(senflag==1&&setsensor==1)
                       {qtstate = 1;}
                       else if(senflag==0&&setsensor==0)
                       {qtstate = 0;}
                        if(qtstate != senflagcopy){
							msgsenlock.msgID = sensorboradLockMessage;
                         if(senflag)
                            msgsenlock.data.senvalue = 0;
                          else 
                            msgsenlock.data.senvalue = 1;
                          msgevent_report_broadcast(&msgsenlock);
                          senflagcopy = qtstate;}
					ret= 0;
					break;
             case SENIOCTL_SETTHOLDS:
				if (copy_from_user(&setsensor, argp, sizeof(int)))
				{ printk(KERN_ALERT "SENIOCTL_SETTHOLD fail\n");  break;   }
                                 
                  set_threshold1(setsensor);
					ret= 0;
					break;
             case SENIOCTL_SETTHOLDB:
				if (copy_from_user(&setsensor, argp, sizeof(int)))
				{ printk(KERN_ALERT "SENIOCTL_SETTHOLD fail\n");  break;   }
                  set_threshold2(setsensor);
					ret= 0;
					break;
		}
		
         return ret;
}

struct file_operations sen_ctl_ops = {
       open:   sen_ctl_open,
       read:   sen_ctl_read,
       write:  sen_ctl_write,
       ioctl:  sen_ctl_ioctl,
       release:   sen_ctl_release,
};

static int __init gpio_keys_init(void)
{
        
        int ret = register_chrdev(98,"gpio_127_ctl",&GPIO_127_ctl_ops);	
        ret = register_chrdev(99,"sen_ctl",&sen_ctl_ops);	
        init_waitqueue_head(&sen_q);
        init_waitqueue_head(&gpio_127_q);
        init_timer(&stimer);
        init_timer(&sentimer);
        
        stimer.function = &timer_handle;
        sentimer.function = &sentimer_handle;
        init_timer(&gpio_sleeptimer);
        gpio_sleeptimer.function = &gpio_sleeptimer_handle;
        init_timer(&sSBtimer);
        sSBtimer.function = &timer_handleS;
        spin_lock_init(&bus_lock);
        //mutex_init(&bus_lock1);

        //add_timer(&stimer); 	
        ret = i2c_add_driver(&i2c_gpio_key_driver);

	
	return platform_driver_register(&gpio_keys_device_driver);
}

static void __exit gpio_keys_exit(void)
{
        del_timer(&stimer);
        del_timer(&sentimer);
        del_timer(&gpio_sleeptimer);
        del_timer(&sSBtimer);
        unregister_chrdev(98,"gpio_127_ctl");
        unregister_chrdev(99,"sen_ctl");
        i2c_del_driver(&i2c_gpio_key_driver);
	platform_driver_unregister(&gpio_keys_device_driver);
}



module_init(gpio_keys_init);
module_exit(gpio_keys_exit);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Phil Blundell <pb@handhelds.org>");
MODULE_DESCRIPTION("Keyboard driver for CPU GPIOs");
