/*
   Copyright (C) 2005, Intel Corporation.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


 *(C) Copyright 2006 Marvell International Ltd.
 * All Rights Reserved
 */

/*****************************************************************************
 * Abstract:
 *    contains all primitive functions for OV2655
 *
 * Notes:Only valid for processor code named Monahans.
 *****************************************************************************/
 
#include <linux/module.h>
#include "camera.h"
#include "ci.h"
#include "ov2655.h"
#include "ov2655_hw.h"
#include <linux/delay.h>
#include <asm/errno.h>

/* LUT Table, TBD */
static unsigned char lut_table[] = {
	/* RED LUT */
	0x00, 0x04, 0x08, 0x0c, 0x10, 0x14, 0x18, 0x1c,
	0x20, 0x24, 0x28, 0x2c, 0x30, 0x34, 0x38, 0x3c,
	0x40, 0x44, 0x48, 0x4c, 0x50, 0x54, 0x58, 0x5c,
	0x60, 0x64, 0x68, 0x6c, 0x70, 0x74, 0x78, 0x7c,
	0x80, 0x84, 0x88, 0x8c, 0x90, 0x94, 0x98, 0x9c,
	0xa0, 0xa4, 0xa8, 0xac, 0xb0, 0xb4, 0xb8, 0xbc,
	0xc0, 0xc4, 0xc8, 0xcc, 0xd0, 0xd4, 0xd8, 0xdc,
	0xe0, 0xe4, 0xe8, 0xec, 0xf0, 0xf4, 0xf8, 0xfc,

	/* BLUE LUT */
	0x00, 0x04, 0x08, 0x0c, 0x10, 0x14, 0x18, 0x1c,
	0x20, 0x24, 0x28, 0x2c, 0x30, 0x34, 0x38, 0x3c,
	0x40, 0x44, 0x48, 0x4c, 0x50, 0x54, 0x58, 0x5c,
	0x60, 0x64, 0x68, 0x6c, 0x70, 0x74, 0x78, 0x7c,
	0x80, 0x84, 0x88, 0x8c, 0x90, 0x94, 0x98, 0x9c,
	0xa0, 0xa4, 0xa8, 0xac, 0xb0, 0xb4, 0xb8, 0xbc,
	0xc0, 0xc4, 0xc8, 0xcc, 0xd0, 0xd4, 0xd8, 0xdc,
	0xe0, 0xe4, 0xe8, 0xec, 0xf0, 0xf4, 0xf8, 0xfc,

	/* GREEN LUT */
	0x00, 0x04, 0x08, 0x0c, 0x10, 0x14, 0x18, 0x1c,
	0x20, 0x24, 0x28, 0x2c, 0x30, 0x34, 0x38, 0x3c,
	0x40, 0x44, 0x48, 0x4c, 0x50, 0x54, 0x58, 0x5c,
	0x60, 0x64, 0x68, 0x6c, 0x70, 0x74, 0x78, 0x7c,
	0x80, 0x84, 0x88, 0x8c, 0x90, 0x94, 0x98, 0x9c,
	0xa0, 0xa4, 0xa8, 0xac, 0xb0, 0xb4, 0xb8, 0xbc,
	0xc0, 0xc4, 0xc8, 0xcc, 0xd0, 0xd4, 0xd8, 0xdc,
	0xe0, 0xe4, 0xe8, 0xec, 0xf0, 0xf4, 0xf8, 0xfc,
};


/* CMU conversion matrixes, they're sensor specific */
static CI_CMU_COE_MATRIX ov2655_cRGB_sRGB_COE = {
	RGB_FLOAT_TO_INT(0),  RGB_FLOAT_TO_INT(0), RGB_FLOAT_TO_INT(1),
	RGB_FLOAT_TO_INT(0) ,  RGB_FLOAT_TO_INT(1), RGB_FLOAT_TO_INT(0),
	RGB_FLOAT_TO_INT(1),  RGB_FLOAT_TO_INT(0), RGB_FLOAT_TO_INT(0)
	
	/*
	RGB_FLOAT_TO_INT(-0.06),  RGB_FLOAT_TO_INT(-0.30), RGB_FLOAT_TO_INT(1.36),
	RGB_FLOAT_TO_INT(-0.12) ,  RGB_FLOAT_TO_INT(1.32), RGB_FLOAT_TO_INT(-0.20),
	RGB_FLOAT_TO_INT(1.59),  RGB_FLOAT_TO_INT(-0.55), RGB_FLOAT_TO_INT(-0.04)
	*/
};

static CI_CMU_COE_MATRIX ov2655_cRGB_YUV_COE = {
	YUV_FLOAT_TO_INT(0.098),  YUV_FLOAT_TO_INT(0.504),
	YUV_FLOAT_TO_INT(0.257),
	
	YUV_FLOAT_TO_INT(0.439),  YUV_FLOAT_TO_INT(-0.291),
	YUV_FLOAT_TO_INT(-0.148),

	YUV_FLOAT_TO_INT(-0.071), YUV_FLOAT_TO_INT(-0.368),
	YUV_FLOAT_TO_INT(0.439),
};

/* Bad pixel table, it's sensor specific, TBD */
#define END_MARKER -1
static int ov2655_badpixel_table[] = {
	/*  column,        row
	 */
	END_MARKER, END_MARKER
};

/*****************************************************************************
 * OV2655 Functions
 *****************************************************************************/

int ov2655_init( p_camera_context_t camera_context )
{
	u8 cm_rev, cm_pid;
	u32 timeout;
	int status;
	camera_context->sensor_status.caps |= SENSOR_CAP_MANUAL_CONTRAST |
		SENSOR_CAP_MANUAL_WHITEBALANCE |
		SENSOR_CAP_MANUAL_EXPOSURE;

	ci_set_mode(CI_MODE_MP, CI_DATA_WIDTH8);
	//ci_set_mode(CI_MODE_MP, CI_DATA_WIDTH10);

	ci_set_clock(1, 1, 2400);//24MHZ
	
	ci_set_polarity(0, 0, 0);
	
	ci_set_fifo(0, CI_FIFO_THL_32, 1, 1);

	ci_cgu_set_black_level(0);

	ci_cgu_set_addr_mux_select(CI_CGU_MUX_0_TO_7);//8bit:CI_CGU_MUX_0_TO_7. 10bti:CI_CGU_MUX_2_TO_9

	ov2655hw_power_down(CAMERA_POWER_FULL);
	
	ov2655hw_reset();

	timeout = 50;
	do {
		cm_pid = cm_rev = 0;
		status = ov2655hw_version_revision(&cm_pid, &cm_rev);

		
		if( cm_pid != PIDH_OV2655 || cm_rev != PIDL_OV2655 )
		{
			printk(KERN_ERR "sensor init first fail\n");
			ov2655hw_power_down( CAMERA_POWER_OFF );
			ov2655hw_power_down( CAMERA_POWER_FULL );
			ov2655hw_reset();
			mdelay(1);
		}
		if (--timeout == 0){
			return -EIO;
		}
	} while( cm_pid != PIDH_OV2655 );

	
	ov2655hw_viewfinder_off();
	return 0;
}

int ov2655_deinit(  p_camera_context_t camera_context )
{
	ov2655hw_power_down( CAMERA_POWER_OFF );
	return 0;
}

int ov2655_sleep(  p_camera_context_t camera_context )
{	
	//sleep on
	/*
	u16 subaddress=0x3086;
	u8 val;

	ov2655hw_read_sensor_reg(subaddress, &val);
	val |= 0x01;
	ov2655hw_write_sensor_reg(subaddress, &val);
	*/
	
	return ov2655_deinit( camera_context );
}

int ov2655_wake(  p_camera_context_t camera_context )
{
	return ov2655_init( camera_context );
}

int ov2655_set_capture_format(p_camera_context_t camera_context )
{
	CI_MP_TIMING timing;
	u32 ovSizeFormat, ovFormat;

	//Set the current mode 
	switch(camera_context->capture_input_format) {
		case CAMERA_IMAGE_FORMAT_YCBCR422_PLANAR:
		case CAMERA_IMAGE_FORMAT_YCBCR422_PACKED:
			ovFormat = OV_FORMAT_YUV_422;//OV_FORMAT_RAW10;//OV_FORMAT_YUV_422;
			break;
		case CAMERA_IMAGE_FORMAT_RGB565:
			ovFormat = OV_FORMAT_RGB_565;
			break;
		case CAMERA_IMAGE_FORMAT_RAW10:
			ovFormat = OV_FORMAT_RAW10;
			break;
		default:
			ovFormat = OV_FORMAT_YUV_422;
			break;
	}
	
	if ( camera_context->capture_input_width == 352 &&
			camera_context->capture_input_height == 288 )
		ovSizeFormat = OV_SIZE_CIF;
	else if ( camera_context->capture_input_width == 800 &&
			camera_context->capture_input_height == 600 )
		ovSizeFormat = OV_SIZE_SVGA;
	else if ( camera_context->capture_input_width == 320 &&
			camera_context->capture_input_height == 240 )
		ovSizeFormat = OV_SIZE_QVGA;
	else if ( camera_context->capture_input_width == 640 &&
			camera_context->capture_input_height == 480 )
		ovSizeFormat = OV_SIZE_VGA;
	else if ( camera_context->capture_input_width == 1280 &&
			camera_context->capture_input_height == 1024 )
		ovSizeFormat = OV_SIZE_SXGA;
	else if ( camera_context->capture_input_width == 1600 &&
			camera_context->capture_input_height == 1200 )
		ovSizeFormat = OV_SIZE_UXGA;
	else if ( camera_context->capture_input_width == 1280 &&
			camera_context->capture_input_height == 960 )
		ovSizeFormat = OV_SIZE_SXGA_2;
	else {
		ovSizeFormat = OV_SIZE_VGA;
		camera_context->capture_input_width=640;
		camera_context->capture_input_height=480;
	}
		
		
	if (camera_context->cmu_usage == CI_CMU_OUTPUT_YUV) {
		ci_cmu_set_color_correction_coe(&ov2655_cRGB_YUV_COE);
	}else if (camera_context->cmu_usage == CI_CMU_OUTPUT_RGB) {
		ci_cmu_set_color_correction_coe(&ov2655_cRGB_sRGB_COE);
	}	
	ci_cmu_enable(camera_context->cmu_usage);

	ov2655hw_set_format(ovSizeFormat, ovFormat,camera_context->capture_mode);
	
	camera_context->psu_enable = 0;

	/* set capture width/height and timing */
	if (ovFormat == OV_FORMAT_RAW10) {
		timing.BFW = 0x01;
		timing.BLW = 0x01; //GRBG to RGGB
	} else {
		timing.BFW = 0x00;
		timing.BLW = 0x00;
	}
	

	ci_configure_mp(camera_context->capture_input_width-1,
		camera_context->capture_input_height-1, &timing);

#if defined(CONFIG_CPU_PXA310)
	ci_set_ycbcr_420_down_sample (camera_context->ycbcr_ds);
#endif

	return 0;
}

int ov2655_start_capture(p_camera_context_t camera_context,unsigned int frames)
{
	(void) camera_context;

	if(camera_context->cgu_enable){
		ci_cgu_load_lut_ram((unsigned int*)camera_context->
					histogram_lut_buffer_virtual,
				camera_context->histogram_lut_buffer_physical,
				(unsigned int*)camera_context->
					histogram_lut_dma_descriptors_virtual,
				camera_context->
					histogram_lut_dma_descriptors_physical,
				lut_table);
	}

	ci_cgu_enable(camera_context->cgu_enable);

	/* turn auto function on only doing continues capture */

	if (frames == 0) {
		ov2655hw_auto_function_on();
	}
	else {
		ov2655hw_auto_function_off();
	}
	
	//ov2655hw_auto_function_on();
	
	/* turn viewfinder on */
	ov2655hw_viewfinder_on();
	
	
	//u8 data=0x01;
	//ov2655hw_write_sensor_reg(0x3308,&data);
	return 0;
}

int ov2655_stop_capture(  p_camera_context_t camera_context )
{
	(void) camera_context;

	/* turn auto function off */
	ov2655hw_auto_function_off();

	/* turn viewfinder off */
	ov2655hw_viewfinder_off();

	return 0;
}

int ov2655_set_power_mode(p_camera_context_t camera_context, u8 mode)
{
	(void) camera_context;

	ov2655hw_power_down(mode );
	return 0;
}

int ov2655_read_16bit( p_camera_context_t camera_context,
		u16 reg_addr,  u16 *reg_val)
{
	(void) camera_context;

	ov2655hw_read_sensor_reg( reg_addr, (u8*)reg_val );
	
	return 0;
}

int ov2655_write_16bit( p_camera_context_t camera_context,
		u16 reg_addr,  u16 reg_val)
{
	u8 buffer;
	int status=0;

	(void) camera_context;

	buffer = (u8)reg_val;
	status = (ov2655hw_write_sensor_reg( reg_addr, &buffer ) == 0) ?
		0 : -EIO;
	
	return status;
}

int ov2655_set_contrast( p_camera_context_t camera_context, u8 mode, u32 value)
{
	(void) camera_context;

	if (mode == SENSOR_MANUAL_CONTRAST)
		return ov2655hw_set_contrast(value);
	else
		return 0;
}

int ov2655_set_exposure( p_camera_context_t camera_context, u8 mode, u32 value)
{
	(void) camera_context;

	if (mode == SENSOR_MANUAL_EXPOSURE)
		return ov2655hw_set_exposure(value);
	else
		return ov2655hw_set_exposure(SENSOR_EXPOSURE_LEVEL_MIDDLE);
}

int ov2655_set_white_balance( p_camera_context_t camera_context,
		u8 mode, u32 value)
{
	(void) camera_context;

	if (mode == SENSOR_MANUAL_WHITEBALANCE)
		return ov2655hw_set_white_balance(value);
	else
		return 0;
}

int ov2655_set_zoom(p_camera_context_t camera_context,u32 value)
{
	(void) camera_context;
	if(value<ZOOM_LEVEL_1x || value>ZOOM_LEVEL_2p5x)
		return -1;
		
	ov2655hw_set_zoom((ZOOM_LEVEL)value);
	
	return 0;
}

int ov2655_set_banding(p_camera_context_t camera_context,u32 value)
{
	(void) camera_context;
	
	ov2655hw_set_banding(value);
	
	return 0;
}

int ov2655_set_sharpness(p_camera_context_t camera_context,u32 value)
{
	(void) camera_context;
	
	ov2655hw_set_sharpness(value);
	
	return 0;
}

int ov2655_set_scene(p_camera_context_t camera_context,u32 value)
{
	(void) camera_context;
	
	ov2655hw_set_scene(value);
	
	return 0;
}

