/*
   Copyright (C) 2005, Intel Corporation.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


 *(C) Copyright 2006 Marvell International Ltd.
 * All Rights Reserved
 */

/*****************************************************************************
 * Abstract:
 * 	contains all hardware related functions for OV2655
 *
 * Notes:Only valid for processor code named Monahans.
 *****************************************************************************/
 
#include <linux/init.h>
#include <linux/module.h>
#include "camera.h"
#include "ov2655_hw.h"
#include <linux/delay.h>

#include <asm/arch/mfp.h>
#include <asm/arch/pxa3xx_gpio.h>
#include <asm/hardware.h>


#include <asm/arch/pxa3xx_pmic.h>

static ZOOM_LEVEL g_zoom_in=ZOOM_LEVEL_1x;
static int g_banding_set=V4L2_CID_POWER_LINE_FREQUENCY_60HZ;

//init regs values
/*24MHz 		;7.5FPS
  1600x1200		;YUV	output
*/
static u16 Init_Regs_Values_UXGA[]={
	//Soft Reset
	0x3012, 0x80,//Add some dealy or wait a few miliseconds after register reset
	
	//IO & Clock & Analog Setup
	0x308c, 0x80,
	0x308d, 0x0e,
	0x360b, 0x00,
	0x30b0, 0xff,
	0x30b1, 0xff,
	0x30b2, 0x24,
	
	0x300e, 0x34,
	0x300f, 0xa6,
	0x3010, 0x81,
	0x3082, 0x01,//Reserved
	0x30f4, 0x01,//Reserved
	0x3090, 0x33,//Reserved for mirror
	0x3091, 0xc0,//Reserved
	0x30ac, 0x42,//Reserved
	
	0x30d1, 0x08,//Reserved
	0x30a8, 0x56,//Reserved
	0x3015, 0x32,
	0x3093, 0x00,//Reserved
	0x307e, 0xe5,
	0x3079, 0x00,//Reserved
	0x30aa, 0x42,//Reserved
	0x3017, 0x40,
	0x30f3, 0x82,//Reserved
	0x306a, 0x0c,//Reserved
	0x306d, 0x00,//Reserved
	0x336a, 0x3c,//Reserved
	0x3076, 0x6a,
	0x30d9, 0x95,//0x80,//Reserved
	0x3016, 0x92,
	0x3601, 0x30,//DVP ctl
	0x304e, 0x88,//Reserved
	0x30f1, 0x82,//Reserved
	
	0x306f, 0x54,
	
	0x3011, 0x01,// on system 00
	//0x3308, 0x10,
	0x3085, 0x20,
	
	//AEC/AGC
	0x3307,	0x00,//AE after Gamma
	0x3013, 0xf7,//0xe7,//0xf7,//auto
	0x3018, 0x78,//0x80,//0x78,//gordon 0610 target 133 level
	0x3019, 0x68,//0x70,//0x68,
	0x301a, 0xd4,//0x83,//0xd4,
	
	//Flicker
	0x301c, 0x0d,
	0x301d, 0x0e,
	0x3070, 0x5f,
	0x3071, 0x00,
	0x3072, 0x4f,
	0x3073, 0x00,
	
	//Property Flicker mode
	0x3014, 0x04,//0x05,//0x04,//0x25,//0x44,//0x0C,
	
	//Property AE Window
	/*
	0x3038, 0x02,
	0x3039, 0x30,
	0x303a, 0x00,
	0x303b, 0xBE,
	0x303c, 0x01,
	0x303d, 0x80,
	0x303e, 0x00,
	0x303F, 0xB4,
	0x3030, 0x96,
	0x3031, 0xBE,
	0x3032, 0xBE,
	0x3033, 0x96,
	*/
	
	//Window Setup
	//sensor x start point:280 offset
	0x3020, 0x01,
	0x3021, 0x18,
	//sensor y start point:10 offset
	0x3022, 0x00,
	0x3023, 0x0a,
	//sensor x output size: 1624
	0x3024, 0x06,
	0x3025, 0x58,
	//sensor y output size: 1212
	0x3026, 0x04,
	0x3027, 0xbc,
	//DVP output x/y size: 1600*1200
	0x3088, 0x06,//
	0x3089, 0x40,//1600
	0x308a, 0x04,//
	0x308b, 0xb0,//1200
	//DSP scale input x/y size: 1600*1200
	0x3316, 0x64,
	0x3317, 0x4b,
	0x3318, 0x00,
	//DSP scale output x/y size: 1600*1200
	0x331a, 0x64,
	0x331b, 0x4b,
	0x331c, 0x00,
	
	0x3100, 0x00,
	//0x3319,	0x2c,//0x6c,//0x0c,//for awb timing
	//0x331d, 0x4c,//0x6c,//0x4c,//for awb timing
	0x3302, 0x01,//0x01,//scale_disable
	
	//AWB
	0x3320, 0xf8,//0xf8,//0x99,
	0x3321, 0x11, 
	0x3322, 0x92, 
	0x3323, 0x01, 
	0x3324, 0x97,//0x97,//0x92
	0x3325, 0x02, 
	0x3326, 0xff,
	0x3327, 0x0c,
	0x3328, 0x0c,//0x0c,//0x10
	0x3329, 0x0f,//0x0f,//0x10
	0x332a, 0x56,//0x56,//0x52
	0x332b, 0x50,
	0x332c, 0xb6,//0xb6,//0xbe
	0x332d, 0xcc,//0xcc,//0xce
	0x332e, 0x3f,//0x3f,//0x3c
	0x332f, 0x36,
	0x3330, 0x46,
	0x3331, 0x3f,//0x3f,//0x44
	0x3332, 0xf0,
	0x3333, 0x10,//0x10,//0x00
	0x3334, 0xf0,
	0x3335, 0xf0,
	0x3336, 0xf0,
	0x3337, 0x40,
	0x3338, 0x40,
	0x3339, 0x40,
	0x333a, 0x00,
	0x333b, 0x00,
	
	//Color Matrix
	0x3380, 0x28,  
	0x3381, 0x48,   
	0x3382, 0x10,  
	0x3383, 0x23,//0x23,//0x3a
	0x3384, 0xc0,//0xc0,//0xad
	0x3385, 0xe5,//0xe5,//0xe6
	0x3386, 0xc2,//0xc2,//0xc0
	0x3387, 0xb3,//0xb3,//0xb6
	0x3388, 0x0e,//0x0e,//0x0a
	0x3389, 0x98, 
	0x338a, 0x01,
	
	//Gamma
	0x3340, 0x0e,
	0x3341, 0x1a,
	0x3342, 0x31,
	0x3343, 0x45,
	0x3344, 0x5a,
	0x3345, 0x69,
	0x3346, 0x75,
	0x3347, 0x7e,
	0x3348, 0x88,
	0x3349, 0x96,
	0x334a, 0xa3,
	0x334b, 0xaf,
	0x334c, 0xc4,
	0x334d, 0xd7,
	0x334e, 0xe8,
	0x334f, 0x20,
	
	//Lens correction
	//R
	0x3350, 0x33,
	0x3351, 0x28,
	0x3352, 0x00,
	0x3353, 0x29,	
	0x3354, 0x30,
	0x3355, 0x85,
	//G
	0x3356, 0x33,
	0x3357, 0x29,
	0x3358, 0x00,
	0x3359, 0x27,
	0x335a, 0x80,
	0x335b, 0x85,
	//B
	0x335c, 0x32,
	0x335d, 0x28,
	0x335e, 0x00,
	0x335f, 0x29,
	0x3360, 0x00,
	0x3361, 0x85,
	//
	0x3362, 0x80,
	0x3363, 0x01,
	0x3364, 0x03,
	0x3365, 0x02,
	0x3366, 0x00,
	
	//UVadjust
	0x3301, 0xff,
	0x338B, 0x0e,
	0x338c, 0x10,
	0x338d, 0x40,
	
	//Sharpness/De-noise
	0x3370, 0xd0,
	0x3371, 0x00,
	0x3372, 0x00,
	0x3373, 0x40,
	0x3374, 0x10,
	0x3375, 0x10,
	0x3376, 0x04,//0x07
	0x3377, 0x00,
	0x3378, 0x04,
	0x3379, 0x80,
	
	//BLC
	0x3069, 0x84,
	0x307c, 0x10,
	0x3087, 0x02,
	
	//Other functions
	0x3300, 0xfc,//ISP
	0x3306, 0x00,//0x80,//Reserved
	0x3400, 0x02,//format ctl: YUV422(UYVY)
	0x3606, 0x20,//DVP_EN
	0x3601, 0x30,//DVP ctl

	0x30f3, 0x82,//0x83,
	0x304e, 0x88,//Reserved
	
	//enable property sat con hue
	0x3391, 0x07,
	0x3390, 0x45,
	0x3398, 0x24,
	0x3399, 0x24,
	
	0x3086, 0x0f,
	0x3086, 0x00,
	
	/*
	0x3376, 0x04,
	//enable property sat con hue
	0x3391, 0x07,
	0x3390, 0x41,    //hue sin
	0x3393, 0x00,    //hue cos
	0x3399, 0x20,    //contrast
	0x3394, 0x40,    //sat u
	0x3395, 0x40,    //sat v
	*/
	
	OV2655_REGEND,0x00
};

//init regs values
/*24MHz 		;15FPS, 18MHZ
  800X600		;YUV(UYVY)	output
*/
/*
static u16 Init_Regs_Values_SVGA[]={
	//Soft Reset
	0x3012, 0x80,//Add some dealy or wait a few miliseconds after register reset
	
	//IO & Clock & Analog Setup
	0x308c, 0x80,
	0x308d, 0x0e,
	0x360b, 0x00,
	0x30b0, 0xff,
	0x30b1, 0xff,
	0x30b2, 0x2c,//0x24,
	
	//15 fps
	0x300e, 0x34,
	0x300f, 0xa6,
	0x3010, 0x81,
	0x3082, 0x01,//Reserved
	0x30f4, 0x01,//Reserved
	0x3090, 0x33,//0x03,//Reserved for mirror
	0x3091, 0xc0,//Reserved
	0x30ac, 0x42,//Reserved
	
	0x30d1, 0x08,//Reserved
	0x30a8, 0x76,//0x56,//Reserved
	0x3015, 0x04,//0x03,//0x32,//0x02,
	0x3093, 0x00,//Reserved
	0x307e, 0xe5,
	0x3079, 0x00,//Reserved
	0x30aa, 0x42,//0x32,//0x42,//Reserved
	0x3017, 0x40,
	0x30f3, 0x81,//0x82,//Reserved
	0x306a, 0x0c,//Reserved
	0x306d, 0x00,//Reserved
	0x336a, 0x3c,//Reserved
	0x3076, 0x6a,
	0x30d9, 0x80,//0x95,//Reserved
	0x3016, 0x82,//0x92,//0x82,
	0x3601, 0x30,//DVP ctl
	0x304e, 0x88,//Reserved
	0x30f1, 0x82,//Reserved
	0x306f, 0x14,
	0x302a, 0x02,
	0x302b, 0x6a,

	0x3012, 0x10,
	0x3011, 0x01,// on system 00
	
	//AEC/AGC
	//0x3307,	0x00,//AE after Gamma
	0x3013, 0xf7,//0xe7,//0xf7,//auto
	0x3018, 0x78,//0x70,//0x82,//0x80,//AE hight limt //gordon 0610 target 133 level
	0x3019, 0x68,//0x60,//0x72,//0x70,//AE low limt
	0x301a, 0xd4,//0xd4,//0xa3,//0x83,
	
	//Flicker
	0x301c, 0x06,
	0x301d, 0x07,
	0x3070, 0x5f,
	0x3071, 0x00,
	0x3072, 0x4f,
	0x3073, 0x00,
	
	//Property Flicker mode
	0x3014, 0x05,//0x05,//0x25,//0x44,//0x0C,//-------------?
	
	0x30a3, 0x80,
	0x30a1, 0x41,
	0x363b, 0x01,
	0x363c, 0xf2,
	
	//D0506
	0x30af,0x00,
	0x3048,0x1f,
	0x3049,0x4e,
	0x304a,0x20,
	0x304f,0x20,
	0x304b,0x02,
	0x304c,0x00,
	0x304d,0x02,
	0x304f,0x20,
	0x30a3,0x10,
	0x3013,0xf7,
	0x3014,0x25,//0x44,//0x25,//R1D bit6 always = 0 , bit[5]=1, bit[0]=1
	0x304d,0x42,
	0x304a,0x40,//0x00, //Disable 50/60 auto detection function, due to ov2650 no this function
	0x304f,0x40,//0x00, //Disable 50/60 auto detection function, due to ov2650 no this function
	0x3095,0x07,
	0x3096,0x16,
	0x3097,0x1d,
	
	//Property AE Window
	0x3038, 0x02,
	0x3039, 0x30,
	0x303a, 0x00,
	0x303b, 0xBE,
	0x303c, 0x01,
	0x303d, 0x80,
	0x303e, 0x00,
	0x303F, 0xB4,
	0x3030, 0x96,
	0x3031, 0xBE,
	0x3032, 0xBE,
	0x3033, 0x96,
	0x30dc, 0x00,
	
	//Window Setup
	0x3020, 0x01,//sensor x start point (h)
	0x3021, 0x18,//sensor x start point (l)---->0x118==280
	0x3022, 0x00,//sensor y start point (h)
	0x3023, 0x06,//sensor y start point (l)----->0x6==6
	0x3024, 0x06,//sensor x output size (h)
	0x3025, 0x58,//sensor x output size (l)----->0x658==1624
	0x3026, 0x02,//sensor y output size (h)
	0x3027, 0x61,//sensor y output size (l)----->0x261==609
	
	0x3088, 0x03,//DVP x output size (h)
	0x3089, 0x20,//DVP x output size (l)----->0x320==800
	0x308a, 0x02,//DVP y output size (h)
	0x308b, 0x58,//DVP y output size (l)----->0x258==600
	
	0x3316, 0x64,//scale x input size (h)
	0x3317, 0x25,//scale y input size (h)
	0x3318, 0x80,//scale y/x input size (l)----->x: 1600, y: 600
	//0x3319, 0x08,//0x08,//0x0c,//add for awb timing
	//0x331d, 0x38,//0x38,//0x4c, for awb timing
	0x331a, 0x32,//scale x output size (h)
	0x331b, 0x25,//scale y output size (h)
	0x331c, 0x80,//scale y/x output size (l)----->x: 800, y: 600
	0x3100, 0x00,

	//AWB
	0x3320, 0xf8,
	0x3321, 0x11, 
	0x3322, 0x92, 
	0x3323, 0x01, 
	0x3324, 0x97,
	0x3325, 0x02, 
	0x3326, 0xff,
	0x3327, 0x0c,
	0x3328, 0x0c,
	0x3329, 0x0f,
	0x332a, 0x56,
	0x332b, 0x50,
	0x332c, 0xb6,
	0x332d, 0xcc,
	0x332e, 0x3f,
	0x332f, 0x36,
	0x3330, 0x46,
	0x3331, 0x3f,
	0x3332, 0xf0,
	0x3333, 0x10, 
	0x3334, 0xf0,
	0x3335, 0xf0,
	0x3336, 0xf0,
	0x3337, 0x40,
	0x3338, 0x40,
	0x3339, 0x40,
	0x333a, 0x00,
	0x333b, 0x00,
	
	//Color Matrix
	0x3380, 0x28,  
	0x3381, 0x48,   
	0x3382, 0x10,  
	0x3383, 0x23, 
	0x3384, 0xc0,
	0x3385, 0xe5,   
	0x3386, 0xc2,  
	0x3387, 0xb3,  
	0x3388, 0x0e,  
	0x3389, 0x98, 
	0x338a, 0x01,
	
	//Gamma
	0x3340, 0x0e,
	0x3341, 0x1a,
	0x3342, 0x31,
	0x3343, 0x45,
	0x3344, 0x5a,
	0x3345, 0x69,
	0x3346, 0x75,
	0x3347, 0x7e,
	0x3348, 0x88,
	0x3349, 0x96,
	0x334a, 0xa3,
	0x334b, 0xaf,
	0x334c, 0xc4,
	0x334d, 0xd7,
	0x334e, 0xe8,
	0x334f, 0x20,
	
	//Lens correction
	//R
	0x3350, 0x33,
	0x3351, 0x28,
	0x3352, 0x00,
	0x3353, 0x29,	
	0x3354, 0x30,
	0x3355, 0x85,
	//G
	0x3356, 0x33,
	0x3357, 0x29,
	0x3358, 0x00,
	0x3359, 0x27,
	0x335a, 0x80,
	0x335b, 0x85,
	//B
	0x335c, 0x32,
	0x335d, 0x28,
	0x335e, 0x00,
	0x335f, 0x29,
	0x3360, 0x00,
	0x3361, 0x85,
	//
	0x3362, 0x80,
	0x3363, 0x01,
	0x3364, 0x03,
	0x3365, 0x02,
	0x3366, 0x00,
	
	//UVadjust
	0x3301, 0xff,
	0x338B, 0x0e,
	0x338c, 0x10,
	0x338d, 0x40,
	
	//Sharpness/De-noise
	0x3370, 0xd0,
	0x3371, 0x00,
	0x3372, 0x00,
	0x3373, 0x40,
	0x3374, 0x10,
	0x3375, 0x10,
	0x3376, 0x04,//0x07,//04
	0x3377, 0x00,
	0x3378, 0x04,
	0x3379, 0x80,
	
	//BLC
	0x3069, 0x84,
	0x307c, 0x10,
	0x3087, 0x02,
	
	//Other functions
	0x3300, 0xfc,//ISP
	0x3306, 0x80,//Reserved
	0x3400, 0x02,//format ctl: YUV422(UYVY)
	0x3606, 0x20,//DVP_EN
	0x3601, 0x30,//DVP ctl
	0x30f3, 0x83,//Reserved
	0x304e, 0x88,//Reserved

	0x3086, 0x0f,
	0x3086, 0x00,
	
	//enable property sat con hue
	0x3391, 0x07,
	0x3390, 0x41,    //hue sin
	0x3393, 0x00,    //hue cos
	0x3399, 0x20,    //contrast
	0x3394, 0x40,    //sat u
	0x3395, 0x40,    //sat v
	
	OV2655_REGEND,0x00
};
*/
/*
static u16 SVGA_To_UXGA[]={
	//pclk:36Mhz, fps:7.5
	0x300e, 0x34,
	0x3011, 0x01,// on system 00
	
	0x3012, 0x00,
	0x302a, 0x04,
	0x302b, 0xd4,
	0x306f, 0x54,
	
	//0x3015, 0x32,
	//0x3014,	0x05,//0x44,//0x0c,
	
	//Flicker
	0x301c, 0x0d,
	0x301d, 0x0e,
	0x3070, 0x5f,
	0x3071, 0x00,
	0x3072, 0x4f,
	0x3073, 0x00,
	
	//Window Setup
	0x3020, 0x01,
	0x3021, 0x18,
	0x3022, 0x00,
	0x3023, 0x0a,
	0x3024, 0x06,
	0x3025, 0x58,
	0x3026, 0x04,
	0x3027, 0xbc,
	
	0x3088, 0x06,//
	0x3089, 0x40,//1600
	0x308a, 0x04,//
	0x308b, 0xb0,//1200
	
	0x331a, 0x64,
	0x331b, 0x4b,
	0x331c, 0x00,
	
	0x3316, 0x64,
	0x3317, 0x4b,
	0x3318, 0x00,
	
	//0x3319, 0x6c,//0x2c,
	//0x331d, 0x6c,//0x4c,
	0x3302, 0x01,//0x00,//0x01,
	0x3100, 0x00,
	
	OV2655_REGEND,0x00
};
static u16 UXGA_To_SVGA[]={
	//pclk:18Mhz, fps:15
	0x300e, 0x34,
	0x3011, 0x01,// on system 00
	
	0x3012, 0x10,
	0x302a, 0x02,
	0x302b, 0x6a,
	0x306f, 0x14,
	
	//0x3015, 0x04,//0x32,
	//0x3014,	0x05,//0x44,//0x0c,
	
	//Flicker
	0x301c, 0x06,
	0x301d, 0x07,
	0x3070, 0x5f,
	0x3071, 0x00,
	0x3072, 0x4f,
	0x3073, 0x00,
	
	0x3020, 0x01,//sensor x start point (h)
	0x3021, 0x18,//sensor x start point (l)---->0x118==280
	0x3022, 0x00,//sensor y start point (h)
	0x3023, 0x06,//sensor y start point (l)----->0x6==6
	0x3024, 0x06,//sensor x output size (h)
	0x3025, 0x58,//sensor x output size (l)----->0x658==1624
	0x3026, 0x02,//sensor y output size (h)
	0x3027, 0x61,//sensor y output size (l)----->0x261==609
	
	0x3088, 0x03,//DVP x output size (h)
	0x3089, 0x20,//DVP x output size (l)----->0x320==800
	0x308a, 0x02,//DVP y output size (h)
	0x308b, 0x58,//DVP y output size (l)----->0x258==600
	
	0x3316, 0x64,//scale x input size (h)
	0x3317, 0x25,//scale y input size (h)
	0x3318, 0x80,//scale y/x input size (l)----->x: 1600, y: 600
	
	0x331a, 0x32,//scale x output size (h)
	0x331b, 0x25,//scale y output size (h)
	0x331c, 0x80,//scale y/x output size (l)----->x: 800, y: 600
	
	//0x3319, 0x6c,//0x2c,
	//0x331d, 0x6c,//0x4c,
	0x3302, 0x11,//0x10,//0x11,
	0x3100, 0x00,
	
	OV2655_REGEND,0x00
};
*/

static u16 QVGA_Regs[]={
	0x3302, 0x11,//0x10,//0x11,
	//banding
	0x301c, 0x06,
	0x301d, 0x07,
	
	0x3088, 0x01,
	0x3089, 0x40,//320
	0x308a, 0x00,
	0x308b, 0xf0,//240
	0x331a, 0x14,
	0x331b, 0x0f,
	0x331c, 0x00,
	OV2655_REGEND,0x00
};
static u16 CIF_Regs[]={
	0x3302, 0x11,//0x10,//0x11,
	0x3088, 0x01,
	0x3089, 0x60,//352
	0x308a, 0x01,
	0x308b, 0x20,
	0x331a, 0x16,//288
	0x331b, 0x12,
	0x331c, 0x00,
	OV2655_REGEND,0x00
};
static u16 VGA_Regs[]={
	0x3302, 0x11,//0x10,//0x11,
	
	0x3010, 0x82,
	
	//banding
	0x301c, 0x06,
	0x301d, 0x07,
	
	//DSP scale output: 640*480
	0x331a, 0x28,
	0x331b, 0x1e,
	0x331c, 0x00,
	
	//DVP output to ci: 640*480
	0x3088, 0x02,
	0x3089, 0x80,//640
	0x308a, 0x01,
	0x308b, 0xe0,//480
	
	OV2655_REGEND,0x00
};
//SVGA ;800*600
static u16 SVGA_Regs[]={
	0x3302, 0x11,//0x10,//0x11,
	
	0x3010, 0x82,
	
	//banding
	0x301c, 0x06,
	0x301d, 0x07,
	
	0x3088, 0x03,
	0x3089, 0x20,
	0x308a, 0x02,
	0x308b, 0x58,
	0x331a, 0x32,
	0x331b, 0x25,
	0x331c, 0x80,
	OV2655_REGEND,0x00
};
//SXGA ;1280*1024
static u16 SXGA_Regs[]={	           
	0x3302, 0x11,//0x10,//0x11,
	
	0x3010, 0x81,
	
	//banding
	0x301c, 0x0d,
	0x301d, 0x0e,
	
	0x3088, 0x05,
	0x3089, 0x00,
	0x308a, 0x04,
	0x308b, 0x00,
	0x331a, 0x50,
	0x331b, 0x40,
	0x331c, 0x00,
	OV2655_REGEND,0x00
};
//SXGA ;1280*960
static u16 SXGA_2_Regs[]={	           
	0x3302, 0x11,//0x10,//0x11,
	
	0x3010, 0x81,
	
	//banding
	0x301c, 0x0d,
	0x301d, 0x0e,
	
	0x3088, 0x05,
	0x3089, 0x00,
	0x308a, 0x03,
	0x308b, 0xc0,
	0x331a, 0x50,
	0x331b, 0x3c,
	0x331c, 0x00,
	OV2655_REGEND,0x00
};
//UXGA
static u16 UXGA_Regs[]={	           
	0x3302, 0x01,//0x00,//0x01,//scale_disable
	
	0x3010, 0x81,
	
	//banding
	0x301c, 0x0d,
	0x301d, 0x0e,
	
	0x3088, 0x06,//
	0x3089, 0x40,//1600
	0x308a, 0x04,//
	0x308b, 0xb0,//1200
	0x331a, 0x64,
	0x331b, 0x4b,
	0x331c, 0x00,
	OV2655_REGEND,0x00
};

static u16 VGA_Zoom_2p5x[]={
	0x3302, 0x11,//0x10,//0x11, //scale disable
	0x3012, 0x00,//0x24
	
	//adjust pclk
	0x300e, 0x38,//0x34,
	0x3011, 0x00,
	
	//sensor x start:(1600-640)/2+280==760
	0x3020, 0x02,
	0x3021, 0xf8,//0xec,//0xf8,
	//sensor y start:(1200-480)/2+10=370
	0x3022, 0x01,//0x00,//0x01,
	0x3023, 0x72,//0x0a,//0x72,
	
	//sensor x output:640+24
	0x3024, 0x02,
	0x3025, 0x98,
	//sensor y output:480+12
	0x3026, 0x01,
	0x3027, 0xec,
	
	//scale input: 640*480
	0x3316, 0x28,
	0x3317, 0x1e,
	0x3318, 0x00,
	
	OV2655_REGEND,0x00
};
static u16 VGA_Zoom_2x[]={
	0x3302, 0x11,//0x10,//0x11,//scale enable
	0x3012, 0x00,
	
	//adjust pclk:24Mhz 10fps
	0x300e, 0x38,//0x34
	0x3011, 0x00,
	
	//sensor x start:(1600-800)/2+280==680
	0x3020, 0x02,
	0x3021, 0xa8,
	//sensor y start:(1200-600)/2+10=310
	0x3022, 0x01,
	0x3023, 0x36,
	
	//800+24 * 600+12
	//sensor x output
	0x3024, 0x03,
	0x3025, 0x38,
	//sensor y output
	0x3026, 0x02,
	0x3027, 0x64,
	
	//scale input:800*600
	0x3316, 0x32,
	0x3317, 0x25,
	0x3318, 0x80,
	
	OV2655_REGEND,0x00
};
static u16 VGA_Zoom_1p25x[]={
	0x3302, 0x11,//0x10,//0x11, //scale enable
	0x3012, 0x00,
	
	//adjust pclk
	0x300e, 0x34,
	0x3011, 0x00,
	
	//sensor x start:(1600-1280)/2+280==440
	0x3020, 0x01,
	0x3021, 0xb8,
	//sensor y start:(1200-960)/2+10=130
	0x3022, 0x00,
	0x3023, 0x82,
	
	//sensor x output:1280+24
	0x3024, 0x05,
	0x3025, 0x18,
	//sensor y output:960+12
	0x3026, 0x03,
	0x3027, 0xcc,
	
	//scale input: 1280*960
	0x3316, 0x50,
	0x3317, 0x3c,
	0x3318, 0x00,
	
	OV2655_REGEND,0x00
};
static u16 VGA_Zoom_1x[]={
	0x3302, 0x11,//0x10,//0x11,
	0x3012, 0x00,
	
	//adjust pclk:36Mhz 15fps
	0x300e, 0x34,
	0x3011, 0x00,
	
	//sensor x start:280
	0x3020, 0x01,
	0x3021, 0x18,
	//sensor y start:10
	0x3022, 0x00,
	0x3023, 0x0a,
	
	//sensor output size: 1624*1212
	0x3024, 0x06,
	0x3025, 0x58,
	0x3026, 0x04,
	0x3027, 0xbc,
	
	//DSP scale input size: 1600*1200
	0x3316, 0x64,
	0x3317, 0x4b,
	0x3318, 0x00,
	
	//DSP scale output: 640*480
	0x331a, 0x28,
	0x331b, 0x1e,
	0x331c, 0x00,
	
	//DVP output to ci: 640*480
	0x3088, 0x02,
	0x3089, 0x80,//640
	0x308a, 0x01,
	0x308b, 0xe0,//480
	
	OV2655_REGEND,0x00
};

static int read_sensor_reg( const u16 subAddress, u8 *bufP );
static int write_sensor_reg( const u16 subAddress, u8 bufP );

/*
static int rmw_sensor_reg( const u16 subAddress, u8 *bufP,
		u8 andMask, u8 orMask )
{
	int status;
	status = read_sensor_reg( subAddress, bufP );
	if (!status) {
		*bufP &= andMask;
		*bufP |= orMask;
		status = write_sensor_reg( subAddress, bufP );
	}
	return status;
}
*/

//static u8 preview_reg0x3000=0;
//static int preview_exposure=0,preview_extra_exposure=0;

#define SVGA_MAX_SHUTTER 618
#define UXGA_MAX_SHUTTER 1236
static void modify_capture_exposure_gain(u32 size)
{
	u16 subadr;
	u8 exposure_line_h,exposure_line_l,extra_exposure_line_h,extra_exposure_line_l;
	
	subadr=0x3002;
	read_sensor_reg(subadr,&exposure_line_h);
	subadr=0x3003;
	read_sensor_reg(subadr,&exposure_line_l);
	subadr=0x302D;
	read_sensor_reg(subadr,&extra_exposure_line_h);
	subadr=0x302E;
	read_sensor_reg(subadr,&extra_exposure_line_l);
	
	int preview_shutter = \
	(exposure_line_l & 0xFF) | (exposure_line_h << 8)+(extra_exposure_line_l & 0xFF) | (extra_exposure_line_h << 8);
	
	//preview_exposure=preview_shutter;
	//preview_extra_exposure=(extra_exposure_line_l & 0xFF) | (extra_exposure_line_h << 8);
	
	u8 reg0x3000;
	subadr=0x3000;
	read_sensor_reg(subadr,&reg0x3000);
	int preview_gain16 = (((reg0x3000 & 0xf0)>>4) + 1) * (16 + (reg0x3000 & 0x0f));
	//preview_reg0x3000=reg0x3000;
	
	//max capture gain16: 58
	int capture_max_gain16=(((0x1d & 0xf0)>>4) + 1) * (16 + (0x1d & 0x0f));//28*16;//0x1d*16
	int capture_gain16;
	
	//pclk 18*1000000
	int capture_frequency=18;//640*480,reg0x3011=0x01
	
	//remove for expourse issue when captrue 1600X1200 out of room
	//if(size>OV_SIZE_SVGA)
		//capture_frequency=18*2;
		
	int preview_frequency=18*2;//640*480,reg0x3011=0x00
	switch(g_zoom_in)
	{
		//reg0x3011=0x00
		//case ZOOM_DISABLE:
		case ZOOM_LEVEL_1x:
		break;
		case ZOOM_LEVEL_1p25x:
		break;
		
		//reg0x3011=0x01
		case ZOOM_LEVEL_2x:
		case ZOOM_LEVEL_2p5x:
		preview_frequency=24;//18;
		break;
	}
	
	//banding
	int capture_banding_filter;
	if(g_banding_set==V4L2_CID_POWER_LINE_FREQUENCY_50HZ){
		capture_banding_filter = (capture_frequency*1000)/(2*194);
	}else{
		capture_banding_filter = (capture_frequency*2500 /(6*194));
	}
	
	int capture_shutter =preview_shutter*capture_frequency/preview_frequency;
	
	int capture_max_shutter=UXGA_MAX_SHUTTER;
	
	int gain_shutter=preview_gain16*capture_shutter;
	if(gain_shutter<capture_banding_filter*16){
		capture_shutter=gain_shutter/16;
		capture_gain16=((gain_shutter*2+1)/capture_shutter)/2;
	}else{
		if(gain_shutter>capture_max_shutter*16){
			capture_shutter=capture_max_shutter;
			capture_gain16=(gain_shutter*2+1)/capture_max_shutter/2;
			if(capture_gain16>capture_max_gain16){
				
				capture_gain16=capture_max_gain16;
				capture_shutter=(gain_shutter*2+1)/capture_gain16/2;
				capture_shutter=capture_shutter/capture_banding_filter;
				capture_shutter=capture_shutter*capture_banding_filter;
				
				/*
				capture_shutter=(gain_shutter*1.1)/capture_max_gain16;
				capture_shutter=(gain_shutter/16)/capture_banding_filter;
				capture_shutter=capture_shutter*capture_banding_filter;
				capture_gain16=((gain_shutter*2+1)/capture_shutter)/2;
				*/
			}else{
				capture_shutter=capture_shutter/capture_banding_filter;
				capture_shutter=capture_shutter*capture_banding_filter;
				capture_gain16=((gain_shutter*2+1)/capture_max_shutter)/2;
			}
		}else{
			capture_shutter=(gain_shutter/16)/capture_banding_filter;
			capture_shutter=capture_shutter*capture_banding_filter;
			capture_gain16=((gain_shutter*2+1)/capture_shutter)/2;		
		}
	}
		
	
	if(capture_shutter<1)
		capture_shutter=1;
	
	//switch to UXGA
	//ov2655hw_set_regs(SVGA_To_UXGA);
	
	//write dummy pixels and dummy lines
	/*
	u8 reg302b, reg302a;
	u16 temp_reg;
	
	u8 dummy_pixels=0,dummy_lines=0;
	if(dummy_lines<0)
		dummy_lines=0;
	write_sensor_reg(0x302c,(dummy_pixels&0xFF)); //EXHTS[7:0]

	// read out and + line
	subadr=0x302B;
	read_sensor_reg(subadr,&reg302b);
	subadr=0x302A;
	read_sensor_reg(subadr,&reg302a);
	temp_reg = (reg302b & 0xFF) | (reg302a << 8);

	temp_reg += dummy_lines;

	write_sensor_reg(0x302B,(temp_reg&0xFF));         //VTS[7:0]
	write_sensor_reg(0x302A,((temp_reg&0xFF00)>>8));  //VTS[15:8]
	
	mdelay(10);
	*/
	
    int extra_exposure_lines;
    if(capture_shutter <= capture_max_shutter){
		extra_exposure_lines = 0;
     }else{
        extra_exposure_lines = capture_shutter - capture_max_shutter;
        capture_shutter=capture_max_shutter;
     }
     
	 // set extra exposure line
    write_sensor_reg(0x302E, extra_exposure_lines & 0xFF);
    write_sensor_reg(0x302D, (extra_exposure_lines & 0xFF00) >> 8);

    /* Max exporsure time is 1 frmae period event if Tex is set longer than 1 frame period */
    write_sensor_reg(0x3003, capture_shutter & 0xFF);           //AEC[7:0]
    write_sensor_reg(0x3002, (capture_shutter & 0xFF00) >> 8);  //AEC[8:15]
    mdelay(5);
	
    u8 gain=0;
    int capture_gain=capture_gain16/16;
    if(capture_gain16 > 16) {
		capture_gain16 = capture_gain /2;
		gain = 0x10;
	}
	if(capture_gain16 > 16) {
		capture_gain=capture_gain16/16;
		capture_gain16 = capture_gain /2;
		gain = gain | 0x20;
	}
	if (capture_gain16 > 16) {
		capture_gain=capture_gain16/16;
		capture_gain16 = capture_gain /2;
		gain = gain | 0x40;
    }
    if (capture_gain16 > 16) {
		capture_gain=capture_gain16/16;
		capture_gain16 = capture_gain /2;
		gain = gain | 0x80;
    }
    
   gain = gain | capture_gain16;
   //gain = gain | (capture_gain16-16);
   
   write_sensor_reg(0x3000, gain);
}

int ov2655hw_read_sensor_reg( const u16 subAddress, u8 *bufP )
{
	return read_sensor_reg(subAddress, bufP);
}

int ov2655hw_write_sensor_reg( const u16 subAddress, u8 *bufP )
{
	return write_sensor_reg(subAddress, *bufP);
}

//16bit subaddress and 8bit value data
int ov2655hw_set_regs( const u16 *regP )
{
	u32 curReg = 0;
	int    status = 0;

	/* The list is a register number followed by the value */
	while( regP[curReg << 1] < OV2655_REGEND )
	{
		u8 regVal = (u8)(regP[(curReg << 1) + 1]);

		status =write_sensor_reg(regP[curReg << 1], regVal);
		
		if( regP[curReg << 1] == 0x3012 && regVal==0x80 ){
			ov2655hw_wait(8);
		}

		if(status!=0)
			printk(KERN_ERR "write_sensor_reg fail: addr=0x%lx, value=0x%lx",regP[curReg << 1],regVal);
			
		curReg++;
	}

	return status;
}

/*
int ov2655hw_read_all_regs( u8 *bufP, u32 numRegs )
{
	u32 curReg;

	for( curReg = 0; curReg < numRegs; curReg++, bufP++ )
		read_sensor_reg( (u16)curReg, bufP );


	return 0;
}
*/

/*****************************************************************************
 *  Settings
 *****************************************************************************/
int ov2655hw_version_revision(u8 * pCmRevision, u8 *pSensorRevision)
{
	read_sensor_reg( OV2655_PIDH, pCmRevision );
	read_sensor_reg( OV2655_PIDL, pSensorRevision );
	return 0;
}

int ov2655hw_set_contrast(u32 value)
{
	if(value>SENSOR_CONTRAST_LEVEL_HIGH_3)
		value=SENSOR_CONTRAST_LEVEL_HIGH_3;
	else if(value<SENSOR_CONTRAST_LEVEL_LOW_3)
		value=SENSOR_CONTRAST_LEVEL_LOW_3;
	
	u8 data1,data2,data3;
	
	read_sensor_reg(0x3391,&data1);
	read_sensor_reg(0x3390,&data2);
	
	switch(value)
	{
		case SENSOR_CONTRAST_LEVEL_LOW_3:
			data1 |=0x04;//bit[2]=1
			data2 |=0x04;
			data3 =0x14;
			break;
		case SENSOR_CONTRAST_LEVEL_LOW_2:
			data1 |=0x04;
			data2 |=0x04;
			data3 =0x18;
			break;
		case SENSOR_CONTRAST_LEVEL_LOW_1:
			data1 |=0x04;
			data2 |=0x04;
			data3 =0x1c;
			break;
		case SENSOR_CONTRAST_LEVEL_MIDDLE:
			data1 &=~0x04;
			data2 &=~0x04;
			data3 =0x20;
			break;
		case SENSOR_CONTRAST_LEVEL_HIGH_1:
			data1 |=0x04;
			data2 |=0x04;
			data3 =0x24;
			break;
		case SENSOR_CONTRAST_LEVEL_HIGH_2:
			data1 |=0x04;
			data2 |=0x04;
			data3=0x28;
			break;
		case SENSOR_CONTRAST_LEVEL_HIGH_3:
			data1 |=0x04;
			data2 |=0x04;
			data3 =0x2c;
			break;
	}
	
	write_sensor_reg(0x3391,data1);//bit[2] enable contrast/brightness
	
	write_sensor_reg(0x3390,data2);//bit[2] Yoffset sign
	write_sensor_reg(0x3398,data3);
	write_sensor_reg(0x3399,data3);
	
	return 0;
}
int ov2655hw_set_exposure(u32 value)
{
	if(value>SENSOR_EXPOSURE_LEVEL_HIGH_3)
		value=SENSOR_EXPOSURE_LEVEL_HIGH_3;
	else if(value<SENSOR_EXPOSURE_LEVEL_LOW_3)
		value=SENSOR_EXPOSURE_LEVEL_LOW_3;
	
	// Average-based Algorithm
	write_sensor_reg(0x3047,0x05);
	
	switch(value)
	{
		case SENSOR_EXPOSURE_LEVEL_LOW_3:
			write_sensor_reg(0x3018, 0x40);
			write_sensor_reg(0x3019, 0x30);
			write_sensor_reg(0x301a, 0x71);
		break;
		
		case SENSOR_EXPOSURE_LEVEL_LOW_2:
			write_sensor_reg(0x3018, 0x5a);
			write_sensor_reg(0x3019, 0x4a);
			write_sensor_reg(0x301a, 0xc2);
		break;
		
		//-0.3EV
		case SENSOR_EXPOSURE_LEVEL_LOW_1:
			write_sensor_reg(0x3018, 0x6a);
			write_sensor_reg(0x3019, 0x5a);
			write_sensor_reg(0x301a, 0xd4);
		break;
		
		//default
		case SENSOR_EXPOSURE_LEVEL_MIDDLE:
			write_sensor_reg(0x3018, 0x78);
			write_sensor_reg(0x3019, 0x68);
			write_sensor_reg(0x301a, 0xd4);
		break;
		
		case SENSOR_EXPOSURE_LEVEL_HIGH_1:
			write_sensor_reg(0x3018, 0x88);
			write_sensor_reg(0x3019, 0x78);
			write_sensor_reg(0x301a, 0xd5);
		break;
		
		case SENSOR_EXPOSURE_LEVEL_HIGH_2:
			write_sensor_reg(0x3018, 0xa8);
			write_sensor_reg(0x3019, 0x98);
			write_sensor_reg(0x301a, 0xe6);
		break;
		
		case SENSOR_EXPOSURE_LEVEL_HIGH_3:
			write_sensor_reg(0x3018, 0xc8);
			write_sensor_reg(0x3019, 0xb8);
			write_sensor_reg(0x301a, 0xf7);
		break;	
	}	
	
	return 0;
}
int ov2655hw_set_white_balance(u32 value)
{
	return 0;
}
/*
int ov2655hw_set_brightness(int value)
{
	write_sensor_reg(0x3391, 0x00);
	
	u8 data1=0x00;
	u8 data2=0x41;
	switch(value)
	{
		case 3:
			data1=0x30;
			data2=0x41;
		break;
		
		case 2:
			data1=0x20;
			data2=0x41;
		break;
		
		case 1:
			data1=0x10;
			data2=0x41;
		break;
		
		case 0:
			data1=0x00;
			data2=0x41;
		break;
		
		case -1:
			data1=0x10;
			data2=0x49;
		break;
		
		case -2:
			data1=0x20;
			data2=0x49;
		break;
		
		case -3:
			data1=0x30;
			data2=0x49;
		break;
	}
	
	write_sensor_reg(0x3390, data2);
	write_sensor_reg(0x339a, data1);

	return 0;
}
*/

void ov2655hw_set_zoom(ZOOM_LEVEL index)
{
	const u16	*defaultDataP=NULL;
	switch(index)
	{
		/*
		case ZOOM_DISABLE:
		if(g_zoom_in!=ZOOM_DISABLE){
			defaultDataP=VGA_Zoom_1x;
		}
		break;
		*/
		case ZOOM_LEVEL_1x:
			defaultDataP=VGA_Zoom_1x;
		break;
		case ZOOM_LEVEL_1p25x:
			defaultDataP=VGA_Zoom_1p25x;
		break;
		case ZOOM_LEVEL_2x:
			defaultDataP=VGA_Zoom_2x;
		break;
		case ZOOM_LEVEL_2p5x:
			defaultDataP=VGA_Zoom_2p5x;
		break;
	}
	
	g_zoom_in=index;
	
	if(defaultDataP!=NULL){
		ov2655hw_set_regs( defaultDataP );
	}
	
	//adjust banding for 50hz/60hz
	if(g_zoom_in<ZOOM_LEVEL_2x){
		write_sensor_reg(0x301c,0x03);
		write_sensor_reg(0x301d,0x04);
		write_sensor_reg(0x3070,0xBE);
		write_sensor_reg(0x3072,0x9E);
	}else{
		write_sensor_reg(0x301c,0x04);
		write_sensor_reg(0x301d,0x05);
		write_sensor_reg(0x3070,0x7E);
		write_sensor_reg(0x3072,0x6E);
	}
}

void ov2655hw_set_banding(u32 value)
{	
	if(value>V4L2_CID_POWER_LINE_FREQUENCY_60HZ)
		value=V4L2_CID_POWER_LINE_FREQUENCY_60HZ;
	else if(value<V4L2_CID_POWER_LINE_FREQUENCY_DISABLED)
		value=V4L2_CID_POWER_LINE_FREQUENCY_DISABLED;
		
	u16 subaddress=0x3014;
	u8 val;
	
	read_sensor_reg(subaddress, &val);
	
	switch(value)
	{
		case V4L2_CID_POWER_LINE_FREQUENCY_DISABLED:
		//auto
		val |=0x40;//bit[6]=1
		break;
		
		case V4L2_CID_POWER_LINE_FREQUENCY_50HZ:
		//50hz
		val |= 0x80; //bit[7]=1
		val &= ~0x40;//bit[6]=0
		break;
		
		case V4L2_CID_POWER_LINE_FREQUENCY_60HZ:
		//60hz
		val &= ~0xc0;//bit[7][6]=0
		break;
	}
	g_banding_set=value;
	write_sensor_reg(subaddress, val);
}

void ov2655hw_set_sharpness(u32 value)
{
	if(value>SENSOR_SHARPNESS_HIGH_2)
		value=SENSOR_SHARPNESS_HIGH_2;
	else if(value<SENSOR_SHARPNESS_AUTO)
		value=SENSOR_SHARPNESS_AUTO;
		
	u16 subaddress=0x3306;
	u8 val;
	
	read_sensor_reg(subaddress, &val);
	if(value==0){
		//auto
		val &= ~0x08;
		write_sensor_reg(subaddress, val);
	}else{
		val |= 0x08;
		write_sensor_reg(subaddress, val);
		
		switch(value)
		{
			case 1:
			write_sensor_reg(0x3371, 0x00);
			break;
			case 2:
			write_sensor_reg(0x3371, 0x01);
			break;
			case 3:
			write_sensor_reg(0x3371, 0x02);//0x332d
			break;
			case 4:
			write_sensor_reg(0x3371, 0x03);//0x332d
			break;
			case 5:
			write_sensor_reg(0x3371, 0x04);//0x332d
			break;
		}
	}
}

void ov2655hw_set_scene(u32 value)
{
	//back up for auto mode
	//static u8 reg3015=0;
	static u8 reg3014=0;
	static u8 cur_mode=0;
	
	if(value==0){
		//default: scence auto mode
		if(cur_mode==1){
			write_sensor_reg(0x3014, reg3014);
			//write_sensor_reg(0x3015, 0x32);
			write_sensor_reg(0x302e, 0x00);
			write_sensor_reg(0x302d, 0x00);
		}
		
		cur_mode=0;
		
	}else if(value==1){
		//night mode
		if(cur_mode==0){
			read_sensor_reg(0x3014,&reg3014);
			//read_sensor_reg(0x3015,&reg3015);
			
			write_sensor_reg(0x3014, 0x0c);
			//write_sensor_reg(0x3015, 0x50);
		}
		
		cur_mode=1;
	}
}

void ov2655hw_auto_function_on()
{
	u16 subaddress=0x3013;
	u8 val;
	
	read_sensor_reg(subaddress, &val);
	val |= 0x05;	//enable AEC AGC auto mode
	write_sensor_reg(subaddress, val);
	
	subaddress=0x3324;
	read_sensor_reg(subaddress,&val);
	val &= ~0x40;
	write_sensor_reg(subaddress,val);
	
	//subaddress=0x3306;
	//read_sensor_reg(subaddress, &val);
	//val = 0x0;	//enable AWB auto mode
	//write_sensor_reg(subaddress, val);
}

void ov2655hw_auto_function_off()
{
	u16 subaddress=0x3013;
	u8 val;
	
	read_sensor_reg(subaddress, &val);
	val &= ~0x05;	//disable AEC AGC auto mode
	write_sensor_reg(subaddress, val);
	
	//fix AWB
	subaddress=0x3324;
	read_sensor_reg(subaddress,&val);
	val |= 0x40;
	write_sensor_reg(subaddress,val);
	
	//subaddress=0x3306;
	//read_sensor_reg(subaddress, &val);
	//val = 0x02;	//disable AWB auto mode
	//write_sensor_reg(subaddress, val);
	
	//sunny when AWB manual
	//write_i2c(0x3337, 0x5e);
	//write_i2c(0x3338, 0x40);
	//write_i2c(0x3339, 0x46);
}

/*****************************************************************************
 *  Viewfinder, still
 *****************************************************************************/
int ov2655hw_viewfinder_on()
{
	/*
	//sleep off
	u16 subaddress=0x3086;
	u8 val;

	read_sensor_reg(subaddress, &val);
	val &= ~0x01;
	write_sensor_reg(subaddress, val);
	
	//val=0x20;
	//write_sensor_reg(0x3606, val);
	*/
	return OV_ERR_NONE;
}


int ov2655hw_viewfinder_off()
{
	/*
	//sleep on
	u16 subaddress=0x3086;
	u8 val;

	read_sensor_reg(subaddress, &val);
	val |= 0x01;
	write_sensor_reg(subaddress, val);

	//val=0x00;
	//write_sensor_reg(0x3606, val);
	*/
	return OV_ERR_NONE;
}

void print_exposure()
{
	static int i=0;
	i++;
	u16 subadr;
	
	u8 exposure_line_h,exposure_line_l,extra_exposure_line_h,extra_exposure_line_l;
	
	subadr=0x3002;
	read_sensor_reg(subadr,&exposure_line_h);
	printk(KERN_INFO "################exposure_line_h=0x%02x\n",exposure_line_h);
	subadr=0x3003;
	read_sensor_reg(subadr,&exposure_line_l);
	printk(KERN_INFO "################exposure_line_l=0x%02x\n",exposure_line_l);
	subadr=0x302D;
	read_sensor_reg(subadr,&extra_exposure_line_h);
	printk(KERN_INFO "################extra_exposure_line_h=0x%02x\n",extra_exposure_line_h);
	subadr=0x302E;
	read_sensor_reg(subadr,&extra_exposure_line_l);
	printk(KERN_INFO "################extra_exposure_line_l=0x%02x\n",extra_exposure_line_l);
	
	u8 reg302b,reg302a,reg302c;
	subadr=0x302B;
	read_sensor_reg(subadr,&reg302b);
	printk(KERN_INFO "################dummy line reg302b=0x%02x\n",reg302b);
	subadr=0x302A;
	read_sensor_reg(subadr,&reg302a);
	printk(KERN_INFO "################dummy line reg302a=0x%02x\n",reg302a);
	
	subadr=0x302C;
	read_sensor_reg(subadr,&reg302c);
	printk(KERN_INFO "################dummy pixel reg302c=0x%02x\n",reg302c);
	
	u16 capture_shutter = \
	(exposure_line_l & 0xFF) | (exposure_line_h << 8)+(extra_exposure_line_l & 0xFF) | (extra_exposure_line_h << 8);
	printk(KERN_INFO "################print_expouse=%d, count=%d\n",capture_shutter,i);
	
	u8 reg0x3000;
	subadr=0x3000;
	read_sensor_reg(subadr,&reg0x3000);
	u16 Preview_Gain16 = (((reg0x3000 & 0xf0)>>4) + 1) * (16 + (reg0x3000 & 0x0f));
	printk(KERN_INFO "################print_gain reg0x3000=0x%02x, Preview_Gain16=%d, count=%d\n",reg0x3000,Preview_Gain16,i);
}
/*****************************************************************************
 *  Format
 *****************************************************************************/
int ov2655hw_set_format(u32 captureSizeFormat,u32 colorFormat,u32 capturemode){
	const u16	*defaultDataP=NULL;
	
	if(capturemode==CAMERA_MODE_STILL){
		//capture still image
		
		//print_exposure();
		modify_capture_exposure_gain(captureSizeFormat);
		
	}
	
	switch(captureSizeFormat){
		case OV_SIZE_CIF:
			defaultDataP=CIF_Regs;
		case OV_SIZE_QVGA:
			defaultDataP=QVGA_Regs;
			break;
		case OV_SIZE_VGA:
			defaultDataP=VGA_Regs;
			break;
		case OV_SIZE_SVGA:
			defaultDataP=SVGA_Regs;
			break;
		case OV_SIZE_SXGA:
			defaultDataP=SXGA_Regs;
			break;
		case OV_SIZE_UXGA:
			defaultDataP=UXGA_Regs;
			break;
		case OV_SIZE_SXGA_2:
			defaultDataP=SXGA_2_Regs;
			break;
	}
	
	if(defaultDataP!=NULL){
		//Get the pointer to the basic setting.The pointer must be freed after exiting.
		//Blast the entire parameter tree into the part
		ov2655hw_set_regs( defaultDataP );
		defaultDataP=NULL;
	}
	
	//modify PCLK, FPS.
	if(capturemode==CAMERA_MODE_STILL){
		//7.5 fps
		write_sensor_reg(0x300e,0x34);
		write_sensor_reg(0x3011,0x01);
		
		//write_sensor_reg(0x301c,0x0d);
		//write_sensor_reg(0x301d,0x0e);
		write_sensor_reg(0x3070,0x5f);
		write_sensor_reg(0x3072,0x4f);
	}else{
		//15 fps
		if(g_zoom_in<ZOOM_LEVEL_2x){
			//pclk:36Mhz, fps:15
			write_sensor_reg(0x300e,0x34);
			write_sensor_reg(0x3011,0x00);
	
			//adjust banding for 50hz/60hz
			write_sensor_reg(0x301c,0x03);
			write_sensor_reg(0x301d,0x04);
			write_sensor_reg(0x3070,0xBE);
			write_sensor_reg(0x3072,0x9E);
		}else{
			//pclk:24Mhz, fps:10
			write_sensor_reg(0x300e,0x38);
			write_sensor_reg(0x3011,0x00);
			
			write_sensor_reg(0x301c,0x04);
			write_sensor_reg(0x301d,0x05);
			write_sensor_reg(0x3070,0x7E);
			write_sensor_reg(0x3072,0x6E);
		}
	}

	return 0;
}

/*****************************************************************************
 *  Power & Reset
 *****************************************************************************/
void ov2655hw_power_down( u8 powerMode )
{
	/* OV2655 PWRDWN, 0 = NORMAL, 1=POWER DOWN */
#ifdef CONFIG_PXA3xx_GPIOEX
	if( powerMode == CAMERA_POWER_OFF )
		pxa3xx_gpio_set_level(MFP_CIF_HI_PWDN_GPIO, GPIO_LEVEL_HIGH);
	else
		pxa3xx_gpio_set_level(MFP_CIF_HI_PWDN_GPIO, GPIO_LEVEL_LOW);
#endif

	mdelay(100);
}

void ov2655hw_reset()
{
	g_zoom_in=ZOOM_LEVEL_1x;
	g_banding_set=V4L2_CID_POWER_LINE_FREQUENCY_60HZ;
	
	ov2655hw_wait(50);
	ov2655hw_set_regs(Init_Regs_Values_UXGA);
	
	return;
}

void ov2655hw_wait( int ms )
{
	mdelay( ms );
}

/******************************************************************************
 *                              OV2655 I2C Client Driver
 ******************************************************************************/
#include <linux/i2c.h>
static int i2c_ov2655_attach_adapter(struct i2c_adapter *adapter);
static int i2c_ov2655_detect_client(struct i2c_adapter *, int,  int);
static int i2c_ov2655_detach_client(struct i2c_client *client);
#define I2C_DRIVERID_OV2655   (96)
#define	OV2655_ADDRESS	0x30 //write addresss: 0x60>>1, read address: 0x61>>1

struct i2c_driver ov2655_driver  =
{
	.driver = {
		.name	= "ov2655 i2c client driver",
	},
	.id		= I2C_DRIVERID_OV2655,
	.attach_adapter	= &i2c_ov2655_attach_adapter,
	.detach_client	= &i2c_ov2655_detach_client,
};

/* Unique ID allocation */
static struct i2c_client *g_client;
static unsigned short normal_i2c[] = {OV2655_ADDRESS, I2C_CLIENT_END };
I2C_CLIENT_INSMOD_1(ov2655);

static int read_sensor_reg( const u16 subAddress, u8 *bufP )
{
	int ret;
	if( g_client == NULL )	/* No global client pointer? */
		return -1;
		
	//ret=i2c_smbus_read_byte_data_u16(g_client,subAddress);
	ret=i2c_smbus_read_byte_data_exp(g_client,subAddress,REG_ADDRESS_SIZE_16);
	if(ret==-1)
		printk(KERN_ERR "read_sensor_reg error: addr=0x%04x\n",subAddress);
		
	*bufP=ret;
		
	return ret;
}

static int write_sensor_reg( const u16 subAddress, u8 bufP )
{
	int ret;
	
	if( g_client == NULL )	/* No global client pointer? */
		return -1;

	//ret=i2c_smbus_write_byte_data_u16(g_client,subAddress,bufP);
	ret=i2c_smbus_write_byte_data_exp(g_client,subAddress,bufP,REG_ADDRESS_SIZE_16);
	if(ret!=0){
		ret=-1;
		printk(KERN_ERR "write_sensor_reg error: addr=0x%04x, value=0x%02x\n",subAddress,bufP);
	}
		
    return ret;
}


static int i2c_ov2655_attach_adapter(struct i2c_adapter *adap)
{
	return i2c_probe(adap,&addr_data,i2c_ov2655_detect_client);
}

static int i2c_ov2655_detect_client(struct i2c_adapter *adapter,
		int address, int kind)
{
	struct i2c_client *new_client;
	int err = 0;

	/* Let's see whether this adapter can support what we need.
	   Please substitute the things you need here!  */
	if ( !i2c_check_functionality(adapter,I2C_FUNC_SMBUS_BYTE_DATA) ) {
		printk(KERN_ERR "byte op is not permited.\n");
		goto ERROR0;
	}

	/* OK. For now, we presume we have a valid client. We now create the
	 * client structure, even though we cannot fill it completely yet.
	 * But it allows us to access several i2c functions safely
	 */

	/* Note that we reserve some space for ov2655_data too. If you don't
	 * need it, remove it. We do it here to help to lessen memory
	 * fragmentation.
	 */
	new_client = kmalloc(sizeof(struct i2c_client), GFP_KERNEL );

	if ( !new_client ){
		err = -ENOMEM;
		goto ERROR0;
	}

	//enable V6
#ifdef PMIC_MANUAL_V6
	if(pxa3xx_pmic_enable_voltage(VCC_CIF, 1)!=0)
		printk(KERN_ERR "pxacamera enable max8660 V6 Fail!\n");
#endif
		
	new_client->addr = address;
	new_client->adapter = adapter;
	new_client->driver = &ov2655_driver;
	new_client->flags = 0;
	
	/* detect OV2655 */
	pxa_set_cken(CKEN_CAMERA, 1);
	pxa3xx_mfp_set_afds(MFP_CIF_MCLK, MFP_AF0, MFP_DS04X);
	ci_set_clock(1, 1, 2600);
#ifdef CONFIG_PXA3xx_GPIOEX
	pxa3xx_gpio_set_direction(MFP_CIF_HI_PWDN_GPIO, GPIO_DIR_OUT);
	pxa3xx_gpio_set_level(MFP_CIF_HI_PWDN_GPIO, GPIO_LEVEL_LOW);
#endif
	mdelay(10);
	
	int idH=0,idL=0;
	
	idH=i2c_smbus_read_byte_data_exp(new_client,0x300a,REG_ADDRESS_SIZE_16);//i2c_smbus_read_byte_data_u16(new_client,0x300a);
	idL=i2c_smbus_read_byte_data_exp(new_client,0x300b,REG_ADDRESS_SIZE_16);//i2c_smbus_read_byte_data_u16(new_client,0x300b);
	if(idH<0 || idL<0)
		printk(KERN_ERR "ov2655: i2c_smbus_read_byte_data_u16 fail 1\n");
	
	
	if ((idH != PIDH_OV2655 ) ||
		(idL != PIDL_OV2655 )){
		ci_set_clock(0, 0, 2600);
#ifdef CONFIG_PXA3xx_GPIOEX
		pxa3xx_gpio_set_level(MFP_CIF_HI_PWDN_GPIO, GPIO_LEVEL_HIGH);
		pxa3xx_gpio_set_direction(MFP_CIF_HI_PWDN_GPIO, GPIO_DIR_IN);
#endif
		pxa_set_cken(CKEN_CAMERA, 0);
		
		//disable V6
#ifdef PMIC_MANUAL_V6
		if(pxa3xx_pmic_enable_voltage(VCC_CIF, 0)!=0)
			printk(KERN_ERR "pxacamera disable max8660 V6 Fail!\n");
#endif
			
		goto ERROR1;
	}
	else {
		extern int ov2655_detected;
		ov2655_detected = 1;
				
		printk(KERN_INFO "OV2655 detected.\n");
	}

	ci_set_clock(0, 0, 2600);
#ifdef CONFIG_PXA3xx_GPIOEX
	pxa3xx_gpio_set_level(MFP_CIF_HI_PWDN_GPIO, GPIO_LEVEL_HIGH);
	pxa3xx_gpio_set_direction(MFP_CIF_HI_PWDN_GPIO, GPIO_DIR_IN);
#endif
	pxa_set_cken(CKEN_CAMERA, 0);

	//disable V6
#ifdef PMIC_MANUAL_V6
	if(pxa3xx_pmic_enable_voltage(VCC_CIF, 0)!=0)
		printk(KERN_ERR "pxacamera disable max8660 V6 Fail!\n");
#endif
			
	g_client = new_client;

	strcpy(new_client->name, "OV2655");

	/* Tell the i2c layer a new client has arrived */
	if ((err = i2c_attach_client(new_client)))
		goto ERROR1;

	return 0;

ERROR1:
	kfree(new_client);
ERROR0:
	return err;
}

static int i2c_ov2655_detach_client(struct i2c_client *client)
{
	int err;

	/* Try to detach the client from i2c space */
	if ((err = i2c_detach_client(client))) {
		return err;
	}

	/* Frees client data too, if allocated at the same time */
	kfree(client);
	g_client = NULL;
	return 0;
}

static int __init i2c_ov2655_init(void)
{
	int ret;
	//ov2655hw_wait(50);
	if ( (ret = i2c_add_driver(&ov2655_driver)) ) {
		printk(KERN_ERR "ov2655: Driver registration failed,"
			"module not inserted.\n");
		return ret;
	}

	return 0;
}

static void __exit i2c_ov2655_exit(void)
{
	if (i2c_del_driver(&ov2655_driver)) {
		printk(KERN_ERR "ov2655: Driver registration failed,"
			"module not removed.\n");
	}
}

MODULE_DESCRIPTION("I2C OV2655 driver");
MODULE_LICENSE("GPL");

module_init(i2c_ov2655_init);
module_exit(i2c_ov2655_exit);
