/*
    Copyright (C) 2005, Intel Corporation.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/*++

Abstract:
      contains all OV2655 specific macros, typedefs, and prototypes.
    Declares no storage.

Notes:Only valid for processor code named Monahans.

--*/

#ifndef __CAM_OV2655_HW_HEADER__
#define __CAM_OV2655_HW_HEADER__

/*****************************************************************************
 * Constants & Structures
 *****************************************************************************/
/* Revision constants */
#define PIDH_OV2655           0x26
#define PIDL_OV2655           0x56

/* Return codes */
#define OV_ERR_NONE       	0x00
#define OV_ERR_TIMEOUT    	-1
#define OV_ERR_PARAMETER  	-2
#define OV_COMM_ERR			-3

/* OV2655 Mode */
typedef enum {
	OV2655_UXGA = 0,  //UXGA  mode for capture
	OV2655_SVGA,      //SVGA  mode for preview
	OV2655_INVALID 
}OV2655_MODE;

#define OV_SIZE_CIF				0x01				//352*288
#define OV_SIZE_QQVGA			( OV_SIZE_CIF + 1 ) //160*120
#define OV_SIZE_QVGA			( OV_SIZE_CIF + 2 )	//320*240
#define OV_SIZE_VGA				( OV_SIZE_CIF + 3 )	//640*480
#define OV_SIZE_SVGA			( OV_SIZE_CIF + 4 )	//800*600
#define OV_SIZE_SXGA			( OV_SIZE_CIF + 5 )	//1280*1024
#define OV_SIZE_UXGA			( OV_SIZE_CIF + 6 )	//1600*1200
#define OV_SIZE_SXGA_2			( OV_SIZE_CIF + 7 )	//1280*960

#define OV_FORMAT_YUV_422		1
#define OV_FORMAT_RGB_565		2
#define OV_FORMAT_RAW8   		3
#define OV_FORMAT_RAW10 		4

#define OV2655_PIDH             0x300A    /* Product ID Number MSBs */
#define OV2655_PIDL             0x300B    /* Product ID Number LSBs */
#define OV2655_REGEND     (0x3805)

typedef enum{
	//ZOOM_DISABLE=0,
	ZOOM_LEVEL_1x=1,
	ZOOM_LEVEL_1p25x,
	ZOOM_LEVEL_2x,
	ZOOM_LEVEL_2p5x,
}ZOOM_LEVEL;

//add 0811 for contarst
#define SENSOR_CONTRAST_LEVEL_LOW_3      1
#define SENSOR_CONTRAST_LEVEL_LOW_2      2
#define SENSOR_CONTRAST_LEVEL_LOW_1      3
#define SENSOR_CONTRAST_LEVEL_MIDDLE     4
#define SENSOR_CONTRAST_LEVEL_HIGH_1     5
#define SENSOR_CONTRAST_LEVEL_HIGH_2     6
#define SENSOR_CONTRAST_LEVEL_HIGH_3     7

//add 0811 for exposure
#define SENSOR_EXPOSURE_LEVEL_LOW_3      1
#define SENSOR_EXPOSURE_LEVEL_LOW_2      2
#define SENSOR_EXPOSURE_LEVEL_LOW_1      3
#define SENSOR_EXPOSURE_LEVEL_MIDDLE     4
#define SENSOR_EXPOSURE_LEVEL_HIGH_1     5
#define SENSOR_EXPOSURE_LEVEL_HIGH_2     6
#define SENSOR_EXPOSURE_LEVEL_HIGH_3     7

//0812: sharpness setting
#define SENSOR_SHARPNESS_AUTO			0
#define SENSOR_SHARPNESS_LOW_2			1
#define SENSOR_SHARPNESS_LOW_1			2
#define SENSOR_SHARPNESS_MIDDLE			3
#define SENSOR_SHARPNESS_HIGH_1			4
#define SENSOR_SHARPNESS_HIGH_2			5

/*****************************************************************************
 * Function Prototype
 *****************************************************************************/
int ov2655hw_set_regs( const u16 *regP );
int ov2655hw_read_all_regs( u8 *bufP, u32 numRegs );

void ov2655hw_power_down( u8 powerMode );
void ov2655hw_reset(void);
void ov2655hw_wait( int ms );

int  ov2655hw_version_revision(u8 * pCmRevision, u8 *pSensorRevision);
void ov2655hw_set_hsync(void);
void ov2655hw_auto_function_on(void);
void ov2655hw_auto_function_off(void);

int ov2655hw_viewfinder_on(void);
int ov2655hw_viewfinder_off(void);
int ov2655hw_halt_video_output(void);
int ov2655hw_resumeto_full_output_mode(void);
int ov2655hw_get_single_image(void);

int ov2655hw_set_format(u32 capturesize,u32 outformat,u32 capturemode);
/*
int ov2655hw_set_format(u32 outformat,u32 captureWidth,
		u32 captureHeight,
		u32 *winStartX,
		u32 *winStartY,
		u32 *winEndX,
		u32 *winEndY);
		*/

int ov2655hw_read_sensor_reg( const u16 subAddress, u8 *bufP );
int ov2655hw_write_sensor_reg( const u16 subAddress, u8 *bufP );
int ov2655hw_set_contrast(u32 value);
int ov2655hw_set_exposure(u32 value);
int ov2655hw_set_white_balance(u32 value);

//void print_exposure();
void ov2655hw_set_zoom(ZOOM_LEVEL index);
void ov2655hw_set_sharpness(u32 value);
void ov2655hw_set_banding(u32 value);
void ov2655hw_set_scene(u32 value);
#endif
