/*
   Copyright (C) 2005, Intel Corporation.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


 *(C) Copyright 2006 Marvell International Ltd.
 * All Rights Reserved
 */

/*****************************************************************************
 * Abstract:
 * 	contains all hardware related functions for OV9665
 *
 * Notes:Only valid for processor code named Monahans.
 *****************************************************************************/
 
#include <linux/init.h>
#include <linux/module.h>
#include "camera.h"
#include "ov9665_hw.h"
#include <linux/delay.h>

#include <asm/arch/mfp.h>
#include <asm/arch/pxa3xx_gpio.h>
#include <asm/hardware.h>

//init regs values
static u8 REG_INIT_VALUES[]={
	//Reset
	0x12, 0x80,
	
	//IO output
	0xd5, 0xff,
	0xd6, 0x3f,

	//pixel Clock 24Mhz, frame rate 30 FPS
	0x11, 0x80,//0x81
	0x3d, 0x3c,
	0x2a, 0x00,
	0x2b, 0x00,//0x88,
	
	//pixel Clock 24Mhz, frame rate 15 FPS
	//0x11, 0x00,
	//0x3d, 0x3c,
	//0x2a, 0x00,
	//0x2b, 0x00,

	//Power control
	0x3a, 0xd9,//0xf1,//0xd9,
	0x3b, 0x00,
	0x3c, 0x58,
	0x3e, 0x50,
	0x71, 0x00,

	//Sync signal
	0x15, 0x00,//0x00

	//Data Format YUV
	0xD7, 0x00,
	0x85, 0xE7,
	0xD8, 0xc4,//0xC4,

	//Sample Option
	0x63, 0x00,

	//Windowing
	0x12, 0x40,
	0x4d, 0x09,
	0x17, 0x0d,
	0x18, 0x5d,
	0x19, 0x02,
	0x1a, 0x3f,
	0x03, 0x03,
	0x32, 0x99,//0xad,//0x99,
	0x2b, 0x00,
	0x5c, 0x80,

	//BLC
	0x36, 0xb4,
	0x65, 0x10,
	0x70, 0x02,
	0x71, 0x9f,
	0x64, 0xa4,//0x24,

	//AEC; 	Average;9 zone;
	0x43, 0x00,
	0x5D, 0x55,
	0x5E, 0x57,
	0x5F, 0x21,

	// Brightness
	0x24, 0x3e,
	0x25, 0x38,
	0x26, 0x72,

	//BF 60Hz
	0x14, 0x68,//0x28,//0x68, AGC gain 16X
	0x0C, 0x38,//0x3a, 50MHZ//0x38,60MHZ
	0x4F, 0x4f,
	0x50, 0x42,
	0x5A, 0x67,//0x23,//0x67,

	//LC enable	Largon9306;
	0x7d, 0x30,
	0x7e, 0x00,
	0x82, 0x03,
	0x7f, 0x00,
	0x83, 0x07,
	0x80, 0x03,
	0x81, 0x04,

	//AWB advance	Largon9306;
	0x96, 0xf0,
	0x97, 0x00,
	0x92, 0x33,
	0x94, 0x5a,
	0x93, 0x3a,
	0x95, 0x48,
	0x91, 0xfc,
	0x90, 0xff,
	0x8e, 0x4e,
	0x8f, 0x4e,
	0x8d, 0x13,
	0x8c, 0x0c,
	0x8b, 0x0c,
	0x86, 0x9e,
	0x87, 0x11,
	0x88, 0x22,
	0x89, 0x05,
	0x8a, 0x03,

	// Gamma enable
	0x9b, 0x0e,//0x04,//0x08,//0x0e,
	0x9c, 0x1c,//0x07,//0x0e,//0x1c,
	0x9d, 0x34,//0x10,//0x1e,//0x34,
	0x9e, 0x5a,//0x28,//0x40,//0x5a,
	0x9f, 0x68,//0x36,//0x54,//0x68,
	0xa0, 0x76,//0x44,//0x66,//0x76,
	0xa1, 0x82,//0x52,//0x78,//0x82,
	0xa2, 0x8e,//0x60,//0x86,//0x8e,
	0xa3, 0x98,//0x6c,//0x92,//0x98,
	0xa4, 0xa0,//0x78,//0x9e,//0xa0,
	0xa5, 0xb0,//0x8c,//0xb0,//0xb0,
	0xa6, 0xbe,//0x9e,//0xbe,//0xbe,
	0xa7, 0xd2,//0xbb,//0xd4,//0xd2,
	0xa8, 0xe2,//0xd2,//0xd4,//0xe2,
	0xa9, 0xee,//0xe5,//0xf0,//0xee,
	0xaa, 0x18,//0x24,//0x15,//0x18,
	
	//fixed U=0 Y=0 if enable fixed output
	//0xc8, 0x1e,
	//0xcd, 0x00,
	//0xce, 0x00,
	//0x13, 0xe0,
	
	//De-noise enable auto
	0xAB, 0xe7,
	0xb0, 0x43,
	0xac, 0x04,
	0x84, 0x40,

	//Sharpness
	0xad, 0x82,
	0xd9, 0x11,
	0xda, 0x00,
	0xae, 0x10,

	//Scaling
	0xab, 0xe7,
	0xb9, 0x50,
	0xba, 0x3c,
	0xbb, 0x50,
	0xbc, 0x3c,

	//CMX
	0xbd, 0x08,
	0xbe, 0x19,
	0xbf, 0x02,
	0xc0, 0x08,
	0xc1, 0x2a,
	0xc2, 0x34,
	0xc3, 0x2d,
	0xc4, 0x2d,
	0xc5, 0x00,
	0xc6, 0x98,
	0xc7, 0x18,
	0x69, 0x48,

	//UVave
	0x74, 0xc0,

	//UVadj
	0x7c, 0x18,//0x28,//0x18,
	0x65, 0x11,
	0x66, 0x00,
	0x41, 0xa0,
	0x5b, 0x28,
	0x60, 0x84,
	0x05, 0x07,
	0x03, 0x03,
	0xd2, 0x8c,//0x94,//0x8c,

	//SAT & Brightness
	0xc7, 0x90,
	0xc8, 0x06,
	0xcb, 0x40,
	0xcc, 0x40,
	0xcf, 0x00,
	0xd0, 0x20,
	0xd1, 0x00,
	0xc7, 0x18,
	
	//for test color bar
	//0x69, 0x4c,
	
	//BLC
	0x0d, 0x92,
	0x0d, 0x90,
	
	OV9665_REGEND, 0x00
};

static u8 gYUV_CIF_15fps_INITIALIZE[]={
	//0x3E, 0x80,	//disable PLL
	//Reset
	0x12, 0x80,
	
	//IO output
	0xd5, 0xff,
	0xd6, 0x3f,

	//Clock 24Mhz 15 FPS
	0x11, 0x80,
	0x3d, 0x3c,
	0x2a, 0x00,
	0x2b, 0x00,
	0x2d, 0x00,
	0x2e, 0x00,	

	//Power control
	0x3a, 0xd9,//0xf1,//0xd9,
	0x3b, 0x00,
	0x3c, 0x58,
	0x3e, 0x50,
	0x71, 0x00,

	//Sync signal
	0x15, 0x00,

	//Data Format YUV
	0xD7, 0x00,
	0x85, 0xe7,//0xef, //for scale
	0xD8, 0xC4,

	//Sample Option
	0x63, 0x00,

	//Windowing
	0x12, 0x40,
	0x4d, 0x09,
	0x17, 0x0c,//0x0d,
	0x18, 0x5d,
	0x19, 0x02,
	0x1a, 0x3f,
	0x03, 0x03,
	0x32, 0x99,//0xad,//0x99,
	0x2b, 0x00,
	0x5c, 0x80,

	//BLC
	0x36, 0xb4,
	0x65, 0x11,//0x10,
	0x70, 0x02,
	0x71, 0x9f,
	0x64, 0xa4,//0x24,

	//AEC; 	Average;9 zone;
	0x43, 0x00,
	0x5D, 0x55,
	0x5E, 0x57,
	0x5F, 0x21,

	// Brightness
	0x24, 0x3e,
	0x25, 0x38,
	0x26, 0x72,

	//BF 60Hz
	0x14, 0x68,//0x28,//0x68, AGC gain 16X
	0x0C, 0x38,//0x3a, 50MHZ//0x38,60MHZ
	0x4F, 0x4f,
	0x50, 0x42,
	0x5A, 0x67,//0x23,//0x67,

	//LC enable	Largon9306;
	0x7d, 0x30,
	0x7e, 0x00,
	0x82, 0x03,
	0x7f, 0x00,
	0x83, 0x07,
	0x80, 0x03,
	0x81, 0x04,

	//AWB advance	Largon9306;
	0x96, 0xf0,
	0x97, 0x00,
	0x92, 0x33,
	0x94, 0x5a,
	0x93, 0x3a,
	0x95, 0x48,
	0x91, 0xfc,
	0x90, 0xff,
	0x8e, 0x4e,
	0x8f, 0x4e,
	0x8d, 0x13,
	0x8c, 0x0c,
	0x8b, 0x0c,
	0x86, 0x9e,
	0x87, 0x11,
	0x88, 0x22,
	0x89, 0x05,
	0x8a, 0x03,

	// Gamma enable
	0x9b, 0x0e,//0x04,//0x08,//0x0e,
	0x9c, 0x1c,//0x07,//0x0e,//0x1c,
	0x9d, 0x34,//0x10,//0x1e,//0x34,
	0x9e, 0x5a,//0x28,//0x40,//0x5a,
	0x9f, 0x68,//0x36,//0x54,//0x68,
	0xa0, 0x76,//0x44,//0x66,//0x76,
	0xa1, 0x82,//0x52,//0x78,//0x82,
	0xa2, 0x8e,//0x60,//0x86,//0x8e,
	0xa3, 0x98,//0x6c,//0x92,//0x98,
	0xa4, 0xa0,//0x78,//0x9e,//0xa0,
	0xa5, 0xb0,//0x8c,//0xb0,//0xb0,
	0xa6, 0xbe,//0x9e,//0xbe,//0xbe,
	0xa7, 0xd2,//0xbb,//0xd4,//0xd2,
	0xa8, 0xe2,//0xd2,//0xd4,//0xe2,
	0xa9, 0xee,//0xe5,//0xf0,//0xee,
	0xaa, 0x18,//0x24,//0x15,//0x18,
	
	//fixed U=0 Y=0 if enable fixed output
	//0xc8, 0x3e,
	//0xcd, 0x00,
	//0xce, 0x00,
	//0x13, 0xe0,
	
	//De-noise enable auto
	0xAB, 0xe7,
	0xb0, 0x43,
	0xac, 0x04,
	0x84, 0x40,

	//Sharpness
	0xad, 0x86,//0x82, 
	0xd9, 0x44,//0x11,
	0xda, 0x00,
	0xae, 0x10,

	//Scaling to 176*144
	0xab, 0xe7,//0xef,//for scale
	0xb9, 0x50,
	0xba, 0x3c,
	0xbb, 0x16,//176,
	0xbc, 0x12,//144,

	//CMX
	0xbd, 0x08,
	0xbe, 0x19,
	0xbf, 0x02,
	0xc0, 0x08,
	0xc1, 0x2a,
	0xc2, 0x34,
	0xc3, 0x2d,
	0xc4, 0x2d,
	0xc5, 0x00,
	0xc6, 0x98,
	0xc7, 0x18,
	0x69, 0x48,

	//UVave
	0x74, 0xc0,

	//UVadj
	0x7c, 0x18,//0x28,//0x18,
	0x65, 0x11,
	0x66, 0x00,
	0x41, 0xa0,
	0x5b, 0x28,
	0x60, 0x84,
	0x05, 0x07,
	0x03, 0x03,
	0xd2, 0x8c,//0x94,//0x8c,

	//SAT & Brightness
	0xc7, 0x90,
	0xc8, 0x06,
	0xcb, 0x40,
	0xcc, 0x40,
	0xcf, 0x00,
	0xd0, 0x20,
	0xd1, 0x00,
	0xc7, 0x18,
	
	//BLC
	0x0d, 0x92,
	0x0d, 0x90,
};
static u8 gYUV_QQVGA_15fps_INITIALIZE[]={
	//0x3E, 0x80,	//disable PLL
	//Reset
	0x12, 0x80,
	
	//IO output
	0xd5, 0xff,
	0xd6, 0x3f,

	//Clock 24Mhz 15 FPS
	0x11, 0x80,
	0x3d, 0x3c,
	0x2a, 0x00,
	0x2b, 0x00,
	0x2d, 0x00,
	0x2e, 0x00,	

	//Power control
	0x3a, 0xd9,//0xf1,//0xd9,
	0x3b, 0x00,
	0x3c, 0x58,
	0x3e, 0x50,
	0x71, 0x00,

	//Sync signal
	0x15, 0x00,

	//Data Format YUV
	0xD7, 0x00,
	0x85, 0xe7,//0xef, //for scale
	0xD8, 0xC4,

	//Sample Option
	0x63, 0x00,

	//Windowing
	0x12, 0x40,
	0x4d, 0x09,
	0x17, 0x0c,//0x0d,
	0x18, 0x5d,
	0x19, 0x02,
	0x1a, 0x3f,
	0x03, 0x03,
	0x32, 0x99,//0xad,//0x99,
	0x2b, 0x00,
	0x5c, 0x80,

	//BLC
	0x36, 0xb4,
	0x65, 0x11,//0x10,
	0x70, 0x02,
	0x71, 0x9f,
	0x64, 0xa4,//0x24,

	//AEC; 	Average;9 zone;
	0x43, 0x00,
	0x5D, 0x55,
	0x5E, 0x57,
	0x5F, 0x21,

	// Brightness
	0x24, 0x3e,
	0x25, 0x38,
	0x26, 0x72,

	//BF 60Hz
	0x14, 0x68,//0x28,//0x68, AGC gain 16X
	0x0C, 0x38,//0x3a, 50MHZ//0x38,60MHZ
	0x4F, 0x4f,
	0x50, 0x42,
	0x5A, 0x67,//0x23,//0x67,

	//LC enable	Largon9306;
	0x7d, 0x30,
	0x7e, 0x00,
	0x82, 0x03,
	0x7f, 0x00,
	0x83, 0x07,
	0x80, 0x03,
	0x81, 0x04,

	//AWB advance	Largon9306;
	0x96, 0xf0,
	0x97, 0x00,
	0x92, 0x33,
	0x94, 0x5a,
	0x93, 0x3a,
	0x95, 0x48,
	0x91, 0xfc,
	0x90, 0xff,
	0x8e, 0x4e,
	0x8f, 0x4e,
	0x8d, 0x13,
	0x8c, 0x0c,
	0x8b, 0x0c,
	0x86, 0x9e,
	0x87, 0x11,
	0x88, 0x22,
	0x89, 0x05,
	0x8a, 0x03,

	// Gamma enable
	0x9b, 0x0e,//0x04,//0x08,//0x0e,
	0x9c, 0x1c,//0x07,//0x0e,//0x1c,
	0x9d, 0x34,//0x10,//0x1e,//0x34,
	0x9e, 0x5a,//0x28,//0x40,//0x5a,
	0x9f, 0x68,//0x36,//0x54,//0x68,
	0xa0, 0x76,//0x44,//0x66,//0x76,
	0xa1, 0x82,//0x52,//0x78,//0x82,
	0xa2, 0x8e,//0x60,//0x86,//0x8e,
	0xa3, 0x98,//0x6c,//0x92,//0x98,
	0xa4, 0xa0,//0x78,//0x9e,//0xa0,
	0xa5, 0xb0,//0x8c,//0xb0,//0xb0,
	0xa6, 0xbe,//0x9e,//0xbe,//0xbe,
	0xa7, 0xd2,//0xbb,//0xd4,//0xd2,
	0xa8, 0xe2,//0xd2,//0xd4,//0xe2,
	0xa9, 0xee,//0xe5,//0xf0,//0xee,
	0xaa, 0x18,//0x24,//0x15,//0x18,
	
	//fixed U=0 Y=0 if enable fixed output
	//0xc8, 0x3e,
	//0xcd, 0x00,
	//0xce, 0x00,
	//0x13, 0xe0,
	
	//De-noise enable auto
	0xAB, 0xe7,
	0xb0, 0x43,
	0xac, 0x04,
	0x84, 0x40,

	//Sharpness
	0xad, 0x86,//0x82, 
	0xd9, 0x44,//0x11,
	0xda, 0x00,
	0xae, 0x10,

	//Scaling to 160*120
	0xab, 0xe7,//0xef,//for scale
	0xb9, 0x50,
	0xba, 0x3c,
	0xbb, 0x14,//0x50,
	0xbc, 0x0f,//0x3c,

	//CMX
	0xbd, 0x08,
	0xbe, 0x19,
	0xbf, 0x02,
	0xc0, 0x08,
	0xc1, 0x2a,
	0xc2, 0x34,
	0xc3, 0x2d,
	0xc4, 0x2d,
	0xc5, 0x00,
	0xc6, 0x98,
	0xc7, 0x18,
	0x69, 0x48,

	//UVave
	0x74, 0xc0,

	//UVadj
	0x7c, 0x18,//0x28,//0x18,
	0x65, 0x11,
	0x66, 0x00,
	0x41, 0xa0,
	0x5b, 0x28,
	0x60, 0x84,
	0x05, 0x07,
	0x03, 0x03,
	0xd2, 0x8c,//0x94,//0x8c,

	//SAT & Brightness
	0xc7, 0x90,
	0xc8, 0x06,
	0xcb, 0x40,
	0xcc, 0x40,
	0xcf, 0x00,
	0xd0, 0x20,
	0xd1, 0x00,
	0xc7, 0x18,
	
	//BLC
	0x0d, 0x92,
	0x0d, 0x90,
	
	OV9665_REGEND, 0x00
};
static u8 gYUV_QVGA_15fps_INITIALIZE[]={
	//0x3E, 0x80,	//disable PLL
	//Reset
	0x12, 0x80,
	
	//IO output
	0xd5, 0xff,
	0xd6, 0x3f,

	//Clock 24Mhz 15 FPS
	0x11, 0x81,//0x80,
	0x3d, 0x3c,
	0x2a, 0x00,
	0x2b, 0x88,//0x00,
	0x2d, 0x00,
	0x2e, 0x00,	

	//Power control
	0x3a, 0xd9,//0xf1,//0xd9,
	0x3b, 0x00,
	0x3c, 0x58,
	0x3e, 0x50,
	0x71, 0x00,

	//Sync signal
	0x15, 0x00,

	//Data Format YUV
	0xD7, 0x00,
	0x85, 0xe7,//0xef, //for scale
	0xD8, 0xC4,

	//Sample Option
	0x63, 0x00,

	//Windowing
	0x12, 0x40,
	0x4d, 0x09,
	0x17, 0x0c,//0x0d,
	0x18, 0x5d,
	0x19, 0x02,
	0x1a, 0x3f,
	0x03, 0x03,
	0x32, 0x99,//0xad,//0x99,
	0x2b, 0x00,
	0x5c, 0x80,

	//BLC
	0x36, 0xb4,
	0x65, 0x11,//0x10,
	0x70, 0x02,
	0x71, 0x9f,
	0x64, 0xa4,//0x24,

	//AEC; 	Average;9 zone;
	0x43, 0x00,
	0x5D, 0x55,
	0x5E, 0x57,
	0x5F, 0x21,

	// Brightness
	0x24, 0x3e,
	0x25, 0x38,
	0x26, 0x72,

	//BF 60Hz
	0x14, 0x68,//0x28,//0x68, AGC gain 16X
	0x0C, 0x38,//0x3a, 50MHZ//0x38,60MHZ
	0x4F, 0x4f,
	0x50, 0x42,
	0x5A, 0x67,//0x23,//0x67,

	//LC enable	Largon9306;
	0x7d, 0x30,
	0x7e, 0x00,
	0x82, 0x03,
	0x7f, 0x00,
	0x83, 0x07,
	0x80, 0x03,
	0x81, 0x04,

	//AWB advance	Largon9306;
	0x96, 0xf0,
	0x97, 0x00,
	0x92, 0x33,
	0x94, 0x5a,
	0x93, 0x3a,
	0x95, 0x48,
	0x91, 0xfc,
	0x90, 0xff,
	0x8e, 0x4e,
	0x8f, 0x4e,
	0x8d, 0x13,
	0x8c, 0x0c,
	0x8b, 0x0c,
	0x86, 0x9e,
	0x87, 0x11,
	0x88, 0x22,
	0x89, 0x05,
	0x8a, 0x03,

	// Gamma enable
	0x9b, 0x0e,//0x04,//0x08,//0x0e,
	0x9c, 0x1c,//0x07,//0x0e,//0x1c,
	0x9d, 0x34,//0x10,//0x1e,//0x34,
	0x9e, 0x5a,//0x28,//0x40,//0x5a,
	0x9f, 0x68,//0x36,//0x54,//0x68,
	0xa0, 0x76,//0x44,//0x66,//0x76,
	0xa1, 0x82,//0x52,//0x78,//0x82,
	0xa2, 0x8e,//0x60,//0x86,//0x8e,
	0xa3, 0x98,//0x6c,//0x92,//0x98,
	0xa4, 0xa0,//0x78,//0x9e,//0xa0,
	0xa5, 0xb0,//0x8c,//0xb0,//0xb0,
	0xa6, 0xbe,//0x9e,//0xbe,//0xbe,
	0xa7, 0xd2,//0xbb,//0xd4,//0xd2,
	0xa8, 0xe2,//0xd2,//0xd4,//0xe2,
	0xa9, 0xee,//0xe5,//0xf0,//0xee,
	0xaa, 0x18,//0x24,//0x15,//0x18,
	
	//fixed U=0 Y=0 if enable fixed output
	//0xc8, 0x3e,
	//0xcd, 0x00,
	//0xce, 0x00,
	//0x13, 0xe0,
	
	//De-noise enable auto
	0xAB, 0xe7,
	0xb0, 0x43,
	0xac, 0x04,
	0x84, 0x40,

	//Sharpness
	0xad, 0x82,//0x86,//0x82, 
	0xd9, 0x11,//0x44,//0x11,
	0xda, 0x00,
	0xae, 0x10,

	//Scaling to 320*240
	0xab, 0xe7,//0xef,//for scale
	0xb9, 0x50,
	0xba, 0x3c,
	0xbb, 0x28,//320,
	0xbc, 0x1e,//240,

	//CMX
	0xbd, 0x08,
	0xbe, 0x19,
	0xbf, 0x02,
	0xc0, 0x08,
	0xc1, 0x2a,
	0xc2, 0x34,
	0xc3, 0x2d,
	0xc4, 0x2d,
	0xc5, 0x00,
	0xc6, 0x98,
	0xc7, 0x18,
	0x69, 0x48,

	//UVave
	0x74, 0xc0,

	//UVadj
	0x7c, 0x18,//0x28,//0x18,
	0x65, 0x11,
	0x66, 0x00,
	0x41, 0xa0,
	0x5b, 0x28,
	0x60, 0x84,
	0x05, 0x07,
	0x03, 0x03,
	0xd2, 0x8c,//0x94,//0x8c,

	//SAT & Brightness
	0xc7, 0x90,
	0xc8, 0x06,
	0xcb, 0x40,
	0xcc, 0x40,
	0xcf, 0x00,
	0xd0, 0x20,
	0xd1, 0x00,
	0xc7, 0x18,
	
	//BLC
	0x0d, 0x92,
	0x0d, 0x90,
	
	OV9665_REGEND, 0x00
};
static u8 gYUV_VGA_15fps_INITIALIZE[]={
	//0x3E, 0x80,	//disable PLL
	//Reset
	0x12, 0x80,
	
	//IO output
	0xd5, 0xff,
	0xd6, 0x3f,

	//Clock 24Mhz 15 FPS
	0x11, 0x81,//0x80
	0x3d, 0x3c,
	0x2a, 0x00,
	0x2b, 0x88,//0x00,
	//0x2d, 0x00,//0x10,
	//0x2e, 0x00,//0x02,	

	//Power control
	0x3a, 0xd9,//0xf1,//0xd9,
	0x3b, 0x00,
	0x3c, 0x58,
	0x3e, 0x50,
	0x71, 0x00,

	//Sync signal
	0x15, 0x00,//0x00

	//Data Format YUV
	0xD7, 0x00,
	0x85, 0xE7,
	0xD8, 0xc4,//0xC4,

	//Sample Option
	0x63, 0x00,

	//Windowing
	0x12, 0x40,
	0x4d, 0x09,
	0x17, 0x0d,
	0x18, 0x5d,
	0x19, 0x02,
	0x1a, 0x3f,
	0x03, 0x03,
	0x32, 0x99,//0xad,//0x99,
	0x2b, 0x00,
	0x5c, 0x80,

	//BLC
	0x36, 0xb4,
	0x65, 0x10,
	0x70, 0x02,
	0x71, 0x9f,
	0x64, 0xa4,//0x24,

	//AEC; 	Average;9 zone;
	0x43, 0x00,
	0x5D, 0x55,
	0x5E, 0x57,
	0x5F, 0x21,

	// Brightness
	0x24, 0x3e,
	0x25, 0x38,
	0x26, 0x72,

	//BF 60Hz
	0x14, 0x68,//0x28,//0x68, AGC gain 16X
	0x0C, 0x38,//0x3a, 50MHZ//0x38,60MHZ
	0x4F, 0x4f,
	0x50, 0x42,
	0x5A, 0x67,//0x23,//0x67,

	//LC enable	Largon9306;
	0x7d, 0x30,
	0x7e, 0x00,
	0x82, 0x03,
	0x7f, 0x00,
	0x83, 0x07,
	0x80, 0x03,
	0x81, 0x04,

	//AWB advance	Largon9306;
	0x96, 0xf0,
	0x97, 0x00,
	0x92, 0x33,
	0x94, 0x5a,
	0x93, 0x3a,
	0x95, 0x48,
	0x91, 0xfc,
	0x90, 0xff,
	0x8e, 0x4e,
	0x8f, 0x4e,
	0x8d, 0x13,
	0x8c, 0x0c,
	0x8b, 0x0c,
	0x86, 0x9e,
	0x87, 0x11,
	0x88, 0x22,
	0x89, 0x05,
	0x8a, 0x03,

	// Gamma enable
	0x9b, 0x0e,//0x04,//0x08,//0x0e,
	0x9c, 0x1c,//0x07,//0x0e,//0x1c,
	0x9d, 0x34,//0x10,//0x1e,//0x34,
	0x9e, 0x5a,//0x28,//0x40,//0x5a,
	0x9f, 0x68,//0x36,//0x54,//0x68,
	0xa0, 0x76,//0x44,//0x66,//0x76,
	0xa1, 0x82,//0x52,//0x78,//0x82,
	0xa2, 0x8e,//0x60,//0x86,//0x8e,
	0xa3, 0x98,//0x6c,//0x92,//0x98,
	0xa4, 0xa0,//0x78,//0x9e,//0xa0,
	0xa5, 0xb0,//0x8c,//0xb0,//0xb0,
	0xa6, 0xbe,//0x9e,//0xbe,//0xbe,
	0xa7, 0xd2,//0xbb,//0xd4,//0xd2,
	0xa8, 0xe2,//0xd2,//0xd4,//0xe2,
	0xa9, 0xee,//0xe5,//0xf0,//0xee,
	0xaa, 0x18,//0x24,//0x15,//0x18,
	
	//fixed U=0 Y=0 if enable fixed output
	//0xc8, 0x1e,
	//0xcd, 0x00,
	//0xce, 0x00,
	//0x13, 0xe0,
	
	//De-noise enable auto
	0xAB, 0xe7,
	0xb0, 0x43,
	0xac, 0x04,
	0x84, 0x40,

	//Sharpness
	0xad, 0x82,//0x82, 
	0xd9, 0x11,//0x11,
	0xda, 0x00,
	0xae, 0x10,

	//Scaling
	0xab, 0xe7,
	0xb9, 0x50,
	0xba, 0x3c,
	0xbb, 0x50,
	0xbc, 0x3c,

	//CMX
	0xbd, 0x08,
	0xbe, 0x19,
	0xbf, 0x02,
	0xc0, 0x08,
	0xc1, 0x2a,
	0xc2, 0x34,
	0xc3, 0x2d,
	0xc4, 0x2d,
	0xc5, 0x00,
	0xc6, 0x98,
	0xc7, 0x18,
	0x69, 0x48,

	//UVave
	0x74, 0xc0,

	//UVadj
	0x7c, 0x18,//0x28,//0x18,
	0x65, 0x11,
	0x66, 0x00,
	0x41, 0xa0,
	0x5b, 0x28,
	0x60, 0x84,
	0x05, 0x07,
	0x03, 0x03,
	0xd2, 0x8c,//0x94,//0x8c,

	//SAT & Brightness
	0xc7, 0x90,
	0xc8, 0x06,
	0xcb, 0x40,
	0xcc, 0x40,
	0xcf, 0x00,
	0xd0, 0x20,
	0xd1, 0x00,
	0xc7, 0x18,
	
	//for test color bar
	//0x69, 0x4c,
	
	//BLC
	0x0d, 0x92,
	0x0d, 0x90,
	
	OV9665_REGEND, 0x00
};
static u8 gYUV_SXGA_15fps_INITIALIZE[]={
	//0x3E, 0x80,	//disable PLL
	//Reset
	0x12, 0x80,
	
	//IO output
	0xd5, 0xff,
	0xd6, 0x3f,

	//Clock 24Mhz 7.5 FPS
	0x11, 0x00,//0x80,
	0x3d, 0x3c,
	0x2a, 0x00,
	0x2b, 0x00,
	0x2d, 0x00,
	0x2e, 0x00,	

	//Power control
	0x3a, 0xd9,//0xf1,//0xd9,
	0x3b, 0x00,
	0x3c, 0x58,
	0x3e, 0x50,
	0x71, 0x00,

	//Sync signal
	0x15, 0x00,

	//Data Format YUV
	0xD7, 0x00,
	0x85, 0xE7,
	0xD8, 0xC4,

	//Sample Option
	0x63, 0x01,//0x00

	//Windowing
	0x12, 0x00,
	//0x4d, 0x11,//0x09,
	0x17, 0x0c,
	0x18, 0x5c,
	0x19, 0x01,
	0x1a, 0x82,//0x3f,
	0x03, 0x0f,//0x03,
	0x32, 0x34,//0xad,//0x99,
	0x2b, 0x00,
	0x5c, 0x80,

	//BLC
	0x36, 0xb4,
	0x65, 0x10,
	0x70, 0x02,
	0x71, 0x9c,//0x9f,
	0x64, 0x24,//0xa4

	//AEC; 	Average;9 zone;
	0x43, 0x00,
	0x5D, 0x55,
	0x5E, 0x57,
	0x5F, 0x21,

	// Brightness
	0x24, 0x3e,
	0x25, 0x38,
	0x26, 0x72,

	//BF 60Hz
	0x14, 0x68,//0x28,//0x68, AGC gain 16X
	0x0C, 0x38,//0x3a, 50MHZ//0x38,60MHZ
	0x4F, 0x9e,//0x4f,
	0x50, 0x84,//0x42,
	0x5A, 0x67,//0x23,//0x67,

	//LC enable	Largon9306;
	0x7d, 0x30,
	0x7e, 0x00,
	0x82, 0x03,
	0x7f, 0x00,
	0x83, 0x07,
	0x80, 0x03,
	0x81, 0x04,

	//AWB advance	Largon9306;
	0x96, 0xf0,
	0x97, 0x00,
	0x92, 0x33,
	0x94, 0x5a,
	0x93, 0x3a,
	0x95, 0x48,
	0x91, 0xfc,
	0x90, 0xff,
	0x8e, 0x4e,
	0x8f, 0x4e,
	0x8d, 0x13,
	0x8c, 0x0c,
	0x8b, 0x0c,
	0x86, 0x9e,
	0x87, 0x11,
	0x88, 0x22,
	0x89, 0x05,
	0x8a, 0x03,

	// Gamma enable
	0x9b, 0x0e,//0x08,//0x0e,
	0x9c, 0x1c,//0x0e,//0x1c,
	0x9d, 0x34,//0x1e,//0x34,
	0x9e, 0x5a,//0x40,//0x5a,
	0x9f, 0x68,//0x54,//0x68,
	0xa0, 0x76,//0x66,//0x76,
	0xa1, 0x82,//0x78,//0x82,
	0xa2, 0x8e,//0x86,//0x8e,
	0xa3, 0x98,//0x92,//0x98,
	0xa4, 0xa0,//0x9e,//0xa0,
	0xa5, 0xb0,//0xb0,//0xb0,
	0xa6, 0xbe,//0xbe,//0xbe,
	0xa7, 0xd2,//0xd4,//0xd2,
	0xa8, 0xe2,//0xd4,//0xe2,
	0xa9, 0xee,//0xf0,//0xee,
	0xaa, 0x18,//0x15,//0x18,
	
	//fixed U=0 Y=0 if enable fixed output
	//0xc8, 0x3e,
	//0xcd, 0x00,
	//0xce, 0x00,
	//0x13, 0xe0,
	
	//De-noise enable auto
	0xAB, 0xe7,
	0xb0, 0x43,
	0xac, 0x04,
	0x84, 0x40,

	//Sharpness
	0xad, 0x84,//0x86,//0x82, 
	0xd9, 0x24,//0x44,//0x11,
	0xda, 0x00,
	0xae, 0x10,

	//Scaling
	0xab, 0xe7,
	0xb9, 0xa0,//1280
	0xba, 0x80,//1024
	0xbb, 0xa0,//1280
	0xbc, 0x80,//1024

	//CMX
	0xbd, 0x08,
	0xbe, 0x19,
	0xbf, 0x02,
	0xc0, 0x08,
	0xc1, 0x2a,
	0xc2, 0x34,
	0xc3, 0x2d,
	0xc4, 0x2d,
	0xc5, 0x00,
	0xc6, 0x98,
	0xc7, 0x18,
	0x69, 0x48,

	//UVave
	0x74, 0xc0,

	//UVadj
	0x7c, 0x18,//0x28,//0x18,
	0x65, 0x11,
	0x66, 0x00,
	0x41, 0xa0,
	0x5b, 0x28,
	0x60, 0x84,
	0x05, 0x07,
	0x03, 0x03,
	0xd2, 0x8c,//0x94,//0x8c,

	//SAT & Brightness
	0xc7, 0x90,
	0xc8, 0x06,
	0xcb, 0x40,
	0xcc, 0x40,
	0xcf, 0x00,
	0xd0, 0x20,
	0xd1, 0x00,
	0xc7, 0x18,
	
	//BLC
	0x0d, 0x82,
	0x0d, 0x80,
	
	OV9665_REGEND, 0x00
};

//#################for RGB565 output##########################
static u8 gRGB565_QQVGA_15fps_INITIALIZE[]={
	//0x3E, 0x80,	//disable PLL
	//Reset
	0x12, 0x80,
	
	//IO output
	0xd5, 0xff,
	0xd6, 0x3f,

	//Clock 24Mhz 15 FPS
	0x11, 0x80,
	0x3d, 0x3c,
	0x2a, 0x00,
	0x2b, 0x00,
	0x2d, 0x00,
	0x2e, 0x00,	

	//Power control
	0x3a, 0xd9,//0xf1,//0xd9,
	0x3b, 0x00,
	0x3c, 0x58,
	0x3e, 0x50,
	0x71, 0x00,

	//Sync signal
	0x15, 0x00,

	//Data Format RGB565
	0x85, 0xef, //for scale
	0xD7, 0x01,//0x00, 
	0xD8, 0xc1,//0xC4,

	//Sample Option
	0x63, 0x00,

	//Windowing
	0x12, 0x40,
	0x4d, 0x09,
	0x17, 0x0c,//0x0d,
	0x18, 0x5d,
	0x19, 0x02,
	0x1a, 0x3f,
	0x03, 0x03,
	0x32, 0xad,//0xad,//0x99,
	0x2b, 0x00,
	0x5c, 0x80,

	//BLC
	0x36, 0xb4,
	0x65, 0x11,//0x10,
	0x70, 0x02,
	0x71, 0x9f,
	0x64, 0xa4,//0x24,

	//AEC; 	Average;9 zone;
	0x43, 0x00,
	0x5D, 0x55,
	0x5E, 0x57,
	0x5F, 0x21,

	// Brightness
	0x24, 0x3e,
	0x25, 0x38,
	0x26, 0x72,

	//BF 60Hz
	0x14, 0x28,//0x28,//0x68, AGC gain 16X
	0x0C, 0x3a,//0x3a, 50MHZ//0x38,60MHZ
	0x4F, 0x4f,
	0x50, 0x42,
	0x5A, 0x23,//0x23,//0x67,

	//LC enable	Largon9306;
	0x7d, 0x30,
	0x7e, 0x00,
	0x82, 0x03,
	0x7f, 0x00,
	0x83, 0x07,
	0x80, 0x03,
	0x81, 0x04,

	//AWB advance	Largon9306;
	0x96, 0xf0,
	0x97, 0x00,
	0x92, 0x33,
	0x94, 0x5a,
	0x93, 0x3a,
	0x95, 0x48,
	0x91, 0xfc,
	0x90, 0xff,
	0x8e, 0x4e,
	0x8f, 0x4e,
	0x8d, 0x13,
	0x8c, 0x0c,
	0x8b, 0x0c,
	0x86, 0x9e,
	0x87, 0x11,
	0x88, 0x22,
	0x89, 0x05,
	0x8a, 0x03,

	// Gamma enable
	0x9b, 0x04,//0x08,//0x0e,
	0x9c, 0x07,//0x0e,//0x1c,
	0x9d, 0x10,//0x1e,//0x34,
	0x9e, 0x28,//0x40,//0x5a,
	0x9f, 0x36,//0x54,//0x68,
	0xa0, 0x44,//0x66,//0x76,
	0xa1, 0x52,//0x78,//0x82,
	0xa2, 0x60,//0x86,//0x8e,
	0xa3, 0x6c,//0x92,//0x98,
	0xa4, 0x78,//0x9e,//0xa0,
	0xa5, 0x8c,//0xb0,//0xb0,
	0xa6, 0x9e,//0xbe,//0xbe,
	0xa7, 0xbb,//0xd4,//0xd2,
	0xa8, 0xd2,//0xd4,//0xe2,
	0xa9, 0xe5,//0xf0,//0xee,
	0xaa, 0x24,//0x15,//0x18,
	
	//fixed U=0 Y=0 if enable fixed output
	//0xc8, 0x3e,
	//0xcd, 0x00,
	//0xce, 0x00,
	//0x13, 0xe0,
	
	//De-noise enable auto
	0xAB, 0xe7,
	0xb0, 0x43,
	0xac, 0x04,
	0x84, 0x40,

	//Sharpness
	0xad, 0x86,//0x82, 
	0xd9, 0x44,//0x11,
	0xda, 0x00,
	0xae, 0x10,

	//Scaling to 160*120
	0xab, 0xef,//for scale
	0xb9, 0x50,
	0xba, 0x3c,
	0xbb, 0x14,//0x50,
	0xbc, 0x0f,//0x3c,

	//CMX
	0xbd, 0x08,
	0xbe, 0x19,
	0xbf, 0x02,
	0xc0, 0x08,
	0xc1, 0x2a,
	0xc2, 0x34,
	0xc3, 0x2d,
	0xc4, 0x2d,
	0xc5, 0x00,
	0xc6, 0x98,
	0xc7, 0x18,
	0x69, 0x48,

	//UVave
	0x74, 0xc0,

	//UVadj
	0x7c, 0x18,//0x28,//0x18,
	0x65, 0x11,
	0x66, 0x00,
	0x41, 0xa0,
	0x5b, 0x28,
	0x60, 0x84,
	0x05, 0x07,
	0x03, 0x03,
	0xd2, 0x8c,//0x94,//0x8c,

	//SAT & Brightness
	0xc7, 0x90,
	0xc8, 0x06,
	0xcb, 0x40,
	0xcc, 0x40,
	0xcf, 0x00,
	0xd0, 0x20,
	0xd1, 0x00,
	0xc7, 0x18,
	
	//BLC
	0x0d, 0x92,
	0x0d, 0x90,
	
	OV9665_REGEND, 0x00
};
static u8 gRGB565_QVGA_15fps_INITIALIZE[]={
	//0x3E, 0x80,	//disable PLL
	//Reset
	0x12, 0x80,
	
	//IO output
	0xd5, 0xff,
	0xd6, 0x3f,

	//Clock 24Mhz 15 FPS
	0x11, 0x80,
	0x3d, 0x3c,
	0x2a, 0x00,
	0x2b, 0x00,
	0x2d, 0x00,
	0x2e, 0x00,	

	//Power control
	0x3a, 0xd9,//0xf1,//0xd9,
	0x3b, 0x00,
	0x3c, 0x58,
	0x3e, 0x50,
	0x71, 0x00,

	//Sync signal
	0x15, 0x00,

	//Data Format RGB565
	0x85, 0xef, //for scale
	0xD7, 0x01,//0x00, 
	0xD8, 0xc1,//0xC4,

	//Sample Option
	0x63, 0x00,

	//Windowing
	0x12, 0x40,
	0x4d, 0x09,
	0x17, 0x0c,//0x0d,
	0x18, 0x5d,
	0x19, 0x02,
	0x1a, 0x3f,
	0x03, 0x03,
	0x32, 0xad,//0xad,//0x99,
	0x2b, 0x00,
	0x5c, 0x80,

	//BLC
	0x36, 0xb4,
	0x65, 0x11,//0x10,
	0x70, 0x02,
	0x71, 0x9f,
	0x64, 0xa4,//0x24,

	//AEC; 	Average;9 zone;
	0x43, 0x00,
	0x5D, 0x55,
	0x5E, 0x57,
	0x5F, 0x21,

	// Brightness
	0x24, 0x3e,
	0x25, 0x38,
	0x26, 0x72,

	//BF 60Hz
	0x14, 0x28,//0x28,//0x68, AGC gain 16X
	0x0C, 0x3a,//0x3a, 50MHZ//0x38,60MHZ
	0x4F, 0x4f,
	0x50, 0x42,
	0x5A, 0x23,//0x23,//0x67,

	//LC enable	Largon9306;
	0x7d, 0x30,
	0x7e, 0x00,
	0x82, 0x03,
	0x7f, 0x00,
	0x83, 0x07,
	0x80, 0x03,
	0x81, 0x04,

	//AWB advance	Largon9306;
	0x96, 0xf0,
	0x97, 0x00,
	0x92, 0x33,
	0x94, 0x5a,
	0x93, 0x3a,
	0x95, 0x48,
	0x91, 0xfc,
	0x90, 0xff,
	0x8e, 0x4e,
	0x8f, 0x4e,
	0x8d, 0x13,
	0x8c, 0x0c,
	0x8b, 0x0c,
	0x86, 0x9e,
	0x87, 0x11,
	0x88, 0x22,
	0x89, 0x05,
	0x8a, 0x03,

	// Gamma enable
	0x9b, 0x04,//0x08,//0x0e,
	0x9c, 0x07,//0x0e,//0x1c,
	0x9d, 0x10,//0x1e,//0x34,
	0x9e, 0x28,//0x40,//0x5a,
	0x9f, 0x36,//0x54,//0x68,
	0xa0, 0x44,//0x66,//0x76,
	0xa1, 0x52,//0x78,//0x82,
	0xa2, 0x60,//0x86,//0x8e,
	0xa3, 0x6c,//0x92,//0x98,
	0xa4, 0x78,//0x9e,//0xa0,
	0xa5, 0x8c,//0xb0,//0xb0,
	0xa6, 0x9e,//0xbe,//0xbe,
	0xa7, 0xbb,//0xd4,//0xd2,
	0xa8, 0xd2,//0xd4,//0xe2,
	0xa9, 0xe5,//0xf0,//0xee,
	0xaa, 0x24,//0x15,//0x18,
	
	//fixed U=0 Y=0 if enable fixed output
	//0xc8, 0x3e,
	//0xcd, 0x00,
	//0xce, 0x00,
	//0x13, 0xe0,
	
	//De-noise enable auto
	0xAB, 0xe7,
	0xb0, 0x43,
	0xac, 0x04,
	0x84, 0x40,

	//Sharpness
	0xad, 0x86,//0x82, 
	0xd9, 0x44,//0x11,
	0xda, 0x00,
	0xae, 0x10,

	//Scaling to 320*240
	0xab, 0xef,//for scale
	0xb9, 0x50,
	0xba, 0x3c,
	0xbb, 0x28,//0x50,
	0xbc, 0x1e,//0x3c,

	//CMX
	0xbd, 0x08,
	0xbe, 0x19,
	0xbf, 0x02,
	0xc0, 0x08,
	0xc1, 0x2a,
	0xc2, 0x34,
	0xc3, 0x2d,
	0xc4, 0x2d,
	0xc5, 0x00,
	0xc6, 0x98,
	0xc7, 0x18,
	0x69, 0x48,

	//UVave
	0x74, 0xc0,

	//UVadj
	0x7c, 0x18,//0x28,//0x18,
	0x65, 0x11,
	0x66, 0x00,
	0x41, 0xa0,
	0x5b, 0x28,
	0x60, 0x84,
	0x05, 0x07,
	0x03, 0x03,
	0xd2, 0x8c,//0x94,//0x8c,

	//SAT & Brightness
	0xc7, 0x90,
	0xc8, 0x06,
	0xcb, 0x40,
	0xcc, 0x40,
	0xcf, 0x00,
	0xd0, 0x20,
	0xd1, 0x00,
	0xc7, 0x18,
	
	//BLC
	0x0d, 0x92,
	0x0d, 0x90,
	
	OV9665_REGEND, 0x00
};
static u8 gRGB565_VGA_15fps_INITIALIZE[]={
	//0x3E, 0x80,	//disable PLL
	//Reset
	0x12, 0x80,
	
	//IO output
	0xd5, 0xff,
	0xd6, 0x3f,

	//Clock 24Mhz 15 FPS
	0x11, 0x80,//0x80, 
	0x3d, 0x3c,
	0x2a, 0x00,
	0x2b, 0x00,//0x00,
	0x2d, 0x00,
	0x2e, 0x00,	

	//Power control
	0x3a, 0xd9,//0xf1,//0xd9,
	0x3b, 0x00,
	0x3c, 0x58,
	0x3e, 0x50,
	0x71, 0x00,

	//Sync signal
	0x15, 0x00,

	//Data Format RGB565
	0x85, 0xE7,
	0xD7, 0x01,//0x00, 
	0xD8, 0xc1,//0xC4,

	//Sample Option
	0x63, 0x00,

	//Windowing
	0x12, 0x40,
	0x4d, 0x09,
	0x17, 0x0c,//0x0d,
	0x18, 0x5d,
	0x19, 0x02,
	0x1a, 0x3f,
	0x03, 0x03,
	0x32, 0xad,//0xad,//0x99,
	0x2b, 0x00,
	0x5c, 0x80,

	//BLC
	0x36, 0xb4,
	0x65, 0x11,//0x10,
	0x70, 0x02,
	0x71, 0x9f,
	0x64, 0xa4,//0x24,

	//AEC; 	Average;9 zone;
	0x43, 0x00,
	0x5D, 0x55,
	0x5E, 0x57,
	0x5F, 0x21,

	// Brightness
	0x24, 0x3e,
	0x25, 0x38,
	0x26, 0x72,

	//BF 60Hz
	0x14, 0x68,//0x28,//0x68, AGC gain 16X
	0x0C, 0x38,//0x3a, 50MHZ//0x38,60MHZ
	0x4F, 0x4f,
	0x50, 0x42,
	0x5A, 0x67,//0x23,//0x67,

	//LC enable	Largon9306;
	0x7d, 0x30,
	0x7e, 0x00,
	0x82, 0x03,
	0x7f, 0x00,
	0x83, 0x07,
	0x80, 0x03,
	0x81, 0x04,

	//AWB advance	Largon9306;
	0x96, 0xf0,
	0x97, 0x00,
	0x92, 0x33,
	0x94, 0x5a,
	0x93, 0x3a,
	0x95, 0x48,
	0x91, 0xfc,
	0x90, 0xff,
	0x8e, 0x4e,
	0x8f, 0x4e,
	0x8d, 0x13,
	0x8c, 0x0c,
	0x8b, 0x0c,
	0x86, 0x9e,
	0x87, 0x11,
	0x88, 0x22,
	0x89, 0x05,
	0x8a, 0x03,

	// Gamma enable
	0x9b, 0x0e,//0x04,//0x08,//0x0e,
	0x9c, 0x1c,//0x07,//0x0e,//0x1c,
	0x9d, 0x34,//0x10,//0x1e,//0x34,
	0x9e, 0x5a,//0x28,//0x40,//0x5a,
	0x9f, 0x68,//0x36,//0x54,//0x68,
	0xa0, 0x76,//0x44,//0x66,//0x76,
	0xa1, 0x82,//0x52,//0x78,//0x82,
	0xa2, 0x8e,//0x60,//0x86,//0x8e,
	0xa3, 0x98,//0x6c,//0x92,//0x98,
	0xa4, 0xa0,//0x78,//0x9e,//0xa0,
	0xa5, 0xb0,//0x8c,//0xb0,//0xb0,
	0xa6, 0xbe,//0x9e,//0xbe,//0xbe,
	0xa7, 0xd2,//0xbb,//0xd4,//0xd2,
	0xa8, 0xe2,//0xd2,//0xd4,//0xe2,
	0xa9, 0xee,//0xe5,//0xf0,//0xee,
	0xaa, 0x18,//0x24,//0x15,//0x18,
	
	//fixed U=0 Y=0 if enable fixed output
	//0xc8, 0x3e,
	//0xcd, 0x00,
	//0xce, 0x00,
	//0x13, 0xe0,
	
	//De-noise enable auto
	0xAB, 0xe7,
	0xb0, 0x43,
	0xac, 0x04,
	0x84, 0x40,

	//Sharpness
	0xad, 0x82,//0x86, 
	0xd9, 0x11,//0x44,
	0xda, 0x00,
	0xae, 0x10,

	//Scaling to 640*480
	0xab, 0xe7,
	0xb9, 0x50,//640
	0xba, 0x3c,//480
	0xbb, 0x50,
	0xbc, 0x3c,

	//CMX
	0xbd, 0x08,
	0xbe, 0x19,
	0xbf, 0x02,
	0xc0, 0x08,
	0xc1, 0x2a,
	0xc2, 0x34,
	0xc3, 0x2d,
	0xc4, 0x2d,
	0xc5, 0x00,
	0xc6, 0x98,
	0xc7, 0x18,
	0x69, 0x48,

	//UVave
	0x74, 0xc0,

	//UVadj
	0x7c, 0x18,//0x28,//0x18,
	0x65, 0x11,
	0x66, 0x00,
	0x41, 0xa0,
	0x5b, 0x28,
	0x60, 0x84,
	0x05, 0x07,
	0x03, 0x03,
	0xd2, 0x8c,//0x94,//0x8c,

	//SAT & Brightness
	0xc7, 0x90,
	0xc8, 0x06,
	0xcb, 0x40,
	0xcc, 0x40,
	0xcf, 0x00,
	0xd0, 0x20,
	0xd1, 0x00,
	0xc7, 0x18,
	
	//BLC
	0x0d, 0x92,
	0x0d, 0x90,
	
	OV9665_REGEND, 0x00
};
static u8 gRGB565_SXGA_15fps_INITIALIZE[]={
	//0x3E, 0x80,	//disable PLL
	//Reset
	0x12, 0x80,
	
	//IO output
	0xd5, 0xff,
	0xd6, 0x3f,

	//Clock 24Mhz 7.5 FPS
	0x11, 0x00,//0x80,
	0x3d, 0x3c,
	0x2a, 0x00,
	0x2b, 0x00,
	0x2d, 0x00,
	0x2e, 0x00,	

	//Power control
	0x3a, 0xd9,//0xf1,//0xd9,
	0x3b, 0x00,
	0x3c, 0x58,
	0x3e, 0x50,
	0x71, 0x00,

	//Sync signal
	0x15, 0x00,

	//Data Format RGB565
	0x85, 0xE7,
	0xD7, 0x01,//0x00, 
	0xD8, 0xc1,//0xC4,

	//Sample Option
	0x63, 0x01,

	//Windowing
	0x12, 0x00,
	//0x4d, 0x09,
	0x17, 0x0c,//0x0d,
	0x18, 0x5d,
	0x19, 0x01,
	0x1a, 0x82,
	0x03, 0x03,
	0x32, 0x34,//0xad,//0x99,
	0x2b, 0x00,
	0x5c, 0x80,

	//BLC
	0x36, 0xb4,//0xb4,
	0x65, 0x10,//0x10,
	0x70, 0x02,
	0x71, 0x9c,//0x9f,
	0x64, 0x24,//0xa4,

	//AEC; 	Average;9 zone;
	0x43, 0x00,
	0x5D, 0x55,
	0x5E, 0x57,
	0x5F, 0x21,

	// Brightness
	0x24, 0x3e,
	0x25, 0x38,
	0x26, 0x72,

	//BF 60Hz
	0x14, 0x68,//0x28,//0x68, AGC gain 16X
	0x0C, 0x38,//0x3a, 50MHZ//0x38,60MHZ
	0x4F, 0x4f,
	0x50, 0x42,
	0x5A, 0x67,//0x23,//0x67,

	//LC enable	Largon9306;
	0x7d, 0x30,
	0x7e, 0x00,
	0x82, 0x03,
	0x7f, 0x00,
	0x83, 0x07,
	0x80, 0x03,
	0x81, 0x04,

	//AWB advance	Largon9306;
	0x96, 0xf0,
	0x97, 0x00,
	0x92, 0x33,
	0x94, 0x5a,
	0x93, 0x3a,
	0x95, 0x48,
	0x91, 0xfc,
	0x90, 0xff,
	0x8e, 0x4e,
	0x8f, 0x4e,
	0x8d, 0x13,
	0x8c, 0x0c,
	0x8b, 0x0c,
	0x86, 0x9e,
	0x87, 0x11,
	0x88, 0x22,
	0x89, 0x05,
	0x8a, 0x03,

	// Gamma enable
	0x9b, 0x0e,//0x08,//0x0e,
	0x9c, 0x1c,//0x0e,//0x1c,
	0x9d, 0x34,//0x1e,//0x34,
	0x9e, 0x5a,//0x40,//0x5a,
	0x9f, 0x68,//0x54,//0x68,
	0xa0, 0x76,//0x66,//0x76,
	0xa1, 0x82,//0x78,//0x82,
	0xa2, 0x8e,//0x86,//0x8e,
	0xa3, 0x98,//0x92,//0x98,
	0xa4, 0xa0,//0x9e,//0xa0,
	0xa5, 0xb0,//0xb0,//0xb0,
	0xa6, 0xbe,//0xbe,//0xbe,
	0xa7, 0xd2,//0xd4,//0xd2,
	0xa8, 0xe2,//0xd4,//0xe2,
	0xa9, 0xee,//0xf0,//0xee,
	0xaa, 0x18,//0x15,//0x18,
	
	//fixed U=0 Y=0 if enable fixed output
	//0xc8, 0x3e,
	//0xcd, 0x00,
	//0xce, 0x00,
	//0x13, 0xe0,
	
	//De-noise enable auto
	0xAB, 0xe7,
	0xb0, 0x43,
	0xac, 0x04,
	0x84, 0x40,

	//Sharpness
	0xad, 0x86,//0x82, 
	0xd9, 0x44,//0x11,
	0xda, 0x00,
	0xae, 0x10,

	//Scaling to 1280*1024
	0xab, 0xe7,
	0xb9, 0xa0,
	0xba, 0x80,
	0xbb, 0xa0,
	0xbc, 0x80,

	//CMX
	0xbd, 0x08,
	0xbe, 0x19,
	0xbf, 0x02,
	0xc0, 0x08,
	0xc1, 0x2a,
	0xc2, 0x34,
	0xc3, 0x2d,
	0xc4, 0x2d,
	0xc5, 0x00,
	0xc6, 0x98,
	0xc7, 0x18,
	0x69, 0x48,

	//UVave
	0x74, 0xc0,

	//UVadj
	0x7c, 0x18,//0x28,//0x18,
	0x65, 0x11,
	0x66, 0x00,
	0x41, 0xa0,
	0x5b, 0x28,
	0x60, 0x84,
	0x05, 0x07,
	0x03, 0x03,
	0xd2, 0x8c,//0x94,//0x8c,

	//SAT & Brightness
	0xc7, 0x90,
	0xc8, 0x06,
	0xcb, 0x40,
	0xcc, 0x40,
	0xcf, 0x00,
	0xd0, 0x20,
	0xd1, 0x00,
	0xc7, 0x18,
	
	//BLC
	0x0d, 0x82,
	0x0d, 0x80,
	
	OV9665_REGEND, 0x00
};

//#################for raw data output##########################
static u8 gRAW_QQVGA_15fps_INITALIZE[]={
	//0x3E, 0x80,	//disable PLL
	//Reset
	0x12, 0x80,
	
	//IO output
	0xd5, 0xff,
	0xd6, 0x3f,

	//Clock 24Mhz 15 FPS
	0x11, 0x80,
	0x3d, 0x3c,
	0x2a, 0x00,
	0x2b, 0x00,
	0x2d, 0x00,
	0x2e, 0x00,	

	//Power control
	0x3a, 0xd9,//0xf1,//0xd9,
	0x3b, 0x00,
	0x3c, 0x58,
	0x3e, 0x50,
	0x71, 0x00,

	//Sync signal
	0x15, 0x00,

	//Data Format RAW 
	0x85, 0xef, //for scale
	0xD7, 0x03,//0x01,//0x00, 
	0xD8, 0xc4,//0xC1,

	//Sample Option
	0x63, 0x00,

	//Windowing
	0x12, 0x40,
	0x4d, 0x09,
	0x17, 0x0c,//0x0d,
	0x18, 0x5d,
	0x19, 0x02,
	0x1a, 0x3f,
	0x03, 0x03,
	0x32, 0xad,//0xad,//0x99,
	0x2b, 0x00,
	0x5c, 0x80,

	//BLC
	0x36, 0xb4,
	0x65, 0x11,//0x10,
	0x70, 0x02,
	0x71, 0x9f,
	0x64, 0xa4,//0x24,

	//AEC; 	Average;9 zone;
	0x43, 0x00,
	0x5D, 0x55,
	0x5E, 0x57,
	0x5F, 0x21,

	// Brightness
	0x24, 0x78,//0x3e,
	0x25, 0x68,//0x38,
	0x26, 0xd4,//0x72,

	//BF 60Hz
	0x14, 0x68,//0x28,//0x68, AGC gain 16X
	0x0C, 0x38,//0x3a, 50MHZ//0x38,60MHZ
	0x4F, 0x4f,//0x4f,
	0x50, 0x42,//0x42,
	0x5A, 0x67,//0x23,//0x67,

	//LC enable	Largon9306;
	0x7d, 0x30,
	0x7e, 0x00,
	0x82, 0x03,
	0x7f, 0x00,
	0x83, 0x07,
	0x80, 0x03,
	0x81, 0x04,

	//AWB advance	Largon9306;
	0x96, 0xf0,
	0x97, 0x00,
	0x92, 0x33,
	0x94, 0x5a,
	0x93, 0x3a,
	0x95, 0x48,
	0x91, 0xfc,
	0x90, 0xff,
	0x8e, 0x4e,
	0x8f, 0x4e,
	0x8d, 0x13,
	0x8c, 0x0c,
	0x8b, 0x0c,
	0x86, 0x9e,
	0x87, 0x11,
	0x88, 0x22,
	0x89, 0x05,
	0x8a, 0x03,

	// Gamma enable
	0x9b, 0x0e,//0x04,//0x08,//0x0e,
	0x9c, 0x1c,//0x07,//0x0e,//0x1c,
	0x9d, 0x34,//0x10,//0x1e,//0x34,
	0x9e, 0x5a,//0x28,//0x40,//0x5a,
	0x9f, 0x68,//0x36,//0x54,//0x68,
	0xa0, 0x76,//0x44,//0x66,//0x76,
	0xa1, 0x82,//0x52,//0x78,//0x82,
	0xa2, 0x8e,//0x60,//0x86,//0x8e,
	0xa3, 0x98,//0x6c,//0x92,//0x98,
	0xa4, 0xa0,//0x78,//0x9e,//0xa0,
	0xa5, 0xb0,//0x8c,//0xb0,//0xb0,
	0xa6, 0xbe,//0x9e,//0xbe,//0xbe,
	0xa7, 0xd2,//0xbb,//0xd4,//0xd2,
	0xa8, 0xe2,//0xd2,//0xd4,//0xe2,
	0xa9, 0xee,//0xe5,//0xf0,//0xee,
	0xaa, 0x18,//0x24,//0x15,//0x18,
	
	//fixed U=0 Y=0 if enable fixed output
	//0xc8, 0x3e,
	//0xcd, 0x00,
	//0xce, 0x00,
	//0x13, 0xe0,
	
	//De-noise enable auto
	0xAB, 0xe7,
	0xb0, 0x43,
	0xac, 0x04,
	0x84, 0x40,

	//Sharpness
	0xad, 0x82,//0x86, 
	0xd9, 0x11,//0x44,
	0xda, 0x00,
	0xae, 0x10,

	//Scaling to 160*120
	0xab, 0xef,//for scale
	0xb9, 0x50,//640
	0xba, 0x3c,//480
	0xbb, 0x14,//160,
	0xbc, 0x0f,//120,

	//CMX
	0xbd, 0x08,
	0xbe, 0x19,
	0xbf, 0x02,
	0xc0, 0x08,
	0xc1, 0x2a,
	0xc2, 0x34,
	0xc3, 0x2d,
	0xc4, 0x2d,
	0xc5, 0x00,
	0xc6, 0x98,
	0xc7, 0x18,
	0x69, 0x48,

	//UVave
	0x74, 0xc0,

	//UVadj
	0x7c, 0x18,//0x28,//0x18,
	0x65, 0x11,
	0x66, 0x00,
	0x41, 0xa0,
	0x5b, 0x28,
	0x60, 0x84,
	0x05, 0x07,
	0x03, 0x03,
	0xd2, 0x8c,//0x94,//0x8c,

	//SAT & Brightness
	0xc7, 0x90,
	0xc8, 0x06,
	0xcb, 0x40,
	0xcc, 0x40,
	0xcf, 0x00,
	0xd0, 0x20,
	0xd1, 0x00,
	0xc7, 0x18,
	
	//H mirror 
	//0x04, 0xa8,
	//0x33, 0xc8,
	
	//BLC
	0x0d, 0x92,
	0x0d, 0x90,
	
	OV9665_REGEND, 0x00
};
static u8 gRAW_QVGA_15fps_INITALIZE[]={
	//0x3E, 0x80,	//disable PLL
	//Reset
	0x12, 0x80,
	
	//IO output
	0xd5, 0xff,
	0xd6, 0x3f,

	//Clock 24Mhz 15 FPS
	0x11, 0x80,
	0x3d, 0x3c,
	0x2a, 0x00,
	0x2b, 0x00,
	0x2d, 0x00,
	0x2e, 0x00,	

	//Power control
	0x3a, 0xd9,//0xf1,//0xd9,
	0x3b, 0x00,
	0x3c, 0x58,
	0x3e, 0x50,
	0x71, 0x00,

	//Sync signal
	0x15, 0x00,

	//Data Format RAW 
	0x85, 0xef, //for scale
	0xD7, 0x03,//0x01,//0x00, 
	0xD8, 0xc4,//0xC1,

	//Sample Option
	0x63, 0x00,

	//Windowing
	0x12, 0x40,
	0x4d, 0x09,
	0x17, 0x0c,//0x0d,
	0x18, 0x5d,
	0x19, 0x02,
	0x1a, 0x3f,
	0x03, 0x03,
	0x32, 0xad,//0xad,//0x99,
	0x2b, 0x00,
	0x5c, 0x80,

	//BLC
	0x36, 0xb4,
	0x65, 0x11,//0x10,
	0x70, 0x02,
	0x71, 0x9f,
	0x64, 0xa4,//0x24,

	//AEC; 	Average;9 zone;
	0x43, 0x00,
	0x5D, 0x55,
	0x5E, 0x57,
	0x5F, 0x21,

	// Brightness
	0x24, 0x78,//0x3e,
	0x25, 0x68,//0x38,
	0x26, 0xd4,//0x72,

	//BF 60Hz
	0x14, 0x68,//0x28,//0x68, AGC gain 16X
	0x0C, 0x38,//0x3a, 50MHZ//0x38,60MHZ
	0x4F, 0x4f,//0x4f,
	0x50, 0x42,//0x42,
	0x5A, 0x67,//0x23,//0x67,

	//LC enable	Largon9306;
	0x7d, 0x30,
	0x7e, 0x00,
	0x82, 0x03,
	0x7f, 0x00,
	0x83, 0x07,
	0x80, 0x03,
	0x81, 0x04,

	//AWB advance	Largon9306;
	0x96, 0xf0,
	0x97, 0x00,
	0x92, 0x33,
	0x94, 0x5a,
	0x93, 0x3a,
	0x95, 0x48,
	0x91, 0xfc,
	0x90, 0xff,
	0x8e, 0x4e,
	0x8f, 0x4e,
	0x8d, 0x13,
	0x8c, 0x0c,
	0x8b, 0x0c,
	0x86, 0x9e,
	0x87, 0x11,
	0x88, 0x22,
	0x89, 0x05,
	0x8a, 0x03,

	// Gamma enable
	0x9b, 0x0e,//0x04,//0x08,//0x0e,
	0x9c, 0x1c,//0x07,//0x0e,//0x1c,
	0x9d, 0x34,//0x10,//0x1e,//0x34,
	0x9e, 0x5a,//0x28,//0x40,//0x5a,
	0x9f, 0x68,//0x36,//0x54,//0x68,
	0xa0, 0x76,//0x44,//0x66,//0x76,
	0xa1, 0x82,//0x52,//0x78,//0x82,
	0xa2, 0x8e,//0x60,//0x86,//0x8e,
	0xa3, 0x98,//0x6c,//0x92,//0x98,
	0xa4, 0xa0,//0x78,//0x9e,//0xa0,
	0xa5, 0xb0,//0x8c,//0xb0,//0xb0,
	0xa6, 0xbe,//0x9e,//0xbe,//0xbe,
	0xa7, 0xd2,//0xbb,//0xd4,//0xd2,
	0xa8, 0xe2,//0xd2,//0xd4,//0xe2,
	0xa9, 0xee,//0xe5,//0xf0,//0xee,
	0xaa, 0x18,//0x24,//0x15,//0x18,
	
	//fixed U=0 Y=0 if enable fixed output
	//0xc8, 0x3e,
	//0xcd, 0x00,
	//0xce, 0x00,
	//0x13, 0xe0,
	
	//De-noise enable auto
	0xAB, 0xe7,
	0xb0, 0x43,
	0xac, 0x04,
	0x84, 0x40,

	//Sharpness
	0xad, 0x82,//0x86, 
	0xd9, 0x11,//0x44,
	0xda, 0x00,
	0xae, 0x10,

	//Scaling to 320*240
	0xab, 0xef,//for scale
	0xb9, 0x50,//640
	0xba, 0x3c,//480
	0xbb, 0x28,//320,
	0xbc, 0x1e,//240,

	//CMX
	0xbd, 0x08,
	0xbe, 0x19,
	0xbf, 0x02,
	0xc0, 0x08,
	0xc1, 0x2a,
	0xc2, 0x34,
	0xc3, 0x2d,
	0xc4, 0x2d,
	0xc5, 0x00,
	0xc6, 0x98,
	0xc7, 0x18,
	0x69, 0x48,

	//UVave
	0x74, 0xc0,

	//UVadj
	0x7c, 0x18,//0x28,//0x18,
	0x65, 0x11,
	0x66, 0x00,
	0x41, 0xa0,
	0x5b, 0x28,
	0x60, 0x84,
	0x05, 0x07,
	0x03, 0x03,
	0xd2, 0x8c,//0x94,//0x8c,

	//SAT & Brightness
	0xc7, 0x90,
	0xc8, 0x06,
	0xcb, 0x40,
	0xcc, 0x40,
	0xcf, 0x00,
	0xd0, 0x20,
	0xd1, 0x00,
	0xc7, 0x18,
	
	//H mirror 
	//0x04, 0xa8,
	//0x33, 0xc8,
	
	//BLC
	0x0d, 0x92,
	0x0d, 0x90,
	
	OV9665_REGEND, 0x00
};
static u8 gRAW_VGA_15fps_INITALIZE[]={
	//0x3E, 0x80,	//disable PLL
	//Reset
	0x12, 0x80,
	
	//IO output
	0xd5, 0xff,
	0xd6, 0x3f,

	//Clock 24Mhz 15 FPS
	0x11, 0x80,
	0x3d, 0x3c,
	0x2a, 0x00,
	0x2b, 0x00,
	0x2d, 0x00,
	0x2e, 0x00,	

	//Power control
	0x3a, 0xd9,//0xf1,//0xd9,
	0x3b, 0x00,
	0x3c, 0x58,
	0x3e, 0x50,
	0x71, 0x00,

	//Sync signal
	0x15, 0x00,

	//Data Format RAW 
	0x85, 0xE7,
	0xD7, 0x03,//0x01,//0x00, 
	0xD8, 0xc4,//0xC1,

	//Sample Option
	0x63, 0x00,

	//Windowing
	0x12, 0x40,
	0x4d, 0x09,
	0x17, 0x0c,//0x0d,
	0x18, 0x5d,
	0x19, 0x02,
	0x1a, 0x3f,
	0x03, 0x03,
	0x32, 0xad,//0xad,//0x99,
	0x2b, 0x00,
	0x5c, 0x80,

	//BLC
	0x36, 0xb4,
	0x65, 0x11,//0x10,
	0x70, 0x02,
	0x71, 0x9f,
	0x64, 0xa4,//0x24,

	//AEC; 	Average;9 zone;
	0x43, 0x00,
	0x5D, 0x55,
	0x5E, 0x57,
	0x5F, 0x21,

	// Brightness
	0x24, 0x78,//0x3e,
	0x25, 0x68,//0x38,
	0x26, 0xd4,//0x72,

	//BF 60Hz
	0x14, 0x68,//0x28,//0x68, AGC gain 16X
	0x0C, 0x38,//0x3a, 50MHZ//0x38,60MHZ
	0x4F, 0x4f,//0x4f,
	0x50, 0x42,//0x42,
	0x5A, 0x67,//0x23,//0x67,

	//LC enable	Largon9306;
	0x7d, 0x30,
	0x7e, 0x00,
	0x82, 0x03,
	0x7f, 0x00,
	0x83, 0x07,
	0x80, 0x03,
	0x81, 0x04,

	//AWB advance	Largon9306;
	0x96, 0xf0,
	0x97, 0x00,
	0x92, 0x33,
	0x94, 0x5a,
	0x93, 0x3a,
	0x95, 0x48,
	0x91, 0xfc,
	0x90, 0xff,
	0x8e, 0x4e,
	0x8f, 0x4e,
	0x8d, 0x13,
	0x8c, 0x0c,
	0x8b, 0x0c,
	0x86, 0x9e,
	0x87, 0x11,
	0x88, 0x22,
	0x89, 0x05,
	0x8a, 0x03,

	// Gamma enable
	0x9b, 0x0e,//0x04,//0x08,//0x0e,
	0x9c, 0x1c,//0x07,//0x0e,//0x1c,
	0x9d, 0x34,//0x10,//0x1e,//0x34,
	0x9e, 0x5a,//0x28,//0x40,//0x5a,
	0x9f, 0x68,//0x36,//0x54,//0x68,
	0xa0, 0x76,//0x44,//0x66,//0x76,
	0xa1, 0x82,//0x52,//0x78,//0x82,
	0xa2, 0x8e,//0x60,//0x86,//0x8e,
	0xa3, 0x98,//0x6c,//0x92,//0x98,
	0xa4, 0xa0,//0x78,//0x9e,//0xa0,
	0xa5, 0xb0,//0x8c,//0xb0,//0xb0,
	0xa6, 0xbe,//0x9e,//0xbe,//0xbe,
	0xa7, 0xd2,//0xbb,//0xd4,//0xd2,
	0xa8, 0xe2,//0xd2,//0xd4,//0xe2,
	0xa9, 0xee,//0xe5,//0xf0,//0xee,
	0xaa, 0x18,//0x24,//0x15,//0x18,
	
	//fixed U=0 Y=0 if enable fixed output
	//0xc8, 0x3e,
	//0xcd, 0x00,
	//0xce, 0x00,
	//0x13, 0xe0,
	
	//De-noise enable auto
	0xAB, 0xe7,
	0xb0, 0x43,
	0xac, 0x04,
	0x84, 0x40,

	//Sharpness
	0xad, 0x82,//0x86, 
	0xd9, 0x11,//0x44,
	0xda, 0x00,
	0xae, 0x10,

	//Scaling to 640*480
	0xab, 0xe7,
	0xb9, 0x50,//640
	0xba, 0x3c,//480
	0xbb, 0x50,
	0xbc, 0x3c,

	//CMX
	0xbd, 0x08,
	0xbe, 0x19,
	0xbf, 0x02,
	0xc0, 0x08,
	0xc1, 0x2a,
	0xc2, 0x34,
	0xc3, 0x2d,
	0xc4, 0x2d,
	0xc5, 0x00,
	0xc6, 0x98,
	0xc7, 0x18,
	0x69, 0x48,

	//UVave
	0x74, 0xc0,

	//UVadj
	0x7c, 0x18,//0x28,//0x18,
	0x65, 0x11,
	0x66, 0x00,
	0x41, 0xa0,
	0x5b, 0x28,
	0x60, 0x84,
	0x05, 0x07,
	0x03, 0x03,
	0xd2, 0x8c,//0x94,//0x8c,

	//SAT & Brightness
	0xc7, 0x90,
	0xc8, 0x06,
	0xcb, 0x40,
	0xcc, 0x40,
	0xcf, 0x00,
	0xd0, 0x20,
	0xd1, 0x00,
	0xc7, 0x18,
	
	//H mirror 
	//0x04, 0xa8,
	//0x33, 0xc8,
	
	//TestPattern
	//0x69, 0x02,
	
	0x69, 0x4a,
	
	//BLC
	0x0d, 0x92,
	0x0d, 0x90,
	
	OV9665_REGEND, 0x00
};
static u8 gRAW_SXGA_15fps_INITALIZE[]={
	//0x3E, 0x80,	//disable PLL
	//Reset
	0x12, 0x80,
	
	//IO output
	0xd5, 0xff,
	0xd6, 0x3f,

	//Clock 24Mhz 7.5 FPS
	0x11, 0x00,//0x80,
	0x3d, 0x3c,
	0x2a, 0x00,
	0x2b, 0x00,
	0x2d, 0x00,
	0x2e, 0x00,	

	//Power control
	0x3a, 0xd9,//0xf1,//0xd9,
	0x3b, 0x00,
	0x3c, 0x58,
	0x3e, 0x50,
	0x71, 0x00,

	//Sync signal
	0x15, 0x00,

	//Data Format RAW 
	0x85, 0xE7,
	0xD7, 0x03,//0x01,//0x00, 
	0xD8, 0xc4,//0xC1,

	//Sample Option
	0x63, 0x01,

	//Windowing
	0x12, 0x00, //for SXGA
	//0x4d, 0x09,
	0x17, 0x0c,//0x0d,
	0x18, 0x5d,
	0x19, 0x01,
	0x1a, 0x82,
	0x03, 0x3f,
	0x32, 0x34,//0xad,//0x99,
	0x2b, 0x00,
	0x5c, 0x80,

	//BLC
	0x36, 0xb4,
	0x65, 0x10,//0x11,
	0x70, 0x02,
	0x71, 0x9c,
	0x64, 0x24,//0xa4,

	//AEC; 	Average;9 zone;
	0x43, 0x00,
	0x5D, 0x55,
	0x5E, 0x57,
	0x5F, 0x21,

	// Brightness
	0x24, 0x78,//0x3e,
	0x25, 0x68,//0x38,
	0x26, 0xd4,//0x72,

	//BF 60Hz
	0x14, 0x68,//0x28,//0x68, AGC gain 16X
	0x0C, 0x38,//0x3a, 50MHZ//0x38,60MHZ
	0x4F, 0x9e,//0x4f,
	0x50, 0x84,//0x42,
	0x5A, 0x67,//0x23,//0x67,

	//LC enable	Largon9306;
	0x7d, 0x30,
	0x7e, 0x00,
	0x82, 0x03,
	0x7f, 0x00,
	0x83, 0x07,
	0x80, 0x03,
	0x81, 0x04,

	//AWB advance	Largon9306;
	0x96, 0xf0,
	0x97, 0x00,
	0x92, 0x33,
	0x94, 0x5a,
	0x93, 0x3a,
	0x95, 0x48,
	0x91, 0xfc,
	0x90, 0xff,
	0x8e, 0x4e,
	0x8f, 0x4e,
	0x8d, 0x13,
	0x8c, 0x0c,
	0x8b, 0x0c,
	0x86, 0x9e,
	0x87, 0x11,
	0x88, 0x22,
	0x89, 0x05,
	0x8a, 0x03,

	// Gamma enable
	0x9b, 0x0e,//0x04,//0x08,//0x0e,
	0x9c, 0x1c,//0x07,//0x0e,//0x1c,
	0x9d, 0x34,//0x10,//0x1e,//0x34,
	0x9e, 0x5a,//0x28,//0x40,//0x5a,
	0x9f, 0x68,//0x36,//0x54,//0x68,
	0xa0, 0x76,//0x44,//0x66,//0x76,
	0xa1, 0x82,//0x52,//0x78,//0x82,
	0xa2, 0x8e,//0x60,//0x86,//0x8e,
	0xa3, 0x98,//0x6c,//0x92,//0x98,
	0xa4, 0xa0,//0x78,//0x9e,//0xa0,
	0xa5, 0xb0,//0x8c,//0xb0,//0xb0,
	0xa6, 0xbe,//0x9e,//0xbe,//0xbe,
	0xa7, 0xd2,//0xbb,//0xd4,//0xd2,
	0xa8, 0xe2,//0xd2,//0xd4,//0xe2,
	0xa9, 0xee,//0xe5,//0xf0,//0xee,
	0xaa, 0x18,//0x24,//0x15,//0x18,
	
	//fixed U=0 Y=0 if enable fixed output
	//0xc8, 0x3e,
	//0xcd, 0x00,
	//0xce, 0x00,
	//0x13, 0xe0,
	
	//De-noise enable auto
	0xAB, 0xe7,
	0xb0, 0x43,
	0xac, 0x04,
	0x84, 0x40,

	//Sharpness
	0xad, 0x84,//0x86, 
	0xd9, 0x24,//0x44,
	0xda, 0x00,
	0xae, 0x10,

	//Scaling to 1280*1024
	0xab, 0xe7,
	0xb9, 0xa0,//1280
	0xba, 0x80,//1024
	0xbb, 0xa0,//1280,
	0xbc, 0x80,//1024,

	//CMX
	0xbd, 0x08,
	0xbe, 0x19,
	0xbf, 0x02,
	0xc0, 0x08,
	0xc1, 0x2a,
	0xc2, 0x34,
	0xc3, 0x2d,
	0xc4, 0x2d,
	0xc5, 0x00,
	0xc6, 0x98,
	0xc7, 0x18,
	0x69, 0x48,

	//UVave
	0x74, 0xc0,

	//UVadj
	0x7c, 0x18,//0x28,//0x18,
	0x65, 0x11,
	0x66, 0x00,
	0x41, 0xa0,
	0x5b, 0x28,
	0x60, 0x84,
	0x05, 0x07,
	0x03, 0x03,
	0xd2, 0x8c,//0x94,//0x8c,

	//SAT & Brightness
	0xc7, 0x90,
	0xc8, 0x06,
	0xcb, 0x40,
	0xcc, 0x40,
	0xcf, 0x00,
	0xd0, 0x20,
	0xd1, 0x00,
	0xc7, 0x18,
	
	//H mirror 
	//0x04, 0xa8,
	//0x33, 0xc8,
	
	//BLC
	0x0d, 0x82,
	0x0d, 0x80,
	
	OV9665_REGEND, 0x00
};

struct OV9665_SETTING
{
	int   format;
	int   resolution;
	const u8 *setting;
};
struct OV9665_SETTING ov9665_setting_table[] ={
	
	{OV_FORMAT_YUV_422, OV_SIZE_CIF  , gYUV_CIF_15fps_INITIALIZE},
	{OV_FORMAT_YUV_422, OV_SIZE_QQVGA  , gYUV_QQVGA_15fps_INITIALIZE},
	{OV_FORMAT_YUV_422, OV_SIZE_QVGA  , gYUV_QVGA_15fps_INITIALIZE},
	{OV_FORMAT_YUV_422, OV_SIZE_VGA  , gYUV_VGA_15fps_INITIALIZE},
	{OV_FORMAT_YUV_422, OV_SIZE_SXGA  , gYUV_SXGA_15fps_INITIALIZE},
	
	{OV_FORMAT_RGB_565, OV_SIZE_QQVGA  , gRGB565_QQVGA_15fps_INITIALIZE},
	{OV_FORMAT_RGB_565, OV_SIZE_QVGA  , gRGB565_QVGA_15fps_INITIALIZE},
	{OV_FORMAT_RGB_565, OV_SIZE_VGA  , gRGB565_VGA_15fps_INITIALIZE},
	{OV_FORMAT_RGB_565, OV_SIZE_SXGA  , gRGB565_SXGA_15fps_INITIALIZE},
	
	{OV_FORMAT_RAW10, OV_SIZE_QQVGA, gRAW_QQVGA_15fps_INITALIZE},
	{OV_FORMAT_RAW10, OV_SIZE_QVGA, gRAW_QVGA_15fps_INITALIZE},
	{OV_FORMAT_RAW10, OV_SIZE_VGA, gRAW_VGA_15fps_INITALIZE},
	{OV_FORMAT_RAW10, OV_SIZE_SXGA, gRAW_SXGA_15fps_INITALIZE},
};


//ov offer: VGA,YUV,15FPS
static u8 REG_INIT[]={
	//0x3E, 0x80,	//disable PLL
	//Reset
	0x12, 0x80,
	
	//IO output
	0xd5, 0xff,
	0xd6, 0x3f,

	//Clock 24Mhz 15 FPS
	0x11, 0x80,
	0x3d, 0x3c,
	0x2a, 0x00,
	0x2b, 0x00,
	0x2d, 0x00,
	0x2e, 0x00,	

	//Power control
	0x3a, 0xd9,//0xf1,//0xd9,
	0x3b, 0x00,
	0x3c, 0x58,
	0x3e, 0x50,
	0x71, 0x00,

	//Sync signal
	0x15, 0x00,

	//Data Format YUV
	0xD7, 0x00,
	//0x6a, 0x24,
	//0x85, 0xe7,

	//Sample Option
	0x63, 0x00,

	//Windowing
	0x12, 0x40,
	0x4d, 0x09,
	0x17, 0x0d,
	0x18, 0x5d,
	0x19, 0x02,
	0x1a, 0x3f,
	0x03, 0x03,
	0x32, 0x99,//0xad,//0x99,
	0x2b, 0x00,
	0x5c, 0x80,

	//BLC
	0x36, 0xb4,
	0x65, 0x10,
	0x70, 0x02,
	0x71, 0x9f,
	0x64, 0xa4,

	//AEC; 	Average;9 zone;
	0x43, 0x00,
	0x5D, 0x55,
	0x5E, 0x57,
	0x5F, 0x21,

	// Brightness
	0x24, 0x3e,
	0x25, 0x38,
	0x26, 0x72,

	//BF 60Hz
	0x14, 0x68,//0x28,//0x68, AGC gain 16X
	0x0C, 0x38,//0x3a, 50MHZ//0x38,60MHZ
	0x4F, 0x4f,
	0x50, 0x42,
	0x5A, 0x67,//0x23,//0x67,

	//LC enable	Largon9306;
	0x7d, 0x30,
	0x7e, 0x00,
	0x82, 0x03,
	0x7f, 0x00,
	0x83, 0x07,
	0x80, 0x03,
	0x81, 0x04,

	//AWB advance	Largon9306;
	0x96, 0xf0,
	0x97, 0x00,
	0x92, 0x33,
	0x94, 0x5a,
	0x93, 0x3a,
	0x95, 0x48,
	0x91, 0xfc,
	0x90, 0xff,
	0x8e, 0x4e,
	0x8f, 0x4e,
	0x8d, 0x13,
	0x8c, 0x0c,
	0x8b, 0x0c,
	0x86, 0x9e,
	0x87, 0x11,
	0x88, 0x22,
	0x89, 0x05,
	0x8a, 0x03,

	// Gamma enable
	0x9b, 0x0e,//0x0e,
	0x9c, 0x1c,//0x1c,
	0x9d, 0x34,//0x34,
	0x9e, 0x5a,//0x5a,
	0x9f, 0x68,//0x68,
	0xa0, 0x76,//0x76,
	0xa1, 0x82,//0x82,
	0xa2, 0x8e,//0x8e,
	0xa3, 0x98,//0x98,
	0xa4, 0xa0,//0xa0,
	0xa5, 0xb0,//0xb0,
	0xa6, 0xbe,//0xbe,
	0xa7, 0xd2,//0xd2,
	0xa8, 0xe2,//0xe2,
	0xa9, 0xee,//0xee,
	0xaa, 0x18,//0x18,
	
	//De-noise enable auto
	0xAB, 0xe7,
	0xb0, 0x43,
	0xac, 0x04,
	0x84, 0x40,

	//Sharpness
	0xad, 0x82, 
	0xd9, 0x11,
	0xda, 0x00,
	0xae, 0x10,

	//Scaling
	0xab, 0xe7,
	0xb9, 0x50,
	0xba, 0x3c,
	0xbb, 0x50,
	0xbc, 0x3c,

	//CMX
	0xbd, 0x08,
	0xbe, 0x19,
	0xbf, 0x02,
	0xc0, 0x08,
	0xc1, 0x2a,
	0xc2, 0x34,
	0xc3, 0x2d,
	0xc4, 0x2d,
	0xc5, 0x00,
	0xc6, 0x98,
	0xc7, 0x18,
	0x69, 0x48,

	//UVave
	0x74, 0xc0,

	//UVadj
	0x7c, 0x18,//0x28,//0x18,
	0x65, 0x11,
	0x66, 0x00,
	0x41, 0xa0,
	0x5b, 0x28,
	0x60, 0x84,
	0x05, 0x07,
	0x03, 0x03,
	0xd2, 0x8c,//0x94,//0x8c,

	//SAT & Brightness
	0xc7, 0x90,
	0xc8, 0x06,
	0xcb, 0x40,
	0xcc, 0x40,
	0xcf, 0x00,
	0xd0, 0x20,
	0xd1, 0x00,
	0xc7, 0x18,
	
	//BLC
	0x0d, 0x92,
	0x0d, 0x90,
	
	OV9665_REGEND, 0x00
};

static int read_sensor_reg( const u8 subAddress, u8 *bufP );
static int write_sensor_reg( const u8 subAddress, u8 *bufP );

static int rmw_sensor_reg( const u8 subAddress, u8 *bufP,
		u8 andMask, u8 orMask )
{
	int status;
	status = read_sensor_reg( subAddress, bufP );
	if (!status) {
		*bufP &= andMask;
		*bufP |= orMask;
		status = write_sensor_reg( subAddress, bufP );
	}
	return status;
}

int ov9665hw_read_sensor_reg( const u8 subAddress, u8 *bufP )
{
	return read_sensor_reg(subAddress, bufP);
}

int ov9665hw_write_sensor_reg( const u8 subAddress, u8 *bufP )
{
	return write_sensor_reg(subAddress, bufP);
}

int ov9665hw_set_regs( const u8 *regP )
{
	u32 curReg = 0;
	int    status = 0;

	/* The list is a register number followed by the value */
	while( regP[curReg << 1] < OV9665_REGEND )
	{
		u8 regVal = regP[(curReg << 1) + 1];

		status = (write_sensor_reg( regP[curReg << 1], &regVal ) == 0) ?
			0 : -EIO;

		if( regP[curReg << 1] == 0x12 && regVal==0x80 ){
			ov9665hw_wait( 5 );
			}

		if(status!=0)
			printk(KERN_ERR "write_sensor_reg fail: addr=0x%lx, value=0x%lx",regP[curReg << 1],regVal);
			
		curReg++;
	}

	return status;
}

int ov9665hw_read_all_regs( u8 *bufP, u32 numRegs )
{
	u32 curReg;

	for( curReg = 0; curReg < numRegs; curReg++, bufP++ )
		read_sensor_reg( (u8)curReg, bufP );


	return 0;
}

/*****************************************************************************
 *  Settings
 *****************************************************************************/
int ov9665hw_version_revision(u8 * pCmRevision, u8 *pSensorRevision)
{
	read_sensor_reg( OV9665_PIDH, pCmRevision );
	read_sensor_reg( OV9665_PIDL, pSensorRevision );
	return 0;
}

int ov9665hw_set_contrast(u32 value)
{
	return 0;
}
int ov9665hw_set_exposure(u32 value)
{
	return 0;
}
int ov9665hw_set_white_balance(u32 value)
{
	return 0;
}

void ov9665hw_auto_function_on()
{
	u8 val;
	read_sensor_reg( OV9665_COM8, &val );
	val |= 0x07;	/* don't disturb AWB AEC AGC*/
	write_sensor_reg( OV9665_COM8, &val );
}

void ov9665hw_auto_function_off()
{
	u8 val;
	read_sensor_reg( OV9665_COM8, &val );
	val &= ~0x07;	/*disturb AWB AEC AGC*/
	write_sensor_reg( OV9665_COM8, &val );
}

/*****************************************************************************
 *  Viewfinder, still
 *****************************************************************************/
int ov9665hw_viewfinder_on()
{
	u8 com2;

	read_sensor_reg( OV9665_COM2, &com2 );
	com2 &= ~0x10;
	write_sensor_reg( OV9665_COM2, &com2 );
	
	/*
	u8	com3;

	read_sensor_reg( OV9665_COM3, &com3 );
	com3 &= ~0x01;
	write_sensor_reg( OV9665_COM3, &com3 );
	*/
	
	return OV_ERR_NONE;
}


int ov9665hw_viewfinder_off()
{
	/*
	u8	com3;

	read_sensor_reg( OV9665_COM3, &com3 );
	com3 |= 0x01;
	write_sensor_reg( OV9665_COM3, &com3 );
	*/
	
	u8 com2;

	read_sensor_reg( OV9665_COM2, &com2 );
	com2 |= 0x10;
	write_sensor_reg( OV9665_COM2, &com2 );

	return OV_ERR_NONE;
}

/*****************************************************************************
 *  Format
 *****************************************************************************/
int ov9665hw_set_format(u32 captureSizeFormat,u32 colorFormat){
	
	int	status;
	u8	*regsP;
	const u8	*defaultDataP;
	struct OV9665_SETTING  *ov9665_setting;

	defaultDataP  = NULL;
	for(ov9665_setting = ov9665_setting_table;
		ov9665_setting->format != 0;ov9665_setting++) {
		if ((colorFormat == ov9665_setting->format) &&
			(captureSizeFormat == ov9665_setting->resolution)) {
			defaultDataP = ov9665_setting->setting;
			break;
		}
	}

	if (defaultDataP == NULL)
		return -EPERM;

	/* Get the pointer to the basic setting.
	 * The pointer must be freed after exiting.
	 */
	regsP = (u8 *) defaultDataP;

	/* Blast the entire parameter tree into the part */
	status = ov9665hw_set_regs( regsP );
	return 0;
}

/*****************************************************************************
 *  Power & Reset
 *****************************************************************************/
void ov9665hw_power_down( u8 powerMode )
{
	
	/* OV9665 PWRDWN, 0 = NORMAL, 1=POWER DOWN */
	if( powerMode == CAMERA_POWER_OFF )
		pxa3xx_gpio_set_level(MFP_CIF_HI_PWDN_GPIO, GPIO_LEVEL_HIGH);
	else
		pxa3xx_gpio_set_level(MFP_CIF_HI_PWDN_GPIO, GPIO_LEVEL_LOW);

	mdelay(100);
}

void ov9665hw_reset( )
{
	//ov9665hw_set_regs( REG_INIT);//_YUV_VGA_INIT REG_INIT
	return;
}

void ov9665hw_wait( int ms )
{
	mdelay( ms );
}

/******************************************************************************
 *                              OV9665 I2C Client Driver
 ******************************************************************************/
#include <linux/i2c.h>
static int i2c_ov9665_attach_adapter(struct i2c_adapter *adapter);
static int i2c_ov9665_detect_client(struct i2c_adapter *, int,  int);
static int i2c_ov9665_detach_client(struct i2c_client *client);
#define I2C_DRIVERID_OV9665   (1047)
#define	OV9665_ADDRESS	0x30 //write addresss: 0x60>>1, read address: 0x61>>1

struct i2c_driver ov9665_driver  =
{
	.driver = {
		.name	= "ov9665 i2c client driver",
	},
	.id		= I2C_DRIVERID_OV9665,
	.attach_adapter	= &i2c_ov9665_attach_adapter,
	.detach_client	= &i2c_ov9665_detach_client,
};

/* Unique ID allocation */
static struct i2c_client *g_client;
static unsigned short normal_i2c[] = {OV9665_ADDRESS, I2C_CLIENT_END };
I2C_CLIENT_INSMOD_1(ov9665);


static int read_sensor_reg( const u8 subAddress, u8 *bufP )
{
	int ret;

	if( g_client == NULL )	/* No global client pointer? */
		return -1;

	ret = i2c_smbus_read_byte_data(g_client, subAddress);
	if (ret >= 0) {
		*bufP = ret;
	}
	return ret;
}

static int write_sensor_reg( const u8 subAddress, u8 *bufP )
{
	if( g_client == NULL )	/* No global client pointer? */
		return -1;

	int ret= i2c_smbus_write_byte_data(g_client, subAddress, *bufP);
	//if (ret == 0x12 && (*bufP & 0x80))
               //msleep(2);
    return ret;
}

static int i2c_ov9665_read(struct i2c_client *client, u8 reg)
{
	return i2c_smbus_read_byte_data(client,reg);
}

static int i2c_ov9665_attach_adapter(struct i2c_adapter *adap)
{
	return i2c_probe(adap,&addr_data,i2c_ov9665_detect_client);
}

static int i2c_ov9665_detect_client(struct i2c_adapter *adapter,
		int address, int kind)
{
	struct i2c_client *new_client;
	int err = 0;

	/* Let's see whether this adapter can support what we need.
	   Please substitute the things you need here!  */
	if ( !i2c_check_functionality(adapter,I2C_FUNC_SMBUS_BYTE_DATA) ) {
		printk(KERN_INFO "byte op is not permited.\n");
		goto ERROR0;
	}

	/* OK. For now, we presume we have a valid client. We now create the
	 * client structure, even though we cannot fill it completely yet.
	 * But it allows us to access several i2c functions safely
	 */

	/* Note that we reserve some space for ov9665_data too. If you don't
	 * need it, remove it. We do it here to help to lessen memory
	 * fragmentation.
	 */
	new_client = kmalloc(sizeof(struct i2c_client), GFP_KERNEL );

	if ( !new_client ){
		err = -ENOMEM;
		goto ERROR0;
	}

	new_client->addr = address;
	new_client->adapter = adapter;
	new_client->driver = &ov9665_driver;
	new_client->flags = 0;

	/* detect OV9665 */
	pxa_set_cken(CKEN_CAMERA, 1);
	pxa3xx_mfp_set_afds(MFP_CIF_MCLK, MFP_AF0, MFP_DS04X);
	ci_set_clock(1, 1, 2600);
	pxa3xx_gpio_set_direction(MFP_CIF_HI_PWDN_GPIO, GPIO_DIR_OUT);
	pxa3xx_gpio_set_level(MFP_CIF_HI_PWDN_GPIO, GPIO_LEVEL_LOW);
	mdelay(1);
	
	
	int idH=i2c_ov9665_read(new_client, OV9665_PIDH);
	int idL=i2c_ov9665_read(new_client, OV9665_PIDL);
	
	if ((idH != PIDH_OV9665 ) ||
		(idL != PIDL_OV9665 )){
		ci_set_clock(0, 0, 2600);
		pxa3xx_gpio_set_level(MFP_CIF_HI_PWDN_GPIO, GPIO_LEVEL_HIGH);
		pxa3xx_gpio_set_direction(MFP_CIF_HI_PWDN_GPIO, GPIO_DIR_IN);
		pxa_set_cken(CKEN_CAMERA, 0);

		goto ERROR1;
	}
	else {
		extern int ov9665_detected;
		ov9665_detected = 1;
				
		//int iGAIN=i2c_ov9665_read(new_client, OV9665_GAIN);
		//printk(KERN_INFO "read OV9665_GAIN=%d\n",iGAIN);
		
		//u8 newGAIN=11;
		//if(i2c_ov9665_write(new_client,OV9665_GAIN,&newGAIN)<0)
			//printk(KERN_INFO "i2c_ov9665_write fail\n");
		//iGAIN=i2c_ov9665_read(new_client, OV9665_COM1);
		//printk(KERN_INFO "read OV9665_COM1=%d\n",iGAIN);
		
		printk(KERN_INFO "OV9665 detected.\n");	
	}

	ci_set_clock(0, 0, 2600);
	pxa3xx_gpio_set_level(MFP_CIF_HI_PWDN_GPIO, GPIO_LEVEL_HIGH);
	pxa3xx_gpio_set_direction(MFP_CIF_HI_PWDN_GPIO, GPIO_DIR_IN);
	pxa_set_cken(CKEN_CAMERA, 0);
	
	g_client = new_client;

	strcpy(new_client->name, "OV9665");

	/* Tell the i2c layer a new client has arrived */
	if ((err = i2c_attach_client(new_client)))
		goto ERROR1;

	return 0;

ERROR1:
	kfree(new_client);
ERROR0:
	return err;
}

static int i2c_ov9665_detach_client(struct i2c_client *client)
{
	int err;

	/* Try to detach the client from i2c space */
	if ((err = i2c_detach_client(client))) {
		return err;
	}

	/* Frees client data too, if allocated at the same time */
	kfree(client);
	g_client = NULL;
	return 0;
}

static int __init i2c_ov9665_init(void)
{
	int ret;

	if ( (ret = i2c_add_driver(&ov9665_driver)) ) {
		printk(KERN_ERR "ov9665: Driver registration failed,"
			"module not inserted.\n");
		return ret;
	}

	return 0;
}

static void __exit i2c_ov9665_exit(void)
{
	if (i2c_del_driver(&ov9665_driver)) {
		printk(KERN_ERR "ov9665: Driver registration failed,"
			"module not removed.\n");
	}
}

MODULE_DESCRIPTION("I2C OV9665 driver");
MODULE_LICENSE("GPL");

module_init(i2c_ov9665_init);
module_exit(i2c_ov9665_exit);
