/*
    Copyright (C) 2005, Intel Corporation.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/*++

Abstract:
      contains all OV9665 specific macros, typedefs, and prototypes.
    Declares no storage.

Notes:Only valid for processor code named Monahans.

--*/

#ifndef __CAM_OV9665_HW_HEADER__
#define __CAM_OV9665_HW_HEADER__

/*****************************************************************************
 * Constants & Structures
 *****************************************************************************/
/* Revision constants */
#define PIDH_OV9665           0x96
#define PIDL_OV9665           0x63

/* Return codes */
#define OV_ERR_NONE       	0x00
#define OV_ERR_TIMEOUT    	-1
#define OV_ERR_PARAMETER  	-2
#define OV_COMM_ERR			-3

/* OV9665 Mode */
typedef enum {
	OV9665_SXGA = 0, //SXGA  mode
	OV9665_VGA,      //VGA  mode
	OV9665_INVALID 
}OV9665_MODE;

#define OV_SIZE_CIF				0x01
#define OV_SIZE_QQVGA			( OV_SIZE_CIF << 1 ) 	//160X120
#define OV_SIZE_QVGA			( OV_SIZE_CIF << 2 )	//320X240
#define OV_SIZE_VGA				( OV_SIZE_CIF << 3 )	//640X480
#define OV_SIZE_SXGA			( OV_SIZE_CIF << 4 )	//1280*1024

#define OV_FORMAT_YUV_422		1
#define OV_FORMAT_RGB_565		2
#define OV_FORMAT_RAW8   		3
#define OV_FORMAT_RAW10 		4

#define OV9665_GAIN             0x00    /* AGC Gain Control */
#define OV9665_BLUE_GAIN        0x01    /* Blue Gain Control */
#define OV9665_RED_GAIN         0x02    /* Red Gain Control */
#define OV9665_COM1             0x03    /* Common Control 1 */
#define OV9665_AECL				0x04
#define OV9665_REG5				0x05
#define OV9665_REG6				0x06
#define OV9665_REG7				0x07
#define OV9665_REG8				0x08	//reserved
#define OV9665_COM2				0x09

#define OV9665_PIDH             0x0A    /* Product ID Number MSBs */
#define OV9665_PIDL             0x0B    /* Product ID Number LSBs */
#define OV9665_COM3				0x0C

#define OV9665_COM8				0x13

#define OV9665_HREFST			0x17	//[10:3]
#define OV9665_HREFEND			0x18	//[10:3]
#define OV9665_VSTRT			0x19	//[9:2]
#define OV9665_VEND				0x19	//[9:2]

#define OV9665_REG32			0x32

#define OV9665_REG64			0x64

#define OV9665_REGD7			0xD7


#define OV9665_REGEND     (0xDE)


/*****************************************************************************
 * Function Prototype
 *****************************************************************************/
int ov9665hw_set_regs( const u8 *regP );
int ov9665hw_read_all_regs( u8 *bufP, u32 numRegs );

void ov9665hw_power_down( u8 powerMode );
void ov9665hw_reset(void);
void ov9665hw_wait( int ms );

int  ov9665hw_version_revision(u8 * pCmRevision, u8 *pSensorRevision);
void ov9665hw_set_hsync(void);
void ov9665hw_auto_function_on(void);
void ov9665hw_auto_function_off(void);

int ov9665hw_viewfinder_on(void);
int ov9665hw_viewfinder_off(void);
int ov9665hw_halt_video_output(void);
int ov9665hw_resumeto_full_output_mode(void);
int ov9665hw_get_single_image(void);

int ov9665hw_set_format(u32 capturesize,u32 outformat);
/*
int ov9665hw_set_format(u32 outformat,u32 captureWidth,
		u32 captureHeight,
		u32 *winStartX,
		u32 *winStartY,
		u32 *winEndX,
		u32 *winEndY);
		*/

int ov9665hw_read_sensor_reg( const u8 subAddress, u8 *bufP );
int ov9665hw_write_sensor_reg( const u8 subAddress, u8 *bufP );
int ov9665hw_set_contrast(u32 value);
int ov9665hw_set_exposure(u32 value);
int ov9665hw_set_white_balance(u32 value);


#endif
