/*
 * wacom_tablet.c -- wacom_tablet driver
 *
 * Copyright (C) 2010 Breeze_Yin
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/delay.h>
#include <linux/ioport.h>
#include <linux/sched.h>
#include <linux/slab.h>
#include <linux/smp_lock.h>
#include <linux/errno.h>
#include <linux/init.h>
#include <linux/timer.h>
#include <linux/list.h>
#include <linux/interrupt.h>
#include <linux/utsname.h>
#include <linux/device.h>
#include <linux/moduleparam.h>
#include <linux/fs.h>
#include <linux/poll.h>
#include <linux/types.h>
#include <linux/ctype.h>
#include <linux/cdev.h>

#include <asm/byteorder.h>
#include <linux/io.h>
#include <linux/irq.h>
#include <asm/system.h>
#include <linux/uaccess.h>
#include <asm/unaligned.h>

#include <linux/usb/ch9.h>
#include <linux/usb_gadget.h>

#include <linux/input.h>
#include <linux/serio.h>

#include <asm/atomic.h>
#include <linux/freezer.h>
#include <linux/kthread.h>

#include "wacom_tablet.h"
#include "gadget_chips.h"

//#define DEBUG

#ifdef DEBUG
#define DBG(fmt, args...) \
	printk(fmt, ## args)
#else
#define DBG(fmt, args...) \
	do { } while (0)
#endif /* DEBUG */

#ifdef VERBOSE
#define VDBG(dev, fmt, args...) \
	printk(fmt, ## args)
#else
#define VDBG(dev, fmt, args...) \
	do { } while (0)
#endif /* VERBOSE */

#define ERROR(fmt, args...) \
	printk(fmt, ## args)
#define WARNING(fmt, args...) \
	printk(fmt, ## args)
#define INFO(fmt, args...) \
	printk(fmt, ## args)


/*-------------------------------------------------------------------------*/

#define DRIVER_DESC		"Eee Note Digitizer"
#define DRIVER_VERSION		"2010.06.04"

static const char shortname [] = "eNote digi";
static const char driver_desc [] = DRIVER_DESC;

//static dev_t wacom_tablet_devno;
static struct class *usb_gadget_class;
/*-------------------------------------------------------------------------*/

static struct wacom_tablet_dev g_wacom_tablet;

/*-------------------------------------------------------------------------*/

/* DO NOT REUSE THESE IDs with a protocol-incompatible driver!!  Ever!!
 * Instead:  allocate your own, using normal USB-IF procedures.
 */

/* Thanks to NetChip Technologies for donating this product ID.
 */
#define DRIVER_VENDOR_NUM	0x0b05   /*UGEE 0x5543*/  	/*0x56a	wacom */		/*0x0525 NetChip */
#define DRIVER_PRODUCT_NUM	0x179f 	 /*RAINBOW 0x05*/   /*0x65	bamboo*/	    /*0x4a46 JF-->JiFeng*/ /*0x0090 tpc*/

/* descriptors that are built on-demand */

static char				manufacturer [50] = "ASUSTek Computer, Inc.";//UC-LOGIC Wacom Co.,Ltd. Tablet
static char				product_desc [40] = DRIVER_DESC; //tpc90 //Tablet WP8060U
static char				serial_num [40] = "1";
static char				pnp_string [1024] = "XXMFG:linux;MDL:SU5E-08S05AS-00X;CLS:MOUSE;SN:1;";


/* Number of requests to allocate per endpoint, not used for ep0. */

static int handle_mode = 1; //default right-handler mode
module_param (handle_mode, int, S_IRUGO|S_IWUSR);

#define QLEN	qlen

#ifdef CONFIG_USB_GADGET_DUALSPEED
#define DEVSPEED	USB_SPEED_HIGH
#else   /* full speed (low speed doesn't do bulk) */
#define DEVSPEED        USB_SPEED_FULL
#endif

#define HID_DEVICET_HID		(USB_TYPE_CLASS | 0x01)
#define HID_DEVICET_REPORT	(USB_TYPE_CLASS | 0x02)

#define USB_REQ_GET_REPORT	0x01
#define USB_REQ_SET_REPORT	0x09

#define STRING_MANUFACTURER		1
#define STRING_PRODUCT			2
#define STRING_SERIALNUM		3
#define STRING_PEN				4
//#define STRING_MOUSE			5

/* holds our biggest descriptor */
#define USB_DESC_BUFSIZE		256
#define USB_BUFSIZE				16

/* This device advertises one configuration. */
#define CONFIG_PEN			1//2
//#define CONFIG_MOUSE		3

#define	WACOM_TABLET_INTERFACE		0

#define REPORTID_MOUSE		1 //8
#define REPORTID_PEN		2 //7

#define REPORTID_MOUSE_2	3 //9
#define REPORTID_KEYBOARD   2

#define REPORT_PEN_LEN		8 //+ 2
#define REPORT_MOUSE_LEN	5
#define REPORT_KEYBOARD_LEN	2     

#define UPDATE_INTERVAL		4

static const char pen_string [] = "pen report data";
//static const char mouse_string [] = "mouse report data";


/*-------------------------------------------------------------------------*/
	
u8 report_desc[] = {
	0x05, 0x01,                         //     USAGE_PAGE (Generic Desktop)     
    0x09, 0x02,                         //     USAGE (Mouse)                    
    0xa1, 0x01,                         //     COLLECTION (Application)        
    0x85, REPORTID_MOUSE,               //     REPORT_ID (Mouse)              
    0x09, 0x01,                         //     USAGE (Pointer)                
    0xa1, 0x00,                         //     COLLECTION (Physical)          
    0x05, 0x09,                         //     USAGE_PAGE (Button)          
    0x19, 0x01,                         //     USAGE_MINIMUM (Button 1)     
    0x29, 0x03,                         //     USAGE_MAXIMUM (Button 3)     
    0x15, 0x00,                         //     LOGICAL_MINIMUM (0)          
    0x25, 0x01,                         //     LOGICAL_MAXIMUM (1)    
    0x95, 0x03,                         //     REPORT_COUNT (3)        
    0x75, 0x01,                         //     REPORT_SIZE (1)                         
    0x81, 0x02,                         //     INPUT (Data,Var,Abs)         
    0x95, 0x01,                         //     REPORT_COUNT (1)    
    0x75, 0x05,                         //     REPORT_SIZE (5)           
    0x81, 0x01,                         //     INPUT (Cnst,Var,Abs)         
    0x05, 0x01,                         //     USAGE_PAGE (Generic Desktop) 
    0x09, 0x30,                         //     USAGE (X)                    
    0x09, 0x31,                         //     USAGE (Y)
    0x09, 0x38,                         //	   USAGE (Wheel)
    0x15, 0x81,                         //     LOGICAL_MINIMUM (-127)       
    0x25, 0x7f,                         //     LOGICAL_MAXIMUM (127)        
    0x75, 0x08,                         //     REPORT_SIZE (8)              
    0x95, 0x03,                         //     REPORT_COUNT (3)             
    0x81, 0x06,                         //     INPUT (Data,Var,Rel)         
    0xc0,                               //     END_COLLECTION                 
    0xc0,                               //     END_COLLECTION       
    // digitizer
    0x05, 0x0d,                         // 	   USAGE_PAGE (Digitizers)    
    0x09, 0x01,                         // 	   USAGE (Digitizer)  
    0xa1, 0x01,                         //     COLLECTION (Application)
    0x85, REPORTID_PEN,                 //     REPORT_ID (pen)
    0xa1, 0x00,                         //     COLLECTION (Physical)  
    0x06, 0x00, 0xff,					//     USAGE_PAGE (vendor defined) ??
    0x09, 0x01,							//     USAGE (Unknown)
    0x15, 0x00,                         //     LOGICAL_MINIMUM (0)          
    0x25, 0xff,                   		//     LOGICAL_MAXIMUM (0xff) 
    0x75, 0x08,                         //     REPORT_SIZE (8)
    0x95, 0x01,                         //     REPORT_COUNT(1) 
    0x81, 0x02,                         //     INPUT (Data,Var,Rel)
    0x05, 0x01,                         //     USAGE_PAGE (Generic Desktop) 
    0x09, 0x30,                         //     USAGE (X)                    
    0x75, 0x10,                         //     REPORT_SIZE (16)             
    0x95, 0x01,                         //     REPORT_COUNT (1)             
    0xa4,                               //     PUSH                         
    0x55, 0x0d,                         //     UNIT_EXPONENT (-3)           
    0x65, 0x33,                         //     UNIT (Inch,EngLinear)        
    0x35, 0x00,                         //     PHYSICAL_MINIMUM (0)         
    0x46, 0x58, 0x19,                   //     PHYSICAL_MAXIMUM (6488)      
    0x26, 0x60, 0x40,                   //     LOGICAL_MAXIMUM (16480)        
    0x81, 0x02,                         //     INPUT (Data,Var,Abs)         
    0x09, 0x31,                         //     USAGE (Y)                    
    0x46, 0x15, 0x13,                   //     PHYSICAL_MAXIMUM (4885)      
    0x26, 0x7a, 0x30,                   //     LOGICAL_MAXIMUM (12410)         
    0x81, 0x02,                         //     INPUT (Data,Var,Abs)         
    0xb4,                               //     POP  
    0x05, 0x0d,                         //     USAGE_PAGE (Digitizers)      
    0x09, 0x30,                         //     USAGE (Tip Pressure)         
    0x26, 0xff, 0x00,                   //     LOGICAL_MAXIMUM (255)        
    0x81, 0x02,                         //     INPUT (Data,Var,Abs)  
    0x06, 0x00, 0xff,					//     USAGE_PAGE (vendor defined) ??
	0x09, 0x01,							//     USAGE (Unknown)      
    0x25, 0xff,                   		//     LOGICAL_MAXIMUM (0xff) 
    0x75, 0x08,                         //     REPORT_SIZE (8)
    0x95, 0x01,                         //     REPORT_COUNT(1) 
    0x81, 0x02,                         //     INPUT (Data,Var,Rel)                       
    0xc0,                               //     END_COLLECTION
    //feature 
    0x09, 0x01,							//	   USAGE (Unknown)
    0x85, 0x02,							//	   REPORT_ID (pen)
    0x95, 0x01,							//	   REPORT_COUNT (1)
    0xb1, 0x02,    						//	   FEATURE (Data, Variable, Absolute, No Wrap, Linear)
    //feature
    0x09, 0x01,							//	   USAGE (Unknown)
    0x85, 0x03,							//	   REPORT_ID (3)
    0x95, 0x01,							//	   REPORT_COUNT (1)
    0xb1, 0x02,    						//	   FEATURE (Data, Variable, Absolute, No Wrap, Linear)
    //feature
    0x09, 0x01,							//	   USAGE (Unknown)
    0x85, 0x04,							//	   REPORT_ID (4)
    0x95, 0x01,							//	   REPORT_COUNT (1)
    0xb1, 0x02,    						//	   FEATURE (Data, Variable, Absolute, No Wrap, Linear)
    //feature
    0x09, 0x01,							//	   USAGE (Unknown)
    0x85, 0x10,							//	   REPORT_ID (16)
    0x95, 0x02,							//	   REPORT_COUNT (2)
    0xb1, 0x02,    						//	   FEATURE (Data, Variable, Absolute, No Wrap, Linear)
    //feature
    0x09, 0x01,							//	   USAGE (Unknown)
    0x85, 0x11,							//	   REPORT_ID (17)
    0x95, 0x10,							//	   REPORT_COUNT (16)
    0xb1, 0x02,    						//	   FEATURE (Data, Variable, Absolute, No Wrap, Linear)
    //feature
    0x09, 0x01,							//	   USAGE (Unknown)
    0x85, 0x13,							//	   REPORT_ID (19)
    0x95, 0x01,							//	   REPORT_COUNT (1)
    0xb1, 0x02,    						//	   FEATURE (Data, Variable, Absolute, No Wrap, Linear)
    //feature
    0x09, 0x01,							//	   USAGE (Unknown)
    0x85, 0x20,							//	   REPORT_ID (32)
    0x95, 0x01,							//	   REPORT_COUNT (1)
    0xb1, 0x02,    						//	   FEATURE (Data, Variable, Absolute, No Wrap, Linear)
    //feature
    0x09, 0x01,							//	   USAGE (Unknown)
    0x85, 0x21,							//	   REPORT_ID (33)
    0x95, 0x01,							//	   REPORT_COUNT (1)
    0xb1, 0x02,    						//	   FEATURE (Data, Variable, Absolute, No Wrap, Linear)
    0xc0                                //     END_COLLECTION    
};

/*-------------------------------------------------------------------------*/

static struct usb_device_descriptor device_desc = {
	.bLength =		sizeof device_desc,
	.bDescriptorType =	USB_DT_DEVICE,
	.bcdUSB =		cpu_to_le16(0x0110), //0x0200
	.bDeviceClass =		0x00,
	.bDeviceSubClass =	0x00,
	.bDeviceProtocol =	0x00,
	.idVendor =		cpu_to_le16(DRIVER_VENDOR_NUM),
	.idProduct =		cpu_to_le16(DRIVER_PRODUCT_NUM),
	.iManufacturer =	STRING_MANUFACTURER,
	.iProduct =		STRING_PRODUCT,
	.iSerialNumber =	STRING_SERIALNUM,
	.bNumConfigurations =	1
};

static struct usb_otg_descriptor otg_desc = {
	.bLength =		sizeof otg_desc,
	.bDescriptorType =	USB_DT_OTG,
	.bmAttributes =		USB_OTG_SRP
};

static struct usb_config_descriptor config_desc_pen = {
	.bLength =		sizeof(config_desc_pen),
	.bDescriptorType =	USB_DT_CONFIG,
	/* compute wTotalLength on the fly */
	//.wTotalLength = cpu_to_le16(sizeof(config_desc_pen)),
	.bNumInterfaces =	0x01,
	.bConfigurationValue =	CONFIG_PEN,
	.iConfiguration =	STRING_PEN,
	.bmAttributes =		USB_CONFIG_ATT_ONE | USB_CONFIG_ATT_SELFPOWER,
	.bMaxPower =		1,
};

/*
static struct usb_config_descriptor config_desc_mouse = {
	.bLength =		sizeof(config_desc_mouse),
	.bDescriptorType =	USB_DT_CONFIG,
	// compute wTotalLength on the fly 
	//.wTotalLength = cpu_to_le16(sizeof(config_desc_mouse)),
	.bNumInterfaces =	0x01,
	.bConfigurationValue =	CONFIG_MOUSE,
	.iConfiguration =	STRING_MOUSE,
	.bmAttributes =		USB_CONFIG_ATT_ONE | USB_CONFIG_ATT_SELFPOWER,
	.bMaxPower =		1,
};
*/

static struct usb_interface_descriptor intf_desc_pen = {
	.bLength =		0x09,
	.bDescriptorType =	USB_DT_INTERFACE,//USB_DT_INTERFACE,
	//.bInterfaceNumber =	0x00,			//the index number of the interface
	.bNumEndpoints =	0x01,
	.bInterfaceClass =	0x03,//USB_CLASS_HID,
	.bInterfaceSubClass =	0x00,	/* No Sub-Class */
	.bInterfaceProtocol =	0x02,	/* Mouse */  //breeze tag
	.iInterface =	STRING_PEN	//STRING_PEN
};

/*
static struct usb_interface_descriptor intf_desc_mouse = {
	.bLength =		0x09,
	.bDescriptorType =	USB_DT_INTERFACE,//USB_DT_INTERFACE,
	//.bInterfaceNumber =	0x00,
	.bNumEndpoints =	0x01,
	.bInterfaceClass =	0x03,//USB_CLASS_HID,
	.bInterfaceSubClass =	0x00,	// No Sub-Class 
	.bInterfaceProtocol =	0x02,	// Mouse   //breeze tag
	.iInterface =		STRING_MOUSE
};
*/

static struct usb_endpoint_descriptor ep_in_desc_pen = {
	.bLength =		USB_DT_ENDPOINT_SIZE,
	.bDescriptorType =	USB_DT_ENDPOINT,
	.bEndpointAddress =	USB_DIR_IN,
	.bmAttributes =		USB_ENDPOINT_XFER_INT,
	.wMaxPacketSize =	__constant_cpu_to_le16(REPORT_PEN_LEN + 1),
	.bInterval =		UPDATE_INTERVAL,	
};

/*
static struct usb_endpoint_descriptor ep_in_desc_mouse = {
	.bLength =		USB_DT_ENDPOINT_SIZE,
	.bDescriptorType =	USB_DT_ENDPOINT,
	.bEndpointAddress =	USB_DIR_IN,
	.bmAttributes =		USB_ENDPOINT_XFER_INT,
	.wMaxPacketSize =	__constant_cpu_to_le16(REPORT_MOUSE_LEN),
	.bInterval =		UPDATE_INTERVAL,	
};
*/

static struct usb_hid_descriptor hid_desc_pen = {
	.bLength	= 0x09,
	.bDescriptorType	= 0x21,
	.bcdHID	=	cpu_to_le16(0x0110),//0x0200
	.bCountyCode = 0x21,
	.bNumDescriptors = 0x01,
	.bSubDescriptorType = 0x22,
	.bDescriptorLength = cpu_to_le16(sizeof(report_desc))  //report_desc_pen

};

/*
static struct usb_hid_descriptor hid_desc_mouse = {
	.bLength	= 0x09,
	.bDescriptorType	= 0x21,
	.bcdHID	=	cpu_to_le16(0x0110),
	.bCountyCode = 0x21,
	.bNumDescriptors = 0x01,
	.bSubDescriptorType = 0x22,
	.bDescriptorLength = cpu_to_le16(sizeof(report_desc)) //report_desc_pen

};
*/

static const struct usb_descriptor_header *pen_wacom_tablet_function [] = {
	(struct usb_descriptor_header *) &otg_desc,
	//(struct usb_descriptor_header *) &config_desc_pen, 
	(struct usb_descriptor_header *) &intf_desc_pen,
	(struct usb_descriptor_header *) &hid_desc_pen,
	(struct usb_descriptor_header *) &ep_in_desc_pen,
	NULL
};

/*
static const struct usb_descriptor_header *mouse_wacom_tablet_function [11] = {
	(struct usb_descriptor_header *) &otg_desc,
	(struct usb_descriptor_header *) &config_desc_mouse, 
	(struct usb_descriptor_header *) &intf_desc_mouse,
	(struct usb_descriptor_header *) &hid_desc_mouse,
	(struct usb_descriptor_header *) &ep_in_desc_mouse,
	NULL
};
*/

#ifdef	CONFIG_USB_GADGET_DUALSPEED
/* maxpacket and other transfer characteristics vary by speed. */
#define ep_desc(g, hs, fs) (((g)->speed == USB_SPEED_HIGH)?(hs):(fs))

#else

/* if there's no high speed support, maxpacket doesn't change. */
#define ep_desc(g, hs, fs) (((void)(g)), (fs))

#endif	/* !CONFIG_USB_GADGET_DUALSPEED */

/*-------------------------------------------------------------------------*/

/* static strings, in UTF-8 */
static struct usb_string		strings [] = {
	{ STRING_MANUFACTURER,	manufacturer, },
	{ STRING_PRODUCT,	product_desc, },
	{ STRING_SERIALNUM,	serial_num, },
	{ STRING_PEN, pen_string, },
	//{ STRING_MOUSE, mouse_string, },
	{  }		/* end of list */
};

static struct usb_gadget_strings	stringtab = {
	.language	= 0x0409,	/* en-us */
	.strings	= strings,
};


static struct usb_request *
wacom_tablet_req_alloc(struct usb_ep *ep, unsigned len, gfp_t gfp_flags)
{
	struct usb_request	*req;

	req = usb_ep_alloc_request(ep, gfp_flags);

	if (req != NULL) {
		req->length = len;
		req->buf =  kmalloc(len, gfp_flags);
		//usb_ep_alloc_buffer(ep, len, &req->dma, gfp_flags);
		if (req->buf == NULL) {
			usb_ep_free_request(ep, req);
			return NULL;
		}
	}

	return req;
}

static void
wacom_tablet_req_free(struct usb_ep *ep, struct usb_request *req)
{
	if (ep != NULL && req != NULL) {
		//usb_ep_free_buffer (ep, req->buf, req->dma, req->length);
		kfree(req->buf);
		usb_ep_free_request(ep, req);
	}
}

static void tx_complete(struct usb_ep *ep, struct usb_request *req)
{
	struct wacom_tablet_dev	*dev = ep->driver_data;
	
	switch (req->status) {
	default:
		/* FALLTHROUGH */
	case -ECONNRESET:		/* unlink */
	case -ESHUTDOWN:		/* disconnect etc */
		break;
	case 0:
		break;
	}
	
	//spin_lock(&dev->lock);
	wacom_tablet_req_free(ep, req);
	//spin_unlock(&dev->lock);
}




/*-------------------------------------------------------------------------*/
static ssize_t
wacom_start_tx(struct wacom_tablet_dev *dev, unsigned char *buf)
{
	unsigned long		flags;
	struct usb_request	*req;
	char 				tmp; // 
	struct usb_ep		*in_ep = dev->in_ep_pen;
	int 				len = 0;
	static int			old_x = -1, old_y = -1;
	int proxi = buf[1] & 0x20 ? 1 : 0 ;
	
	if(!handle_mode)	//left-handler mode
	{
		int xtmp, ytmp;
		xtmp = (((int)buf[2] & 0xff)|(((int)buf[3] << 8) & 0xff00)) & 0xffff;
		ytmp = (((int)buf[4] & 0xff)|(((int)buf[5] << 8) & 0xff00)) & 0xffff;
		xtmp = 16480 - xtmp;
		ytmp = 12410 - ytmp;
		buf[2] =  xtmp & 0xff ;
		buf[3] =  (xtmp >> 8) & 0xff ;
		buf[4] =  ytmp & 0xff;
		buf[5] =  (ytmp >> 8) & 0xff;
	}

	spin_lock_irqsave(&dev->lock, flags);
	
	if(!dev->is_connect || dev->config != CONFIG_PEN)
	{
		spin_unlock_irqrestore(&dev->lock, flags);
		DBG("reset or disconnected\n");
		return -EAGAIN;
	}
	
	if(dev->report_id == REPORTID_MOUSE)
	{
		s16 x = 0, y = 0, dx = 0, dy = 0;
		
		len = REPORT_MOUSE_LEN;
		buf[0] = REPORTID_MOUSE;
		x = ( buf[2] & 0x00ff | ((buf[3] << 8) & 0xff00));
		y = ( buf[4] & 0x00ff | ((buf[5] << 8) & 0xff00));
		
		if(!proxi)
		{
			x  = -1;
			y  = -1;
		}
		else
		{
			if(old_x != -1 && old_y != -1)
			{
				dx = (x - old_x) * 200 / 2540;
				dy = (y - old_y) * 200 / 2540;
				
				if(dx == 0 && x != old_x) 
					dx = x > old_x ? 1 : -1;
					
				if(dx > 127)  dx = 127;
				if(dx < -127) dx = -127;
				
				if(dy == 0 && y != old_y) 
					dy = y > old_y ? 1 : -1;
					
				if(dy > 127)  dy = 127;
				if(dy < -127) dy = -127;
			}
			
		}
		
		old_x = x;
		old_y = y;
		
		buf[1] = buf[1] & 0x07;
		
		buf[2] = dx;
		buf[3] = dy;
		buf[4] = 0;
	}
	else if(dev->report_id == REPORTID_PEN)
	{
		len = REPORT_PEN_LEN + 1;
		buf[0] = REPORTID_PEN;
		tmp = (((buf[1] & 0x20) << 2) | ((buf[1] & 0x04) << 3) | (buf[1] & 0x03));
		if(proxi)
			tmp |= 0x10;
		buf[1] = tmp;
	}
	else
	{
		DBG("unknown report id\n");
		spin_unlock_irqrestore(&dev->lock, flags);
		return -EAGAIN;
	}

		
	req = wacom_tablet_req_alloc(in_ep, USB_BUFSIZE, GFP_KERNEL);
	if(!req)
	{
		spin_unlock_irqrestore(&dev->lock, flags);
		DBG("alloc request failed.\n");
		return -EAGAIN;
	}
					
	req->length = len;

	//if (len > size)
	req->zero = 0;
	//else
	//	req->zero = ((len % in_ep->maxpacket) == 0);

	memcpy(req->buf, buf, len);
	
	//DBG("add req to queue.\n");
	list_add_tail(&req->list, &dev->tx_reqs);
		
	spin_unlock_irqrestore(&dev->lock, flags);
			
	wake_up_interruptible(&dev->tx_wait);
	
	if (len) 
		return len;
	else 
		return -EAGAIN;
}


static int wacom_tablet_handle_event(struct wacom_tablet_dev *dev)
{
	//unsigned long		flags;
	int					ret;
	int					i;
	struct usb_request	*req;
	struct usb_ep		*in_ep;
	int				     flag = 0;
	
	while(1) {
		
		spin_lock(&dev->lock);
			
		if(!dev->is_connect || unlikely(list_empty(&dev->tx_reqs)))
		{
			spin_unlock(&dev->lock);
				//DBG("list %d is empty\n", i);
			//flag++;
			break;
		}
			
		req = container_of(dev->tx_reqs.next, struct usb_request,
				list);
					
		list_del_init(&req->list);
		
		//in_ep = (dev->config == CONFIG_PEN) ? dev->in_ep_pen : dev->in_ep_mouse ;	
		in_ep = dev->in_ep_pen;
			
		req->complete = tx_complete;
		
		ret = usb_ep_queue(in_ep, req, GFP_ATOMIC);
		if(ret < 0) {
			DBG("tablet_thread: Failed to queue usb ep, error: %d\n", ret);
			wacom_tablet_req_free(in_ep, req);
			spin_unlock(&dev->lock);
			return -EAGAIN;
		}
			
		spin_unlock(&dev->lock);
		
	}
	
	return 0;
}

static int tablet_thread(void *dev)
{	 
	struct wacom_tablet_dev	*tablet_dev = (struct wacom_tablet_dev *)dev;
	
	set_user_nice(current, 0);
	
	do {
		wacom_tablet_handle_event(tablet_dev);
		wait_event_interruptible(tablet_dev->tx_wait, 
			kthread_should_stop() || !list_empty(&tablet_dev->tx_reqs));
		try_to_freeze();
	} while (!kthread_should_stop());

	printk(KERN_DEBUG "eTablet: keTablet exiting\n");
	return 0;
}


/*-------------------------------------------------------------------------*/

static int
wacom_tablet_config_buf(enum usb_device_speed speed, u8 *buf, u8 type, unsigned index,
		int is_otg)
{
	int					len;
	const struct usb_descriptor_header	**function;
	struct usb_config_descriptor *config_desc;
	
	DBG("%s, index=%d, is_otg=%d\n", __func__, index, is_otg);
	
	if (index >= device_desc.bNumConfigurations)
		return -EINVAL;
	
	//function = is_pen ? pen_wacom_tablet_function : mouse_wacom_tablet_function;
	function = pen_wacom_tablet_function;
	//config_desc = is_pen ? &config_desc_pen : &config_desc_mouse;
	config_desc = &config_desc_pen;

	/* for now, don't advertise srp-only devices */
	if (!is_otg)
		function++;

	len = usb_gadget_config_buf(config_desc, buf, USB_DESC_BUFSIZE,
			function);
	if (len < 0)
		return len;
	((struct usb_config_descriptor *) buf)->bDescriptorType = type;
	
	
	return len;
}

static int
set_wacom_tablet_interface(struct wacom_tablet_dev *dev, unsigned number)
{
	int			result = 0;
	struct usb_ep		*in_ep;
    struct usb_endpoint_descriptor *in_desc;	
	
	DBG("%s, number=%d\n", __func__, number);
	
	spin_lock (&dev->lock);
	switch(number)
	{
		case CONFIG_PEN:
		dev->ep_desc_pen = ep_desc(dev->gadget, NULL, &ep_in_desc_pen);
		dev->in_ep_pen->driver_data = dev;
		in_desc = (struct usb_endpoint_descriptor *)dev->ep_desc_pen;
		in_ep = dev->in_ep_pen;
		break;
		/*
		case CONFIG_MOUSE:
		dev->ep_desc_mouse = ep_desc(dev->gadget, NULL, &ep_in_desc_mouse);
		dev->in_ep_mouse->driver_data = dev;
		in_desc = (struct usb_endpoint_descriptor *)dev->ep_desc_mouse;
		in_ep = dev->in_ep_mouse;
		break;
		*/
		default:
		spin_unlock (&dev->lock);
		return 0;
	}
	
	
	result = usb_ep_enable(in_ep, in_desc);
	if (result != 0) {
		DBG("enable %s failed --> %d\n", in_ep->name, result);
		goto done;
	}

done:
	/* on error, disable any endpoints  */
	if (result != 0) {
		(void) usb_ep_disable(in_ep);
		/*		
		if(number == CONFIG_PEN)
				dev->ep_desc_pen = NULL;
		else
				dev->ep_desc_mouse = NULL;
		*/
		dev->is_connect = 0;
	}
	else
	{
		dev->is_connect = 1;
	}
	
	
	spin_unlock (&dev->lock);
	/* caller is responsible for cleanup on error */
	return result;
}

static void disable_wacom_tablet_interface(struct wacom_tablet_dev *dev)
{
	DBG("%s, interface-->%d, config-->%d\n", __func__, dev->interface, dev->config);
	
	if (dev->config == 0)
		return;
	
	spin_lock (&dev->lock);
	
	dev->is_connect = 0;
	
	if (dev->in_ep_pen)
	{
		usb_ep_disable(dev->in_ep_pen);	
		//dev->in_ep_pen = NULL;
		//dev->ep_desc_pen = NULL;
	}
		
	/*
	if (dev->in_ep_mouse)
	{
		usb_ep_disable(dev->in_ep_mouse);
		//dev->in_ep_mouse = NULL;
		//dev->ep_desc_mouse = NULL;	
	}
	*/
	
	dev->interface = 0;
	dev->config	   = 0;
	
	spin_unlock (&dev->lock);
	
}

/* change our operational config.  must agree with the code
 * that returns config descriptors, and altsetting code.
 */
static int
wacom_tablet_set_config(struct wacom_tablet_dev *dev, unsigned number)
{
	int			result = 0;
	struct usb_gadget	*gadget = dev->gadget;
	
	DBG("%s, config=%d\n", __func__, number);
	
	if (gadget_is_sa1100(gadget) && dev->config) {
		/* tx fifo is full, but we can't clear it...*/
		INFO("can't change configurations\n");
		return -ESPIPE;
	}

	switch (number) {
	case CONFIG_PEN:
	//case CONFIG_MOUSE:
		result = 0;
		break;
	default:
		result = -EINVAL;
		/* FALL THROUGH */
	case 0:
		return result;
	}

	if (result) {
		usb_gadget_vbus_draw(dev->gadget,
				dev->gadget->is_otg ? 8 : 100);
	} else {
		char *speed;
		unsigned power;

		power = 2 * config_desc_pen.bMaxPower;
		usb_gadget_vbus_draw(dev->gadget, power);

		switch (gadget->speed) {
		case USB_SPEED_FULL:	speed = "full"; break;
		default:		speed = "?"; break;
		}

		dev->config = number;
		INFO("%s speed config #%d: %d mA, %s\n",
				speed, number, power, driver_desc);
	}
	
	set_wacom_tablet_interface(dev, number);
	
	
	return result;
}

/* Change our operational Interface. */
static int
wacom_tablet_set_interface(struct wacom_tablet_dev *dev, unsigned interface)
{
	int			result = 0;
	
	DBG("%s, interface=%d\n", __func__, interface);
	
	if (gadget_is_sa1100(dev->gadget) && dev->interface < 0) {
		/* tx fifo is full, but we can't clear it...*/
		INFO("can't change interfaces\n");
		return -ESPIPE;
	}
	
	/* Free the current interface */
	switch(dev->interface)
	{
		case CONFIG_PEN:
		//case CONFIG_MOUSE:
			disable_wacom_tablet_interface(dev);
			break;
		default:
			break;
	}

	switch (interface) {
	case CONFIG_PEN:
	//case CONFIG_MOUSE:
		result = set_wacom_tablet_interface(dev, interface);
		if (result) {
			disable_wacom_tablet_interface(dev);
		}
		else
		{
			dev->interface = interface;
		} 
		break;
	default:
		result = -EINVAL;
		/* FALL THROUGH */
	}

	if (!result)
		INFO("Using interface %x\n", interface);
		

	return result;
}

static void wacom_tablet_setup_complete(struct usb_ep *ep, struct usb_request *req)
{
	
	if (req->status || req->actual != req->length)
		DBG("setup complete -->%d, %d/%d\n",
				req->status, req->actual, req->length);
							
}

static void wacom_tablet_soft_reset(struct wacom_tablet_dev *dev)
{
	//struct usb_request	*req;
	int result;
	
	DBG("Received wacom_tablet Reset Request\n");
	
	spin_lock (&dev->lock);
	
	dev->is_connect = 0;
	
	if (dev->in_ep_pen)
		usb_ep_disable(dev->in_ep_pen);	
		
	/*
	if (dev->in_ep_mouse)
		usb_ep_disable(dev->in_ep_mouse);
	*/
	
	dev->in_ep_pen->driver_data = dev;
	dev->ep_desc_pen = ep_desc(dev->gadget, NULL, &ep_in_desc_pen);
	
	//dev->in_ep_mouse->driver_data = dev;
	//dev->ep_desc_mouse = ep_desc(dev->gadget, NULL, &ep_in_desc_mouse);
	
	result = usb_ep_enable(dev->in_ep_pen, dev->ep_desc_pen);
	if (result != 0) {
		DBG("enable %s failed--> %d\n", dev->in_ep_pen->name, result);
		//dev->in_ep_pen = NULL;
	}
	
	/*
	result = usb_ep_enable(dev->in_ep_mouse, dev->ep_desc_mouse);
	if (result != 0) {
		DBG("enable %s failed--> %d\n", dev->in_ep_mouse->name, result);
		//dev->in_ep_mouse = NULL;
	}
	*/
	dev->is_connect = 1;
	
	spin_unlock (&dev->lock);
	
	wake_up_interruptible(&dev->tx_wait);
	
	DBG("wacom_tablet_soft_reset end-------------------------\n");
	
}

/*-------------------------------------------------------------------------*/

/*
 * The setup() callback implements all the ep0 functionality that's not
 * handled lower down.
 */
static int
wacom_tablet_setup(struct usb_gadget *gadget, const struct usb_ctrlrequest *ctrl)
{

	struct wacom_tablet_dev	*dev = get_gadget_data(gadget);
	struct usb_request	*req = dev->req;
	int			value = -EOPNOTSUPP;
	u16			wIndex = le16_to_cpu(ctrl->wIndex);
	u16			wValue = le16_to_cpu(ctrl->wValue);
	u16			wLength = le16_to_cpu(ctrl->wLength);
	bool		bend = 0;

	//DBG(dev, "ctrl req%02x.%02x v%04x i%04x l%d\n",
	//	ctrl->bRequestType, ctrl->bRequest, wValue, wIndex, wLength);
		
	req->complete = wacom_tablet_setup_complete;

	switch (ctrl->bRequest) {
		
		case USB_REQ_GET_DESCRIPTOR:
			DBG("USB_REQ_GET_DESCRIPTOR\n");
			if (!(ctrl->bRequestType & USB_DIR_IN))
				goto unknown;
			
			switch (wValue >> 8) {
				case USB_DT_DEVICE:
					DBG("USB_DT_DEVICE\n");
					value = min(wLength, (u16) sizeof device_desc);
					memcpy(req->buf, &device_desc, value);
					break;
					
				case USB_DT_CONFIG:
					DBG("USB_DT_CONFIG\n");
					value = wacom_tablet_config_buf(gadget->speed, req->buf,
							USB_DT_CONFIG,
							wValue & 0xff,
							gadget->is_otg);
					if (value >= 0)
						value = min(wLength, (u16) value);
					break;
					
				case USB_DT_STRING:
					DBG("USB_DT_STRING\n");
					value = usb_gadget_get_string(&stringtab,
							wValue & 0xff, req->buf);
					if (value >= 0)
						value = min(wLength, (u16) value);
					break;
					
				case HID_DEVICET_REPORT:
					DBG("HID_DEVICET_REPORT\n");
					if (!(ctrl->bRequestType & USB_RECIP_INTERFACE))
						goto unknown;
					value = min(wLength, (u16) sizeof report_desc);
					memcpy(req->buf, &report_desc, (u16)sizeof report_desc); 
					break;
			}
			break;
			
		case USB_REQ_SET_CONFIGURATION: //USB_REQ_SET_REPORT
			DBG("USB_REQ_SET_CONFIGURATION\n");
			if (ctrl->bRequestType == (USB_TYPE_CLASS | USB_RECIP_INTERFACE))
			{
				DBG("USB_REQ_SET_REPORT, wValue-->%d, wIndex-->%d, wLength-->%d\n", 
					wValue, wIndex, wLength);
					
				switch(wValue & 0xff)
				{
					case REPORTID_MOUSE:
					case REPORTID_PEN:
						spin_lock (&dev->lock);
						dev->report_id = wValue & 0xff;
						spin_unlock(&dev->lock);
						*((u8 *)req->buf + 0) = (u8)(wValue & 0xff);
						*((u8 *)req->buf + 1) = (u8)(wValue & 0xff);
						value = min(wLength, (u16)2);
						bend = 1;
						break;
					
					default:
						goto unknown;
				}
			}
			else if (ctrl->bRequestType == 0)
			{	
				if (gadget->a_hnp_support)
					DBG("HNP available\n");
				else if (gadget->a_alt_hnp_support)
					DBG("HNP needs a different root port\n");
				else
					DBG("HNP inactive\n");
				value = wacom_tablet_set_config(dev, wValue);
			}
			else
				goto unknown;
				
			break;
			
		case USB_REQ_GET_CONFIGURATION:
			DBG("USB_REQ_GET_CONFIGURATION\n");
			if (ctrl->bRequestType != USB_DIR_IN)
				goto unknown;
			*(u8 *)req->buf = dev->config;
			value = min(wLength, (u16) 1);
			break;

		case USB_REQ_SET_INTERFACE:
			DBG("USB_REQ_SET_INTERFACE\n");
			if (ctrl->bRequestType != USB_RECIP_INTERFACE)
				goto unknown;
			
			if(!dev->config)
			{
				DBG("unknown config\n");
				break;
			}
			
			value = wacom_tablet_set_interface(dev, dev->config);
			break;
			
		case USB_REQ_GET_INTERFACE:
			DBG("USB_REQ_GET_INTERFACE\n");
			if (ctrl->bRequestType != (USB_DIR_IN|USB_RECIP_INTERFACE))
				goto unknown;
			
			if(!dev->interface)
			{
				DBG("unknown interface\n");
				break;
			}
			
			*(u8 *)req->buf = dev->interface;
			value = min(wLength, (u16) 1);
			break;
		/*	
		case USB_REQ_SET_REPORT:
			DBG("USB_REQ_SET_REPORT\n");
			if (ctrl->bRequestType != (USB_TYPE_CLASS | USB_RECIP_INTERFACE))
				goto unknown;
			break;
		*/
			
		case USB_REQ_GET_REPORT:
			DBG("USB_REQ_GET_REPORT, wValue-->%d, wIndex-->%d, wLength-->%d\n", 
					wValue, wIndex, wLength);
			if (!(ctrl->bRequestType & (USB_TYPE_CLASS | USB_RECIP_INTERFACE)))
				goto unknown;
			*((u16 *)req->buf + 0) = (wValue & 0xff00) >> 8;
			value = min(wLength, (u16)2);
			break;

		default:
unknown:
			DBG("unknown ctrl req-->%02x, type-->%02x, value-->%04x, index-->%04x len-->%d\n",
				ctrl->bRequest, ctrl->bRequestType, wValue, wIndex, wLength);
			break;
		}

	/* respond with data transfer before status phase? */
	if (value >= 0) {
		req->length = value;
		req->zero = value < wLength; 
		value = usb_ep_queue(gadget->ep0, req, GFP_ATOMIC);
		if (value < 0) {
			DBG("ep_queue failed --> %d\n", value);
			req->status = 0;
			wacom_tablet_setup_complete(gadget->ep0, req);
		}
	}
	
	if(value >= 0 && bend)
	{
		char buf[9] = {0} ;
		return wacom_start_tx(dev, buf);
	}
	
	DBG("\n");
	/* host either stalls (value < 0) or reports success */
	return value;
}

static void
wacom_tablet_disconnect(struct usb_gadget *gadget)
{
	struct wacom_tablet_dev	*dev = get_gadget_data(gadget);
	unsigned long		flags;
	struct usb_request	*req;
	struct usb_ep		*in_ep;
	
	//in_ep = (dev->config == CONFIG_PEN) ? dev->in_ep_pen : dev->in_ep_mouse ;
	in_ep = dev->in_ep_pen;
	
	DBG("wacom_tablet_disconnect begin---------------\n");
	
	//wac_set_gadget_dev(NULL);

	disable_wacom_tablet_interface(dev);
	
	spin_lock_irqsave(&dev->lock, flags);
	
	dev->report_id = REPORTID_MOUSE;
	
	/* Free all memory for this driver. */
	while (!list_empty(&dev->tx_reqs)) {
		req = container_of(dev->tx_reqs.next, struct usb_request,
				list);
		list_del(&req->list);
		wacom_tablet_req_free(in_ep, req);
	}
	
	spin_unlock_irqrestore(&dev->lock, flags);
	
	DBG("wacom_tablet_disconnect end---------------\n");
}

static void
wacom_tablet_unbind(struct usb_gadget *gadget)
{
	struct wacom_tablet_dev	*dev = get_gadget_data(gadget);
	int    bempty ;

	DBG("wacom_tablet_unbind begin---------------\n");
	
	wac_set_gadget_dev(NULL);
	
	kthread_stop(dev->tablet_task);

	if (dev->req) {
		wacom_tablet_req_free(gadget->ep0, dev->req);
		dev->req = NULL;
	}

	set_gadget_data(gadget, NULL);
	
	DBG("wacom_tablet_unbind end---------------\n");
}

static int __init
wacom_tablet_bind(struct usb_gadget *gadget)
{
	struct wacom_tablet_dev	*dev;
	struct usb_ep		*in_ep_pen, *in_ep_mouse;
	int			status = -ENOMEM;
	int			gcnum;
	size_t			len;
	int			error;
	
	DBG("wacom_tablet_bind begin---------------\n");
	
	dev = &g_wacom_tablet;
	//dev->wac_dev = &g_wac_dev;
	
	spin_lock_init(&dev->lock);
	INIT_LIST_HEAD(&dev->tx_reqs);
	init_waitqueue_head(&dev->tx_wait);
	dev->is_connect = 0;
	
	if (gadget_is_sa1100(gadget)) {
		/* hardware can't write zero length packets. */
		ERROR("SA1100 controller is unsupport by this driver\n");
		goto fail;
	}

	gcnum = usb_gadget_controller_number(gadget);
	if (gcnum >= 0) {
		device_desc.bcdDevice = cpu_to_le16(0x0110 + gcnum);//0x0200
	} else {
		WARNING("controller '%s' not recognized\n",
			gadget->name);
		/* unrecognized, but safe unless bulk is REALLY quirky */
		device_desc.bcdDevice =
			cpu_to_le16(0xFFFF);
	}

	device_desc.idVendor =
		cpu_to_le16(DRIVER_VENDOR_NUM);
	device_desc.idProduct =
		cpu_to_le16(DRIVER_PRODUCT_NUM);
	
	len = strlen(pnp_string);
	pnp_string[0] = (len >> 8) & 0xFF;
	pnp_string[1] = len & 0xFF;
	
	usb_ep_autoconfig_reset(gadget);
	
	/*set pen endpoint */
	in_ep_pen = usb_ep_autoconfig(gadget, &ep_in_desc_pen);
	if (!in_ep_pen) {
		dev_err(&gadget->dev, "can't autoconfigure on %s\n",
			gadget->name);
		return -ENODEV;
	}
	in_ep_pen->driver_data = in_ep_pen;	/* claim */

	dev->in_ep_pen = in_ep_pen;

	device_desc.bMaxPacketSize0 = gadget->ep0->maxpacket;
	
	if (gadget->is_otg) {
		otg_desc.bmAttributes |= USB_OTG_HNP,
		config_desc_pen.bmAttributes |= USB_CONFIG_ATT_WAKEUP;
		//config_desc_mouse.bmAttributes |= USB_CONFIG_ATT_WAKEUP;
	}
	
	usb_gadget_set_selfpowered(gadget);

	dev->config = 0;
	dev->interface = 0;
	dev->wacom_tablet_status = WACOM_TABLET_NOT_ERROR;
	
	/* preallocate control message data and buffer */
	dev->req = wacom_tablet_req_alloc(gadget->ep0, USB_DESC_BUFSIZE,
			GFP_KERNEL);
	if (!dev->req) {
		status = -ENOMEM;
		goto fail;
	}

	dev->req->complete = wacom_tablet_setup_complete;

	/* finish hookup to lower layer ... */
	dev->gadget = gadget;
	set_gadget_data(gadget, dev);
	gadget->ep0->driver_data = dev;
	
	dev->report_id = REPORTID_MOUSE;
	dev->start_tx = wacom_start_tx;
	
	/*init process thread*/
	dev->tablet_task = kthread_run(tablet_thread, (void*)dev, "keTablet");
	if (IS_ERR(dev->tablet_task)) {
		error = PTR_ERR(dev->tablet_task);
		printk(KERN_ERR "wacom_tablet: Failed to start keTablet, error: %d\n", error);
		goto fail;
	}
	
	if(wac_set_gadget_dev(dev))
	{
		ERROR("attach gadget device to touchwacom failed.\n");
		goto fail;
	}
	
	INFO("%s, version: " DRIVER_VERSION "\n", driver_desc);
	INFO("using %s, IN %s \n", gadget->name, in_ep_pen->name);
	
	DBG("wacom_tablet_bind end---------------\n");
	
	return 0;

fail:
	wacom_tablet_unbind(gadget);
	return status;
}

/*-------------------------------------------------------------------------*/

static struct usb_gadget_driver wacom_tablet_driver = {
	.speed		= DEVSPEED,

	.function	= (char *) driver_desc,
	.bind		= wacom_tablet_bind,
	.unbind		= wacom_tablet_unbind,

	.setup		= wacom_tablet_setup,
	.disconnect	= wacom_tablet_disconnect,

	.driver		= {
		.name		= (char *) shortname,
		.owner		= THIS_MODULE,
	},
};

MODULE_DESCRIPTION(DRIVER_DESC);
MODULE_AUTHOR("Touch Down");
MODULE_LICENSE("GPL");

static int __init
init(void)
{
	int status;
	
	DBG("__init begin---------------\n");

	usb_gadget_class = class_create(THIS_MODULE, shortname);
	if (IS_ERR(usb_gadget_class)) {
		status = PTR_ERR(usb_gadget_class);
		ERROR("unable to create wacom_tablet class %d\n", status);
		return status;
	}

	status = usb_gadget_register_driver(&wacom_tablet_driver);
	if (status) {
		class_destroy(usb_gadget_class);
		DBG("wacom_tablet_register_driver %x\n", status);
	}

	DBG("__init end---------------\n");
	
	return status;
}
module_init(init);

static void __exit
cleanup(void)
{
	int 			status;
	
	DBG("cleanup begin---------------\n");
	
	class_destroy(usb_gadget_class);
	
	//serio_unregister_driver(&wac_drv);
	status = usb_gadget_unregister_driver(&wacom_tablet_driver);
	if (status)
		ERROR("wacom_tablet_unregister_driver %x\n", status);

	
	DBG("cleanup end---------------\n");
}
module_exit(cleanup);
