/*
 * g_printer.h -- Header file for MOUSE Printer gadget driver
 *
 * Copyright (C) 2007 Craig W. Nadler
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __LINUX_MOUSE_G_WACOM_TABLET_H
#define __LINUX_MOUSE_G_WACOM_TABLET_H

/*-------------------------------------------------------------------------*/
/* USB_DT_HID: HID descriptor */
struct usb_hid_descriptor {
	__u8  bLength;
	__u8  bDescriptorType;
	__le16 bcdHID;
	__u8  bCountyCode;
	__u8	bNumDescriptors;
	__u8	bSubDescriptorType;
	__u16	bDescriptorLength;

} __attribute__ ((packed));

struct wacom_tablet_dev {
	spinlock_t		lock;		/* lock this structure */
	struct usb_gadget	*gadget;
	struct usb_request	*req;		/* for control responses */
	u8			config;
	s8			interface;
	u16			report_id;
	
	bool		is_connect;
	
	struct usb_ep		*in_ep_pen;
	//struct usb_ep		*in_ep_mouse;
	const struct usb_endpoint_descriptor *ep_desc_pen;
	//const struct usb_endpoint_descriptor *ep_desc_mouse;	
	
	struct task_struct *tablet_task;
	
	struct list_head	tx_reqs;	/* List of free TX structs */
	wait_queue_head_t	tx_wait;		

	u8					wacom_tablet_status;
	void		    	*wac_dev;
	int (*start_tx)(struct wacom_tablet_dev *dev, unsigned char *buf);
};
/*-------------------------------------------------------------------------*/


#define WACOM_TABLET_NOT_ERROR	0x08
#define WACOM_TABLET_SELECTED	0x10
#define WACOM_TABLET_PAPER_EMPTY	0x20

/* The 'g' code is also used by gadgetfs ioctl requests.
 * Don't add any colliding codes to either driver, and keep
 * them in unique ranges (size 0x20 for now).
 */
#define GET_WACOM_TABLET_STATUS	_IOR('g', 0x21, unsigned char)
#define SET_WACOM_TABLET_STATUS	_IOWR('g', 0x22, unsigned char)

#endif /* __LINUX_MOUSE_G_WACOM_TABLET_H */
