/*
 * rt5610.c  --  RT5610 ALSA Soc Audio driver
 *
 * Copyright 2008 Realtek Microelectronics
 *
 * Author: flove <flove@realtek.com>
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */
 
#include <linux/init.h>
#include <linux/module.h> 
#include <linux/fs.h>
#include <linux/proc_fs.h>
#include <linux/device.h>
#include <sound/driver.h>
#include <sound/core.h>
#include <sound/pcm.h>
#include <sound/ac97_codec.h>
#include <sound/initval.h>
#include <sound/pcm_params.h>
#include <sound/soc.h>
#include <sound/soc-dapm.h>
#include "rt5611.h"

#include <asm/irq.h>
#include <linux/interrupt.h>
#include <asm-arm/arch-pxa/gpio.h>
#include <asm-arm/arch-pxa/pxa3xx_gpio.h>
#include <linux/irq.h>
#include <asm/delay.h>
#include <linux/signal.h>
#include <linux/msgevent.h> 

#include <linux/delay.h>
#include <asm/arch/zylonite.h>
#define RT5611_VERSION "0.01"
unsigned int gpio_val_bak=-1;
unsigned int audio_jack_state=-1;

struct rt5611_priv {
	u32 pll_in; /* PLL input frequency */
	u32 pll_out; /* PLL output frequency */
};

static unsigned int ac97_read(struct snd_soc_codec *codec,
	unsigned int reg);
static int ac97_write(struct snd_soc_codec *codec,
	unsigned int reg, unsigned int val);

static int ac97_write_mask(struct snd_soc_codec *codec,
	unsigned int reg, unsigned int val, unsigned int mask);
/*
 * RT5610 register cache
 * Reg 0x3c bit 15 is used by touch driver.
 */
static const u16 rt5611_reg[] = {
	0x59b4, 0x8080, 0x8080, 0x0000, // 6
	0xc880, 0xe808, 0xeC0C, 0x0808, // e
	0xe0e0, 0xf58b, 0x7f7f, 0x0000, // 16
	0xe800, 0x0000, 0x0000, 0x0000, // 1e
	0x0000, 0x0000, 0x0000, 0xef00, // 26
	0x0000, 0x0000, 0xbb80, 0x0000, // 2e
	0x0000, 0xbb80, 0x0000, 0x0000, // 36
	0x0000, 0x0000, 0x0000, 0x0000, // 3e
	0x0428, 0x0000, 0x0000, 0x0000, // 46
	0x0000, 0x0000, 0x2e3e, 0x2e3e, // 4e
	0x0000, 0x0000, 0x003a, 0x0000, // 56
	0x0cff, 0x0000, 0x0000, 0x0000, // 5e
	0x0000, 0x0000, 0x2130, 0x0010, // 66
	0x0053, 0x0000, 0x0000, 0x0000, // 6e
	0x0000, 0x0000, 0x008c, 0x3f00, // 76
	0x0000, 0x0000, 0x10ec, 0x1003, // 7e
	0x0000, 0x0000, 0x0000 // virtual hp & mic mixers
};

/* virtual HP mixers regs */
#define HPL_MIXER	0x80
#define HPR_MIXER	0x82
#define MICB_MUX	0x82


u16 Set_Codec_Reg_Init[][2]={
	
	{RT5611_SPK_OUT_VOL		,0x8080},//default speaker volume to 0db 
	{RT5611_HP_OUT_VOL		,0x8888},//default HP volume to 0db
	{RT5611_ADC_REC_MIXER	,0x3F3F},//default Record is Mic1
	{RT5611_STEREO_DAC_VOL	,0x0808},//default stereo DAC volume to 0db
	{RT5611_MIC_CTRL		,0x0500},//set boost to +20DB
	{RT5611_INDEX_ADDRESS 	,0x0054},//AD_DA_Mixer_internal Register5
	{RT5611_INDEX_DATA		,0xE184},//To reduce power consumption for DAC reference
	{RT5611_TONE_CTRL		,0x0001},//Enable varible sample rate	
	{RT5611_OUTPUT_MIXER_CTRL,0x93C0},//default output mixer control,CLASS AB
	
	{RT5611_OUTPUT_MIXER_CTRL,0xB3C0},//default output mixer control,CLASS D
};
u16 Set_Codec_Reg_EQ[][2]={	
	//for speaker improvement
	{RT5611_INDEX_ADDRESS, 0x0000},
	{RT5611_INDEX_DATA, 0x1CD4},
	
	{RT5611_INDEX_ADDRESS, 0x0001},
	{RT5611_INDEX_DATA, 0xF405},

	{RT5611_INDEX_ADDRESS, 0x0002},
	{RT5611_INDEX_DATA, 0xC115},

	{RT5611_INDEX_ADDRESS, 0x0003},
	{RT5611_INDEX_DATA, 0x1EF8},

	{RT5611_INDEX_ADDRESS, 0x0004},
	{RT5611_INDEX_DATA, 0x1D18},

	{RT5611_INDEX_ADDRESS, 0x0005},
	{RT5611_INDEX_DATA, 0xC278},

	{RT5611_INDEX_ADDRESS, 0x0006},
	{RT5611_INDEX_DATA, 0x1DF6},

	{RT5611_INDEX_ADDRESS, 0x0007},
	{RT5611_INDEX_DATA, 0xF65F},

	{RT5611_INDEX_ADDRESS, 0x0008},
	{RT5611_INDEX_DATA, 0xC433},

	{RT5611_INDEX_ADDRESS, 0x0009},
	{RT5611_INDEX_DATA, 0x1E5E},

	{RT5611_INDEX_ADDRESS, 0x000a},
	{RT5611_INDEX_DATA, 0xF805},

	{RT5611_INDEX_ADDRESS, 0x000b},
	{RT5611_INDEX_DATA, 0xFDE8},

	{RT5611_INDEX_ADDRESS, 0x000c},
	{RT5611_INDEX_DATA, 0x0FEC},

	{RT5611_INDEX_ADDRESS, 0x0010},
	{RT5611_INDEX_DATA, 0x801F},

	{RT5611_INDEX_ADDRESS, 0x0011},
	{RT5611_INDEX_DATA, 0x0000},

	{RT5611_INDEX_ADDRESS, 0x0012},
	{RT5611_INDEX_DATA, 0x0001},
};

static const char *rt5611_spkl_pga[] = {"Vmid","HPL mixer","SPK mixer","Mono Mixer"};
static const char *rt5611_spkr_pga[] = {"Vmid","HPR mixer","SPK mixer","Mono Mixer"};
static const char *rt5611_hpl_pga[]  = {"Vmid","HPL mixer"};
static const char *rt5611_hpr_pga[]  = {"Vmid","HPR mixer"};
static const char *rt5611_mono_pga[] = {"Vmid","HP mixer","SPK mixer","Mono Mixer"};
static const char *rt5611_amp_type_select[] = {"Class AB","Class D"};
static const char *rt5611_mic_boost_select[] = {"Bypass","20db","30db","40db"};


static const struct soc_enum rt5611_enum[] = {
SOC_ENUM_SINGLE(RT5611_OUTPUT_MIXER_CTRL, 14, 4, rt5611_spkl_pga), /* spk left input sel 0 */	
SOC_ENUM_SINGLE(RT5611_OUTPUT_MIXER_CTRL, 11, 4, rt5611_spkr_pga), /* spk right input sel 1 */	
SOC_ENUM_SINGLE(RT5611_OUTPUT_MIXER_CTRL, 9, 2, rt5611_hpl_pga), /* hp left input sel 2 */	
SOC_ENUM_SINGLE(RT5611_OUTPUT_MIXER_CTRL, 8, 2, rt5611_hpr_pga), /* hp right input sel 3 */	
SOC_ENUM_SINGLE(RT5611_OUTPUT_MIXER_CTRL, 6, 4, rt5611_mono_pga), /* mono input sel 4 */
SOC_ENUM_SINGLE(RT5611_MIC_CTRL, 10,4, rt5611_mic_boost_select), /*Mic1 boost sel 5 */
SOC_ENUM_SINGLE(RT5611_MIC_CTRL, 8,4, rt5611_mic_boost_select), /*Mic2 boost sel 6 */
SOC_ENUM_SINGLE(RT5611_OUTPUT_MIXER_CTRL, 12,2,rt5611_amp_type_select), /*Speaker AMP sel 7 */
};

static const struct snd_kcontrol_new rt5611_snd_ac97_controls[] = {
SOC_DOUBLE("Speaker Playback Volume", 	RT5611_SPK_OUT_VOL, 8, 0, 31, 1),	
SOC_DOUBLE("Speaker Playback Switch", 	RT5611_SPK_OUT_VOL, 15, 7, 1, 1),
SOC_DOUBLE("Headphone Playback Volume", RT5611_HP_OUT_VOL, 8, 0, 31, 1),
SOC_DOUBLE("Headphone Playback Switch", RT5611_HP_OUT_VOL,15, 7, 1, 1),
SOC_SINGLE("Mono Playback Volume", 		RT5611_PHONEIN_MONO_OUT_VOL, 0, 31, 1),
SOC_SINGLE("Mono Playback Switch", 		RT5611_PHONEIN_MONO_OUT_VOL, 7, 1, 1),
SOC_DOUBLE("PCM Playback Volume", 		RT5611_STEREO_DAC_VOL, 8, 0, 31, 1),
SOC_DOUBLE("PCM Playback Switch", 		RT5611_STEREO_DAC_VOL,15, 7, 1, 1),
SOC_DOUBLE("Line In Volume", 			RT5611_LINE_IN_VOL, 8, 0, 31, 1),
SOC_SINGLE("Mic 1 Volume", 				RT5611_MIC_VOL, 8, 31, 1),
SOC_SINGLE("Mic 2 Volume", 				RT5611_MIC_VOL, 0, 31, 1),
SOC_ENUM("Mic 1 Boost", 				rt5611_enum[5]),
SOC_ENUM("Mic 2 Boost", 				rt5611_enum[6]),
SOC_ENUM("Speaker AMP", 				rt5611_enum[7]),
SOC_SINGLE("Phone In Volume", 			RT5611_PHONEIN_MONO_OUT_VOL, 8, 31, 1),
SOC_DOUBLE("Capture Volume", 			RT5611_ADC_REC_GAIN, 7, 0, 31, 0),
	};


/* add non dapm controls */
static int rt5611_add_controls(struct snd_soc_codec *codec)
{
	int err, i;

	for (i = 0; i < ARRAY_SIZE(rt5611_snd_ac97_controls); i++) {
		err = snd_ctl_add(codec->card,
				snd_soc_cnew(&rt5611_snd_ac97_controls[i],
				codec, NULL));
		if (err < 0)
			return err;
	}

	return 0;
}

/* We have to create a fake left and right HP mixers because
 * the codec only has a single control that is shared by both channels.
 * This makes it impossible to determine the audio path using the current
 * register map, thus we add a new (virtual) register to help determine the
 * audio route within the device.
 */
static int mixer_event (struct snd_soc_dapm_widget *w, int event)
{
	u16 l, r, lineIn,mic1,mic2, phone, pcm,voice;

	l = ac97_read(w->codec, HPL_MIXER);
	r = ac97_read(w->codec, HPR_MIXER);
	lineIn = ac97_read(w->codec, RT5611_LINE_IN_VOL);
	mic1 = ac97_read(w->codec, RT5611_MIC_ROUTING_CTRL);
	mic2 = ac97_read(w->codec, RT5611_MIC_ROUTING_CTRL);
	phone = ac97_read(w->codec,RT5611_PHONEIN_MONO_OUT_VOL);
	pcm = ac97_read(w->codec, RT5611_STEREO_DAC_VOL);
	voice = ac97_read(w->codec, RT5611_VOICE_DAC_OUT_VOL);

	if (event & SND_SOC_DAPM_PRE_REG)
		return 0;
	if (l & 0x1 || r & 0x1)
		ac97_write(w->codec, RT5611_VOICE_DAC_OUT_VOL, voice & 0x7fff);
	else
		ac97_write(w->codec, RT5611_VOICE_DAC_OUT_VOL, voice | 0x8000);

	if (l & 0x2 || r & 0x2)
		ac97_write(w->codec, RT5611_STEREO_DAC_VOL, pcm & 0x7fff);
	else
		ac97_write(w->codec, RT5611_STEREO_DAC_VOL, pcm | 0x8000);

	if (l & 0x4 || r & 0x4)
		ac97_write(w->codec, RT5611_MIC_ROUTING_CTRL, mic2 & 0xf7ff);
	else
		ac97_write(w->codec, RT5611_MIC_ROUTING_CTRL, mic2 | 0x0800);

	if (l & 0x8 || r & 0x8)
		ac97_write(w->codec, RT5611_MIC_ROUTING_CTRL, mic1 & 0x7fff);
	else
		ac97_write(w->codec, RT5611_MIC_ROUTING_CTRL, mic1 | 0x8000);

	if (l & 0x10 || r & 0x10)
		ac97_write(w->codec, RT5611_PHONEIN_MONO_OUT_VOL, phone & 0x7fff);
	else
		ac97_write(w->codec, RT5611_PHONEIN_MONO_OUT_VOL, phone | 0x8000);

	if (l & 0x20 || r & 0x20)
		ac97_write(w->codec, RT5611_LINE_IN_VOL, lineIn & 0x7fff);
	else
		ac97_write(w->codec, RT5611_LINE_IN_VOL, lineIn | 0x8000);

	return 0;
}

/* Left Headphone Mixers */
static const struct snd_kcontrol_new rt5611_hpl_mixer_controls[] = {
SOC_DAPM_SINGLE("LineIn Playback Switch", HPL_MIXER, 5, 1, 0),
SOC_DAPM_SINGLE("PhoneIn Playback Switch", HPL_MIXER, 4, 1, 0),
SOC_DAPM_SINGLE("Mic1 Playback Switch", HPL_MIXER, 3, 1, 0),
SOC_DAPM_SINGLE("Mic2 Playback Switch", HPL_MIXER, 2, 1, 0),
SOC_DAPM_SINGLE("PCM Playback Switch", HPL_MIXER, 1, 1, 0),
SOC_DAPM_SINGLE("Voice Playback Switch", HPL_MIXER, 0, 1, 0),
SOC_DAPM_SINGLE("RecordL Playback Switch", RT5611_ADC_REC_GAIN, 15, 1,1),
};


/* Right Headphone Mixers */
static const struct snd_kcontrol_new rt5611_hpr_mixer_controls[] = {
SOC_DAPM_SINGLE("LineIn Playback Switch", HPR_MIXER, 5, 1, 0),
SOC_DAPM_SINGLE("PhoneIn Playback Switch", HPR_MIXER, 4, 1, 0),
SOC_DAPM_SINGLE("Mic1 Playback Switch", HPR_MIXER, 3, 1, 0),
SOC_DAPM_SINGLE("Mic2 Playback Switch", HPR_MIXER, 2, 1, 0),
SOC_DAPM_SINGLE("PCM Playback Switch", HPR_MIXER, 1, 1, 0),
SOC_DAPM_SINGLE("Voice Playback Switch", HPR_MIXER, 0, 1, 0),
SOC_DAPM_SINGLE("RecordR Playback Switch", RT5611_ADC_REC_GAIN, 14, 1,1),
};


static const struct snd_kcontrol_new rt5611_captureL_mixer_controls[] = {
SOC_DAPM_SINGLE("Mic1 Capture Switch", RT5611_ADC_REC_MIXER, 14, 1, 1),
SOC_DAPM_SINGLE("Mic2 Capture Switch", RT5611_ADC_REC_MIXER, 13, 1, 1),
SOC_DAPM_SINGLE("LineInL Capture Switch",RT5611_ADC_REC_MIXER,12, 1, 1),
SOC_DAPM_SINGLE("Phone Capture Switch", RT5611_ADC_REC_MIXER, 11, 1, 1),
SOC_DAPM_SINGLE("HPMixerL Capture Switch", RT5611_ADC_REC_MIXER,10, 1, 1),
SOC_DAPM_SINGLE("SPKMixer Capture Switch",RT5611_ADC_REC_MIXER,9, 1, 1),
SOC_DAPM_SINGLE("MonoMixer Capture Switch",RT5611_ADC_REC_MIXER,8, 1, 1),
};


static const struct snd_kcontrol_new rt5611_captureR_mixer_controls[] = {
SOC_DAPM_SINGLE("Mic1 Capture Switch", RT5611_ADC_REC_MIXER, 6, 1, 1),
SOC_DAPM_SINGLE("Mic2 Capture Switch", RT5611_ADC_REC_MIXER, 5, 1, 1),
SOC_DAPM_SINGLE("LineInR Capture Switch",RT5611_ADC_REC_MIXER,4, 1, 1),
SOC_DAPM_SINGLE("Phone Capture Switch", RT5611_ADC_REC_MIXER, 3, 1, 1),
SOC_DAPM_SINGLE("HPMixer Capture Switch", RT5611_ADC_REC_MIXER,2, 1, 1),
SOC_DAPM_SINGLE("SPKMixer Capture Switch",RT5611_ADC_REC_MIXER,1, 1, 1),
SOC_DAPM_SINGLE("MonoMixer Capture Switch",RT5611_ADC_REC_MIXER,0, 1, 1),
};


/* Speaker Mixer */
static const struct snd_kcontrol_new rt5611_speaker_mixer_controls[] = {
SOC_DAPM_SINGLE("LineIn Playback Switch", RT5611_LINE_IN_VOL, 14, 1, 1),
SOC_DAPM_SINGLE("PhoneIn Playback Switch", RT5611_PHONEIN_MONO_OUT_VOL, 14, 1, 1),
SOC_DAPM_SINGLE("Mic1 Playback Switch", RT5611_MIC_ROUTING_CTRL, 14, 1, 1),
SOC_DAPM_SINGLE("Mic2 Playback Switch", RT5611_MIC_ROUTING_CTRL, 6, 1, 1),
SOC_DAPM_SINGLE("PCM Playback Switch", RT5611_STEREO_DAC_VOL, 14, 1, 1),
SOC_DAPM_SINGLE("Voice Playback Switch", RT5611_VOICE_DAC_OUT_VOL, 14, 1, 1),
};


/* Mono Mixer */
static const struct snd_kcontrol_new rt5611_mono_mixer_controls[] = {
SOC_DAPM_SINGLE("LineIn Playback Switch", RT5611_LINE_IN_VOL, 13, 1, 1),
SOC_DAPM_SINGLE("Mic1 Playback Switch", RT5611_MIC_ROUTING_CTRL, 13, 1, 1),
SOC_DAPM_SINGLE("Mic2 Playback Switch", RT5611_MIC_ROUTING_CTRL, 5, 1, 1),
SOC_DAPM_SINGLE("PCM Playback Switch", RT5611_STEREO_DAC_VOL, 13, 1, 1),
SOC_DAPM_SINGLE("RecL Playback Switch", RT5611_ADC_REC_GAIN, 13, 1, 1),
SOC_DAPM_SINGLE("RecR Playback Switch", RT5611_ADC_REC_GAIN, 12, 1, 1),
SOC_DAPM_SINGLE("Voice Playback Switch", RT5611_VOICE_DAC_OUT_VOL, 13, 1, 1),
SOC_DAPM_SINGLE("RecordL Playback Switch", RT5611_ADC_REC_GAIN, 13, 1,1),
SOC_DAPM_SINGLE("RecordR Playback Switch", RT5611_ADC_REC_GAIN, 12, 1,1),
};

/* mono output mux */
static const struct snd_kcontrol_new rt5611_mono_mux_controls =
SOC_DAPM_ENUM("Route", rt5611_enum[4]);


/* speaker left output mux */
static const struct snd_kcontrol_new rt5611_hp_spkl_mux_controls =
SOC_DAPM_ENUM("Route", rt5611_enum[0]);


/* speaker right output mux */
static const struct snd_kcontrol_new rt5611_hp_spkr_mux_controls =
SOC_DAPM_ENUM("Route", rt5611_enum[1]);


/* headphone left output mux */
static const struct snd_kcontrol_new rt5611_hpl_out_mux_controls =
SOC_DAPM_ENUM("Route", rt5611_enum[2]);


/* headphone right output mux */
static const struct snd_kcontrol_new rt5611_hpr_out_mux_controls =
SOC_DAPM_ENUM("Route", rt5611_enum[3]);


static const struct snd_soc_dapm_widget rt5611_dapm_widgets[] = {
SND_SOC_DAPM_MUX("Mono Out Mux", SND_SOC_NOPM, 0, 0,
	&rt5611_mono_mux_controls),
SND_SOC_DAPM_MUX("Left Speaker Out Mux", SND_SOC_NOPM, 0, 0,
	&rt5611_hp_spkl_mux_controls),
SND_SOC_DAPM_MUX("Right Speaker Out Mux", SND_SOC_NOPM, 0, 0,
	&rt5611_hp_spkr_mux_controls),
SND_SOC_DAPM_MUX("Left Headphone Out Mux", SND_SOC_NOPM, 0, 0,
	&rt5611_hpl_out_mux_controls),
SND_SOC_DAPM_MUX("Right Headphone Out Mux", SND_SOC_NOPM, 0, 0,
	&rt5611_hpr_out_mux_controls),
SND_SOC_DAPM_MIXER_E("Left HP Mixer",RT5611_PWR_MANAG_ADD2, 5, 0,
	&rt5611_hpl_mixer_controls[0], ARRAY_SIZE(rt5611_hpl_mixer_controls),
	mixer_event, SND_SOC_DAPM_POST_REG),
SND_SOC_DAPM_MIXER_E("Right HP Mixer",RT5611_PWR_MANAG_ADD2, 4, 0,
	&rt5611_hpr_mixer_controls[0], ARRAY_SIZE(rt5611_hpr_mixer_controls),
	mixer_event, SND_SOC_DAPM_POST_REG),
SND_SOC_DAPM_MIXER("Mono Mixer", RT5611_PWR_MANAG_ADD2, 2, 0,
	&rt5611_mono_mixer_controls[0], ARRAY_SIZE(rt5611_mono_mixer_controls)),
SND_SOC_DAPM_MIXER("Speaker Mixer", RT5611_PWR_MANAG_ADD2,3,0,
	&rt5611_speaker_mixer_controls[0],
	ARRAY_SIZE(rt5611_speaker_mixer_controls)),	
SND_SOC_DAPM_MIXER("Left Record Mixer", RT5611_PWR_MANAG_ADD2,1,0,
	&rt5611_captureL_mixer_controls[0],
	ARRAY_SIZE(rt5611_captureL_mixer_controls)),	
SND_SOC_DAPM_MIXER("Right Record Mixer", RT5611_PWR_MANAG_ADD2,0,0,
	&rt5611_captureR_mixer_controls[0],
	ARRAY_SIZE(rt5611_captureR_mixer_controls)),				
SND_SOC_DAPM_DAC("Left DAC", "Left HiFi Playback", RT5611_PWR_MANAG_ADD2,9, 0),
SND_SOC_DAPM_DAC("Right DAC", "Right HiFi Playback", RT5611_PWR_MANAG_ADD2, 8, 0),	
SND_SOC_DAPM_MIXER("AC97 Mixer", SND_SOC_NOPM, 0, 0, NULL, 0),
SND_SOC_DAPM_MIXER("HP Mixer", SND_SOC_NOPM, 0, 0, NULL, 0),
SND_SOC_DAPM_ADC("Left ADC", "Left HiFi Capture", RT5611_PWR_MANAG_ADD2, 7, 0),
SND_SOC_DAPM_ADC("Right ADC", "Right HiFi Capture", RT5611_PWR_MANAG_ADD2, 6, 0),
SND_SOC_DAPM_PGA("Left Headphone", RT5611_PWR_MANAG_ADD3, 11, 1, NULL, 0),
SND_SOC_DAPM_PGA("Right Headphone", RT5611_PWR_MANAG_ADD3, 10, 1, NULL, 0),
SND_SOC_DAPM_PGA("Left Speaker", RT5611_PWR_MANAG_ADD3, 8, 1, NULL, 0),
SND_SOC_DAPM_PGA("Right Speaker", RT5611_PWR_MANAG_ADD3, 7, 1, NULL, 0),
SND_SOC_DAPM_PGA("Mono Out", RT5611_PWR_MANAG_ADD3, 13, 1, NULL, 0),
SND_SOC_DAPM_PGA("Left Line In", RT5611_PWR_MANAG_ADD3, 6, 1, NULL, 0),
SND_SOC_DAPM_PGA("Right Line In", RT5611_PWR_MANAG_ADD3, 5, 1, NULL, 0),
SND_SOC_DAPM_PGA("Phone In PGA", RT5611_PWR_MANAG_ADD3, 4, 1, NULL, 0),
SND_SOC_DAPM_PGA("Phone In Mixer", RT5611_PWR_MANAG_ADD3, 5, 1, NULL, 0),
SND_SOC_DAPM_PGA("Mic 1 PGA", RT5611_PWR_MANAG_ADD3, 3, 1, NULL, 0),
SND_SOC_DAPM_PGA("Mic 2 PGA", RT5611_PWR_MANAG_ADD3, 2, 1, NULL, 0),
SND_SOC_DAPM_PGA("Mic 1 Pre Amp", RT5611_PWR_MANAG_ADD3, 1, 1, NULL, 0),
SND_SOC_DAPM_PGA("Mic 2 Pre Amp", RT5611_PWR_MANAG_ADD3, 0, 1, NULL, 0),
SND_SOC_DAPM_MICBIAS("Mic Bias1", RT5611_PWR_MANAG_ADD1, 3, 0),
SND_SOC_DAPM_MICBIAS("Mic Bias2", RT5611_PWR_MANAG_ADD1, 2, 0),
SND_SOC_DAPM_OUTPUT("MONO"),
SND_SOC_DAPM_OUTPUT("HPL"),
SND_SOC_DAPM_OUTPUT("HPR"),
SND_SOC_DAPM_OUTPUT("SPKL"),
SND_SOC_DAPM_OUTPUT("SPKR"),
SND_SOC_DAPM_INPUT("LINEL"),
SND_SOC_DAPM_INPUT("LINER"),
SND_SOC_DAPM_INPUT("PHONEIN"),
SND_SOC_DAPM_INPUT("MIC1"),
SND_SOC_DAPM_INPUT("MIC2"),
SND_SOC_DAPM_INPUT("PCMIN"),
SND_SOC_DAPM_VMID("VMID"),
};


static const char *audio_map[][3] = {
	/* left HP mixer */
	{"Left HP Mixer", "LineIn Playback Switch", "LINEL"},
	{"Left HP Mixer", "PhoneIn Playback Switch","PHONEIN"},
	{"Left HP Mixer", "Mic1 Playback Switch","MIC1"},
	{"Left HP Mixer", "Mic2 Playback Switch","MIC2"},
	{"Left HP Mixer", "PCM Playback Switch","Left DAC"},
	{"Left HP Mixer", "Voice Playback Switch","Voice DAC"},
	{"Left HP Mixer", "RecordL Playback Switch","Left Record Mixer"},
	
	/* right HP mixer */
	{"Right HP Mixer", "LineIn Playback Switch", "LINER"},
	{"Right HP Mixer", "PhoneIn Playback Switch","PHONEIN"},
	{"Right HP Mixer", "Mic1 Playback Switch","MIC1"},
	{"Right HP Mixer", "Mic2 Playback Switch","MIC2"},
	{"Right HP Mixer", "PCM Playback Switch","Right DAC"},
	{"Right HP Mixer", "Voice Playback Switch","Voice DAC"},
	{"Right HP Mixer", "RecordR Playback Switch","Right Record Mixer"},
	
	/* virtual mixer - mixes left & right channels for spk and mono */
	{"AC97 Mixer", NULL, "Left DAC"},
	{"AC97 Mixer", NULL, "Right DAC"},
	{"Line Mixer", NULL, "Right Line In"},
	{"Line Mixer", NULL, "Left Line In"},
	{"HP Mixer", NULL, "Left HP Mixer"},
	{"HP Mixer", NULL, "Right HP Mixer"},
	
	/* speaker mixer */
	{"Speaker Mixer", "LineIn Playback Switch","Line Mixer"},
	{"Speaker Mixer", "PhoneIn Playback Switch","PHONEIN"},
	{"Speaker Mixer", "Mic1 Playback Switch","MIC1"},
	{"Speaker Mixer", "Mic2 Playback Switch","MIC2"},
	{"Speaker Mixer", "PCM Playback Switch","AC97 Mixer"},
	{"Speaker Mixer", "Voice Playback Switch","Voice DAC"},

	/* mono mixer */
	{"Mono Mixer", "LineIn Playback Switch","Line Mixer"},
	{"Mono Mixer", "PhoneIn Playback Switch","PHONEIN"},
	{"Mono Mixer", "Mic1 Playback Switch","MIC1"},
	{"Mono Mixer", "Mic2 Playback Switch","MIC2"},
	{"Mono Mixer", "PCM Playback Switch","AC97 Mixer"},
	{"Mono Mixer", "Voice Playback Switch","Voice DAC"},
	{"Mono Mixer", "RecordL Playback Switch","Left Record Mixer"},
	{"Mono Mixer", "RecordR Playback Switch","Right Record Mixer"},
	
	/*Left record mixer */
	{"Left Record Mixer", "Mic1 Capture Switch","Mic 1 Pre Amp"},
	{"Left Record Mixer", "Mic2 Capture Switch","Mic 2 Pre Amp"},
	{"Left Record Mixer", "LineInL Capture Switch","LINEL"},
	{"Left Record Mixer", "Phone Capture Switch","PHONEIN"},
	{"Left Record Mixer", "HPMixerL Capture Switch","Left HP Mixer"},
	{"Left Record Mixer", "SPKMixer Capture Switch","Speaker Mixer"},
	{"Left Record Mixer", "MonoMixer Capture Switch","Mono Mixer"},
	
	/*Right record mixer */
	{"Right Record Mixer", "Mic1 Capture Switch","Mic 1 Pre Amp"},
	{"Right Record Mixer", "Mic2 Capture Switch","Mic 2 Pre Amp"},
	{"Right Record Mixer", "LineInR Capture Switch","LINER"},
	{"Right Record Mixer", "Phone Capture Switch","PHONEIN"},
	{"Right Record Mixer", "HPMixerR Capture Switch","Right HP Mixer"},
	{"Right Record Mixer", "SPKMixer Capture Switch","Speaker Mixer"},
	{"Right Record Mixer", "MonoMixer Capture Switch","Mono Mixer"},	

	/* headphone left mux */
	{"Left Headphone Out Mux", "HPL mixer", "Left HP Mixer"},

	/* headphone right mux */
	{"Right Headphone Out Mux", "HPR mixer", "Right HP Mixer"},

	/* speaker left mux */
	{"Left Speaker Out Mux", "HPL mixer", "Left HP Mixer"},
	{"Left Speaker Out Mux", "SPK mixer", "Speaker Mixer"},
	{"Left Speaker Out Mux", "Mono Mixer", "Mono Mixer"},

	/* speaker right mux */
	{"Right Speaker Out Mux", "HPR mixer", "Right HP Mixer"},
	{"Right Speaker Out Mux", "SPK mixer", "Speaker Mixer"},
	{"Right Speaker Out Mux", "Mono Mixer", "Mono Mixer"},

	/* mono mux */
	{"Mono Out Mux", "HP mixer", "HP Mixer"},
	{"Mono Out Mux", "SPK mixer", "Speaker Mixer"},
	{"Mono Out Mux", "Mono Mixer", "Mono Mixer"},
	
	/* output pga */
	{"HPL", NULL, "Left Headphone"},
	{"Left Headphone", NULL, "Left Headphone Out Mux"},
	{"HPR", NULL, "Right Headphone"},
	{"Right Headphone", NULL, "Right Headphone Out Mux"},
	{"SPKL", NULL, "Left Speaker"},
	{"Left Speaker", NULL, "Left Speaker Out Mux"},
	{"SPKR", NULL, "Right Speaker"},
	{"Right Speaker", NULL, "Right Speaker Out Mux"},
	{"MONO", NULL, "Mono Out"},
	{"Mono Out", NULL, "Mono Out Mux"},

	/* input pga */
	{"Left Line In", NULL, "LINEL"},
	{"Right Line In", NULL, "LINER"},
	{"Phone In PGA", NULL, "PHONEIN"},
	{"Phone In Mixer", NULL, "PHONEIN"},
	{"Mic 1 Pre Amp", NULL, "MIC1"},
	{"Mic 2 Pre Amp", NULL, "MIC2"},	
	{"Mic 1 PGA", NULL, "Mic 1 Pre Amp"},
	{"Mic 2 PGA", NULL, "Mic 2 Pre Amp"},

	/* left ADC */
	{"Left ADC", NULL, "Left Record Mixer"},

	/* right ADC */
	{"Right ADC", NULL, "Right Record Mixer"},
	{NULL, NULL, NULL},	
};



static int rt5611_add_widgets(struct snd_soc_codec *codec)
{
	int i;

	for(i = 0; i < ARRAY_SIZE(rt5611_dapm_widgets); i++) {
		snd_soc_dapm_new_control(codec, &rt5611_dapm_widgets[i]);
	}

	/* set up audio path audio_mapnects */
	for(i = 0; audio_map[i][0] != NULL; i++) {
		snd_soc_dapm_connect_input(codec, audio_map[i][0],
			audio_map[i][1], audio_map[i][2]);
	}

	snd_soc_dapm_new_widgets(codec);

	return 0;
}

static unsigned int ac97_read(struct snd_soc_codec *codec,
	unsigned int reg)
{
	u16 *cache = codec->reg_cache;

	if(reg<=0x7E)
	{
		return soc_ac97_ops.read(codec->ac97, reg);
		}
	else {
		reg = reg >> 1;

		if (reg > (ARRAY_SIZE(rt5611_reg)))
			return -EIO;

		return cache[reg];
	}
}

static int ac97_write(struct snd_soc_codec *codec, unsigned int reg,
	unsigned int val)
{
	u16 *cache = codec->reg_cache;
	if (reg < 0x7c)
		soc_ac97_ops.write(codec->ac97, reg, val);
	reg = reg >> 1;
	if (reg <= (ARRAY_SIZE(rt5611_reg)))
		cache[reg] = val;

	return 0;
}

static int ac97_write_mask(struct snd_soc_codec *codec,unsigned int reg,unsigned int val,unsigned int mask)
{
	unsigned int CodecData;

	if(!mask)
		return 0; 

	if(mask!=0xffff)
	 {
		CodecData=ac97_read(codec,reg);

		CodecData&=~mask;
		CodecData|=(val&mask);
		ac97_write(codec,reg,CodecData);
	 }		
	else
	{
		ac97_write(codec,reg,val);
	}
	return 0;
	
}
int g_hasplay = 0;
static int rt5611_ChangeCodecPowerStatus(struct snd_soc_codec *codec,int power_state)
{
	unsigned short int PowerDownState=0;

	switch(power_state)
	{
		case POWER_STATE_D0:			//FULL ON-----power on all power

			ac97_write(codec,RT5611_PD_CTRL_STAT,PowerDownState);
			ac97_write(codec,RT5611_PWR_MANAG_ADD1,~PowerDownState);
			ac97_write(codec,RT5611_PWR_MANAG_ADD2,~PowerDownState);
			ac97_write(codec,RT5611_PWR_MANAG_ADD3,~PowerDownState);

		break;	

		case POWER_STATE_D1:		//LOW ON-----
			
																																
			ac97_write_mask(codec,RT5611_PWR_MANAG_ADD1,PWR_DAC_REF|PWR_MIC_BIAS1|PWR_HI_R_LOAD_HP
											   		,PWR_DAC_REF|PWR_MIC_BIAS1|PWR_HI_R_LOAD_HP);
									   
											
			ac97_write_mask(codec,RT5611_PWR_MANAG_ADD2,PWR_SPK_MIXER | PWR_R_ADC_REC_MIXER | PWR_L_ADC_REC_MIXER | PWR_R_HP_MIXER | PWR_L_HP_MIXER | PWR_R_ADC_CLK_GAIN | PWR_L_ADC_CLK_GAIN | 
												PWR_R_DAC_CLK | PWR_L_DAC_CLK  | PWR_CLASS_AB
											   ,PWR_SPK_MIXER | PWR_R_ADC_REC_MIXER | PWR_L_ADC_REC_MIXER | PWR_R_HP_MIXER | PWR_L_HP_MIXER | PWR_R_ADC_CLK_GAIN | PWR_L_ADC_CLK_GAIN | 
												PWR_R_DAC_CLK | PWR_L_DAC_CLK | PWR_CLASS_AB);

			ac97_write_mask(codec,RT5611_PWR_MANAG_ADD3,PWR_MIC1_BOOST  | PWR_SPK_R_OUT| PWR_SPK_L_OUT |PWR_HP_R_OUT | PWR_HP_L_OUT |
											    		PWR_SPK_RN_OUT|PWR_SPK_LN_OUT 
											   			,PWR_MIC1_BOOST | PWR_SPK_R_OUT| PWR_SPK_L_OUT |PWR_HP_R_OUT | PWR_HP_L_OUT	|
											    		PWR_SPK_RN_OUT|PWR_SPK_LN_OUT);								
		break;

		case POWER_STATE_D1_PLAYBACK:	//Low on of Playback

			//ac97_write_mask(codec,RT5611_PWR_MANAG_ADD1,PWR_DAC_REF | PWR_HI_R_LOAD_HP,PWR_DAC_REF | PWR_HI_R_LOAD_HP);
											
			ac97_write_mask(codec,RT5611_PWR_MANAG_ADD2,PWR_SPK_MIXER | PWR_R_HP_MIXER | PWR_L_HP_MIXER | PWR_R_DAC_CLK | PWR_L_DAC_CLK  | PWR_CLASS_AB
											   ,PWR_SPK_MIXER | PWR_R_HP_MIXER | PWR_L_HP_MIXER | PWR_R_DAC_CLK | PWR_L_DAC_CLK  | PWR_CLASS_AB);

			//ac97_write_mask(codec,RT5611_PWR_MANAG_ADD3,PWR_SPK_L_OUT|PWR_SPK_R_OUT | PWR_HP_R_OUT | PWR_HP_L_OUT|PWR_SPK_RN_OUT|PWR_SPK_LN_OUT
			//								   		   ,PWR_SPK_L_OUT|PWR_SPK_R_OUT | PWR_HP_R_OUT | PWR_HP_L_OUT|PWR_SPK_RN_OUT|PWR_SPK_LN_OUT);	
	
				if(g_hasplay == 0)
				{
				int i =0 ;
				for(i=0;i<30;i++)
				mdelay(2);
				}
			g_hasplay = 1;
			ac97_write_mask(codec,RT5611_PWR_MANAG_ADD1,PWR_DAC_REF | PWR_HI_R_LOAD_HP,PWR_DAC_REF | PWR_HI_R_LOAD_HP);
		break;

		case POWER_STATE_D1_RECORD:	//Low on of Record
			
			ac97_write_mask(codec,RT5611_PWR_MANAG_ADD1,PWR_MIC_BIAS1,PWR_MIC_BIAS1);
											//
			ac97_write_mask(codec,RT5611_PWR_MANAG_ADD2,PWR_R_ADC_REC_MIXER | PWR_L_ADC_REC_MIXER | PWR_R_ADC_CLK_GAIN | PWR_L_ADC_CLK_GAIN												
											   ,PWR_R_ADC_REC_MIXER | PWR_L_ADC_REC_MIXER | PWR_R_ADC_CLK_GAIN | PWR_L_ADC_CLK_GAIN);

			ac97_write_mask(codec,RT5611_PWR_MANAG_ADD3,PWR_MIC1_BOOST ,PWR_MIC1_BOOST);	
		
		break;

		case POWER_STATE_D2:		//STANDBY----
											//																								
			ac97_write_mask(codec,RT5611_PWR_MANAG_ADD1,0,PWR_DAC_REF | PWR_HI_R_LOAD_HP);
											//
			ac97_write_mask(codec,RT5611_PWR_MANAG_ADD2,0,PWR_SPK_MIXER | PWR_R_ADC_REC_MIXER | PWR_L_ADC_REC_MIXER | PWR_R_HP_MIXER | PWR_L_HP_MIXER | PWR_R_ADC_CLK_GAIN | PWR_L_ADC_CLK_GAIN | 
												PWR_R_DAC_CLK | PWR_L_DAC_CLK  | PWR_CLASS_AB);

			ac97_write_mask(codec,RT5611_PWR_MANAG_ADD3,0,PWR_MIC1_BOOST | PWR_MIC1_VOL_CTRL | PWR_SPK_R_OUT | PWR_SPK_L_OUT |PWR_HP_R_OUT | PWR_HP_L_OUT |
											    PWR_SPK_RN_OUT | PWR_SPK_LN_OUT);	
				
		break;

		case POWER_STATE_D2_PLAYBACK:	//STANDBY of playback

			ac97_write_mask(codec,RT5611_PWR_MANAG_ADD1,0,PWR_DAC_REF | PWR_HI_R_LOAD_HP);
											//
			ac97_write_mask(codec,RT5611_PWR_MANAG_ADD2,0,PWR_SPK_MIXER | PWR_R_HP_MIXER | PWR_L_HP_MIXER | PWR_R_DAC_CLK | PWR_L_DAC_CLK  | PWR_CLASS_AB);

			ac97_write_mask(codec,RT5611_PWR_MANAG_ADD3,0,PWR_SPK_R_OUT |PWR_SPK_L_OUT | PWR_HP_R_OUT | PWR_HP_L_OUT | PWR_SPK_RN_OUT| PWR_SPK_LN_OUT);

		break;

		case POWER_STATE_D2_RECORD:		//STANDBY of record

//			ac97_write_mask(codec,RT5610_PWR_MANAG_ADD1,0,PWR_MIC_BIAS1);
											//
			ac97_write_mask(codec,RT5611_PWR_MANAG_ADD2,0,PWR_R_ADC_REC_MIXER | PWR_L_ADC_REC_MIXER | PWR_R_ADC_CLK_GAIN | PWR_L_ADC_CLK_GAIN);

			ac97_write_mask(codec,RT5611_PWR_MANAG_ADD3,0,PWR_MIC1_BOOST | PWR_MIC1_VOL_CTRL);	

		break;		

		case POWER_STATE_D3:		//SLEEP
		case POWER_STATE_D4:		//OFF----power off all power,include PR0,PR1,PR3,PR4,PR5,PR6,EAPD,and addition power managment
				ac97_write(codec,RT5611_PWR_MANAG_ADD3,PowerDownState);
				ac97_write(codec,RT5611_PWR_MANAG_ADD1,PowerDownState);
				ac97_write(codec,RT5611_PWR_MANAG_ADD2,PowerDownState);
			

				PowerDownState=RT_PWR_PR0 | RT_PWR_PR1 | RT_PWR_PR2 | RT_PWR_PR3 /*| RT_PWR_PR4*/ | RT_PWR_PR5 | RT_PWR_PR6 | RT_PWR_PR7; 		
				ac97_write(codec,RT5611_PD_CTRL_STAT,PowerDownState);		
				
		break;	

		default:

		break;
	}

	return 0;	
}

static int rt5611_AudioOutEnable(struct snd_soc_codec *codec,unsigned short int WavOutPath,int Mute)
{
	int RetVal=0;	

	if(Mute)
	{
		switch(WavOutPath)
		{
			case RT_WAVOUT_ALL_ON:

				RetVal=ac97_write_mask(codec,RT5611_SPK_OUT_VOL,RT_L_MUTE|RT_R_MUTE,RT_L_MUTE|RT_R_MUTE);	//Mute Speaker right/left channel
				RetVal=ac97_write_mask(codec,RT5611_HP_OUT_VOL,RT_L_MUTE|RT_R_MUTE,RT_L_MUTE|RT_R_MUTE);	//Mute headphone right/left channel
				RetVal=ac97_write_mask(codec,RT5611_PHONEIN_MONO_OUT_VOL,RT_R_MUTE,RT_R_MUTE);				//Mute Mono channel
				RetVal=ac97_write_mask(codec,RT5611_STEREO_DAC_VOL,RT_M_HP_MIXER|RT_M_SPK_MIXER|RT_M_MONO_MIXER
															  ,RT_M_HP_MIXER|RT_M_SPK_MIXER|RT_M_MONO_MIXER);	//Mute DAC to HP,Speaker,Mono Mixer
		
			break;
		
			case RT_WAVOUT_HP:

				RetVal=ac97_write_mask(codec,RT5611_HP_OUT_VOL,RT_L_MUTE|RT_R_MUTE,RT_L_MUTE|RT_R_MUTE);	//Mute headphone right/left channel
					
			break;

			case RT_WAVOUT_SPK:
				
				RetVal=ac97_write_mask(codec,RT5611_SPK_OUT_VOL,RT_L_MUTE|RT_R_MUTE,RT_L_MUTE|RT_R_MUTE);	//Mute Speaker right/left channel			

			break;
			

			case RT_WAVOUT_MONO:

				RetVal=ac97_write_mask(codec,RT5611_PHONEIN_MONO_OUT_VOL,RT_R_MUTE,RT_R_MUTE);	//Mute MonoOut channel		

			break;

			case RT_WAVOUT_DAC:

				RetVal=ac97_write_mask(codec,RT5611_STEREO_DAC_VOL,RT_M_HP_MIXER|RT_M_SPK_MIXER|RT_M_MONO_MIXER
															  	,RT_M_HP_MIXER|RT_M_SPK_MIXER|RT_M_MONO_MIXER);	//Mute DAC to HP,Speaker,Mono Mixer

			break;
			default:

				return 0;

		}
	}
	else
	{
		switch(WavOutPath)
		{

			case RT_WAVOUT_ALL_ON:

				RetVal=ac97_write_mask(codec,RT5611_SPK_OUT_VOL	,0,RT_L_MUTE|RT_R_MUTE);	//Mute Speaker right/left channel
				RetVal=ac97_write_mask(codec,RT5611_HP_OUT_VOL 		,0,RT_L_MUTE|RT_R_MUTE);	//Mute headphone right/left channel
				RetVal=ac97_write_mask(codec,RT5611_PHONEIN_MONO_OUT_VOL,0,RT_L_MUTE|RT_R_MUTE);	//Mute Mono channel
				RetVal=ac97_write_mask(codec,RT5611_STEREO_DAC_VOL	,0,RT_M_HP_MIXER|RT_M_SPK_MIXER|RT_M_MONO_MIXER);	//Mute DAC to HP,Speaker,Mono Mixer
		
			break;
		
			case RT_WAVOUT_HP:

				RetVal=ac97_write_mask(codec,RT5611_HP_OUT_VOL,0,RT_L_MUTE|RT_R_MUTE);	//unMute headphone right/left channel
					
			break;

			case RT_WAVOUT_SPK:
				
				RetVal=ac97_write_mask(codec,RT5611_SPK_OUT_VOL,0,RT_L_MUTE|RT_R_MUTE);	//unMute Speaker right/left channel			

			break;			

			case RT_WAVOUT_MONO:

				RetVal=ac97_write_mask(codec,RT5611_PHONEIN_MONO_OUT_VOL,0,RT_R_MUTE);	//unMute MonoOut channel		

			break;
			case RT_WAVOUT_DAC:

				RetVal=ac97_write_mask(codec,RT5611_STEREO_DAC_VOL,0,RT_M_HP_MIXER|RT_M_SPK_MIXER|RT_M_MONO_MIXER);	//unMute DAC to HP,Speaker,Mono Mixer

			break;
			default:
				return 0;
		}

	}
	
	return RetVal;
}



/* PLL divisors */
struct _pll_div {
	u32 pll_in;
	u32 pll_out;
	u16 regvalue;
};

static const struct _pll_div rt5611_pll_div[] = {		
		
	{  2048000,  24576000,	0x2ea0},
	{  3686400,  24576000,	0xee27},	
	{ 12000000,  24576000,	0x2915},   
	{ 13000000,  24576000,	0x772e},
	{ 13100000,	 24576000,	0x0d20},
	{ 24576000,  24576000,  0x0620},
	
};


static int rt5611_set_pll(struct snd_soc_codec *codec,
	int pll_id, unsigned int freq_in, unsigned int freq_out)
{
	int i;
		
	if (!freq_in || !freq_out) 
		return 0;		
		if(g_hasplay ==0)
			{
	for (i = 0; i < ARRAY_SIZE(rt5611_pll_div); i++) {
					
			if (rt5611_pll_div[i].pll_in == freq_in && rt5611_pll_div[i].pll_out == freq_out)
			 {			 	
			 	//printk("%s %d regval:%x \n",__FUNCTION__,__LINE__,rt5610_pll_div[i].regvalue);
			 	ac97_write(codec,RT5611_PLL_CTRL,rt5611_pll_div[i].regvalue);//set PLL parameter			 							 				 	
				break;
			}
		}
	//ac97_write_mask(codec,RT5610_GEN_CTRL_REG1,GP2_PLL_PRE_DIV_2,GP2_PLL_PRE_DIV_2);	
	
	//codec clock source from PLL output		
	ac97_write_mask(codec,RT5611_GEN_CTRL_REG1,GP_CLK_FROM_PLL,GP_CLK_FROM_PLL);	
	//enable PLL power	
	ac97_write_mask(codec,RT5611_PWR_MANAG_ADD2,PWR_PLL,PWR_PLL);						
	//Power off ACLink	
	ac97_write_mask(codec,RT5611_PD_CTRL_STAT,RT_PWR_PR4,RT_PWR_PR4);	 		
	//need ac97 controller to do warm reset	
	soc_ac97_ops.warm_reset(codec->ac97);
		}
	/* wait 10ms AC97 link frames for the link to stabilise */
	schedule_timeout_interruptible(msecs_to_jiffies(10));	

	return 0;	
}	

static int rt5611_set_dai_pll(struct snd_soc_codec_dai *codec_dai,
		int pll_id, unsigned int freq_in, unsigned int freq_out)
{
	struct snd_soc_codec *codec = codec_dai->codec;

	rt5611_set_pll(codec,pll_id,freq_in,freq_out);

	return 0;
}
void rt5611_enable_hp_spk(struct snd_soc_codec *codec)
{
		if(!codec)
		return -1;
		int hp_state =0;

		unsigned int gpio_val = pxa3xx_gpio_get_level(gpio_val );
		hp_state=(gpio_val & (1 << 1)) >> 1;
		if(hp_state==0){
			ac97_write_mask(codec, RT5611_PWR_MANAG_ADD3,
			(~PWR_SPK_L_OUT & ~PWR_SPK_R_OUT & ~PWR_SPK_LN_OUT & ~PWR_SPK_RN_OUT) | (PWR_HP_L_OUT | PWR_HP_R_OUT),
			(PWR_SPK_L_OUT | PWR_SPK_R_OUT|PWR_SPK_LN_OUT|PWR_SPK_RN_OUT) | (PWR_HP_L_OUT | PWR_HP_R_OUT));
		//ac97_write_mask(codec,RT5611_PWR_MANAG_ADD1,PWR_DAC_REF | PWR_HI_R_LOAD_HP,PWR_DAC_REF | PWR_HI_R_LOAD_HP);
			}
		else{
			ac97_write_mask(codec, RT5611_PWR_MANAG_ADD3, PWR_SPK_L_OUT | PWR_SPK_R_OUT|PWR_SPK_LN_OUT|PWR_SPK_RN_OUT ,
			PWR_SPK_L_OUT | PWR_SPK_R_OUT|PWR_SPK_LN_OUT|PWR_SPK_RN_OUT );
			ac97_write_mask(codec, RT5611_PWR_MANAG_ADD3, 
			(~PWR_HP_L_OUT & ~PWR_HP_R_OUT) ,PWR_HP_L_OUT | PWR_HP_R_OUT);
		}
}

void rt5611_enabel_hp_depop(struct snd_soc_codec *codec)
{
	ac97_write_mask(codec,RT5611_PWR_MANAG_ADD1,PWR_HI_R_LOAD_HP, PWR_HI_R_LOAD_HP);
		
}


void rt5611_speaker_power()
{
	 pxa3xx_mfp_set_afds(MFP_AC97_nSPKPWR,0,0);
        pxa3xx_gpio_set_direction(MFP_AC97_nSPKPWR, GPIO_DIR_OUT);
	 pxa3xx_gpio_set_level(MFP_AC97_nSPKPWR, 0);
}
EXPORT_SYMBOL_GPL(rt5611_speaker_power);

void rt5611_speaker_power_on(struct snd_soc_codec *codec)
{
	rt5611_hp_spk_pm(codec);
	pxa3xx_gpio_set_level(MFP_AC97_nSPKPWR, 0);
}
EXPORT_SYMBOL_GPL(rt5611_speaker_power_on);

void rt5611_speaker_power_down(struct snd_soc_codec *codec)
{
 	pxa3xx_gpio_set_level(MFP_AC97_nSPKPWR, 1);
}
EXPORT_SYMBOL_GPL(rt5611_speaker_power_down);

void rt5611_set_EQ(struct snd_soc_codec *codec)
{		
	int i;
	for(i=0;i<ARRAY_SIZE(Set_Codec_Reg_EQ);i++)
	{
		ac97_write(codec,Set_Codec_Reg_EQ[i][0],Set_Codec_Reg_EQ[i][1]);	
	}
	
}
EXPORT_SYMBOL_GPL(rt5611_set_EQ);

//static int rt5611_init(struct snd_soc_codec *codec)
int rt5611_init(struct snd_soc_codec *codec)
{
	int i;
	ac97_write(codec,RT5611_PD_CTRL_STAT,0);	
	//power on main bias of codec
	ac97_write_mask(codec,RT5611_PWR_MANAG_ADD1,PWR_MAIN_BIAS|PWR_MIC_BIAS1,PWR_MAIN_BIAS|PWR_MIC_BIAS1);	
	//power on vref of codec
	ac97_write_mask(codec,RT5611_PWR_MANAG_ADD2,PWR_MIXER_VREF,PWR_MIXER_VREF);	
			
	for(i=0;i<ARRAY_SIZE(Set_Codec_Reg_Init);i++)
	{
		ac97_write(codec,Set_Codec_Reg_Init[i][0],Set_Codec_Reg_Init[i][1]);	
	}
	ac97_write_mask(codec,RT5611_GEN_CTRL_REG1,GP_SPK_D_AMP_CTRL_RATIO_100  ,GP_SPK_D_AMP_CTRL_MASK);
	rt5611_set_EQ( codec);
	rt5611_speaker_power();
	return 0;
}
EXPORT_SYMBOL_GPL(rt5611_init);
static int ac97_hifi_trigger(struct snd_pcm_substream *substream,int cmd)
{
	struct snd_pcm_runtime *runtime = substream->runtime;
	struct snd_soc_pcm_runtime *rtd = substream->private_data;
	struct snd_soc_device *socdev = rtd->socdev;
	struct snd_soc_codec *codec = socdev->codec;
	int ret = 0;
	switch (cmd) {
	case SNDRV_PCM_TRIGGER_START:
		
		if(substream->stream==SNDRV_PCM_STREAM_PLAYBACK)
		{
			rt5611_ChangeCodecPowerStatus(codec,POWER_STATE_D1_PLAYBACK);

			//rt5611_AudioOutEnable(codec,RT_WAVOUT_SPK,0);
			
			//rt5611_AudioOutEnable(codec,RT_WAVOUT_HP,0);
		}
		else
		{
			rt5611_ChangeCodecPowerStatus(codec,POWER_STATE_D1_RECORD);			
		}		
		
		break;

	case SNDRV_PCM_TRIGGER_STOP:
	case SNDRV_PCM_TRIGGER_SUSPEND:
	case SNDRV_PCM_TRIGGER_PAUSE_PUSH:
   		ac97_write_mask(codec,RT5611_PWR_MANAG_ADD1,~PWR_HI_R_LOAD_HP, PWR_HI_R_LOAD_HP);
		break;

	case SNDRV_PCM_TRIGGER_RESUME:

		break;
	case SNDRV_PCM_TRIGGER_PAUSE_RELEASE:
		ac97_write_mask(codec,RT5611_PWR_MANAG_ADD1, PWR_HI_R_LOAD_HP, PWR_HI_R_LOAD_HP);
		break;

	default:
		ret = -EINVAL;
	}

	return ret;
}

//if steram will close,it will call ac97_hifi_shutdown
static void ac97_hifi_shutdown(struct snd_pcm_substream *substream)
{
	struct snd_pcm_runtime *runtime = substream->runtime;
	struct snd_soc_pcm_runtime *rtd = substream->private_data;
	struct snd_soc_device *socdev = rtd->socdev;
	struct snd_soc_codec *codec = socdev->codec;



}

static int ac97_hifi_prepare(struct snd_pcm_substream *substream)
{
	struct snd_pcm_runtime *runtime = substream->runtime;
	struct snd_soc_pcm_runtime *rtd = substream->private_data;
	struct snd_soc_device *socdev = rtd->socdev;
	struct snd_soc_codec *codec = socdev->codec;
	int reg;
	u16 vra;
if(g_hasplay ==0){
	vra = ac97_read(codec, RT5611_TONE_CTRL);
	ac97_write(codec, RT5611_TONE_CTRL, vra | 0x1);
}
	if (substream->stream == SNDRV_PCM_STREAM_PLAYBACK)
		reg = RT5611_STEREO_DAC_RATE;
	else
		reg = RT5611_STEREO_ADC_RATE;

	return ac97_write(codec, reg, runtime->rate);
}


#define RT5611_RATES (SNDRV_PCM_RATE_8000 | SNDRV_PCM_RATE_11025 |\
		SNDRV_PCM_RATE_22050 | SNDRV_PCM_RATE_44100 | SNDRV_PCM_RATE_48000)

#define RT5611_PCM_FORMATS SNDRV_PCM_FMTBIT_S16_LE

struct snd_soc_codec_dai rt5611_dai[] = {
{
	.name = "AC97 HiFi",
	.playback = {
		.stream_name = "HiFi Playback",
		.channels_min = 1,
		.channels_max = 2,
		.rates = RT5611_RATES,
		.formats = RT5611_PCM_FORMATS,},
	.capture = {
		.stream_name = "HiFi Capture",
		.channels_min = 1,
		.channels_max = 2,
		.rates = RT5611_RATES,
		.formats = RT5611_PCM_FORMATS,},

	.ops = {
		.prepare = ac97_hifi_prepare,
		.trigger = ac97_hifi_trigger,
//		.shutdown= ac97_hifi_shutdown,
		},
		
	.dai_ops = {
		.set_pll = rt5611_set_dai_pll,
	},
},

};
EXPORT_SYMBOL_GPL(rt5611_dai);


extern void pxa3xx_enable_ac97_pins(void);
int rt5611_reset(struct snd_soc_codec *codec, int try_warm)
{

	soc_ac97_ops.reset(codec->ac97);

	return 0;
}
	
EXPORT_SYMBOL_GPL(rt5611_reset);

//static int rt5611_dapm_event(struct snd_soc_codec *codec, int event)
int rt5611_dapm_event(struct snd_soc_codec *codec, int event)
{
	switch (event) {
	case SNDRV_CTL_POWER_D0: /* full On */
		/* enable thermal shutdown */

		break;
	case SNDRV_CTL_POWER_D1: /* partial On */
	case SNDRV_CTL_POWER_D2: /* partial On */
		break;
	case SNDRV_CTL_POWER_D3hot: /* Off, with power */
		/* enable master bias and vmid */

		break;
	case SNDRV_CTL_POWER_D3cold: /* Off, without power */
		/* disable everything including AC link */

		break;
	}
	codec->dapm_state = event;

	return 0;
}
EXPORT_SYMBOL_GPL(rt5611_dapm_event);

static int rt5611_soc_suspend(struct platform_device *pdev,
	pm_message_t state)
{
	struct snd_soc_device *socdev = platform_get_drvdata(pdev);
	struct snd_soc_codec *codec = socdev->codec;

	rt5611_dapm_event(codec, SNDRV_CTL_POWER_D3cold);

	return 0;
}

static int rt5611_soc_resume(struct platform_device *pdev)
{
	struct snd_soc_device *socdev = platform_get_drvdata(pdev);
	struct snd_soc_codec *codec = socdev->codec;
	struct rt5611_priv *rt5611= codec->private_data;
	int i, ret;
	u16 *cache = codec->reg_cache;

	if ((ret = rt5611_reset(codec, 1)) < 0){
		printk(KERN_ERR "could not reset AC97 codec\n");
		return ret;
	}

	rt5611_dapm_event(codec, SNDRV_CTL_POWER_D3hot);

	/* do we need to re-start the PLL ? */
	if (rt5611->pll_out)
		rt5611_set_pll(codec, 0, rt5611->pll_in, rt5611->pll_out);

	/* only synchronise the codec if warm reset failed */
	if (ret == 0) {
		for (i = 2; i < ARRAY_SIZE(rt5611_reg) << 1; i+=2) {
			if (i > 0x66)
				continue;
			soc_ac97_ops.write(codec->ac97, i, cache[i>>1]);
		}
	}

	if (codec->suspend_dapm_state == SNDRV_CTL_POWER_D0)
		rt5611_dapm_event(codec, SNDRV_CTL_POWER_D0);

	return ret;
}

//broad cast audio jack state for resume
void rt5611_query_audiojack(int resume)
{
	unsigned int gpio_val = GPLR0;
	gpio_val_bak=gpio_val & (1 << 1) >> 1;
	audio_jack_state = !gpio_val_bak;
	if(gpio_val_bak == 0x0)//hp
	{
		gpio_val_bak=(gpio_val & (1 << 1)) >> 1 ;
		
		audio_jack_state = !gpio_val_bak;
		struct MSG_DATA msg;
		memset(&msg,0,sizeof(struct MSG_DATA));
		msg.msgID=AudioMessage;
		msg.data.jackstate=audio_jack_state;
		msgevent_report_broadcast(&msg);
	}	
        return ;
}
EXPORT_SYMBOL_GPL(rt5611_query_audiojack); 
/*
codec : IN
*/
void rt5611_hp_spk_pm( codec)
{
	int hp_state =0;

unsigned int gpio_val = pxa3xx_gpio_get_level(MFP_AC97_HP_DETECT_GPIO);

hp_state=gpio_val;
if(hp_state == 0)
{
ac97_write_mask(codec, RT5611_PWR_MANAG_ADD3,
(~PWR_SPK_L_OUT & ~PWR_SPK_R_OUT & ~PWR_SPK_LN_OUT & ~PWR_SPK_RN_OUT) |
(PWR_HP_L_OUT | PWR_HP_R_OUT),
(PWR_SPK_L_OUT | PWR_SPK_R_OUT|PWR_SPK_LN_OUT|PWR_SPK_RN_OUT) |
(PWR_HP_L_OUT | PWR_HP_R_OUT));
audio_jack_state = 1;
}
else
{
ac97_write_mask(codec, RT5611_PWR_MANAG_ADD3, PWR_SPK_L_OUT |
PWR_SPK_R_OUT|PWR_SPK_LN_OUT|PWR_SPK_RN_OUT ,
PWR_SPK_L_OUT | PWR_SPK_R_OUT|PWR_SPK_LN_OUT|PWR_SPK_RN_OUT );
ac97_write_mask(codec, RT5611_PWR_MANAG_ADD3,
(~PWR_HP_L_OUT & ~PWR_HP_R_OUT) ,PWR_HP_L_OUT | PWR_HP_R_OUT);
audio_jack_state = 0;
}

}
EXPORT_SYMBOL_GPL(rt5611_hp_spk_pm); 
/*
codec : IN
*/
int ModifyOutputPath(struct snd_soc_codec *codec, int send)
{
	if(!codec)
		return -1;

	unsigned int gpio_val = GPLR0;
	gpio_val=gpio_val & (1 << 1);

	if(gpio_val != gpio_val_bak){
		gpio_val_bak=(gpio_val & (1 << 1)) >> 1 ;
		if((gpio_val & (1 << 1)) == 0x0)//00401800,00001800
		{	//low indicate insert jack, disable speaker	,power down spk	
			ac97_write_mask(codec, RT5611_PWR_MANAG_ADD3,
			(~PWR_SPK_L_OUT & ~PWR_SPK_R_OUT & ~PWR_SPK_LN_OUT & ~PWR_SPK_RN_OUT) | (PWR_HP_L_OUT | PWR_HP_R_OUT),
			(PWR_SPK_L_OUT | PWR_SPK_R_OUT|PWR_SPK_LN_OUT|PWR_SPK_RN_OUT) | (PWR_HP_L_OUT | PWR_HP_R_OUT));
			audio_jack_state = 1;
			if(send == 1)
				{
					struct MSG_DATA msg;
					memset(&msg,0,sizeof(struct MSG_DATA));
					msg.msgID=AudioMessage;
					msg.data.jackstate=audio_jack_state;
					msgevent_report_broadcast(&msg);
				}
		}
		else
		{//high indicate move jack, enable speaker,power down hp
			ac97_write_mask(codec, RT5611_PWR_MANAG_ADD3, PWR_SPK_L_OUT | PWR_SPK_R_OUT|PWR_SPK_LN_OUT|PWR_SPK_RN_OUT ,
			PWR_SPK_L_OUT | PWR_SPK_R_OUT|PWR_SPK_LN_OUT|PWR_SPK_RN_OUT );
			ac97_write_mask(codec, RT5611_PWR_MANAG_ADD3, 
			(~PWR_HP_L_OUT & ~PWR_HP_R_OUT) ,PWR_HP_L_OUT | PWR_HP_R_OUT);
			audio_jack_state = 0;
			if(send == 1)
				{
					struct MSG_DATA msg;
					memset(&msg,0,sizeof(struct MSG_DATA));
					msg.msgID=AudioMessage;
					msg.data.jackstate=audio_jack_state;
					msgevent_report_broadcast(&msg);
				}
		}
	}
		
        return 0;
}
EXPORT_SYMBOL_GPL(ModifyOutputPath); 

/*
irq: IN    irq num
dev_id : IN
this function is the irq handler	
*/
static irqreturn_t pxa3xx_JackDetect_irq(int irq, void *dev_id)
{
        struct snd_soc_codec *codec = dev_id;
	int i=0;
	for(i=0;i<100;i++)
		udelay(500);
        //pxa3xx_gpio_clear_edge_detect_status(MFP_KP_MKIN_3);
	
	if(!codec)
		{
			printk("%s %d \n",__func__,__LINE__);
			return IRQ_NONE;
		}
	//disable_irq(IRQ_GPIO(MFP2GPIO(MFP_KP_MKIN_3)));
        ModifyOutputPath(codec, 1);
	//enable_irq(IRQ_GPIO(MFP2GPIO(MFP_KP_MKIN_3)));
       
        return IRQ_NONE;

}

/* 
pdev: IN    
codec : IN
*/
static int JackInsertIntrrupt(struct platform_device *pdev,struct snd_soc_codec *codec)
{               
        GCR |= GCR_GIE;
	gpio_val_bak=-1;
        pxa3xx_mfp_set_afds(MFP_AC97_HP_DETECT_GPIO,MFP_AC97_HP_DETECT_GPIO_AF,0);
        pxa3xx_gpio_set_direction(MFP_AC97_HP_DETECT_GPIO, GPIO_DIR_IN);
        pxa3xx_gpio_clear_edge_detect_status(MFP_AC97_HP_DETECT_GPIO);
        set_irq_type(IRQ_GPIO(MFP2GPIO(MFP_AC97_HP_DETECT_GPIO)), IRQT_BOTHEDGE);
        int ret = request_irq(IRQ_GPIO(MFP2GPIO(MFP_AC97_HP_DETECT_GPIO)), pxa3xx_JackDetect_irq,IRQF_DISABLED, "JackDetect", codec);

        if(ret < 0)
                {
                printk("jack insert request_irq failed\n");
               
                }
        return 0; 
}

static int rt5611_soc_probe(struct platform_device *pdev)
{
	struct snd_soc_device *socdev = platform_get_drvdata(pdev);
	struct snd_soc_codec *codec;
	int ret = 0;

	printk(KERN_INFO "RT5610/RT5611 SoC Audio Codec %s\n", RT5611_VERSION);

	socdev->codec = kzalloc(sizeof(struct snd_soc_codec), GFP_KERNEL);
	if (socdev->codec == NULL)
		return -ENOMEM;
	codec = socdev->codec;
	mutex_init(&codec->mutex);

	codec->reg_cache =
			kzalloc(sizeof(u16) * ARRAY_SIZE(rt5611_reg), GFP_KERNEL);
	if (codec->reg_cache == NULL){
		ret = -ENOMEM;
		goto cache_err;
	}
	memcpy(codec->reg_cache, rt5611_reg,
		sizeof(u16) * ARRAY_SIZE(rt5611_reg));
	codec->reg_cache_size = sizeof(u16) * ARRAY_SIZE(rt5611_reg);
	codec->reg_cache_step = 2;

	codec->private_data = kzalloc(sizeof(struct rt5611_priv), GFP_KERNEL);
	if (codec->private_data == NULL) {
		ret = -ENOMEM;
		goto priv_err;
	}

	codec->name = "RT5611";
	codec->owner = THIS_MODULE;
	codec->dai = rt5611_dai;
	codec->num_dai = ARRAY_SIZE(rt5611_dai);
	codec->write = ac97_write;
	codec->read = ac97_read;
	codec->dapm_event = rt5611_dapm_event;
	INIT_LIST_HEAD(&codec->dapm_widgets);
	INIT_LIST_HEAD(&codec->dapm_paths);

	ret = snd_soc_new_ac97_codec(codec, &soc_ac97_ops, 0);
	if (ret < 0)
		goto codec_err;

	/* register pcms */
	ret = snd_soc_new_pcms(socdev, SNDRV_DEFAULT_IDX1, SNDRV_DEFAULT_STR1);
	if (ret < 0)
		goto pcm_err;

	ret = rt5611_reset(codec, 1);
	if (ret < 0) {
		printk(KERN_ERR "AC97 link error\n");
		goto reset_err;
	}

	rt5611_dapm_event(codec, SNDRV_CTL_POWER_D0);


	rt5611_add_controls(codec);	
//	rt5610_add_widgets(codec);	


	rt5611_init(codec);

	
	ret = snd_soc_register_card(socdev);
	if (ret < 0)
		goto reset_err;
	JackInsertIntrrupt(pdev,codec);
	//int val = ac97_read(codec, RT5611_STEREO_DAC_VOL);

	//val = val & 0xe0e0;
	//val = val | 0x0c0c;
	//ac97_write(codec, RT5611_STEREO_DAC_VOL, val);

	return 0;

reset_err:
	snd_soc_free_pcms(socdev);

pcm_err:
	snd_soc_free_ac97_codec(codec);

codec_err:
	kfree(codec->private_data);

priv_err:
	kfree(codec->reg_cache);

cache_err:
	kfree(socdev->codec);
	socdev->codec = NULL;

	return ret;
}

static int rt5611_soc_remove(struct platform_device *pdev)
{
	struct snd_soc_device *socdev = platform_get_drvdata(pdev);
	struct snd_soc_codec *codec = socdev->codec;

	if (codec == NULL)
		return 0;

	snd_soc_dapm_free(socdev);
	snd_soc_free_pcms(socdev);
	snd_soc_free_ac97_codec(codec);
	kfree(codec->private_data);
	kfree(codec->reg_cache);
	kfree(codec->dai);
	kfree(codec);

	return 0;
}

struct snd_soc_codec_device soc_codec_dev_rt5611 = {
	.probe = 	rt5611_soc_probe,
	.remove = 	rt5611_soc_remove,
	.suspend =	rt5611_soc_suspend,
	.resume = 	rt5611_soc_resume,
};

EXPORT_SYMBOL_GPL(soc_codec_dev_rt5611);

MODULE_DESCRIPTION("ASoC RT5610/RT5611 driver");
MODULE_AUTHOR("flove");
MODULE_LICENSE("GPL");
