/** @file wlan_test.c
  * @brief This file contains functions for special test.
  * 
  *  (c) Copyright  2003-2006, Marvell International Ltd.  
  *   
  *  This software file (the "File") is distributed by Marvell International 
  *  Ltd. under the terms of the GNU General Public License Version 2, June 1991 
  *  (the "License").  You may use, redistribute and/or modify this File in 
  *  accordance with the terms and conditions of the License, a copy of which 
  *  is available along with the File in the gpl.txt file or by writing to 
  *  the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 
  *  02111-1307 or on the worldwide web at http://www.gnu.org/licenses/gpl.txt.
  *
  *  THE FILE IS DISTRIBUTED AS-IS, WITHOUT WARRANTY OF ANY KIND, AND THE 
  *  IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE 
  *  ARE EXPRESSLY DISCLAIMED.  The License provides additional details about 
  *  this warranty disclaimer.
  *
  */
/********************************************************
Change log:
	10/04/05: Add Doxygen format comments
	
********************************************************/
#include	"include.h"

#ifdef WLAN_LOOPBACK
/********************************************************
		Global Functions
********************************************************/
/** 
 *  @brief Set loopback patern   
 *  @param priv                 A pointer to wlan_private structure
 *  @param req			A pointer to ifreq structure
 *  @return 	   		WLAN_STATUS_SUCCESS --success, otherwise fail
 */
int wlan_subcmd_setloopback_ioctl(wlan_private *priv, struct ifreq *req)
{
	struct	ifreq	*treq;
	u8 pattern;
	ENTER();
	PRINTM(INFO, "WLAN_SUBCMD_LOOPBACK\n");
	treq = (struct ifreq *)((u8 *)req + 4);
	pattern = (u8)((u32)treq->ifr_data);
	PRINTM(INFO, "Sending one loopback cmd: %x\n", pattern);
	PrepareAndSendCommand(priv,
			HostCmd_CMD_802_11_LOOPBACK,
			pattern, HostCmd_OPTION_WAITFORRSP, 0, NULL);
	LEAVE();
	return WLAN_STATUS_SUCCESS;
}

/** 
 *  @brief This function handles the loopback command response
 *  
 *  @param priv    A pointer to wlan_private structure
 *  @return 	   WLAN_STATUS_SUCCESS or WLAN_STATUS_FAILURE
 */
int wlan_process_rx_loopbackcommand(wlan_private * priv)
{
	
	wlan_adapter   *Adapter = priv->adapter;
	static int 			loop=0;
	int					RespSize;
	u32			flags;

	ENTER();
	RespSize = MIN(priv->wlan_dev.upld_len, MRVDRV_ETH_RX_PACKET_BUFFER_SIZE);
	HEXDUMP("Cmd", Adapter->CurCmd->BufVirtualAddr, 
						MIN(200, RespSize));
	HEXDUMP("Cmd Response", priv->mspiodev.upld_buf,
						MIN(200, RespSize));
	if (Adapter->CurCmd) {
		int i;
		u8 *src, *dst;
		dst = priv->wlan_dev.upld_buf;
		src = Adapter->CurCmd->BufVirtualAddr;
		for ( i=0; i< RespSize-S_DS_GEN; i++ ) {
			if( src[i] != dst[i] ) {
				PRINTM(MSG,"loop[%d], pos[%d] cmd=%x resp=%x\n", loop, i, src[i], dst[i]);
				break;
			} 
		}
		if ( i >= RespSize-S_DS_GEN )
			PRINTM(INFO,"Loop[%d] OK,len=%d\n", ++loop,RespSize);
		else
			PRINTM(MSG,"Loop[%d] Failure,len=%d\n", ++loop,RespSize);

		CleanupAndInsertCmd(priv, Adapter->CurCmd);
		spin_lock_irqsave(&Adapter->QueueSpinLock, flags);
		Adapter->CurCmd = NULL;
		spin_unlock_irqrestore(&Adapter->QueueSpinLock, flags);
	}
	LEAVE();
	return WLAN_STATUS_SUCCESS;
}
#endif
