/** @file wlan_fw.h
 * 
 * @brief This file contains firmware specific defines. 
 *  
 * (c) Copyright  2003-2007, Marvell International Ltd. 
 *
 * This software file (the "File") is distributed by Marvell International 
 * Ltd. under the terms of the GNU General Public License Version 2, June 1991 
 * (the "License").  You may use, redistribute and/or modify this File in 
 * accordance with the terms and conditions of the License, a copy of which 
 * is available along with the File in the gpl.txt file or by writing to 
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 
 * 02111-1307 or on the worldwide web at http://www.gnu.org/licenses/gpl.txt.
 *
 * THE FILE IS DISTRIBUTED AS-IS, WITHOUT WARRANTY OF ANY KIND, AND THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE 
 * ARE EXPRESSLY DISCLAIMED.  The License provides additional details about 
 * this warranty disclaimer.
 *
 */
/********************************************************
Change log:
	05/30/07: Initial creation
********************************************************/

#ifndef _WLAN_FW_H
#define _WLAN_FW_H

/** Macros for Data Alignment */
#define ALIGN_SZ(p, a)	\
	(((p) + ((a) - 1)) & ~((a) - 1))

#define ALIGN_ADDR(p, a)	\
	((((u32)(p)) + (((u32)(a)) - 1)) & ~(((u32)(a)) - 1))

/** PUBLIC DEFINITIONS */
#define DEFAULT_AD_HOC_CHANNEL       6
#define DEFAULT_AD_HOC_CHANNEL_A    36

#define HostCmd_OPTION_WAITFORRSP             0x0002

/** Host Command ID */
#define HostCmd_CMD_GET_HW_SPEC               0x0003
#define HostCmd_CMD_802_11_RESET              0x0005
#define HostCmd_CMD_802_11_SCAN               0x0006
#define HostCmd_CMD_802_11_GET_LOG            0x000b
#define HostCmd_CMD_MAC_MULTICAST_ADR         0x0010
#define HostCmd_CMD_802_11_EEPROM_ACCESS      0x0059
#define HostCmd_CMD_802_11_ASSOCIATE          0x0012
#define HostCmd_CMD_802_11_SET_WEP            0x0013
#define HostCmd_CMD_802_11_SNMP_MIB           0x0016
#define HostCmd_CMD_MAC_REG_ACCESS            0x0019
#define HostCmd_CMD_BBP_REG_ACCESS            0x001a
#define HostCmd_CMD_RF_REG_ACCESS             0x001b
#define HostCmd_CMD_802_11_RADIO_CONTROL      0x001c
#define HostCmd_CMD_802_11_RF_CHANNEL         0x001d
#define HostCmd_CMD_802_11_RF_TX_POWER        0x001e
#define HostCmd_CMD_802_11_RSSI               0x001f
#define HostCmd_CMD_802_11_RF_ANTENNA         0x0020

#define HostCmd_CMD_802_11_PS_MODE	      0x0021

#define HostCmd_CMD_802_11_DEAUTHENTICATE     0x0024
#define HostCmd_CMD_MAC_CONTROL               0x0028
#define HostCmd_CMD_802_11_AD_HOC_START       0x002b
#define HostCmd_CMD_802_11_AD_HOC_JOIN        0x002c

#define HostCmd_CMD_802_11_KEY_MATERIAL       0x005e

#define HostCmd_CMD_802_11_DEEP_SLEEP         0x003e

#define HostCmd_CMD_802_11_AD_HOC_STOP        0x0040

#define HostCmd_CMD_802_11_HOST_SLEEP_CFG     0x0043
#define HostCmd_CMD_802_11_WAKEUP_CONFIRM     0x0044
#define HostCmd_CMD_802_11_HOST_SLEEP_ACTIVATE	0x0045

#define HostCmd_CMD_802_11_MAC_ADDRESS        0x004D
#define HostCmd_CMD_802_11_EEPROM_ACCESS      0x0059

#define HostCmd_CMD_802_11D_DOMAIN_INFO       0x005b

#define HostCmd_CMD_802_11_TPC_INFO           0x005f
#define HostCmd_CMD_802_11_TPC_ADAPT_REQ      0x0060
#define HostCmd_CMD_802_11_CHAN_SW_ANN        0x0061
#define HostCmd_CMD_MEASUREMENT_REQUEST       0x0062
#define HostCmd_CMD_MEASUREMENT_REPORT        0x0063

#define HostCmd_CMD_802_11_SLEEP_PARAMS          0x0066

#define HostCmd_CMD_802_11_INACTIVITY_TIMEOUT    0x0067

#define HostCmd_CMD_802_11_SLEEP_PERIOD          0x0068
#define HostCmd_CMD_802_11_BCA_CONFIG_TIMESHARE  0x0069

#define HostCmd_CMD_802_11_BG_SCAN_CONFIG        0x006b
#define HostCmd_CMD_802_11_BG_SCAN_QUERY         0x006c

#define HostCmd_CMD_802_11_CAL_DATA_EXT          0x006d

#define HostCmd_CMD_WMM_ADDTS_REQ                0x006E
#define HostCmd_CMD_WMM_DELTS_REQ                0x006F
#define HostCmd_CMD_WMM_QUEUE_CONFIG             0x0070
#define HostCmd_CMD_WMM_GET_STATUS               0x0071

#define HostCmd_CMD_802_11_TPC_CFG               0x0072

#define HostCmd_CMD_802_11_FW_WAKE_METHOD        0x0074

#define HostCmd_CMD_802_11_LED_CONTROL           0x004e

#define HostCmd_CMD_802_11_SUBSCRIBE_EVENT       0x0075

#define HostCmd_CMD_802_11_RATE_ADAPT_RATESET    0x0076

#define HostCmd_CMD_802_11_CRYPTO                0x0078

#define HostCmd_CMD_802_11_TX_RATE_QUERY	0x007f

#define HostCmd_CMD_802_11_POWER_ADAPT_CFG_EXT	0x007e

#define HostCmd_CMD_GET_TSF                      0x0080

#define HostCmd_CMD_WMM_QUEUE_STATS              0x0081

#define HostCmd_CMD_802_11_AUTO_TX		0x0082
#define HostCmd_CMD_802_11_IBSS_COALESCING_STATUS 0x0083

#define HostCmd_CMD_MEM_ACCESS			0x0086

#ifdef MFG_CMD_SUPPORT
#define HostCmd_CMD_MFG_COMMAND               0x0089
#endif

#define HostCmd_CMD_DBGS_CFG		      0x008b
#define HostCmd_CMD_GET_MEM		      0x008c

#define HostCmd_CMD_TX_PKT_STATS              0x008d

#define HostCmd_CMD_CFG_DATA                  0x008f

#define HostCmd_CMD_SDIO_PULL_CTRL		      0x0093

#define HostCmd_CMD_VERSION_EXT               0x0097

#define HostCmd_CMD_MODULE_TYPE_CONFIG        0x0099

#define HostCmd_CMD_MEF_CFG		      0x009a

/* For the IEEE Power Save */
#define HostCmd_SubCmd_Enter_PS               0x0030
#define HostCmd_SubCmd_Exit_PS                0x0031
#define HostCmd_SubCmd_Sleep_Confirmed        0x0034
#define HostCmd_SubCmd_Full_PowerDown         0x0035
#define HostCmd_SubCmd_Full_PowerUp           0x0036

/* Command RET code, MSB is set to 1 */
#define HostCmd_RET_BIT                       0x8000

/** General Result Code*/
/* OK */
#define HostCmd_RESULT_OK                    0x0000
/* Genenral error */
#define HostCmd_RESULT_ERROR                 0x0001
/* Command is not valid */
#define HostCmd_RESULT_NOT_SUPPORT           0x0002
/* Command is pending */
#define HostCmd_RESULT_PENDING               0x0003
/* System is busy (command ignored) */
#define HostCmd_RESULT_BUSY                  0x0004
/* Data buffer is not big enough */
#define HostCmd_RESULT_PARTIAL_DATA          0x0005

/* Definition of action or option for each command */

/* Define general purpose action */
#define HostCmd_ACT_GEN_READ                    0x0000
#define HostCmd_ACT_GEN_WRITE                   0x0001
#define HostCmd_ACT_GEN_GET                     0x0000
#define HostCmd_ACT_GEN_SET                     0x0001
#define HostCmd_ACT_GEN_REMOVE                  0x0002
#define HostCmd_ACT_GEN_OFF                     0x0000
#define HostCmd_ACT_GEN_ON                      0x0001

/* Define action or option for HostCmd_CMD_802_11_SET_WEP */
#define HostCmd_ACT_ADD                         0x0002
#define HostCmd_ACT_REMOVE                      0x0004

#define HostCmd_TYPE_WEP_40_BIT                 0x0001
#define HostCmd_TYPE_WEP_104_BIT                0x0002

#define HostCmd_WEP_KEY_INDEX_MASK              0x3fff

/* Define action or option for HostCmd_CMD_802_11_SCAN */
#define HostCmd_BSS_TYPE_BSS                    0x0001
#define HostCmd_BSS_TYPE_IBSS                   0x0002
#define HostCmd_BSS_TYPE_ANY                    0x0003

/* Define action or option for HostCmd_CMD_802_11_SCAN */
#define HostCmd_SCAN_TYPE_ACTIVE                0x0000
#define HostCmd_SCAN_TYPE_PASSIVE               0x0001

/* Radio type definitions for the channel TLV */
#define HostCmd_SCAN_RADIO_TYPE_BG		0
#define HostCmd_SCAN_RADIO_TYPE_A		1

/* Define action or option for HostCmd_CMD_MAC_CONTROL */
#define HostCmd_ACT_MAC_RX_ON                   0x0001
#define HostCmd_ACT_MAC_TX_ON                   0x0002
#define HostCmd_ACT_MAC_LOOPBACK_ON             0x0004
#define HostCmd_ACT_MAC_WEP_ENABLE              0x0008
#define HostCmd_ACT_MAC_ETHERNETII_ENABLE       0x0010
#define HostCmd_ACT_MAC_PROMISCUOUS_ENABLE      0x0080
#define HostCmd_ACT_MAC_ALL_MULTICAST_ENABLE    0x0100
#define HostCmd_ACT_MAC_STRICT_PROTECTION_ENABLE  0x0400
#define HostCmd_ACT_MAC_ADHOC_G_PROTECTION_ON	  0x2000

/* Define action or option or constant for HostCmd_CMD_MAC_MULTICAST_ADR */
#define HostCmd_SIZE_MAC_ADR                    6
#define HostCmd_MAX_MCAST_ADRS                  32

#define RADIO_ON                                0x01
#define RADIO_OFF                               0x00

/* Define action or option for CMD_802_11_RF_CHANNEL */
#define HostCmd_OPT_802_11_RF_CHANNEL_GET       0x00
#define HostCmd_OPT_802_11_RF_CHANNEL_SET       0x01

#define HostCmd_ACT_SET_RX                      0x0001
#define HostCmd_ACT_SET_TX                      0x0002
#define HostCmd_ACT_SET_BOTH                    0x0003
#define HostCmd_ACT_GET_RX                      0x0004
#define HostCmd_ACT_GET_TX                      0x0008
#define HostCmd_ACT_GET_BOTH                    0x000c

/** Card Event definition */
#define MACREG_INT_CODE_DUMMY_HOST_WAKEUP_SIGNAL       0x00000001
#define MACREG_INT_CODE_LINK_LOST_WITH_SCAN     0x00000002
#define MACREG_INT_CODE_LINK_LOST       	0x00000003
#define MACREG_INT_CODE_LINK_SENSED             0x00000004
#define MACREG_INT_CODE_MIB_CHANGED             0x00000006
#define MACREG_INT_CODE_INIT_DONE               0x00000007
#define MACREG_INT_CODE_DEAUTHENTICATED         0x00000008
#define MACREG_INT_CODE_DISASSOCIATED           0x00000009
#define MACREG_INT_CODE_PS_AWAKE                0x0000000a
#define MACREG_INT_CODE_PS_SLEEP                0x0000000b
#define MACREG_INT_CODE_MIC_ERR_MULTICAST       0x0000000d
#define MACREG_INT_CODE_MIC_ERR_UNICAST         0x0000000e
#define MACREG_INT_CODE_WM_AWAKE                0x0000000f
#define MACREG_INT_CODE_DEEP_SLEEP_AWAKE        0x00000010
#define MACREG_INT_CODE_ADHOC_BCN_LOST          0x00000011
#define MACREG_INT_CODE_HOST_SLEEP_AWAKE        0x00000012
#define MACREG_INT_CODE_STOP_TX                 0x00000013
#define MACREG_INT_CODE_START_TX                0x00000014
#define MACREG_INT_CODE_CHANNEL_SWITCH          0x00000015
#define MACREG_INT_CODE_MEAS_REPORT_RDY         0x00000016
#define MACREG_INT_CODE_WMM_STATUS_CHANGE	0x00000017
#define MACREG_INT_CODE_BG_SCAN_REPORT		0x00000018
#define MACREG_INT_CODE_RSSI_LOW		0x00000019
#define MACREG_INT_CODE_SNR_LOW			0x0000001a
#define MACREG_INT_CODE_MAX_FAIL		0x0000001b
#define MACREG_INT_CODE_RSSI_HIGH		0x0000001c
#define MACREG_INT_CODE_SNR_HIGH		0x0000001d
#define MACREG_INT_CODE_IBSS_COALESCED		0x0000001e

/* Define bitmap conditions for HOST_SLEEP_CFG */
#define HOST_SLEEP_CFG_CANCEL			0xffffffff
#define HOST_SLEEP_CFG_GPIO_FF			0xff
#define HOST_SLEEP_CFG_GAP_FF			0xff

#define MRVDRV_MAX_MULTICAST_LIST_SIZE	32
#define MRVDRV_MAX_CHANNEL_SIZE		14
#define MRVDRV_ETH_ADDR_LEN             6
#define MRVDRV_MAX_SSID_LENGTH			32
#define MRVDRV_MAX_BSS_DESCRIPTS		16
/** WEP list macros & data structures */
#define MRVL_KEY_BUFFER_SIZE_IN_BYTE  16
#define MRVL_MAX_KEY_WPA_KEY_LENGTH     32

#define BS_802_11B	0
#define BS_802_11G	1
#define BS_802_11A	2
/* Setup the number of rates pased in the driver/firmware API.*/
#define A_SUPPORTED_RATES		14

#define FW_MULTI_BANDS_SUPPORT	(BIT(8) | BIT(9) | BIT(10))
#define	BAND_B			(0x01)
#define	BAND_G			(0x02)
#define BAND_A			(0x04)
#define ALL_802_11_BANDS	(BAND_B | BAND_G | BAND_A)

#define	IS_SUPPORT_MULTI_BANDS(_adapter) \
				(_adapter->fwCapInfo & FW_MULTI_BANDS_SUPPORT)
#define GET_FW_DEFAULT_BANDS(_adapter)	\
				((_adapter->fwCapInfo >> 8) & ALL_802_11_BANDS)

/* Setup the number of rates pased in the driver/firmware API.*/
#define HOSTCMD_SUPPORTED_RATES A_SUPPORTED_RATES

#define B_SUPPORTED_RATES		8
#define G_SUPPORTED_RATES		14

#define	WLAN_SUPPORTED_RATES		14
#define WLAN_MAX_SSID_LENGTH		32

#define	MAX_POWER_ADAPT_GROUP		5

typedef u8 WLAN_802_11_RATES[WLAN_SUPPORTED_RATES];
typedef u8 WLAN_802_11_MAC_ADDRESS[ETH_ALEN];

/** WLAN_802_11_NETWORK_TYPE */
typedef enum _WLAN_802_11_NETWORK_TYPE
{
    Wlan802_11FH,
    Wlan802_11DS,
    /*defined as upper bound */
    Wlan802_11NetworkTypeMax
} WLAN_802_11_NETWORK_TYPE, *PWLAN_802_11_NETWORK_TYPE;

/** WLAN_802_11_NETWORK_INFRASTRUCTURE */
typedef enum _WLAN_802_11_NETWORK_INFRASTRUCTURE
{
    Wlan802_11IBSS,
    Wlan802_11Infrastructure,
    Wlan802_11AutoUnknown,
    /*defined as upper bound */
    Wlan802_11InfrastructureMax
} WLAN_802_11_NETWORK_INFRASTRUCTURE, *PWLAN_802_11_NETWORK_INFRASTRUCTURE;

#define IEEE_MAX_IE_SIZE  256

/** IEEE Type definitions  */
typedef enum _IEEEtypes_ElementId_e
{
    SSID = 0,
    SUPPORTED_RATES = 1,
    FH_PARAM_SET = 2,
    DS_PARAM_SET = 3,
    CF_PARAM_SET = 4,

    IBSS_PARAM_SET = 6,

    COUNTRY_INFO = 7,

    POWER_CONSTRAINT = 32,
    POWER_CAPABILITY = 33,
    TPC_REQUEST = 34,
    TPC_REPORT = 35,
    SUPPORTED_CHANNELS = 36,
    CHANNEL_SWITCH_ANN = 37,
    QUIET = 40,
    IBSS_DFS = 41,
    ERP_INFO = 42,
    EXTENDED_SUPPORTED_RATES = 50,

    VENDOR_SPECIFIC_221 = 221,
    WMM_IE = VENDOR_SPECIFIC_221,

    WPS_IE = VENDOR_SPECIFIC_221,

    WPA_IE = VENDOR_SPECIFIC_221,
    RSN_IE = 48,

} __ATTRIB_PACK__ IEEEtypes_ElementId_e;

#define CAPINFO_MASK    (~( BIT(15) | BIT(14) |               \
                            BIT(12) | BIT(11) | BIT(9)) )

typedef struct _IEEEtypes_CapInfo_t
{
    u8 Ess:1;
    u8 Ibss:1;
    u8 CfPollable:1;
    u8 CfPollRqst:1;
    u8 Privacy:1;
    u8 ShortPreamble:1;
    u8 Pbcc:1;
    u8 ChanAgility:1;
    u8 SpectrumMgmt:1;
    u8 Rsrvd3:1;
    u8 ShortSlotTime:1;
    u8 Apsd:1;
    u8 Rsvrd2:1;
    u8 DSSSOFDM:1;
    u8 Rsrvd1:2;
} __ATTRIB_PACK__ IEEEtypes_CapInfo_t;

typedef struct
{
    u8 ElementId;
    u8 Len;
} __ATTRIB_PACK__ IEEEtypes_Header_t;

/** IEEEtypes_CfParamSet_t */
typedef struct _IEEEtypes_CfParamSet_t
{
    u8 ElementId;
    u8 Len;
    u8 CfpCnt;
    u8 CfpPeriod;
    u16 CfpMaxDuration;
    u16 CfpDurationRemaining;
} __ATTRIB_PACK__ IEEEtypes_CfParamSet_t;

typedef struct IEEEtypes_IbssParamSet_t
{
    u8 ElementId;
    u8 Len;
    u16 AtimWindow;
} __ATTRIB_PACK__ IEEEtypes_IbssParamSet_t;

/** IEEEtypes_SsParamSet_t */
typedef union _IEEEtypes_SsParamSet_t
{
    IEEEtypes_CfParamSet_t CfParamSet;
    IEEEtypes_IbssParamSet_t IbssParamSet;
} __ATTRIB_PACK__ IEEEtypes_SsParamSet_t;

/** IEEEtypes_FhParamSet_t */
typedef struct _IEEEtypes_FhParamSet_t
{
    u8 ElementId;
    u8 Len;
    u16 DwellTime;
    u8 HopSet;
    u8 HopPattern;
    u8 HopIndex;
} __ATTRIB_PACK__ IEEEtypes_FhParamSet_t;

typedef struct _IEEEtypes_DsParamSet_t
{
    u8 ElementId;
    u8 Len;
    u8 CurrentChan;
} __ATTRIB_PACK__ IEEEtypes_DsParamSet_t;

/** IEEEtypes_DsParamSet_t */
typedef union IEEEtypes_PhyParamSet_t
{
    IEEEtypes_FhParamSet_t FhParamSet;
    IEEEtypes_DsParamSet_t DsParamSet;
} __ATTRIB_PACK__ IEEEtypes_PhyParamSet_t;

typedef struct _IEEEtypes_ERPInfo_t
{
    u8 ElementId;
    u8 Len;
    u8 ERPFlags;
} __ATTRIB_PACK__ IEEEtypes_ERPInfo_t;

typedef u16 IEEEtypes_AId_t;
typedef u16 IEEEtypes_StatusCode_t;

typedef struct
{
    IEEEtypes_CapInfo_t Capability;
    IEEEtypes_StatusCode_t StatusCode;
    IEEEtypes_AId_t AId;
    u8 IEBuffer[1];
} __ATTRIB_PACK__ IEEEtypes_AssocRsp_t;

typedef struct
{
    u8 ElementId;
    u8 Len;
    u8 Oui[3];
    u8 OuiType;
    u8 OuiSubtype;
    u8 Version;
} __ATTRIB_PACK__ IEEEtypes_VendorHeader_t;

typedef struct
{
    IEEEtypes_VendorHeader_t VendHdr;

    /* IE Max - size of previous fields */
    u8 Data[IEEE_MAX_IE_SIZE - sizeof(IEEEtypes_VendorHeader_t)];

}
__ATTRIB_PACK__ IEEEtypes_VendorSpecific_t;

typedef struct
{
    IEEEtypes_Header_t IeeeHdr;

    /* IE Max - size of previous fields */
    u8 Data[IEEE_MAX_IE_SIZE - sizeof(IEEEtypes_Header_t)];

}
__ATTRIB_PACK__ IEEEtypes_Generic_t;

/** TLV  type ID definition */
#define PROPRIETARY_TLV_BASE_ID		0x0100

/* Terminating TLV Type */
#define MRVL_TERMINATE_TLV_ID		0xffff

#define TLV_TYPE_SSID				0x0000
#define TLV_TYPE_RATES				0x0001
#define TLV_TYPE_PHY_FH				0x0002
#define TLV_TYPE_PHY_DS				0x0003
#define TLV_TYPE_CF				    0x0004
#define TLV_TYPE_IBSS				0x0006

#define TLV_TYPE_DOMAIN				0x0007

#define TLV_TYPE_POWER_CONSTRAINT   0x0020
#define TLV_TYPE_POWER_CAPABILITY   0x0021

#define TLV_TYPE_KEY_MATERIAL       (PROPRIETARY_TLV_BASE_ID + 0)
#define TLV_TYPE_CHANLIST           (PROPRIETARY_TLV_BASE_ID + 1)
#define TLV_TYPE_NUMPROBES          (PROPRIETARY_TLV_BASE_ID + 2)
#define TLV_TYPE_RSSI_LOW           (PROPRIETARY_TLV_BASE_ID + 4)
#define TLV_TYPE_SNR_LOW            (PROPRIETARY_TLV_BASE_ID + 5)
#define TLV_TYPE_FAILCOUNT          (PROPRIETARY_TLV_BASE_ID + 6)
#define TLV_TYPE_BCNMISS            (PROPRIETARY_TLV_BASE_ID + 7)
#define TLV_TYPE_LED_GPIO           (PROPRIETARY_TLV_BASE_ID + 8)
#define TLV_TYPE_LEDBEHAVIOR        (PROPRIETARY_TLV_BASE_ID + 9)
#define TLV_TYPE_PASSTHROUGH        (PROPRIETARY_TLV_BASE_ID + 10)
#define TLV_TYPE_POWER_TBL_2_4GHZ   (PROPRIETARY_TLV_BASE_ID + 12)
#define TLV_TYPE_POWER_TBL_5GHZ     (PROPRIETARY_TLV_BASE_ID + 13)
#define TLV_TYPE_WMMQSTATUS         (PROPRIETARY_TLV_BASE_ID + 16)
#define TLV_TYPE_CRYPTO_DATA        (PROPRIETARY_TLV_BASE_ID + 17)
#define TLV_TYPE_WILDCARDSSID       (PROPRIETARY_TLV_BASE_ID + 18)
#define TLV_TYPE_TSFTIMESTAMP       (PROPRIETARY_TLV_BASE_ID + 19)
#define TLV_TYPE_POWERADAPTCFGEXT   (PROPRIETARY_TLV_BASE_ID + 20)
#define TLV_TYPE_RSSI_HIGH          (PROPRIETARY_TLV_BASE_ID + 22)
#define TLV_TYPE_SNR_HIGH           (PROPRIETARY_TLV_BASE_ID + 23)
#define TLV_TYPE_AUTO_TX            (PROPRIETARY_TLV_BASE_ID + 24)
#define TLV_TYPE_WPS_ENROLLEE_PROBE_REQ_TLV    (PROPRIETARY_TLV_BASE_ID + 27)

#define TLV_TYPE_STARTBGSCANLATER   (PROPRIETARY_TLV_BASE_ID + 30)
#define TLV_TYPE_AUTH_TYPE          (PROPRIETARY_TLV_BASE_ID + 31)

/** TLV related data structures*/
/** MrvlIEtypesHeader_t */
typedef struct _MrvlIEtypesHeader
{
    u16 Type;
    u16 Len;
} __ATTRIB_PACK__ MrvlIEtypesHeader_t;

/** MrvlIEtypes_Data_t */
typedef struct _MrvlIEtypes_Data_t
{
    MrvlIEtypesHeader_t Header;
    u8 Data[1];
} __ATTRIB_PACK__ MrvlIEtypes_Data_t;

/** MrvlIEtypes_RatesParamSet_t */
typedef struct _MrvlIEtypes_RatesParamSet_t
{
    MrvlIEtypesHeader_t Header;
    u8 Rates[1];
} __ATTRIB_PACK__ MrvlIEtypes_RatesParamSet_t;

/** MrvlIEtypes_SsIdParamSet_t */
typedef struct _MrvlIEtypes_SsIdParamSet_t
{
    MrvlIEtypesHeader_t Header;
    u8 SsId[1];
} __ATTRIB_PACK__ MrvlIEtypes_SsIdParamSet_t;

/** MrvlIEtypes_WildCardSsIdParamSet_t */
typedef struct _MrvlIEtypes_WildCardSsIdParamSet_t
{
    MrvlIEtypesHeader_t Header;
    u8 MaxSsidLength;
    u8 SsId[1];
} __ATTRIB_PACK__ MrvlIEtypes_WildCardSsIdParamSet_t;

/** ChanScanMode_t */
typedef struct
{
    u8 PassiveScan:1;
    u8 DisableChanFilt:1;
    u8 Reserved_2_7:6;
} __ATTRIB_PACK__ ChanScanMode_t;

/** ChanScanParamSet_t */
typedef struct _ChanScanParamSet_t
{
    u8 RadioType;
    u8 ChanNumber;
    ChanScanMode_t ChanScanMode;
    u16 MinScanTime;
    u16 MaxScanTime;
} __ATTRIB_PACK__ ChanScanParamSet_t;

/** MrvlIEtypes_ChanListParamSet_t */
typedef struct _MrvlIEtypes_ChanListParamSet_t
{
    MrvlIEtypesHeader_t Header;
    ChanScanParamSet_t ChanScanParam[1];
} __ATTRIB_PACK__ MrvlIEtypes_ChanListParamSet_t;

/** CfParamSet_t */
typedef struct _CfParamSet_t
{
    u8 CfpCnt;
    u8 CfpPeriod;
    u16 CfpMaxDuration;
    u16 CfpDurationRemaining;
} __ATTRIB_PACK__ CfParamSet_t;

/** IbssParamSet_t */
typedef struct _IbssParamSet_t
{
    u16 AtimWindow;
} __ATTRIB_PACK__ IbssParamSet_t;

/** MrvlIEtypes_SsParamSet_t */
typedef struct _MrvlIEtypes_SsParamSet_t
{
    MrvlIEtypesHeader_t Header;
    union
    {
        CfParamSet_t CfParamSet[1];
        IbssParamSet_t IbssParamSet[1];
    } cf_ibss;
} __ATTRIB_PACK__ MrvlIEtypes_SsParamSet_t;

/** FhParamSet_t */
typedef struct _FhParamSet_t
{
    u16 DwellTime;
    u8 HopSet;
    u8 HopPattern;
    u8 HopIndex;
} __ATTRIB_PACK__ FhParamSet_t;

/** DsParamSet_t */
typedef struct _DsParamSet_t
{
    u8 CurrentChan;
} __ATTRIB_PACK__ DsParamSet_t;

/** MrvlIEtypes_PhyParamSet_t */
typedef struct _MrvlIEtypes_PhyParamSet_t
{
    MrvlIEtypesHeader_t Header;
    union
    {
        FhParamSet_t FhParamSet[1];
        DsParamSet_t DsParamSet[1];
    } fh_ds;
} __ATTRIB_PACK__ MrvlIEtypes_PhyParamSet_t;

/** MrvlIEtypes_RsnParamSet_t */
typedef struct _MrvlIEtypes_RsnParamSet_t
{
    MrvlIEtypesHeader_t Header;
    u8 RsnIE[1];
} __ATTRIB_PACK__ MrvlIEtypes_RsnParamSet_t;

/** MrvlIEtypes_WmmParamSet_t */
typedef struct _MrvlIEtypes_WmmParamSet_t
{
    MrvlIEtypesHeader_t Header;
    u8 WmmIE[1];
} __ATTRIB_PACK__ MrvlIEtypes_WmmParamSet_t;

typedef struct
{
    MrvlIEtypesHeader_t Header;
    u8 QueueIndex;
    u8 Disabled;
    u8 Reserved1;
    u8 Reserved2;
    u8 FlowRequired;
    u8 FlowCreated;
    u32 Reserved3;
} __ATTRIB_PACK__ MrvlIEtypes_WmmQueueStatus_t;

/** Table of TSF values returned in the scan result */
typedef struct
{
    MrvlIEtypesHeader_t Header;
    u64 tsfTable[1];
} __ATTRIB_PACK__ MrvlIEtypes_TsfTimestamp_t;

/**  Local Power Capability */
typedef struct _MrvlIEtypes_PowerCapability_t
{
    MrvlIEtypesHeader_t Header;
    s8 MinPower;
    s8 MaxPower;
} __ATTRIB_PACK__ MrvlIEtypes_PowerCapability_t;

/** MrvlIEtypes_RssiParamSet_t */
typedef struct _MrvlIEtypes_RssiThreshold_t
{
    MrvlIEtypesHeader_t Header;
    u8 RSSIValue;
    u8 RSSIFreq;
} __ATTRIB_PACK__ MrvlIEtypes_RssiThreshold_t;

/** MrvlIEtypes_SnrThreshold_t */
typedef struct _MrvlIEtypes_SnrThreshold_t
{
    MrvlIEtypesHeader_t Header;
    u8 SNRValue;
    u8 SNRFreq;
} __ATTRIB_PACK__ MrvlIEtypes_SnrThreshold_t;

/** MrvlIEtypes_FailureCount_t */
typedef struct _MrvlIEtypes_FailureCount_t
{
    MrvlIEtypesHeader_t Header;
    u8 FailValue;
    u8 FailFreq;
} __ATTRIB_PACK__ MrvlIEtypes_FailureCount_t;

/** MrvlIEtypes_BeaconsMissed_t */
typedef struct _MrvlIEtypes_BeaconsMissed_t
{
    MrvlIEtypesHeader_t Header;
    u8 BeaconMissed;
    u8 Reserved;
} __ATTRIB_PACK__ MrvlIEtypes_BeaconsMissed_t;

/** MrvlIEtypes_NumProbes_t */
typedef struct _MrvlIEtypes_NumProbes_t
{
    MrvlIEtypesHeader_t Header;
    u16 NumProbes;
} __ATTRIB_PACK__ MrvlIEtypes_NumProbes_t;

/** MrvlIEtypes_StartBGScanLater_t */
typedef struct _MrvlIEtypes_StartBGScanLater_t
{
    MrvlIEtypesHeader_t Header;
    u16 StartLater;
} __ATTRIB_PACK__ MrvlIEtypes_StartBGScanLater_t;

typedef struct _LedGpio_t
{
    u8 LedNum;                  /* LED # mapped to GPIO pin # below */
    u8 GpioNum;                 /* GPIO pin # used to control LED # above */
} __ATTRIB_PACK__ LedGpio_t;

/** MrvlIEtypes_LedGpio_t */
typedef struct _MrvlIEtypes_LedGpio_t
{
    MrvlIEtypesHeader_t Header;
    LedGpio_t LedGpio[1];
} __ATTRIB_PACK__ MrvlIEtypes_LedGpio_t;

/** MrvlIEtypes_LedBehavior_t */
typedef struct _MrvlIEtypes_LedBehavior_t
{
    MrvlIEtypesHeader_t Header;
    u8 FirmwareState;           /* Firmware State */
    u8 LedNum;                  /* LED # */
    u8 LedState;                /* LED State corresponding to Firmware State */
    u8 LedArgs;                 /* Arguments for LED State */
} __ATTRIB_PACK__ MrvlIEtypes_LedBehavior_t;

typedef struct _PA_Group_t
{
    u16 PowerAdaptLevel;
    u16 RateBitmap;
    u32 Reserved;
} __ATTRIB_PACK__ PA_Group_t;

/** MrvlIEtypes_PA_Group_t */
typedef struct _MrvlIEtypes_PowerAdapt_Group_t
{
    MrvlIEtypesHeader_t Header;
    PA_Group_t PA_Group[MAX_POWER_ADAPT_GROUP];
} __ATTRIB_PACK__ MrvlIEtypes_PowerAdapt_Group_t;

typedef struct _AutoTx_MacFrame_t
{
    u16 Interval;               /* in seconds */
    u8 Priority;                /* User Priority: 0~7, ignored if non-WMM */
    u8 Reserved;                /* set to 0 */
    u16 FrameLen;               /* Length of MAC frame payload */
    u8 DestMacAddr[ETH_ALEN];
    u8 SrcMacAddr[ETH_ALEN];
    u8 Payload[];
} __ATTRIB_PACK__ AutoTx_MacFrame_t;

/** MrvlIEtypes_AutoTx_t */
typedef struct _MrvlIEtypes_AutoTx_t
{
    MrvlIEtypesHeader_t Header;
    AutoTx_MacFrame_t AutoTx_MacFrame;
} __ATTRIB_PACK__ MrvlIEtypes_AutoTx_t;

typedef struct
{
    u8 value;
    u8 Freq;
} Threshold;

typedef struct
{
    u16 EventsBitmap;           /* bit 0: RSSI low,  bit 1: SNR low,
                                 * bit 2: RSSI high, bit 3: SNR high
                                 */
    Threshold Rssi_low;
    Threshold Snr_low;
    Threshold Rssi_high;
    Threshold Snr_high;
    u8 Rssi_low_count;
    u8 Snr_low_count;
    u8 Rssi_high_count;
    u8 Snr_high_count;
} wlan_subscribe_event;

/** Auth type to be used in the Authentication portion of an Assoc seq */
typedef struct
{
    MrvlIEtypesHeader_t Header;
    u16 AuthType;
} __ATTRIB_PACK__ MrvlIEtypes_AuthType_t;

#define MRVDRV_MAX_SUBBAND_802_11D		83
#define COUNTRY_CODE_LEN			3

/** Data structure for Country IE*/
typedef struct _IEEEtypes_SubbandSet
{
    u8 FirstChan;
    u8 NoOfChan;
    u8 MaxTxPwr;
} __ATTRIB_PACK__ IEEEtypes_SubbandSet_t;

typedef struct _IEEEtypes_CountryInfoSet
{
    u8 ElementId;
    u8 Len;
    u8 CountryCode[COUNTRY_CODE_LEN];
    IEEEtypes_SubbandSet_t Subband[1];
} __ATTRIB_PACK__ IEEEtypes_CountryInfoSet_t;

typedef struct _IEEEtypes_CountryInfoFullSet
{
    u8 ElementId;
    u8 Len;
    u8 CountryCode[COUNTRY_CODE_LEN];
    IEEEtypes_SubbandSet_t Subband[MRVDRV_MAX_SUBBAND_802_11D];
} __ATTRIB_PACK__ IEEEtypes_CountryInfoFullSet_t;

typedef struct _MrvlIEtypes_DomainParamSet
{
    MrvlIEtypesHeader_t Header;
    u8 CountryCode[COUNTRY_CODE_LEN];
    IEEEtypes_SubbandSet_t Subband[1];
} __ATTRIB_PACK__ MrvlIEtypes_DomainParamSet_t;

/** Size of a TSPEC.  Used to allocate necessary buffer space in commands */
#define WMM_TSPEC_SIZE              63

/** Extra IE bytes allocated in messages for appended IEs after a TSPEC */
#define WMM_ADDTS_EXTRA_IE_BYTES    256

/** Extra TLV bytes allocated in messages for configuring WMM Queues */
#define WMM_QUEUE_CONFIG_EXTRA_TLV_BYTES 64
/** Maximum number of AC QOS queues available in the driver/firmware */
#define MAX_AC_QUEUES 4

/** enum of WMM AC_QUEUES */
typedef enum
{
    WMM_AC_BK,
    WMM_AC_BE,
    WMM_AC_VI,
    WMM_AC_VO,
} __ATTRIB_PACK__ wlan_wmm_ac_e;

/** data structure of WMM QoS information */
typedef struct
{
    u8 ParaSetCount:4;
    u8 Reserved:3;
    u8 QosUAPSD:1;
} __ATTRIB_PACK__ IEEEtypes_WmmQosInfo_t;

typedef struct
{
    u8 Aifsn:4;
    u8 Acm:1;
    u8 Aci:2;
    u8 Reserved:1;
} __ATTRIB_PACK__ IEEEtypes_WmmAciAifsn_t;

/**  data structure of WMM ECW */
typedef struct
{
    u8 EcwMin:4;
    u8 EcwMax:4;
} __ATTRIB_PACK__ IEEEtypes_WmmEcw_t;

/** data structure of WMM AC parameters  */
typedef struct
{
    IEEEtypes_WmmAciAifsn_t AciAifsn;
    IEEEtypes_WmmEcw_t Ecw;
    u16 TxopLimit;
} __ATTRIB_PACK__ IEEEtypes_WmmAcParameters_t;

/** data structure of WMM Info IE  */
typedef struct
{

    /**
     * WMM Info IE - Vendor Specific Header:
     *   ElementId   [221/0xdd]
     *   Len         [7] 
     *   Oui         [00:50:f2]
     *   OuiType     [2]
     *   OuiSubType  [0]
     *   Version     [1]
     */
    IEEEtypes_VendorHeader_t VendHdr;

    IEEEtypes_WmmQosInfo_t QoSInfo;

} __ATTRIB_PACK__ IEEEtypes_WmmInfo_t;

/** data structure of WMM parameter IE  */
typedef struct
{
    /**
     * WMM Parameter IE - Vendor Specific Header:
     *   ElementId   [221/0xdd]
     *   Len         [24] 
     *   Oui         [00:50:f2]
     *   OuiType     [2]
     *   OuiSubType  [1]
     *   Version     [1]
     */
    IEEEtypes_VendorHeader_t VendHdr;

    IEEEtypes_WmmQosInfo_t QoSInfo;
    u8 Reserved;

    /** AC Parameters Record WMM_AC_BE, WMM_AC_BK, WMM_AC_VI, WMM_AC_VO */
    IEEEtypes_WmmAcParameters_t AcParams[MAX_AC_QUEUES];

} __ATTRIB_PACK__ IEEEtypes_WmmParameter_t;

/**
 *  @brief Firmware command structure to retrieve the firmware WMM status.
 *
 *  Used to retrieve the status of each WMM AC Queue in TLV 
 *    format (MrvlIEtypes_WmmQueueStatus_t) as well as the current WMM
 *    parameter IE advertised by the AP.  
 *  
 *  Used in response to a MACREG_INT_CODE_WMM_STATUS_CHANGE event signalling
 *    a QOS change on one of the ACs or a change in the WMM Parameter in
 *    the Beacon.
 *
 *  TLV based command, byte arrays used for max sizing purpose. There are no 
 *    arguments sent in the command, the TLVs are returned by the firmware.
 */
typedef struct
{
    u8 queueStatusTlv[sizeof(MrvlIEtypes_WmmQueueStatus_t) * MAX_AC_QUEUES];
    u8 wmmParamTlv[sizeof(IEEEtypes_WmmParameter_t) + 2];

}
__ATTRIB_PACK__ HostCmd_DS_WMM_GET_STATUS;

/**
 *  @brief Enumeration for the command result from an ADDTS or DELTS command 
 */
typedef enum
{
    TSPEC_RESULT_SUCCESS = 0,
    TSPEC_RESULT_EXEC_FAILURE = 1,
    TSPEC_RESULT_TIMEOUT = 2,
    TSPEC_RESULT_DATA_INVALID = 3,

} __ATTRIB_PACK__ wlan_wmm_tspec_result_e;

/**
 *  @brief IOCTL structure to send an ADDTS request and retrieve the response.
 *
 *  IOCTL structure from the application layer relayed to firmware to 
 *    instigate an ADDTS management frame with an appropriate TSPEC IE as well
 *    as any additional IEs appended in the ADDTS Action frame.
 *
 *  @sa wlan_wmm_addts_req_ioctl
 */
typedef struct
{
    wlan_wmm_tspec_result_e commandResult;
    u32 timeout_ms;

    u8 ieeeStatusCode;

    u8 tspecData[WMM_TSPEC_SIZE];

    u8 addtsExtraIEBuf[WMM_ADDTS_EXTRA_IE_BYTES];

} __ATTRIB_PACK__ wlan_ioctl_wmm_addts_req_t;

/**
 *  @brief IOCTL structure to send a DELTS request.
 *
 *  IOCTL structure from the application layer relayed to firmware to 
 *    instigate an DELTS management frame with an appropriate TSPEC IE.
 *
 *  @sa wlan_wmm_delts_req_ioctl
 */
typedef struct
{
    wlan_wmm_tspec_result_e commandResult;      //!< Firmware execution result

    u8 ieeeReasonCode;          //!< IEEE reason code sent, unused for WMM 

    u8 tspecData[WMM_TSPEC_SIZE];       //!< TSPEC to send in the DELTS

} __ATTRIB_PACK__ wlan_ioctl_wmm_delts_req_t;

/**
 *  @brief Internal command structure used in executing an ADDTS command.
 *
 *  Relay information between the IOCTL layer and the firmware command and 
 *    command response procedures.
 *
 *  @sa wlan_wmm_addts_req_ioctl
 *  @sa wlan_cmd_wmm_addts_req
 *  @sa wlan_cmdresp_wmm_addts_req
 */
typedef struct
{
    wlan_wmm_tspec_result_e commandResult;
    u32 timeout_ms;

    u8 dialogToken;
    u8 ieeeStatusCode;

    int tspecDataLen;
    u8 tspecData[WMM_TSPEC_SIZE];
    u8 addtsExtraIEBuf[WMM_ADDTS_EXTRA_IE_BYTES];

} wlan_cmd_wmm_addts_req_t;

/**
 *  @brief Internal command structure used in executing an DELTS command.
 *
 *  Relay information between the IOCTL layer and the firmware command and 
 *    command response procedures.
 *
 *  @sa wlan_wmm_delts_req_ioctl
 *  @sa wlan_cmd_wmm_delts_req
 *  @sa wlan_cmdresp_wmm_delts_req
 */
typedef struct
{
    wlan_wmm_tspec_result_e commandResult;

    u8 dialogToken;

    u8 ieeeReasonCode;

    int tspecDataLen;
    u8 tspecData[WMM_TSPEC_SIZE];

} wlan_cmd_wmm_delts_req_t;

/**
 *  @brief Command structure for the HostCmd_CMD_WMM_ADDTS_REQ firmware command
 *
 */
typedef struct
{
    wlan_wmm_tspec_result_e commandResult;
    u32 timeout_ms;

    u8 dialogToken;
    u8 ieeeStatusCode;
    u8 tspecData[WMM_TSPEC_SIZE];
    u8 addtsExtraIEBuf[WMM_ADDTS_EXTRA_IE_BYTES];

} __ATTRIB_PACK__ HostCmd_DS_WMM_ADDTS_REQ;

/**
 *  @brief Command structure for the HostCmd_CMD_WMM_DELTS_REQ firmware command
 */
typedef struct
{
    wlan_wmm_tspec_result_e commandResult;
    u8 dialogToken;
    u8 ieeeReasonCode;
    u8 tspecData[WMM_TSPEC_SIZE];

} __ATTRIB_PACK__ HostCmd_DS_WMM_DELTS_REQ;

/**
 *  @brief Enumeration for the action field in the Queue configure command
 */
typedef enum
{
    WMM_QUEUE_CONFIG_ACTION_GET = 0,
    WMM_QUEUE_CONFIG_ACTION_SET = 1,
    WMM_QUEUE_CONFIG_ACTION_DEFAULT = 2,

    WMM_QUEUE_CONFIG_ACTION_MAX
} __ATTRIB_PACK__ wlan_wmm_queue_config_action_e;

/**
 *  @brief Command structure for the HostCmd_CMD_WMM_QUEUE_CONFIG firmware cmd
 *
 *  Set/Get/Default the Queue parameters for a specific AC in the firmware.
 *
 */
typedef struct
{
    wlan_wmm_queue_config_action_e action;      //!< Set, Get, or Default
    wlan_wmm_ac_e accessCategory;       //!< WMM_AC_BK(0) to WMM_AC_VO(3)

    /** @brief MSDU lifetime expiry per 802.11e
     *
     *   - Ignored if 0 on a set command 
     *   - Set to the 802.11e specified 500 TUs when defaulted
     */
    u16 msduLifetimeExpiry;

    u8 tlvBuffer[WMM_QUEUE_CONFIG_EXTRA_TLV_BYTES];     //!< Not supported yet

} __ATTRIB_PACK__ HostCmd_DS_WMM_QUEUE_CONFIG;

/**
 *  @brief Internal command structure used in executing a queue config command.
 *
 *  Relay information between the IOCTL layer and the firmware command and 
 *    command response procedures.
 *
 *  @sa wlan_wmm_queue_config_ioctl
 *  @sa wlan_cmd_wmm_queue_config
 *  @sa wlan_cmdresp_wmm_queue_config
 */
typedef struct
{
    wlan_wmm_queue_config_action_e action;      //!< Set, Get, or Default
    wlan_wmm_ac_e accessCategory;       //!< WMM_AC_BK(0) to WMM_AC_VO(3)
    u16 msduLifetimeExpiry;     //!< lifetime expiry in TUs

    int tlvBufLen;              //!< Not supported yet
    u8 tlvBuffer[WMM_QUEUE_CONFIG_EXTRA_TLV_BYTES];     //!< Not supported yet

} wlan_cmd_wmm_queue_config_t;

/**
 *  @brief IOCTL structure to configure a specific AC Queue's parameters
 *
 *  IOCTL structure from the application layer relayed to firmware to 
 *    get, set, or default the WMM AC queue parameters.
 *
 *  - msduLifetimeExpiry is ignored if set to 0 on a set command
 *
 *  @sa wlan_wmm_queue_config_ioctl
 */
typedef struct
{
    wlan_wmm_queue_config_action_e action;      //!< Set, Get, or Default
    wlan_wmm_ac_e accessCategory;       //!< WMM_AC_BK(0) to WMM_AC_VO(3)
    u16 msduLifetimeExpiry;     //!< lifetime expiry in TUs

    u8 supportedRates[10];      //!< Not supported yet

} __ATTRIB_PACK__ wlan_ioctl_wmm_queue_config_t;

/**
 *   @brief Enumeration for the action field in the queue stats command
 */
typedef enum
{
    WMM_STATS_ACTION_START = 0,
    WMM_STATS_ACTION_STOP = 1,
    WMM_STATS_ACTION_GET_CLR = 2,
    WMM_STATS_ACTION_SET_CFG = 3,       /* Not currently used */
    WMM_STATS_ACTION_GET_CFG = 4,       /* Not currently used */

    WMM_STATS_ACTION_MAX
} __ATTRIB_PACK__ wlan_wmm_stats_action_e;

/** Number of bins in the histogram for the HostCmd_DS_WMM_QUEUE_STATS */
#define WMM_STATS_PKTS_HIST_BINS  7

/**
 *  @brief Command structure for the HostCmd_CMD_WMM_QUEUE_STATS firmware cmd
 *
 *  Turn statistical collection on/off for a given AC or retrieve the 
 *    accumulated stats for an AC and clear them in the firmware.
 */
typedef struct
{
    wlan_wmm_stats_action_e action;     //!< Start, Stop, or Get 
    wlan_wmm_ac_e accessCategory;       //!< WMM_AC_BK(0) to WMM_AC_VO(3)

    u16 pktCount;               //!< Number of successful packets transmitted
    u16 pktLoss;                //!< Packets lost; not included in pktCount
    u32 avgQueueDelay;          //!< Average Queue delay in microseconds
    u32 avgTxDelay;             //!< Average Transmission delay in microseconds
    u32 usedTime;               //!< Calculated medium time - Not currently used

    /** @brief Queue Delay Histogram; number of packets per queue delay range
     * 
     *  [0] -  0ms <= delay < 5ms
     *  [1] -  5ms <= delay < 10ms
     *  [2] - 10ms <= delay < 20ms
     *  [3] - 20ms <= delay < 30ms
     *  [4] - 30ms <= delay < 40ms
     *  [5] - 40ms <= delay < 50ms
     *  [6] - 50ms <= delay < msduLifetime (TUs)
     */
    u16 delayHistogram[WMM_STATS_PKTS_HIST_BINS];

    u16 reserved_u16_1;

} __ATTRIB_PACK__ HostCmd_DS_WMM_QUEUE_STATS;

/**
 *  @brief IOCTL structure to start, stop, and get statistics for a WMM AC
 *
 *  IOCTL structure from the application layer relayed to firmware to 
 *    start or stop statistical collection for a given AC.  Also used to 
 *    retrieve and clear the collected stats on a given AC.
 *
 *  @sa wlan_wmm_queue_stats_ioctl
 */
typedef struct
{
    wlan_wmm_stats_action_e action;     //!< Start, Stop, or Get 
    wlan_wmm_ac_e accessCategory;       //!< WMM_AC_BK(0) to WMM_AC_VO(3)
    u16 pktCount;               //!< Number of successful packets transmitted  
    u16 pktLoss;                //!< Packets lost; not included in pktCount    
    u32 avgQueueDelay;          //!< Average Queue delay in microseconds
    u32 avgTxDelay;             //!< Average Transmission delay in microseconds
    u32 usedTime;               //!< Calculated medium time 

    /** @brief Queue Delay Histogram; number of packets per queue delay range
     * 
     *  [0] -  0ms <= delay < 5ms
     *  [1] -  5ms <= delay < 10ms
     *  [2] - 10ms <= delay < 20ms
     *  [3] - 20ms <= delay < 30ms
     *  [4] - 30ms <= delay < 40ms
     *  [5] - 40ms <= delay < 50ms
     *  [6] - 50ms <= delay < msduLifetime (TUs)
     */
    u16 delayHistogram[WMM_STATS_PKTS_HIST_BINS];
} __ATTRIB_PACK__ wlan_ioctl_wmm_queue_stats_t;

/** 
 *  @brief IOCTL sub structure for a specific WMM AC Status
 */
typedef struct
{
    u8 wmmAcm;
    u8 flowRequired;
    u8 flowCreated;
    u8 disabled;
} __ATTRIB_PACK__ wlan_ioctl_wmm_queue_status_ac_t;

/**
 *  @brief IOCTL structure to retrieve the WMM AC Queue status
 *
 *  IOCTL structure from the application layer to retrieve:
 *     - ACM bit setting for the AC
 *     - Firmware status (flow required, flow created, flow disabled)
 *
 *  @sa wlan_wmm_queue_status_ioctl
 */
typedef struct
{
    wlan_ioctl_wmm_queue_status_ac_t acStatus[MAX_AC_QUEUES];
} __ATTRIB_PACK__ wlan_ioctl_wmm_queue_status_t;

/** Firmware status for a specific AC */
typedef struct
{
    u8 Disabled;
    u8 FlowRequired;
    u8 FlowCreated;
} WmmAcStatus_t;

/* hostcmd.h */
/*  802.11-related definitions */

/** TxPD descriptor */
typedef struct _TxPD
{
    /** Current Tx packet status */
    u32 TxStatus;
    /** Tx Control */
    u32 TxControl;
    u32 TxPacketLocation;
    /** Tx packet length */
    u16 TxPacketLength;
        /**Destination MAC address */
    u8 TxDestAddr[MRVDRV_ETH_ADDR_LEN];
    /** Pkt Priority */
    u8 Priority;
    /** Trasnit Pkt Flags*/
    u8 Flags;
    /** Amount of time the packet has been queued in the driver (units = 2ms)*/
    u8 PktDelay_2ms;
    /** Reserved */
    u8 Reserved1;

} __ATTRIB_PACK__ TxPD, *PTxPD;

/** RxPD Descriptor */
typedef struct _RxPD
{
        /** Current Rx packet status */
    u16 RxStatus;

        /** SNR */
    u8 SNR;

        /** Tx Control */
    u8 RxControl;

        /** Pkt Length */
    u16 PktLen;

        /** Noise Floor */
    u8 NF;

        /** Rx Packet Rate */
    u8 RxRate;

        /** Pkt offset */
    u32 PktOffset;
    u8 RxPacketType;
    u8 Reserved_1[3];
        /** Pkt Priority */
    u8 Priority;
    u8 Reserved[3];

} __ATTRIB_PACK__ RxPD, *PRxPD;

#if defined(__KERNEL__)

/** CmdCtrlNode */
typedef struct _CmdCtrlNode
{
    /* CMD link list */
    struct list_head list;

    u32 Status;

    /* CMD ID */
    WLAN_OID cmd_oid;

    /*CMD wait option: wait for finish or no wait */
    u16 wait_option;

    /* command parameter */
    void *pdata_buf;

    /*command data */
    u8 *BufVirtualAddr;

    u16 CmdFlags;

    /* wait queue */
    u16 CmdWaitQWoken;
    wait_queue_head_t cmdwait_q __ATTRIB_ALIGN__;
} __ATTRIB_PACK__ CmdCtrlNode, *PCmdCtrlNode;

#endif

/** MRVL_WEP_KEY */
typedef struct _MRVL_WEP_KEY
{
    u32 Length;
    u32 KeyIndex;
    u32 KeyLength;
    u8 KeyMaterial[MRVL_KEY_BUFFER_SIZE_IN_BYTE];
} __ATTRIB_PACK__ MRVL_WEP_KEY, *PMRVL_WEP_KEY;

typedef ULONGLONG WLAN_802_11_KEY_RSC;

/** WLAN_802_11_KEY */
typedef struct _WLAN_802_11_KEY
{
    u32 Length;
    u32 KeyIndex;
    u32 KeyLength;
    WLAN_802_11_MAC_ADDRESS BSSID;
    WLAN_802_11_KEY_RSC KeyRSC;
    u8 KeyMaterial[MRVL_MAX_KEY_WPA_KEY_LENGTH];
} __ATTRIB_PACK__ WLAN_802_11_KEY;

/** MRVL_WPA_KEY */
typedef struct _MRVL_WPA_KEY
{
    u32 KeyIndex;
    u32 KeyLength;
    u32 KeyRSC;
    u8 KeyMaterial[MRVL_MAX_KEY_WPA_KEY_LENGTH];
} MRVL_WPA_KEY, *PMRVL_WPA_KEY;

/** MRVL_WLAN_WPA_KEY */
typedef struct _MRVL_WLAN_WPA_KEY
{
    u8 EncryptionKey[16];
    u8 MICKey1[8];
    u8 MICKey2[8];
} MRVL_WLAN_WPA_KEY, *PMRVL_WLAN_WPA_KEY;

/* Received Signal Strength Indication  in dBm*/
typedef LONG WLAN_802_11_RSSI;

/** WLAN_802_11_WEP */
typedef struct _WLAN_802_11_WEP
{
    /* Length of this structure */
    u32 Length;

    /* 0 is the per-client key, 1-N are the global keys */
    u32 KeyIndex;

    /* length of key in bytes */
    u32 KeyLength;

    /* variable length depending on above field */
    u8 KeyMaterial[1];
} __ATTRIB_PACK__ WLAN_802_11_WEP;

/** WLAN_802_11_SSID */
typedef struct _WLAN_802_11_SSID
{
    /* SSID Length */
    u32 SsidLength;

    /* SSID information field */
    u8 Ssid[WLAN_MAX_SSID_LENGTH];
} __ATTRIB_PACK__ WLAN_802_11_SSID;

typedef u32 WLAN_802_11_FRAGMENTATION_THRESHOLD;
typedef u32 WLAN_802_11_RTS_THRESHOLD;
typedef u32 WLAN_802_11_ANTENNA;

/** wlan_offset_value */
typedef struct _wlan_offset_value
{
    u32 offset;
    u32 value;
} wlan_offset_value;

/** WLAN_802_11_FIXED_IEs */
typedef struct _WLAN_802_11_FIXED_IEs
{
    u8 Timestamp[8];
    u16 BeaconInterval;
    u16 Capabilities;
} WLAN_802_11_FIXED_IEs;

/** WLAN_802_11_VARIABLE_IEs */
typedef struct _WLAN_802_11_VARIABLE_IEs
{
    u8 ElementID;
    u8 Length;
    u8 data[1];
} WLAN_802_11_VARIABLE_IEs;

/** WLAN_802_11_AI_RESFI */
typedef struct _WLAN_802_11_AI_RESFI
{
    u16 Capabilities;
    u16 StatusCode;
    u16 AssociationId;
} WLAN_802_11_AI_RESFI;

/** WLAN_802_11_AI_REQFI */
typedef struct _WLAN_802_11_AI_REQFI
{
    u16 Capabilities;
    u16 ListenInterval;
    WLAN_802_11_MAC_ADDRESS CurrentAPAddress;
} WLAN_802_11_AI_REQFI;

/* Define general data structure */
/** HostCmd_DS_GEN */
typedef struct _HostCmd_DS_GEN
{
    u16 Command;
    u16 Size;
    u16 SeqNum;
    u16 Result;
} __ATTRIB_PACK__ HostCmd_DS_GEN, HostCmd_DS_802_11_DEEP_SLEEP;

#define S_DS_GEN    sizeof(HostCmd_DS_GEN)
/*
 * Define data structure for HostCmd_CMD_GET_HW_SPEC
 * This structure defines the response for the GET_HW_SPEC command
 */
/** HostCmd_DS_GET_HW_SPEC */
typedef struct _HostCmd_DS_GET_HW_SPEC
{
    /* HW Interface version number */
    u16 HWIfVersion;

    /* HW version number */
    u16 Version;

    /* Max number of TxPD FW can handle */
    u16 NumOfTxPD;

    /* Max no of Multicast address  */
    u16 NumOfMCastAdr;

    /* MAC address */
    u8 PermanentAddr[6];

    /* Region Code */
    u16 RegionCode;

    /* Number of antenna used */
    u16 NumberOfAntenna;

    /* FW release number, example 0x1234=1.2.3.4 */
    u32 FWReleaseNumber;

    u32 Reserved_1;

    u32 Reserved_2;

    u32 Reserved_3;

    /*FW/HW Capability */
    u32 fwCapInfo;
} __ATTRIB_PACK__ HostCmd_DS_GET_HW_SPEC;

typedef struct _HostCmd_DS_802_11_SUBSCRIBE_EVENT
{
    u16 Action;
    u16 Events;
} __ATTRIB_PACK__ HostCmd_DS_802_11_SUBSCRIBE_EVENT;

/* 
 * This scan handle Country Information IE(802.11d compliant) 
 * Define data structure for HostCmd_CMD_802_11_SCAN 
 */
/** HostCmd_DS_802_11_SCAN */
typedef struct _HostCmd_DS_802_11_SCAN
{
    u8 BSSType;
    u8 BSSID[ETH_ALEN];
    u8 TlvBuffer[1];
    /* MrvlIEtypes_SsIdParamSet_t   SsIdParamSet; 
     * MrvlIEtypes_ChanListParamSet_t       ChanListParamSet;
     * MrvlIEtypes_RatesParamSet_t  OpRateSet; 
     * */
} __ATTRIB_PACK__ HostCmd_DS_802_11_SCAN;

typedef struct _HostCmd_DS_802_11_SCAN_RSP
{
    u16 BSSDescriptSize;
    u8 NumberOfSets;
    u8 BssDescAndTlvBuffer[1];

} __ATTRIB_PACK__ HostCmd_DS_802_11_SCAN_RSP;

/** HostCmd_CMD_802_11_GET_LOG */
typedef struct _HostCmd_DS_802_11_GET_LOG
{
    u32 mcasttxframe;
    u32 failed;
    u32 retry;
    u32 multiretry;
    u32 framedup;
    u32 rtssuccess;
    u32 rtsfailure;
    u32 ackfailure;
    u32 rxfrag;
    u32 mcastrxframe;
    u32 fcserror;
    u32 txframe;
    u32 reserved;
} __ATTRIB_PACK__ HostCmd_DS_802_11_GET_LOG;

/**  HostCmd_CMD_MAC_CONTROL */
typedef struct _HostCmd_DS_MAC_CONTROL
{
    u16 Action;
    u16 Reserved;
} __ATTRIB_PACK__ HostCmd_DS_MAC_CONTROL;

/**  HostCmd_CMD_MAC_MULTICAST_ADR */
typedef struct _HostCmd_DS_MAC_MULTICAST_ADR
{
    u16 Action;
    u16 NumOfAdrs;
    u8 MACList[MRVDRV_ETH_ADDR_LEN * MRVDRV_MAX_MULTICAST_LIST_SIZE];
} __ATTRIB_PACK__ HostCmd_DS_MAC_MULTICAST_ADR;

/**  HostCmd_CMD_802_11_DEAUTHENTICATE */
typedef struct _HostCmd_DS_802_11_DEAUTHENTICATE
{
    u8 MacAddr[6];
    u16 ReasonCode;
} __ATTRIB_PACK__ HostCmd_DS_802_11_DEAUTHENTICATE;

/** HostCmd_DS_802_11_ASSOCIATE */
typedef struct _HostCmd_DS_802_11_ASSOCIATE
{
    u8 PeerStaAddr[6];
    IEEEtypes_CapInfo_t CapInfo;
    u16 ListenInterval;
    u8 Reserved1[3];

    /*
     *  MrvlIEtypes_SsIdParamSet_t  SsIdParamSet;
     *  MrvlIEtypes_PhyParamSet_t   PhyParamSet;
     *  MrvlIEtypes_SsParamSet_t    SsParamSet;
     *  MrvlIEtypes_RatesParamSet_t RatesParamSet;
     */
} __ATTRIB_PACK__ HostCmd_DS_802_11_ASSOCIATE;

/** HostCmd_CMD_802_11_ASSOCIATE response */
typedef struct
{
    IEEEtypes_AssocRsp_t assocRsp;
} __ATTRIB_PACK__ HostCmd_DS_802_11_ASSOCIATE_RSP;

/**  HostCmd_CMD_802_11_AD_HOC_START response */
typedef struct _HostCmd_DS_802_11_AD_HOC_RESULT
{
    u8 PAD[3];
    u8 BSSID[MRVDRV_ETH_ADDR_LEN];
} __ATTRIB_PACK__ HostCmd_DS_802_11_AD_HOC_RESULT;

/**  HostCmd_CMD_802_11_SET_WEP */
typedef struct _HostCmd_DS_802_11_SET_WEP
{
    /* ACT_ADD, ACT_REMOVE or ACT_ENABLE  */
    u16 Action;

    /* Key Index selected for Tx */
    u16 KeyIndex;

    /* 40, 128bit or TXWEP */
    u8 WEPTypeForKey1;

    u8 WEPTypeForKey2;
    u8 WEPTypeForKey3;
    u8 WEPTypeForKey4;
    u8 WEP1[16];
    u8 WEP2[16];
    u8 WEP3[16];
    u8 WEP4[16];
} __ATTRIB_PACK__ HostCmd_DS_802_11_SET_WEP;

/** HostCmd_DS_802_11_AD_HOC_STOP */
typedef struct _HostCmd_DS_802_11_AD_HOC_STOP
{

} __ATTRIB_PACK__ HostCmd_DS_802_11_AD_HOC_STOP;

/**  HostCmd_CMD_802_11_SNMP_MIB */
typedef struct _HostCmd_DS_802_11_SNMP_MIB
{
    u16 QueryType;
    u16 OID;
    u16 BufSize;
    u8 Value[128];
} __ATTRIB_PACK__ HostCmd_DS_802_11_SNMP_MIB;

/** HostCmd_CMD_MAC_REG_ACCESS */
typedef struct _HostCmd_DS_MAC_REG_ACCESS
{
    u16 Action;
    u16 Offset;
    u32 Value;
} __ATTRIB_PACK__ HostCmd_DS_MAC_REG_ACCESS;

/** HostCmd_CMD_BBP_REG_ACCESS */
typedef struct _HostCmd_DS_BBP_REG_ACCESS
{
    u16 Action;
    u16 Offset;
    u8 Value;
    u8 Reserved[3];
} __ATTRIB_PACK__ HostCmd_DS_BBP_REG_ACCESS;

/**  HostCmd_CMD_RF_REG_ACCESS */
typedef struct _HostCmd_DS_RF_REG_ACCESS
{
    u16 Action;
    u16 Offset;
    u8 Value;
    u8 Reserved[3];
} __ATTRIB_PACK__ HostCmd_DS_RF_REG_ACCESS;

/** HostCmd_CMD_802_11_RADIO_CONTROL */
typedef struct _HostCmd_DS_802_11_RADIO_CONTROL
{
    u16 Action;
    u16 Control;
} __ATTRIB_PACK__ HostCmd_DS_802_11_RADIO_CONTROL;

/* HostCmd_DS_802_11_SLEEP_PARAMS */
typedef struct _HostCmd_DS_802_11_SLEEP_PARAMS
{
    /* ACT_GET/ACT_SET */
    u16 Action;

    /* Sleep clock error in ppm */
    u16 Error;

    /* Wakeup offset in usec */
    u16 Offset;

    /* Clock stabilization time in usec */
    u16 StableTime;

    /* Control periodic calibration */
    u8 CalControl;

    /* Control the use of external sleep clock */
    u8 ExternalSleepClk;

    /* Reserved field, should be set to zero */
    u16 Reserved;
} __ATTRIB_PACK__ HostCmd_DS_802_11_SLEEP_PARAMS;

/* HostCmd_DS_802_11_SLEEP_PERIOD */
typedef struct _HostCmd_DS_802_11_SLEEP_PERIOD
{
    /* ACT_GET/ACT_SET */
    u16 Action;

    /* Sleep Period in msec */
    u16 Period;
} __ATTRIB_PACK__ HostCmd_DS_802_11_SLEEP_PERIOD;

/* HostCmd_DS_802_11_BCA_TIMESHARE */
typedef struct _HostCmd_DS_802_11_BCA_TIMESHARE
{
    /* ACT_GET/ACT_SET */
    u16 Action;

    /* Type: WLAN, BT */
    u16 TrafficType;

    /* 20msec - 60000msec */
    u32 TimeShareInterval;

    /* PTA arbiter time in msec */
    u32 BTTime;
} __ATTRIB_PACK__ HostCmd_DS_802_11_BCA_TIMESHARE;

/* HostCmd_DS_802_11_INACTIVITY_TIMEOUT */
typedef struct _HostCmd_DS_802_11_INACTIVITY_TIMEOUT
{
    /* ACT_GET/ACT_SET */
    u16 Action;

    /* Inactivity timeout in msec */
    u16 Timeout;
} __ATTRIB_PACK__ HostCmd_DS_802_11_INACTIVITY_TIMEOUT;

/** HostCmd_CMD_802_11_RF_CHANNEL */
typedef struct _HostCmd_DS_802_11_RF_CHANNEL
{
    u16 Action;
    u16 CurrentChannel;
    u16 RFType;
    u16 Reserved;
    u8 ChannelList[32];
} __ATTRIB_PACK__ HostCmd_DS_802_11_RF_CHANNEL;

/**  HostCmd_CMD_802_11_RSSI */
typedef struct _HostCmd_DS_802_11_RSSI
{
    /* weighting factor */
    u16 N;

    u16 Reserved_0;
    u16 Reserved_1;
    u16 Reserved_2;
} __ATTRIB_PACK__ HostCmd_DS_802_11_RSSI;

/** HostCmd_DS_802_11_RSSI_RSP */
typedef struct _HostCmd_DS_802_11_RSSI_RSP
{
    u16 SNR;
    u16 NoiseFloor;
    u16 AvgSNR;
    u16 AvgNoiseFloor;
} __ATTRIB_PACK__ HostCmd_DS_802_11_RSSI_RSP;

/** HostCmd_DS_802_11_MAC_ADDRESS */
typedef struct _HostCmd_DS_802_11_MAC_ADDRESS
{
    u16 Action;
    u8 MacAdd[ETH_ALEN];
} __ATTRIB_PACK__ HostCmd_DS_802_11_MAC_ADDRESS;

/** HostCmd_CMD_802_11_RF_TX_POWER */
typedef struct _HostCmd_DS_802_11_RF_TX_POWER
{
    u16 Action;
    u16 CurrentLevel;
    u8 MaxPower;
    u8 MinPower;
} __ATTRIB_PACK__ HostCmd_DS_802_11_RF_TX_POWER;

/** HostCmd_CMD_802_11_RF_ANTENNA */
typedef struct _HostCmd_DS_802_11_RF_ANTENNA
{
    u16 Action;

    /*  Number of antennas or 0xffff(diversity) */
    u16 AntennaMode;

} __ATTRIB_PACK__ HostCmd_DS_802_11_RF_ANTENNA;

/** HostCmd_CMD_802_11_PS_MODE */
typedef struct _HostCmd_DS_802_11_PS_MODE
{
    u16 Action;
    u16 NullPktInterval;
    u16 MultipleDtim;
    u16 BCNMissTimeOut;
    u16 LocalListenInterval;
    u16 AdhocAwakePeriod;
} __ATTRIB_PACK__ HostCmd_DS_802_11_PS_MODE;

/** PS_CMD_ConfirmSleep */
typedef struct _PS_CMD_ConfirmSleep
{
    u16 Command;
    u16 Size;
    u16 SeqNum;
    u16 Result;

    u16 Action;
    u16 Reserved1;
    u16 MultipleDtim;
    u16 Reserved;
    u16 LocalListenInterval;
} __ATTRIB_PACK__ PS_CMD_ConfirmSleep, *PPS_CMD_ConfirmSleep;

/** HostCmd_CMD_802_11_FW_WAKE_METHOD */
typedef struct _HostCmd_DS_802_11_FW_WAKEUP_METHOD
{
    u16 Action;
    u16 Method;
} __ATTRIB_PACK__ HostCmd_DS_802_11_FW_WAKEUP_METHOD;

/** HostCmd_DS_802_11_RATE_ADAPT_RATESET */
typedef struct _HostCmd_DS_802_11_RATE_ADAPT_RATESET
{
    u16 Action;
    u16 HWRateDropMode;
    u16 Bitmap;
    u16 Threshold;
    u16 FinalRate;
} __ATTRIB_PACK__ HostCmd_DS_802_11_RATE_ADAPT_RATESET;

/** HostCmd_DS_802_11_AD_HOC_START*/
typedef struct _HostCmd_DS_802_11_AD_HOC_START
{
    u8 SSID[MRVDRV_MAX_SSID_LENGTH];
    u8 BSSType;
    u16 BeaconPeriod;
    u8 DTIMPeriod;
    IEEEtypes_SsParamSet_t SsParamSet;
    IEEEtypes_PhyParamSet_t PhyParamSet;
    u16 Reserved1;
    IEEEtypes_CapInfo_t Cap;
    u8 DataRate[HOSTCMD_SUPPORTED_RATES];
} __ATTRIB_PACK__ HostCmd_DS_802_11_AD_HOC_START;

/** AdHoc_BssDesc_t */
typedef struct _AdHoc_BssDesc_t
{
    u8 BSSID[6];
    u8 SSID[32];
    u8 BSSType;
    u16 BeaconPeriod;
    u8 DTIMPeriod;
    u8 TimeStamp[8];
    u8 LocalTime[8];
    IEEEtypes_PhyParamSet_t PhyParamSet;
    IEEEtypes_SsParamSet_t SsParamSet;
    IEEEtypes_CapInfo_t Cap;
    u8 DataRates[HOSTCMD_SUPPORTED_RATES];

    /* DO NOT ADD ANY FIELDS TO THIS STRUCTURE.      It is used below in the
     *      Adhoc join command and will cause a binary layout mismatch with 
     *      the firmware 
     */
} __ATTRIB_PACK__ AdHoc_BssDesc_t;

/** HostCmd_DS_802_11_AD_HOC_JOIN */
typedef struct _HostCmd_DS_802_11_AD_HOC_JOIN
{
    AdHoc_BssDesc_t BssDescriptor;
    u16 Reserved1;
    u16 Reserved2;

} __ATTRIB_PACK__ HostCmd_DS_802_11_AD_HOC_JOIN;

typedef union _KeyInfo_WEP_t
{
    u8 Reserved;

    /* bits 1-4: Specifies the index of key */
    u8 WepKeyIndex;

    /* bit 0: Specifies that this key is 
     * to be used as the default for TX data packets 
     * */
    u8 isWepDefaultKey;
} __ATTRIB_PACK__ KeyInfo_WEP_t;

typedef union _KeyInfo_TKIP_t
{
    u8 Reserved;

    /* bit 2: Specifies that this key is 
     * enabled and valid to use */
    u8 isKeyEnabled;

    /* bit 1: Specifies that this key is
     * to be used as the unicast key */
    u8 isUnicastKey;

    /* bit 0: Specifies that this key is 
     * to be used as the multicast key */
    u8 isMulticastKey;
} __ATTRIB_PACK__ KeyInfo_TKIP_t;

typedef union _KeyInfo_AES_t
{
    u8 Reserved;

    /* bit 2: Specifies that this key is
     * enabled and valid to use */
    u8 isKeyEnabled;

    /* bit 1: Specifies that this key is
     * to be used as the unicast key */
    u8 isUnicastKey;

    /* bit 0: Specifies that this key is 
     * to be used as the multicast key */
    u8 isMulticastKey;
} __ATTRIB_PACK__ KeyInfo_AES_t;

/** KeyMaterial_TKIP_t */
typedef struct _KeyMaterial_TKIP_t
{
    /* TKIP encryption/decryption key */
    u8 TkipKey[16];

    /* TKIP TX MIC Key */
    u8 TkipTxMicKey[16];

    /* TKIP RX MIC Key */
    u8 TkipRxMicKey[16];
} __ATTRIB_PACK__ KeyMaterial_TKIP_t, *PKeyMaterial_TKIP_t;

/** KeyMaterial_AES_t */
typedef struct _KeyMaterial_AES_t
{
    /* AES encryption/decryption key */
    u8 AesKey[16];
} __ATTRIB_PACK__ KeyMaterial_AES_t, *PKeyMaterial_AES_t;

/** MrvlIEtype_KeyParamSet_t */
typedef struct _MrvlIEtype_KeyParamSet_t
{
    /* Type ID */
    u16 Type;

    /* Length of Payload */
    u16 Length;

    /* Type of Key: WEP=0, TKIP=1, AES=2 */
    u16 KeyTypeId;

    /* Key Control Info specific to a KeyTypeId */
    u16 KeyInfo;

    /* Length of key */
    u16 KeyLen;

    /* Key material of size KeyLen */
    u8 Key[32];
} __ATTRIB_PACK__ MrvlIEtype_KeyParamSet_t, *PMrvlIEtype_KeyParamSet_t;

/** HostCmd_DS_802_11_KEY_MATERIAL */
typedef struct _HostCmd_DS_802_11_KEY_MATERIAL
{
    u16 Action;

    MrvlIEtype_KeyParamSet_t KeyParamSet;
} __ATTRIB_PACK__ HostCmd_DS_802_11_KEY_MATERIAL;

/** HostCmd_DS_802_11_HOST_SLEEP_CFG */
typedef struct _HostCmd_DS_HOST_802_11_HOST_SLEEP_CFG
{
    /* bit0=1: non-unicast data
     * bit1=1: unicast data
     * bit2=1: mac events
     * bit3=1: magic packet 
     */
    u32 conditions;

    u8 gpio;

    /* in milliseconds */
    u8 gap;
} __ATTRIB_PACK__ HostCmd_DS_802_11_HOST_SLEEP_CFG;

#define CAL_DATA_HEADER_LEN	6       /* sizeof(HostCmd_DS_802_11_CAL_DATA_EXT)-sizeof(CalData) */

/** HostCmd_DS_802_11_CAL_DATA_EXT */
typedef struct _HostCmd_DS_802_11_CAL_DATA_EXT
{
    u16 Action;
    u16 Revision;
    u16 CalDataLen;
    u8 CalData[1];
} __ATTRIB_PACK__ HostCmd_DS_802_11_CAL_DATA_EXT;

typedef struct _HostCmd_DS_802_11_CFG_DATA
{
    u16 u16Action;
    u16 u16Type;
    u16 u16DataLen;
    u8 u8Data[1];
} __ATTRIB_PACK__ HostCmd_DS_802_11_CFG_DATA, *pHostCmd_DS_802_11_CFG_DATA;

/** HostCmd_DS_802_11_EEPROM_ACCESS */
typedef struct _HostCmd_DS_802_11_EEPROM_ACCESS
{
    u16 Action;

    /* multiple 4 */
    u16 Offset;
    u16 ByteCount;
    u8 Value;
} __ATTRIB_PACK__ HostCmd_DS_802_11_EEPROM_ACCESS;

/** HostCmd_DS_802_11_BG_SCAN_CONFIG */
typedef struct _HostCmd_DS_802_11_BG_SCAN_CONFIG
{
    /** Action */
    u16 Action;

    /** 
     *  Enable/Disable
     *  0 - Disable
     *  1 - Enable 
     */
    u8 Enable;

    /**
     *  bssType
     *  1 - Infrastructure
     *  2 - IBSS
     *  3 - any 
     */
    u8 BssType;

    /** 
     * ChannelsPerScan 
     *   Number of channels to scan during a single scanning opportunity
     */
    u8 ChannelsPerScan;

    u8 Reserved1[3];

    /** ScanInterval */
    u32 ScanInterval;

    u8 Reserved2[4];

    /** 
     * ReportConditions
     * - SSID Match
     * - Exceed SNR Threshold
     * - Exceed RSSI Threshold
     * - Complete all channels
     */
    u32 ReportConditions;

    u8 Reserved3[2];

    /*  Attach TLV based parameters as needed:
     *
     *  MrvlIEtypes_SsIdParamSet_t          Set specific SSID filter
     *  MrvlIEtypes_ChanListParamSet_t      Set the channels & channel params
     *  MrvlIEtypes_NumProbes_t             Number of probes per SSID/broadcast
     *  MrvlIEtypes_WildCardSsIdParamSet_t  Wildcard SSID matching patterns
     *  MrvlIEtypes_SnrThreshold_t          SNR Threshold for match/report  
     */

} __ATTRIB_PACK__ HostCmd_DS_802_11_BG_SCAN_CONFIG;

/** HostCmd_DS_802_11_BG_SCAN_QUERY */
typedef struct _HostCmd_DS_802_11_BG_SCAN_QUERY
{
    u8 Flush;
} __ATTRIB_PACK__ HostCmd_DS_802_11_BG_SCAN_QUERY;

/** HostCmd_DS_802_11_BG_SCAN_QUERY_RSP */
typedef struct _HostCmd_DS_802_11_BG_SCAN_QUERY_RSP
{
    u32 ReportCondition;
    HostCmd_DS_802_11_SCAN_RSP scanresp;
} __ATTRIB_PACK__ HostCmd_DS_802_11_BG_SCAN_QUERY_RSP;

/** HostCmd_DS_802_11_TPC_CFG */
typedef struct _HostCmd_DS_802_11_TPC_CFG
{
    u16 Action;
    u8 Enable;
    s8 P0;
    s8 P1;
    s8 P2;
    u8 UseSNR;
} __ATTRIB_PACK__ HostCmd_DS_802_11_TPC_CFG;

/** HostCmd_DS_802_11_LED_CTRL */
typedef struct _HostCmd_DS_802_11_LED_CTRL
{
    u16 Action;                 /* 0 = ACT_GET; 1 = ACT_SET; */
    u16 LedNums;                /* Numbers of LEDs supported */
    MrvlIEtypes_LedGpio_t LedGpio;
    MrvlIEtypes_LedBehavior_t LedBehavior[1];
} __ATTRIB_PACK__ HostCmd_DS_802_11_LED_CTRL;

/** HostCmd_DS_802_11_POWER_ADAPT_CFG_EXT */
typedef struct _HostCmd_DS_802_11_POWER_ADAPT_CFG_EXT
{
        /** Action */
    u16 Action;                 /* 0 = ACT_GET; 1 = ACT_SET; */
    u16 EnablePA;               /* 0 = disable; 1 = enable; */
    MrvlIEtypes_PowerAdapt_Group_t PowerAdaptGroup;
} __ATTRIB_PACK__ HostCmd_DS_802_11_POWER_ADAPT_CFG_EXT;

typedef struct _HostCmd_DS_SDIO_PULL_CTRL
{
    u16 Action;                 /* 0: get; 1: set */
    u16 PullUp;                 /* the delay of pulling up in us */
    u16 PullDown;               /* the delay of pulling down in us */
} __ATTRIB_PACK__ HostCmd_DS_SDIO_PULL_CTRL;
typedef struct _HostCmd_DS_802_11_IBSS_Status
{
    u16 Action;
    u16 Enable;
    u8 BSSID[ETH_ALEN];
    u16 BeaconInterval;
    u16 ATIMWindow;
    u16 UseGRateProtection;
} __ATTRIB_PACK__ HostCmd_DS_802_11_IBSS_Status;

#define CIPHER_TEST_RC4 (1)
#define CIPHER_TEST_AES (2)
#define CIPHER_TEST_AES_KEY_WRAP (3)

/** HostCmd_DS_802_11_CRYPTO */
typedef struct _HostCmd_DS_802_11_CRYPTO
{
    u16 EncDec;                 //Decrypt=0, Encrypt=1
    u16 Algorithm;              // RC4=1 AES=2 , AES_KEY_WRAP=3
    u16 KeyIVLength;            //Length of Key IV (bytes)              
    u8 KeyIV[32];               //Key IV
    u16 KeyLength;              //Length of Key (bytes)
    u8 Key[32];                 //Key
    /* MrvlIEtypes_Data_t   data  Plain text if EncDec=Encrypt, Ciphertext data if EncDec=Decrypt */
} __ATTRIB_PACK__ HostCmd_DS_802_11_CRYPTO;

typedef struct _HostCmd_TX_RATE_QUERY
{
    u16 TxRate;
} __ATTRIB_PACK__ HostCmd_TX_RATE_QUERY;

/** HostCmd_DS_802_11_AUTO_TX */
typedef struct _HostCmd_DS_802_11_AUTO_TX
{
        /** Action */
    u16 Action;                 /* 0 = ACT_GET; 1 = ACT_SET; */
    MrvlIEtypes_AutoTx_t AutoTx;
} __ATTRIB_PACK__ HostCmd_DS_802_11_AUTO_TX;

/** HostCmd_MEM_ACCESS */
typedef struct _HostCmd_DS_MEM_ACCESS
{
        /** Action */
    u16 Action;                 /* 0 = ACT_GET; 1 = ACT_SET; */
    u16 Reserved;
    u32 Addr;
    u32 Value;
} __ATTRIB_PACK__ HostCmd_DS_MEM_ACCESS;

typedef struct
{
    u64 TsfValue;
} __ATTRIB_PACK__ HostCmd_DS_GET_TSF;

typedef struct _HostCmd_DS_MODULE_TYPE_CONFIG
{
    u16 Action;                 /* 0 = ACT_GET; 1 = ACT_SET; */
    u16 Module;
} __ATTRIB_PACK__ HostCmd_DS_MODULE_TYPE_CONFIG;

typedef struct _HostCmd_DS_VERSION_EXT
{
    u8 versionStrSel;
    char versionStr[128];
} __ATTRIB_PACK__ HostCmd_DS_VERSION_EXT;

/** Define data structure for HostCmd_CMD_802_11D_DOMAIN_INFO */
typedef struct _HostCmd_DS_802_11D_DOMAIN_INFO
{
    u16 Action;
    MrvlIEtypes_DomainParamSet_t Domain;
} __ATTRIB_PACK__ HostCmd_DS_802_11D_DOMAIN_INFO;

/** Define data structure for HostCmd_CMD_802_11D_DOMAIN_INFO response */
typedef struct _HostCmd_DS_802_11D_DOMAIN_INFO_RSP
{
    u16 Action;
    MrvlIEtypes_DomainParamSet_t Domain;
} __ATTRIB_PACK__ HostCmd_DS_802_11D_DOMAIN_INFO_RSP;

typedef struct _HostCmd_DS_MEF_CFG
{
    u32 Criteria;
    u16 NumEntries;
} __ATTRIB_PACK__ HostCmd_DS_MEF_CFG;

typedef struct _MEF_CFG_DATA
{
    u16 size;
    HostCmd_DS_MEF_CFG data;
} __ATTRIB_PACK__ MEF_CFG_DATA;

typedef struct _HostCmd_DS_DBGS_CFG
{
    u8 Destination;
    u8 ToAirChan;
    u8 reserved;
    u8 En_NumEntries;
} __ATTRIB_PACK__ HostCmd_DS_DBGS_CFG;

typedef struct _DBGS_ENTRY_DATA
{
    u16 ModeAndMaskorID;
    u16 BaseOrID;
} __ATTRIB_PACK__ DBGS_ENTRY_DATA;

typedef struct _DBGS_CFG_DATA
{
    u16 size;
    HostCmd_DS_DBGS_CFG data;
} __ATTRIB_PACK__ DBGS_CFG_DATA;

typedef struct _HostCmd_DS_GET_MEM
{
    u32 StartAddr;
    u16 Len;
} __ATTRIB_PACK__ HostCmd_DS_GET_MEM;

typedef struct _FW_MEM_DATA
{
    u16 size;
    HostCmd_DS_GET_MEM data;
} __ATTRIB_PACK__ FW_MEM_DATA;

typedef struct
{
    u32 PktInitCnt;
    u32 PktSuccessCnt;
    u32 TxAttempts;
    u32 RetryFailure;
    u32 ExpiryFailure;
} __ATTRIB_PACK__ HostCmd_DS_TX_PKT_STAT_Entry;

typedef struct
{
    HostCmd_DS_TX_PKT_STAT_Entry StatEntry[HOSTCMD_SUPPORTED_RATES];
} __ATTRIB_PACK__ HostCmd_DS_TX_PKT_STATS;

/**
 * @brief 802.11h Local Power Constraint Marvell extended TLV
 */
typedef struct
{
    MrvlIEtypesHeader_t Header; //!< Marvell TLV header: ID/Len
    u8 Chan;                    //!< Channel local constraint applies to

    //! Power constraint included in beacons and used by fw to offset 11d info
    u8 Constraint;

} __ATTRIB_PACK__ MrvlIEtypes_LocalPowerConstraint_t;

/*
 *
 * Data structures for driver/firmware command processing
 *
 */

//!  TPC Info structure sent in CMD_802_11_TPC_INFO command to firmware
typedef struct
{
    MrvlIEtypes_LocalPowerConstraint_t localConstraint; //!< Local constraint
    MrvlIEtypes_PowerCapability_t powerCap;     //!< Power Capability

} __ATTRIB_PACK__ HostCmd_DS_802_11_TPC_INFO;

//!  TPC Request structure sent in CMD_802_11_TPC_ADAPT_REQ command to firmware
typedef struct
{
    u8 destMac[MRVDRV_ETH_ADDR_LEN];    //!< Destination STA address 
    u16 timeout;                //!< Response timeout in ms
    u8 rateIndex;               //!< IEEE Rate index to send request

} __ATTRIB_PACK__ HostCmd_TpcRequest;

//!  TPC Response structure received from the CMD_802_11_TPC_ADAPT_REQ command
typedef struct
{
    u8 tpcRetCode;              //!< Firmware command result status code
    s8 txPower;                 //!< Reported TX Power from the TPC Report element
    s8 linkMargin;              //!< Reported link margin from the TPC Report element
    s8 rssi;                    //!< RSSI of the received TPC Report frame

} __ATTRIB_PACK__ HostCmd_TpcResponse;

//!  CMD_802_11_TPC_ADAPT_REQ substruct. Union of the TPC request and response
typedef union
{
    HostCmd_TpcRequest req;     //!< Request struct sent to firmware
    HostCmd_TpcResponse resp;   //!< Response struct received from firmware

} __ATTRIB_PACK__ HostCmd_DS_802_11_TPC_ADAPT_REQ;

//!  CMD_802_11_CHAN_SW_ANN firmware command substructure
typedef struct
{
    u8 switchMode;              //!< Set to 1 for a quiet switch request, no STA tx
    u8 newChan;                 //!< Requested new channel                       
    u8 switchCount;             //!< Number of TBTTs until the switch is to occur
} __ATTRIB_PACK__ HostCmd_DS_802_11_CHAN_SW_ANN;

/**        
*** @brief Include extra 11h measurement types (not currently supported 
***                                             in firmware)
**/
#define  WLAN_MEAS_EXTRA_11H_TYPES   0

/**        
*** @brief Include extra 11k measurement types (not currently implemented)
**/
#define  WLAN_MEAS_EXTRA_11K_TYPES   0

/**        
*** @brief Enumeration of measurement types, including max supported 
***        enum for 11h/11k
**/
typedef enum
{
    WLAN_MEAS_BASIC = 0,        //!< 11h: Basic
#if WLAN_MEAS_EXTRA_11H_TYPES
    /* Not supported in firmware */
    WLAN_MEAS_CCA = 1,          //!< 11h: CCA
    WLAN_MEAS_RPI = 2,          //!< 11h: RPI
#endif
#if WLAN_MEAS_EXTRA_11K_TYPES
    /* Future 11k extensions */
    WLAN_MEAS_CHAN_LOAD = 3,    //!< 11k: Channel Load
    WLAN_MEAS_NOISE_HIST = 4,   //!< 11k: Noise Histogram 
    WLAN_MEAS_BEACON = 5,       //!< 11k: Beacon 
    WLAN_MEAS_FRAME = 6,        //!< 11k: Frame
    WLAN_MEAS_HIDDEN_NODE = 7,  //!< 11k: Hidden Node
    WLAN_MEAS_MEDIUM_SENS_HIST = 8,     //!< 11k: Medium Sense Histogram
    WLAN_MEAS_STA_STATS = 9,    //!< 11k: Station Statistics
#endif

    WLAN_MEAS_NUM_TYPES,        //!< Number of enumerated measurements

#if WLAN_MEAS_EXTRA_11H_TYPES
    WLAN_MEAS_11H_MAX_TYPE = WLAN_MEAS_RPI,     //!< Max 11h measurement
#else
    WLAN_MEAS_11H_MAX_TYPE = WLAN_MEAS_BASIC,   //!< Max 11h measurement
#endif

#if WLAN_MEAS_EXTRA_11K_TYPES
    /* Future 11k extensions */
    WLAN_MEAS_11K_MAX_TYPE = WLAN_MEAS_STA_STATS,       //!< Max 11k measurement
#endif

} __ATTRIB_PACK__ MeasType_t;

/**        
*** @brief Mode octet of the measurement request element (7.3.2.21)
**/
typedef struct
{
    u8 parallel:1;              //!< 11k: series or parallel with previous meas
    u8 enable:1;                //!< 11h: enable report/request bits
    u8 request:1;               //!< 11h: en/disable requests of specified type
    u8 report:1;                //!< 11h: en/disable report rcpt. of spec. type
    u8 durationMandatory:1;     //!< 11k: duration spec. for meas. is mandatory
    u8 rsvd5_7:3;               //!< Reserved

} __ATTRIB_PACK__ MeasReqMode_t;

/**        
*** @brief Common measurement request structure (7.3.2.21.1 to 7.3.2.21.3)
**/
typedef struct
{
    u8 channel;                 //!< Channel to measure
    u64 startTime;              //!< TSF Start time of measurement (0 for immediate)
    u16 duration;               //!< TU duration of the measurement

} __ATTRIB_PACK__ MeasReqCommonFormat_t;

/**        
*** @brief Basic measurement request structure (7.3.2.21.1)
**/
typedef MeasReqCommonFormat_t MeasReqBasic_t;

/**        
*** @brief CCA measurement request structure (7.3.2.21.2)
**/
typedef MeasReqCommonFormat_t MeasReqCCA_t;

/**        
*** @brief RPI measurement request structure (7.3.2.21.3)
**/
typedef MeasReqCommonFormat_t MeasReqRPI_t;

/**        
*** @brief Union of the availble measurement request types.  Passed in the 
***        driver/firmware interface.
**/
typedef union
{
    MeasReqBasic_t basic;       //!< Basic measurement request
    MeasReqCCA_t cca;           //!< CCA measurement request
    MeasReqRPI_t rpi;           //!< RPI measurement request

} MeasRequest_t;

/**        
*** @brief Mode octet of the measurement report element (7.3.2.22)
**/
typedef struct
{
    u8 late:1;                  //!< Start TSF time missed for measurement
    u8 incapable:1;             //!< Incapable of performin measurement
    u8 refused:1;               //!< Measurement refused
    u8 rsvd3_7:5;               //!< Reserved

} __ATTRIB_PACK__ MeasRptMode_t;

/**
***  @brief Map octet of the basic measurment report (7.3.2.22.1)
**/
typedef struct
{
    u8 BSS:1;                   //!< At least one valid MPDU received on channel
    u8 OFDM_Preamble:1;         //!< OFDM preamble detected on channel
    u8 unidentifiedSig:1;       //!< Unidentified signal found on channel
    u8 radar:1;                 //!< Radar detected on channel
    u8 unmeasured:1;            //!< Channel is unmeasured
    u8 rsvd5_7:3;               //!< Reserved

} __ATTRIB_PACK__ MeasRptBasicMap_t;

/**        
*** @brief Basic measurement report (7.3.2.22.1)
**/
typedef struct
{
    u8 channel;                 //!< Channel to measured
    u64 startTime;              //!< Start time (TSF) of measurement
    u16 duration;               //!< Duration of measurement in TUs
    MeasRptBasicMap_t map;      //!< Basic measurement report

} __ATTRIB_PACK__ MeasRptBasic_t;

/**        
*** @brief CCA measurement report (7.3.2.22.2)
**/
typedef struct
{
    u8 channel;                 //!< Channel to measured            
    u64 startTime;              //!< Start time (TSF) of measurement
    u16 duration;               //!< Duration of measurement in TUs 
    u8 busyFraction;            //!< Fractional duration CCA indicated chan busy

} __ATTRIB_PACK__ MeasRptCCA_t;

/**        
*** @brief RPI measurement report (7.3.2.22.3)
**/
typedef struct
{
    u8 channel;                 //!< Channel to measured            
    u64 startTime;              //!< Start time (TSF) of measurement
    u16 duration;               //!< Duration of measurement in TUs 
    u8 density[8];              //!< RPI Density histogram report

} __ATTRIB_PACK__ MeasRptRPI_t;

/**        
*** @brief Union of the availble measurement report types.  Passed in the 
***        driver/firmware interface.
**/
typedef union
{
    MeasRptBasic_t basic;       //!< Basic measurement report
    MeasRptCCA_t cca;           //!< CCA measurement report
    MeasRptRPI_t rpi;           //!< RPI measurement report

} MeasReport_t;

/**        
*** @brief Structure passed to firmware to perform a measurement
**/
typedef struct
{
    u8 macAddr[MRVDRV_ETH_ADDR_LEN];    //!< Reporting STA address  
    u8 dialogToken;             //!< Measurement dialog toke
    MeasReqMode_t reqMode;      //!< Report mode            
    MeasType_t measType;        //!< Measurement type       
    MeasRequest_t req;          //!< Measurement request data

} __ATTRIB_PACK__ HostCmd_DS_MEASUREMENT_REQUEST;

/**        
*** @brief Structure passed back from firmware with a measurement report,
***        also can be to send a measurement report to another STA
**/
typedef struct
{
    u8 macAddr[MRVDRV_ETH_ADDR_LEN];    //!< Reporting STA address
    u8 dialogToken;             //!< Measurement dialog token
    MeasRptMode_t rptMode;      //!< Report mode 
    MeasType_t measType;        //!< Measurement type
    MeasReport_t rpt;           //!< Measurement report data

} __ATTRIB_PACK__ HostCmd_DS_MEASUREMENT_REPORT;

/** _HostCmd_DS_COMMAND*/
struct _HostCmd_DS_COMMAND
{

    /** Command Header */
    u16 Command;
    u16 Size;
    u16 SeqNum;
    u16 Result;

    /** Command Body */
    union
    {
        HostCmd_DS_GET_HW_SPEC hwspec;
        HostCmd_DS_802_11_PS_MODE psmode;
        HostCmd_DS_802_11_SCAN scan;
        HostCmd_DS_802_11_SCAN_RSP scanresp;
        HostCmd_DS_MAC_CONTROL macctrl;
        HostCmd_DS_802_11_ASSOCIATE associate;
        HostCmd_DS_802_11_ASSOCIATE_RSP associatersp;
        HostCmd_DS_802_11_DEAUTHENTICATE deauth;
        HostCmd_DS_802_11_SET_WEP wep;
        HostCmd_DS_802_11_AD_HOC_START ads;
        HostCmd_DS_802_11_AD_HOC_RESULT result;
        HostCmd_DS_802_11_GET_LOG glog;
        HostCmd_DS_802_11_SNMP_MIB smib;
        HostCmd_DS_802_11_RF_TX_POWER txp;
        HostCmd_DS_802_11_RF_ANTENNA rant;
        HostCmd_DS_802_11_RATE_ADAPT_RATESET rateset;
        HostCmd_DS_MAC_MULTICAST_ADR madr;
        HostCmd_DS_802_11_AD_HOC_JOIN adj;
        HostCmd_DS_802_11_RADIO_CONTROL radio;
        HostCmd_DS_802_11_RF_CHANNEL rfchannel;
        HostCmd_DS_802_11_RSSI rssi;
        HostCmd_DS_802_11_RSSI_RSP rssirsp;
        HostCmd_DS_802_11_AD_HOC_STOP adhoc_stop;
        HostCmd_DS_802_11_MAC_ADDRESS macadd;
        HostCmd_DS_802_11_KEY_MATERIAL keymaterial;
        HostCmd_DS_MAC_REG_ACCESS macreg;
        HostCmd_DS_BBP_REG_ACCESS bbpreg;
        HostCmd_DS_RF_REG_ACCESS rfreg;
        HostCmd_DS_802_11_CAL_DATA_EXT caldataext;
        HostCmd_DS_802_11_HOST_SLEEP_CFG hostsleepcfg;
        HostCmd_DS_802_11_EEPROM_ACCESS rdeeprom;

        HostCmd_DS_802_11D_DOMAIN_INFO domaininfo;
        HostCmd_DS_802_11D_DOMAIN_INFO_RSP domaininforesp;
        HostCmd_DS_802_11_TPC_ADAPT_REQ tpcReq;
        HostCmd_DS_802_11_TPC_INFO tpcInfo;
        HostCmd_DS_802_11_CHAN_SW_ANN chan_sw_ann;
        HostCmd_DS_MEASUREMENT_REQUEST meas_req;
        HostCmd_DS_MEASUREMENT_REPORT meas_rpt;
        HostCmd_DS_802_11_BG_SCAN_CONFIG bgscancfg;
        HostCmd_DS_802_11_BG_SCAN_QUERY bgscanquery;
        HostCmd_DS_802_11_BG_SCAN_QUERY_RSP bgscanqueryresp;
        HostCmd_DS_WMM_GET_STATUS getWmmStatus;
        HostCmd_DS_WMM_ADDTS_REQ addTsReq;
        HostCmd_DS_WMM_DELTS_REQ delTsReq;
        HostCmd_DS_WMM_QUEUE_CONFIG queueConfig;
        HostCmd_DS_WMM_QUEUE_STATS queueStats;
        HostCmd_DS_TX_PKT_STATS txPktStats;
        HostCmd_DS_802_11_SLEEP_PARAMS sleep_params;
        HostCmd_DS_802_11_BCA_TIMESHARE bca_timeshare;
        HostCmd_DS_802_11_INACTIVITY_TIMEOUT inactivity_timeout;
        HostCmd_DS_802_11_SLEEP_PERIOD ps_sleeppd;
        HostCmd_DS_802_11_TPC_CFG tpccfg;
        HostCmd_DS_802_11_LED_CTRL ledgpio;
        HostCmd_DS_802_11_FW_WAKEUP_METHOD fwwakeupmethod;

        HostCmd_DS_802_11_CRYPTO crypto;
        HostCmd_TX_RATE_QUERY txrate;
        HostCmd_DS_GET_TSF gettsf;
        HostCmd_DS_802_11_IBSS_Status ibssCoalescing;
        HostCmd_DS_SDIO_PULL_CTRL sdiopullctl;
        HostCmd_DS_MODULE_TYPE_CONFIG moduletypecfg;
        HostCmd_DS_VERSION_EXT verext;
        HostCmd_DS_MEF_CFG mefcfg;
        HostCmd_DS_DBGS_CFG dbgcfg;
        HostCmd_DS_GET_MEM getmem;
    } params;
} __ATTRIB_PACK__;

#ifndef MIN
#define MIN(a,b)		((a) < (b) ? (a) : (b))
#endif

#ifndef MAX
#define MAX(a,b)		((a) > (b) ? (a) : (b))
#endif

#ifndef NELEMENTS
#define NELEMENTS(x) (sizeof(x)/sizeof(x[0]))
#endif

/** Buffer Constants */

/*	The size of SQ memory PPA, DPA are 8 DWORDs, that keep the physical
*	addresses of TxPD buffers. Station has only 8 TxPD available, Whereas
*	driver has more local TxPDs. Each TxPD on the host memory is associated 
*	with a Tx control node. The driver maintains 8 RxPD descriptors for 
*	station firmware to store Rx packet information.
*
*	Current version of MAC has a 32x6 multicast address buffer.
*
*	802.11b can have up to  14 channels, the driver keeps the
*	BSSID(MAC address) of each APs or Ad hoc stations it has sensed.
*/

#define MRVDRV_SIZE_OF_PPA		0x00000008
#define MRVDRV_SIZE_OF_DPA		0x00000008
#define MRVDRV_NUM_OF_TxPD		0x00000020
#define MRVDRV_NUM_OF_CMD_BUFFER        10
#define MRVDRV_SIZE_OF_CMD_BUFFER       (2 * 1024)
#define MRVDRV_MAX_BSSID_LIST		64
#define MRVDRV_TIMER_10S		10000
#define MRVDRV_TIMER_5S			5000
#define MRVDRV_TIMER_1S			1000
#define MRVDRV_SNAP_HEADER_LEN          8
#define MRVDRV_ETH_HEADER_SIZE          14

#define ARP_FILTER_MAX_BUF_SIZE		20

#define	WLAN_UPLD_SIZE			2312
#define DEV_NAME_LEN			32

#ifndef	ETH_ALEN
#define ETH_ALEN			6
#endif

/** Misc constants */
/* This section defines 802.11 specific contants */
#define SDIO_HEADER_LEN		4

#define MRVDRV_MAX_REGION_CODE			7

#define MRVDRV_MAX_SSID_LIST_LENGTH     10

#define MRVDRV_IGNORE_MULTIPLE_DTIM		0xfffe
#define MRVDRV_MIN_MULTIPLE_DTIM		1
#define MRVDRV_MAX_MULTIPLE_DTIM		5
#define MRVDRV_DEFAULT_MULTIPLE_DTIM		1

#define MRVDRV_DEFAULT_LISTEN_INTERVAL		10
#define MRVDRV_DEFAULT_LOCAL_LISTEN_INTERVAL		0

#define	MRVDRV_CHANNELS_PER_ACTIVE_SCAN		14
#define MRVDRV_MIN_BEACON_INTERVAL		20
#define MRVDRV_MAX_BEACON_INTERVAL		1000
#define MRVDRV_BEACON_INTERVAL			100

#define MRVDRV_DEFAULT_WATCHDOG_TIMEOUT (2 * HZ)
#define MRVDRV_SCAN_WATCHDOG_TIMEOUT    (10 * HZ)
#define MRVDRV_DEEP_SLEEP_EXIT_TIMEOUT  (10 * HZ)

/** TxPD Status */

/*	Station firmware use TxPD status field to report final Tx transmit
*	result, Bit masks are used to present combined situations.
*/

#define MRVDRV_TxPD_POWER_MGMT_NULL_PACKET 0x01
#define MRVDRV_TxPD_POWER_MGMT_LAST_PACKET 0x08

/** Tx control node status */

#define MRVDRV_TX_CTRL_NODE_STATUS_IDLE      0x0000

/* Link spped */
#define MRVDRV_LINK_SPEED_1mbps          10000  /* in unit of 100bps */
#define MRVDRV_LINK_SPEED_11mbps         110000

/** RSSI-related defines */
/*	RSSI constants are used to implement 802.11 RSSI threshold 
*	indication. if the Rx packet signal got too weak for 5 consecutive
*	times, miniport driver (driver) will report this event to wrapper
*/

#define MRVDRV_NF_DEFAULT_SCAN_VALUE		(-96)

/** RTS/FRAG related defines */
#define MRVDRV_RTS_MIN_VALUE		0
#define MRVDRV_RTS_MAX_VALUE		2347
#define MRVDRV_FRAG_MIN_VALUE		256
#define MRVDRV_FRAG_MAX_VALUE		2346

/* Fixed IE size is 8 bytes time stamp + 2 bytes beacon interval +
 * 2 bytes cap */
#define MRVL_FIXED_IE_SIZE      12

/* This is for firmware specific length */
#define EXTRA_LEN	36
#define MRVDRV_MAXIMUM_ETH_PACKET_SIZE	1514

#define PXA3XX_DMA_ALIGNMENT	8

#define MRVDRV_ETH_TX_PACKET_BUFFER_SIZE \
	(MRVDRV_MAXIMUM_ETH_PACKET_SIZE + sizeof(TxPD) + EXTRA_LEN)

#define MRVDRV_ETH_RX_PACKET_BUFFER_SIZE \
	(MRVDRV_MAXIMUM_ETH_PACKET_SIZE + sizeof(RxPD) \
	 + MRVDRV_SNAP_HEADER_LEN + EXTRA_LEN)

#define	CMD_F_HOSTCMD		(1 << 0)

/* to resolve CISCO AP extension */
#define MRVDRV_SCAN_LIST_VAR_IE_SPACE  	256
#define FW_IS_WPA_ENABLED(_adapter) \
		(_adapter->fwCapInfo & FW_CAPINFO_WPA)

#define FW_CAPINFO_WPA  	(1 << 0)
#define WLAN_802_11_AI_REQFI_CAPABILITIES 	1
#define WLAN_802_11_AI_REQFI_LISTENINTERVAL 	2
#define WLAN_802_11_AI_REQFI_CURRENTAPADDRESS 	4

#define WLAN_802_11_AI_RESFI_CAPABILITIES 	1
#define WLAN_802_11_AI_RESFI_STATUSCODE 	2
#define WLAN_802_11_AI_RESFI_ASSOCIATIONID 	4

#define MRVL_NUM_WEP_KEY		4

/** WPA Key LENGTH*/
/* Support 4 keys per key set */
#define MRVL_NUM_WPA_KEY_PER_SET        4
#define MRVL_MAX_WPA_KEY_LENGTH 	32

#define WPA_AES_KEY_LEN 		16
#define WPA_TKIP_KEY_LEN 		32

/* A few details needed for WEP (Wireless Equivalent Privacy) */
/* 104 bits */
#define MAX_WEP_KEY_SIZE	13
/*40 bits RC4 - WEP*/
#define MIN_WEP_KEY_SIZE	5

#define RF_ANTENNA_1		0x1
#define RF_ANTENNA_2		0x2
#define RF_ANTENNA_AUTO		0xFFFF

#define KEY_INFO_ENABLED	0x01

#define SNR_BEACON		0
#define SNR_RXPD		1
#define NF_BEACON		2
#define NF_RXPD			3

/** MACRO DEFINITIONS */
#define CAL_NF(NF)			((s32)(-(s32)(NF)))
#define CAL_RSSI(SNR, NF) 		((s32)((s32)(SNR) + CAL_NF(NF)))
#define SCAN_RSSI(RSSI)			(0x100 - ((u8)(RSSI)))

#define DEFAULT_BCN_AVG_FACTOR		8
#define DEFAULT_DATA_AVG_FACTOR		8
#define MIN_BCN_AVG_FACTOR		1
#define MAX_BCN_AVG_FACTOR		8
#define MIN_DATA_AVG_FACTOR		1
#define MAX_DATA_AVG_FACTOR		8
#define AVG_SCALE			100
#define CAL_AVG_SNR_NF(AVG, SNRNF, N)         \
                        (((AVG) == 0) ? ((u16)(SNRNF) * AVG_SCALE) : \
                        ((((int)(AVG) * (N -1)) + ((u16)(SNRNF) * \
                        AVG_SCALE))  / N))

#define WLAN_STATUS_SUCCESS			(0)
#define WLAN_STATUS_FAILURE			(-1)
#define WLAN_STATUS_NOT_ACCEPTED                (-2)

#define	MAX_LEDS			3
#define	LED_DISABLED			16
#define	LED_BLINKING			2

/* S_SWAP : To swap 2 u8 */
#define S_SWAP(a,b) 	do { \
				u8  t = SArr[a]; \
				SArr[a] = SArr[b]; SArr[b] = t; \
			} while(0)

/* SWAP: swap u8 */
#define SWAP_U8(a,b)	{u8 t; t=a; a=b; b=t;}

/* SWAP: swap u8 */
#define SWAP_U16(a,b)	{u16 t; t=a; a=b; b=t;}

#define wlan_le16_to_cpu(x) x
#define wlan_le32_to_cpu(x) x
#define wlan_le64_to_cpu(x) x
#define wlan_cpu_to_le16(x) x
#define wlan_cpu_to_le32(x) x
#define wlan_cpu_to_le64(x) x

#define endian_convert_TxPD(x)
#define endian_convert_RxPD(x)
#define endian_convert_GET_LOG(x)

/** Global Varibale Declaration */
typedef struct _wlan_private wlan_private;
typedef struct _wlan_adapter wlan_adapter;
typedef struct _HostCmd_DS_COMMAND HostCmd_DS_COMMAND;

extern u32 DSFreqList[15];
extern const char driver_version[];
extern u32 DSFreqList[];
extern u16 RegionCodeToIndex[MRVDRV_MAX_REGION_CODE];

extern u8 WlanDataRates[WLAN_SUPPORTED_RATES];

extern u8 SupportedRates_B[B_SUPPORTED_RATES];
extern u8 SupportedRates_G[G_SUPPORTED_RATES];
extern u8 SupportedRates_A[A_SUPPORTED_RATES];
extern u8 AdhocRates_A[A_SUPPORTED_RATES];

extern u8 AdhocRates_G[G_SUPPORTED_RATES];

extern u8 AdhocRates_B[4];
extern wlan_private *wlanpriv;

#ifdef MFG_CMD_SUPPORT
#define SIOCCFMFG SIOCDEVPRIVATE
#endif /* MFG_CMD_SUPPORT */

/** ENUM definition*/
/** SNRNF_TYPE */
typedef enum _SNRNF_TYPE
{
    TYPE_BEACON = 0,
    TYPE_RXPD,
    MAX_TYPE_B
} SNRNF_TYPE;

/** SNRNF_DATA*/
typedef enum _SNRNF_DATA
{
    TYPE_NOAVG = 0,
    TYPE_AVG,
    MAX_TYPE_AVG
} SNRNF_DATA;

/** WLAN_802_11_AUTH_ALG*/
typedef enum _WLAN_802_11_AUTH_ALG
{
    AUTH_ALG_OPEN_SYSTEM = 1,
    AUTH_ALG_SHARED_KEY = 2,
    AUTH_ALG_NETWORK_EAP = 8,
} WLAN_802_11_AUTH_ALG;

/** WLAN_802_11_ENCRYPTION_MODE */
typedef enum _WLAN_802_11_ENCRYPTION_MODE
{
    CIPHER_NONE,
    CIPHER_WEP40,
    CIPHER_TKIP,
    CIPHER_CCMP,
    CIPHER_WEP104,
} WLAN_802_11_ENCRYPTION_MODE;

/** WLAN_802_11_POWER_MODE */
typedef enum _WLAN_802_11_POWER_MODE
{
    Wlan802_11PowerModeCAM,
    Wlan802_11PowerModeMAX_PSP,
    Wlan802_11PowerModeFast_PSP,

    /*not a real mode, defined as an upper bound */
    Wlan802_11PowerModeMax
} WLAN_802_11_POWER_MODE;

/** PS_STATE */
typedef enum _PS_STATE
{
    PS_STATE_FULL_POWER,
    PS_STATE_AWAKE,
    PS_STATE_PRE_SLEEP,
    PS_STATE_SLEEP
} PS_STATE;

/** DNLD_STATE */
typedef enum _DNLD_STATE
{
    DNLD_RES_RECEIVED,
    DNLD_DATA_SENT,
    DNLD_CMD_SENT
} DNLD_STATE;

/** WLAN_MEDIA_STATE */
typedef enum _WLAN_MEDIA_STATE
{
    WlanMediaStateDisconnected,
    WlanMediaStateConnected
} WLAN_MEDIA_STATE;

/** WLAN_802_11_PRIVACY_FILTER */
typedef enum _WLAN_802_11_PRIVACY_FILTER
{
    Wlan802_11PrivFilterAcceptAll,
    Wlan802_11PrivFilter8021xWEP
} WLAN_802_11_PRIVACY_FILTER;

/** mv_ms_type */
typedef enum _mv_ms_type
{
    MVMS_DAT = 0,
    MVMS_CMD = 1,
    /* 2: reserved */
    MVMS_EVENT = 3
} mv_ms_type;

/* Hardware status codes */
typedef enum _WLAN_HARDWARE_STATUS
{
    WlanHardwareStatusReady,
    WlanHardwareStatusInitializing,
    WlanHardwareStatusReset,
    WlanHardwareStatusClosing,
    WlanHardwareStatusNotReady
} WLAN_HARDWARE_STATUS;

/** WLAN_802_11_AUTHENTICATION_MODE */
typedef enum _WLAN_802_11_AUTHENTICATION_MODE
{
    Wlan802_11AuthModeOpen = 0x00,
    Wlan802_11AuthModeShared = 0x01,
    Wlan802_11AuthModeNetworkEAP = 0x80,
} WLAN_802_11_AUTHENTICATION_MODE;

/** WLAN_802_11_WEP_STATUS */
typedef enum _WLAN_802_11_WEP_STATUS
{
    Wlan802_11WEPEnabled,
    Wlan802_11WEPDisabled,
    Wlan802_11WEPKeyAbsent,
    Wlan802_11WEPNotSupported
} WLAN_802_11_WEP_STATUS;

/** SNMP_MIB_INDEX_e */
typedef enum _SNMP_MIB_INDEX_e
{
    DesiredBssType_i = 0,
    OpRateSet_i,
    BcnPeriod_i,
    DtimPeriod_i,
    AssocRspTimeOut_i,
    RtsThresh_i,
    ShortRetryLim_i,
    LongRetryLim_i,
    FragThresh_i,
    Dot11D_i,
    Dot11H_i,
    ManufId_i,
    ProdId_i,
    ManufOui_i,
    ManufName_i,
    ManufProdName_i,
    ManufProdVer_i
} SNMP_MIB_INDEX_e;

/** KEY_TYPE_ID */
typedef enum _KEY_TYPE_ID
{
    KEY_TYPE_ID_WEP = 0,
    KEY_TYPE_ID_TKIP,
    KEY_TYPE_ID_AES
} KEY_TYPE_ID;

/** KEY_INFO_WEP*/
typedef enum _KEY_INFO_WEP
{
    KEY_INFO_WEP_DEFAULT_KEY = 0x01
} KEY_INFO_WEP;

/** KEY_INFO_TKIP */
typedef enum _KEY_INFO_TKIP
{
    KEY_INFO_TKIP_MCAST = 0x01,
    KEY_INFO_TKIP_UNICAST = 0x02,
    KEY_INFO_TKIP_ENABLED = 0x04
} KEY_INFO_TKIP;

/** KEY_INFO_AES*/
typedef enum _KEY_INFO_AES
{
    KEY_INFO_AES_MCAST = 0x01,
    KEY_INFO_AES_UNICAST = 0x02,
    KEY_INFO_AES_ENABLED = 0x04
} KEY_INFO_AES;

/** SNMP_MIB_VALUE_e */
typedef enum _SNMP_MIB_VALUE_e
{
    SNMP_MIB_VALUE_INFRA = 1,
    SNMP_MIB_VALUE_ADHOC
} SNMP_MIB_VALUE_e;

/** HWRateDropMode */
typedef enum _HWRateDropMode
{
    NO_HW_RATE_DROP,
    HW_TABLE_RATE_DROP,
    HW_SINGLE_RATE_DROP
} HWRateDropMode;

#ifdef __KERNEL__
extern struct iw_handler_def wlan_handler_def;
struct iw_statistics *wlan_get_wireless_stats(struct net_device *dev);
int wlan_do_ioctl(struct net_device *dev, struct ifreq *req, int i);
#endif

#endif /* _WLAN_FW_H */
